--[[----------------------------------------------------------------------------

Bablu.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author erwright 2011-01-31

@reviewed epoley 2011-03-08
@reviewed mhamburg 2011-03-11

------------------------------------------------------------------------------]]

local WFWorkerThread = require 'WFWorkerThread'

local KSPrefs = require 'KSPrefs'
						
local BabluEP = WFWorkerThread( "BabluImp.lua" )
						
--------------------------------------------------------------------------------

local Bablu = {}

--------------------------------------------------------------------------------

-- FIXME [mhamburg 2011-03-11]: We obviously have a repeated pattern here that
-- probably should get standardized into a utility routine or routines. No
-- nominations for that code at this time, however.

--==============================================================================

-- Behavior for tracking whether auto-sync with Oz is enabled

function Bablu:setOzAutoSyncEnabled( isEnabled )

	BabluEP:requestP( 'setOzAutoSyncEnabled', isEnabled ):runUntilDone()
	
end

--------------------------------------------------------------------------------

local ozAutoSyncEnabledB = nil

function Bablu:getOzAutoSyncEnabledB()

	if not ozAutoSyncEnabledB then
		ozAutoSyncEnabledB = BabluEP:requestB {
			message = 'getOzAutoSyncEnabled',
			startingValue = WFWorkerThread.kNilValue,
		}
	end

	return ozAutoSyncEnabledB:relayB()
	
end

--------------------------------------------------------------------------------

function Bablu:getImsClientIdAndSecretP()
	
	return BabluEP:requestP( 'getImsClientIdAndSecret' )
	
end

--------------------------------------------------------------------------------

-- Behavior for tracking whether original image download is enabled

function Bablu:setOzDownloadOriginalsEnabled( isEnabled )

	BabluEP:requestP( 'setOzDownloadOriginalsEnabled', isEnabled ):runUntilDone()
	
end

local ozDownloadOriginalsEnabledB = nil

function Bablu:getOzDownloadOriginalsEnabledB()

	if not ozDownloadOriginalsEnabledB then
		ozDownloadOriginalsEnabledB = BabluEP:requestB {
			message = 'getOzDownloadOriginalsEnabled',
			startingValue = WFWorkerThread.kNilValue,
		}
	end

	return ozDownloadOriginalsEnabledB:relayB()
	
end

--------------------------------------------------------------------------------
	
--[[

	get/setAutoImportCatalogForUser are the public APIs the Bablu exposes
	for handling auto-import catalog preferences.
	
	This is saved as a KSPrefs key per userId.
	
]]

--------------------------------------------------------------------------------

-- userId must be a non-empty string
-- catalogId can be either a string or nil

function Bablu:setAutoImportCatalogForUser( userId, catalogId )

	assert( type( userId ) == 'string' )
	assert( type( catalogId ) == 'string' or catalogId == nil )
	
	local autoImportPrefsKey = "AutoImportCatalogId" .. userId
	KSPrefs[ autoImportPrefsKey ] = catalogId
	
end

--------------------------------------------------------------------------------

function Bablu:getAutoImportCatalogForUser( userId )

	assert( type( userId ) == 'string' )

	local autoImportPrefsKey = "AutoImportCatalogId" .. userId
	return KSPrefs[ autoImportPrefsKey ]
	
end

--------------------------------------------------------------------------------

function Bablu:getAutoImportCatalogForUserB( userId )

	assert( type( userId ) == 'string' )

	local autoImportPrefsKey = "AutoImportCatalogId" .. userId
	return KSPrefs.prefB( autoImportPrefsKey )
	
end

--------------------------------------------------------------------------------

local purgeStatusReqs = {}
local doPurgeReqs = {}

function Bablu:isNowOrWillBePurgingB( catalogId )
	
	if not purgeStatusReqs[ catalogId ] then
		purgeStatusReqs[ catalogId ] = BabluEP:requestB {
			message = 'isNowOrWillBePurgingB',
			arguments = catalogId,
			startingValue = WFWorkerThread.kNilValue,
		}
	end
	
	return purgeStatusReqs[ catalogId ]:relayB()
	
end

--------------------------------------------------------------------------------

function Bablu:purgeAllEligible( catalogId )
	
	if not doPurgeReqs[ catalogId ] then
		doPurgeReqs[ catalogId ] = BabluEP:requestE( 'purgeAllEligible', catalogId )
		doPurgeReqs[ catalogId ]:onDataE( function( data )
			if data == nil then return end
			doPurgeReqs[ catalogId ]:close()
			doPurgeReqs[ catalogId ] = nil
		end )
	end

end

return Bablu
