--[[----------------------------------------------------------------------------

BabluImp.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author erwright 2011-02-15

@reviewed scouten 2011-03-02

@reviewed epoley 2011-03-08
@reviewed mhamburg 2011-03-11

------------------------------------------------------------------------------]]

local KSReactive = require 'KSReactive'

local KSPrefs = require 'KSPrefs'

local WFRequestDispatcher = require 'WFRequestDispatcher'
local WFPlatformInfo = require 'WFPlatformInfo'

local BabluImsInfoImp = require 'BabluImsInfoImp'

local assert = assert


--------------------------------------------------------------------------------

require 'OzBackupFlagAgentManager'.installMessageHandlers()
require 'WFOzPurgeAgentManager'.installMessageHandlers()
require 'OzSyncManager'.installMessageHandlers()
require 'OzOriginalsAgentManager'.installMessageHandlers()

--------------------------------------------------------------------------------

-- Messages for getting/setting the Oz auto-sync value

--debug.print( "Read KSPrefs['OzAutoSyncEnabled']: ", KSPrefs[ "OzAutoSyncEnabled" ] )
local ozAutoSyncEnabled = KSPrefs[ "OzAutoSyncEnabled" ]
if ozAutoSyncEnabled == nil then
	-- if this is first-launch, set auto-sync to be enabled
	ozAutoSyncEnabled = true
end

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessageNoCancel( 'setOzAutoSyncEnabled', function( isEnabled )

	assert( type( isEnabled ) == 'boolean' )
	
	ozAutoSyncEnabled = isEnabled
	KSPrefs.OzAutoSyncEnabled = isEnabled
	
end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessage( 'getOzAutoSyncEnabled', function()

	return ozAutoSyncEnabled
	
end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessage( 'getImsClientIdAndSecret', function()

	return BabluImsInfoImp.getImsClientIdAndSecretP()
	
end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessage( 'setImsClientIdAndSecret', function( ... )

	return BabluImsInfoImp.setImsClientIdAndSecret( ... )
	
end )

--------------------------------------------------------------------------------

-- Messages for getting/setting the original image download setting

--debug.print( "Read KSPrefs['DownloadOriginals']: ", KSPrefs[ "DownloadOriginals" ] )
local downloadOriginals = KSPrefs[ "DownloadOriginals" ]
if downloadOriginals == nil then
	-- if this is first-launch, set original image download to true on Mac, false on iOS
	downloadOriginals = WFPlatformInfo.isMac and true or false
	KSPrefs[ "DownloadOriginals" ] = downloadOriginals
end

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessageNoCancel( 'setOzDownloadOriginalsEnabled', function( isEnabled )

	assert( type( isEnabled ) == 'boolean' )
	
	KSPrefs.DownloadOriginals = isEnabled
	
end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessage( 'getOzDownloadOriginalsEnabled', function()

	return KSPrefs.prefB( 'DownloadOriginals' )

end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessage( 'uploadP', function( args )

	local OzCentralUploadTask = require 'OzCentralUploadTask'
	local OzUser = require 'OzUser'
	local OzDocStore = require 'OzDocStore'
	
	args.ozUser = OzUser.make( args.ozUserId )
	args.docStore = OzDocStore( args.catalogId )
	return OzCentralUploadTask.uploadP( args )

end )

--------------------------------------------------------------------------------

local hackPhotoBinContentsB = KSReactive.variableB( {} )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessage( 'photoBinContentsE', function()

	return hackPhotoBinContentsB

end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessageNoCancel( 'setPhotoBinContents', function( newBinContents )

	hackPhotoBinContentsB:set( newBinContents or {} )

end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessage( 'DEBUG_dumpCoverageP', function( path )

	require 'SetupCoverage'.dumpCoverage( path )

	return require 'KSPromises'.resolvedPromiseP( 'data', true )

end )