--[[----------------------------------------------------------------------------

KSAssetsLibraryImportService.lua

@author arahn 2012-10-24

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

FIXME [erwright 2011-05-5]: future work on import: we need to revisit the overall
import structure for handling assets from the asset library, data straight from
the camera sensor, and filesystem files on the desktop.  Are file paths and
camera sensor data two distinct import paths?  Or are file paths just a high-level
layer on top of a core data-import component?

------------------------------------------------------------------------------]]

local KSPromises = require "KSPromises"
local KSIdGenerator = require "KSIdGenerator"
local KSAssetsLibraryImportServiceImp = require "KSAssetsLibraryImportServiceImp"
local KSClass = require "KSClass"
local WFCloseSupport = require "WFCloseSupport"

--==============================================================================

local AutoImportEnumerationResolver = KSClass "AutoImportEnumerationResolver" {
	__init = function()
	end,
	
	stopResolution = function( self )
		self:close()
	end,
	
	close = function( self )
		if self:canResolvePromise() then
			self:resolvePromise( 'error', 'canceled' )
		end
	end,
	
	KSPromises.addPromiseResolverMethods,
	WFCloseSupport.addCloseSupportMethods,
}

--==============================================================================

local assetCallbacks = {}
local autoImportEnumerationResolvers = { }

--------------------------------------------------------------------------------

local function makeAssetDataRequestP( assetUrl )
	
	local assetPromise, assetResolver = KSPromises.promiseP()
		
	local requestId = KSIdGenerator.uuid()
		
	KSAssetsLibraryImportServiceImp.getAsset( requestId, assetUrl )
	
	assetCallbacks[ requestId ] = assetResolver
		
	return assetPromise
		
end

--------------------------------------------------------------------------------

local function getAssetsListP( importHandler )

	local requestId = KSIdGenerator.uuid()
	local resolver = AutoImportEnumerationResolver()
	autoImportEnumerationResolvers[ requestId ] = resolver
	
	KSAssetsLibraryImportServiceImp.getAssetsList( requestId )
	
	return KSPromises.promiseForResolverP( resolver, 'retained | closeTarget' )
	
end

--------------------------------------------------------------------------------

if KSAssetsLibraryImportServiceImp then

	KSAssetsLibraryImportServiceImp.registerGetAssetCompletionHandler( function ( requestId, ... )
	
		local resolver = assetCallbacks[ requestId ]
		
		if resolver and resolver:canResolvePromise() then

			assetCallbacks[ requestId ] = nil
			
			resolver( ... )

		end
				
	end )
	
	KSAssetsLibraryImportServiceImp.registerGetAssetListCompletionHandler( function( requestId, ... ) 
		
		local enumerationResolver = autoImportEnumerationResolvers[ requestId ]
		if not enumerationResolver or not enumerationResolver:canResolvePromise() then
			return
		end
		
		autoImportEnumerationResolvers[ requestId ] = nil
		enumerationResolver:resolvePromise( ... )
		
	end )
	
end

--------------------------------------------------------------------------------

return {
	makeAssetDataRequestP = makeAssetDataRequestP,
	getAssetsListP = getAssetsListP,
}