local WFAsyncTestCase = require "WFAsyncTestCase"
local WFIdle = require "WFIdle"

--==============================================================================

local wantAsset
local wantAssetList

local function buildMocks()
	
	local getAssetCompletionHandler, getAssetListCompletionHandler
	
	return {
		KSAssetsLibraryImportServiceImp = {
			getAsset = function( requestId, assetUrl )
				
				local assetResult = wantAsset
				
				WFIdle.doLater( function()
					getAssetCompletionHandler( requestId, unpack( assetResult ) )
				end )
				
			end,
			
			registerGetAssetCompletionHandler = function( fn )
				
				getAssetCompletionHandler = fn
				
			end,
			
			getAssetsList = function( requestId )
				
				local result = wantAssetList
				
				WFIdle.doLater( function()
					getAssetListCompletionHandler( requestId, unpack( result ) )
				end )
			
			end,
			
			registerGetAssetListCompletionHandler = function( fn )
				
				getAssetListCompletionHandler = fn
				
			end,
		}
	}
end

local KSAssetsLibraryImportService

local TestSuite = {}

--==============================================================================
-- @mark setup
--==============================================================================

function TestSuite.setUpClass()

	local LOADED = debug.getregistry()[ "_LOADED" ]

	for k, namespace in pairs( buildMocks() ) do
		LOADED[ k ] = namespace	
	end
	
	LOADED.KSAssetsLibraryImportService = nil
	KSAssetsLibraryImportService = require 'KSAssetsLibraryImportService'

end

--------------------------------------------------------------------------------

function TestSuite.tearDownClass()

	local LOADED = debug.getregistry()[ "_LOADED" ]

	for k, namespace in pairs( buildMocks() ) do
		LOADED[ k ] = nil	
	end
	
	LOADED.KSAssetsLibraryImportService = nil
	
end

--==============================================================================

function TestSuite.testZero()
	-- Here so I could see what coverage is like when simply requiring the namespace
end

--------------------------------------------------------------------------------

function TestSuite.testGetAsset( test )
	
	local asset = { filename = "filename" }
	wantAsset = { 'data', asset }
	local assetListP = KSAssetsLibraryImportService:makeAssetDataRequestP( "asset-url:/this/is?totally=fake" )
	
	WFAsyncTestCase( test )
		:waitFor( assetListP )
		:continueWhenValueMatches( 'data', asset )
	
end

--------------------------------------------------------------------------------

function TestSuite.testGetAssetError( test )
	
	wantAsset = { 'error', "error message" }
	local assetListP = KSAssetsLibraryImportService:makeAssetDataRequestP( "asset-url:/this/is?totally=fake" )
	
	WFAsyncTestCase( test )
		:waitFor( assetListP )
		:continueWhenValueMatches( 'error', "error message" )
	
end

--------------------------------------------------------------------------------

function TestSuite.testGetAssetList( test )
	
	local assets = { { url = "test" } }
	wantAssetList = { 'data', assets }
	
	WFAsyncTestCase( test )
		:waitFor( KSAssetsLibraryImportService:getAssetsListP() )
		:continueWhenValueMatches( 'data', assets )
	
end

--------------------------------------------------------------------------------

function TestSuite.testGetAssetListError( test )
	
	wantAssetList = { 'error', "error message" }
	
	WFAsyncTestCase( test )
		:waitFor( KSAssetsLibraryImportService:getAssetsListP() )
		:continueWhenValueMatches( 'error', "error message" )
	
end

--==============================================================================

return TestSuite