--[[----------------------------------------------------------------------------

KSDataLuaTest.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author scouten 2011-01-31

@reviewed aksherry 2011-03-08

------------------------------------------------------------------------------]]

local KSData = require 'KSData'
	
local KSDataLuaTest = {}

--------------------------------------------------------------------------------

function KSDataLuaTest.testKSDataNamespace()

	local data = assert( KSData.make( "123abc" ) )
	
	assert( KSData.isData( data ) == true, "isData returned false for data" )
	assert( KSData.isData( "xyz" ) == false, "isData returned true for string" )
	assert( KSData.isData( 19 ) == false, "isData returned true for number" )
	assert( KSData.isData( nil ) == false, "isData returned true for nil" )

	assert( tostring( data ) == "123abc", "tostring not overloaded" )
	
	assert( KSData.length( data ) == 6, "length returned incorrect result" )
	assert( KSData.length( "xyz" ) == 0, "isData returned non-zero for string" )
	assert( KSData.length( 19 ) == 0, "isData returned non-zero for number" )
	assert( KSData.length( nil ) == 0, "isData returned non-zero for nil" )

	data:close()

end

--------------------------------------------------------------------------------

function KSDataLuaTest.testMakeFromData()

	local stringData = assert( KSData.make( "123abc" ) )
	
	local data = assert( KSData.make( stringData ) )
	
	assert( KSData.isData( data ) == true, "isData returned false for data" )
	assert( KSData.isData( "xyz" ) == false, "isData returned true for string" )
	assert( KSData.isData( 19 ) == false, "isData returned true for number" )
	assert( KSData.isData( nil ) == false, "isData returned true for nil" )

	assert( tostring( data ) == "123abc", "tostring not overloaded" )
	
	assert( KSData.length( data ) == 6, "length returned incorrect result" )
	assert( KSData.length( "xyz" ) == 0, "isData returned non-zero for string" )
	assert( KSData.length( 19 ) == 0, "isData returned non-zero for number" )
	assert( KSData.length( nil ) == 0, "isData returned non-zero for nil" )
	
	stringData:close()
	data:close()
	
end

--------------------------------------------------------------------------------

function KSDataLuaTest.testDigest( test )
	
	-- digest values tested against mac os command line tool md5 and shasum
	
	local stringData = assert( KSData.make( "123abc" ) )
	
	local digests = {
		MD5 = { "a906449d5769fa7361d7ecc6aa3f6d28", "422a1740794ba84190fe6e4d178f5c0d" },
		SHA1 = { "4be30d9814c6d4e9800e0d2ea9ec9fb00efa887b", "a6e6f03a0c8a4e6c8096dda7c7ab199b7edcc850" },
		SHA256 = {
			"dd130a849d7b29e5541b05d2f7f86a4acd4f1ec598c1c9438783f56bc4f0ff80",
			"ebc83f059131be564238b82207818f5d843a812155d9b85efb640a445a8ba1ce" },
		SHA384 = {
			"40889bd7aee5be35f9d713c4f51ff00fe2099fd901d094284d6edc9aa39ec096cebe9547d3cbaf1f35101a6489f033c3",
			"c86ee2fac94384f2225ffc3b399fa2771ad67c0af3c0901a290d626c0e16b06e3143c9f9bf521252ac174340b1d8353a" },
		SHA512 = {
			"7b6ad79b346fb6951275343948e13c1b4ebca82a5452a6c5d15684377f096ca927506a23a847e6e046061399631b16fc2820c8b0e02d0ea87aa5a203a77c2a7e",
			"976ce700d2b79bf089e40305e965426505434d62c4f2bf8d6c53a04bfe002df76e53274a0b84747b79693cdc3c045aa2999b901c1ae0a8aaebe3bae35d10c1c0" },
	}
	
	local shortData = assert( KSData.make( stringData ), "Fail to make KSData" )
	local longString = string.rep( "smile!! ", 256 )
	local longData = assert( KSData.make( longString ), "Fail to make KSData" )

	for alg, expected in pairs( digests ) do

		local digest = shortData:digest( alg )
		test:log( digest )
		assert( digest == expected[ 1 ], alg .. " returned bad value for short string" )
		
		digest = longData:digest( alg )
		test:log( digest )
		assert( digest == expected[ 2 ], alg .. " returned bad value for long string" )
	
	end

	stringData:close()
	shortData:close()
	longData:close()

end

----------------------------------------------------------------------------------

function KSDataLuaTest.testMakeRejectsOtherDataTypes()

	local function assertFails( dataType, func )
	
		local pcallResults = pcall( func )
		assert( pcallResults == false, string.format( "make didn't reject %s", dataType ) )
	
	end

	assertFails( "number", function() KSData.make( 4 ) end )
	assertFails( "nil", function() KSData.make( nil ) end )
	assertFails( "table", function() KSData.make( {} ) end )

end

--------------------------------------------------------------------------------

return KSDataLuaTest
