--[[----------------------------------------------------------------------------

KSDateTest.lua

@author scouten 2011-02-17
@reviewed aksherry 2011-03-09
@reviewed scouten 2013-02-12

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- wichitafoundation.core.os
local KSDate = require 'KSDate'
local WFPlatformInfo = require 'WFPlatformInfo'

	-- wichitafoundation.core.utils
local KSTestUtils = require 'KSTestUtils'


--============================================================================--

local KSDateTest = {}

--------------------------------------------------------------------------------

function KSDateTest.testTimeNow( test )

	local timeNow = KSDate.timeNow()
	
	KSTestUtils.assertEquals( test, 'return value is wrong type' )( 'number' )( type( timeNow ) )
	
	test:log( "timeNow is " .. tostring( timeNow ) )

end

--------------------------------------------------------------------------------

function KSDateTest.testTimeNowForAnimation( test )

	local timeNowForAnimation = KSDate.timeNowForAnimation()
	
	KSTestUtils.assertEquals( test, "return value is wrong type" )( 'number' )( type( timeNowForAnimation ) )

	test:log( "timeNowForAnimation is " .. tostring( timeNowForAnimation ) )

end

--------------------------------------------------------------------------------

function KSDateTest.testTimeForIso8601Format( test )

	local result = KSDate.timeInIso8601Format( 319659344.54388 )
	
	KSTestUtils.assertEquals( test, "wrong date" )( '2011-02-17T18:15:44.544Z' )( result )

	test:log( "time at 319659344.54388 in ISO 8601 is " .. tostring( result ) )

end

--------------------------------------------------------------------------------

function KSDateTest.testAddDaysInIso8601Format( test )

	-- FIXME_WINRT: Returned date does not have fractional seconds.

	if WFPlatformInfo.isWindows then
		debug.print( "testAddDaysInIso8601Format not working on Win RT" )
		return
	end

	local result = KSDate.addDaysInIso8601Format( 319659344.54388, false, 3 )

	KSTestUtils.assertEquals( test, "date was wrong" )( '2011-02-20T18:15:44.544Z' )( result )

	test:log( "time at 319659344.54388 + 3 days in ISO 8601 is " .. tostring( result ) )

	
end

--------------------------------------------------------------------------------

function KSDateTest.testDifferenceInDays( test )

	local result = KSDate.differenceInDays( "2012-03-03T09:12:34", "2012-03-21T08:34:45" )

	KSTestUtils.assertEquals( test, "difference was wrong" )( 18 )( result )

	test:log( "differenceInDays between 2012-03-03T09:12:34 and 2012-03-21T08:34:45 is "
			  .. tostring( result ) )
	
	-- Make sure the reverse test yields a negative value...
	result = KSDate.differenceInDays( "2012-03-21T08:34:45", "2012-03-03T09:12:34" )

	KSTestUtils.assertEquals( test, "difference was wrong" )( -18 )( result )

	test:log( "differenceInDays between 2012-03-21T08:34:45 and 2012-03-03T09:12:34 is "
			  .. tostring( result ) )
	
end

--------------------------------------------------------------------------------

function KSDateTest.testDifferenceInSeconds( test )

	local result = KSDate.differenceInSeconds( "2012-06-03T09:12:34.0Z",
											   "2012-06-21T08:34:45.0Z" )

	KSTestUtils.assertEquals( test, "difference A was wrong" )( 1552931 )( result )

	test:log( "differenceInSeconds between 2012-06-03T09:12:34Z and 2012-06-21T08:34:45Z is "
			  .. tostring( result ) )
	
	if not WFPlatformInfo.isWinRT then	-- FIXME_WINRT: This test is off by one hour on WinRT.

		-- Now test difference across DST change boundary...

		result = KSDate.differenceInSeconds( "2012-03-03T09:12:34.0Z",
											 "2012-04-21T08:34:45.0Z" )

		KSTestUtils.assertEquals( test, "difference B was wrong" )( 4227731 )( result )

		test:log( "differenceInSeconds between 2012-03-03T09:12:34Z and 2012-04-21T08:34:45Z is "
				  .. tostring( result ) )
	end
	
	-- Now reverse the dates and set useLocalTime to true...
	
	result = KSDate.differenceInSeconds( "2012-06-21T08:34:45", "2012-06-03T09:12:34", true )

	KSTestUtils.assertEquals( test, "difference C was wrong" )( -1552931 )( result )

	test:log( "differenceInSeconds between 2012-06-21T08:34:45 and 2012-06-03T09:12:34 is "
			  .. tostring( result ) )
	
	if not WFPlatformInfo.isWinRT then	-- FIXME_WINRT: This test is off by one hour on WinRT.

		-- Now test difference across DST change boundary...

		result = KSDate.differenceInSeconds( "2012-04-21T08:34:45",
											 "2012-03-03T09:12:34", true )

		KSTestUtils.assertEquals( test, "difference D was wrong" )( -4227731 )( result )

		test:log( "differenceInSeconds between 2012-04-21T08:34:45 and 2012-03-03T09:12:34 is "
				  .. tostring( result ) )
	end

end

--------------------------------------------------------------------------------

function KSDateTest.testAddTimeInSeconds( test )

	-- FIXME_WINRT:

	if WFPlatformInfo.isWindows then
		debug.print( "testAddTimeInSeconds not working on Win RT" )
		return
	end

	local result = KSDate.addTimeInSeconds( "2012-06-03T09:12:34", 1552931 )

	KSTestUtils.assertEquals( test, "date was wrong" )( "2012-06-21T08:34:45" )( result )

	test:log( "addTimeInSeconds 2012-06-03T09:12:34 + 1552931 is " .. tostring( result ) )

	-- Now test adding across DST change boundary...

	result = KSDate.addTimeInSeconds( "2012-03-03T09:12:34", 4227731 )
	
	KSTestUtils.assertEquals( test, "date was wrong" )( "2012-04-21T08:34:45" )( result )

	test:log( "addTimeInSeconds 2012-03-03T09:12:34 + 1549331 is " .. tostring( result ) )

end

--------------------------------------------------------------------------------

-- FIXME arahn -> rseeler This test does not work in Minnesota
--
--function KSDateTest.testTimeZoneInIso8601Format()
--	-- FIXME_WINRT: Daylight savings time modifier should not be added to result in KSDate.timeZoneInIso8601Format()
--
--	if not WFPlatformInfo.isWindows then
--		local result = KSDate.timeZoneInIso8601Format()
--		debug.print( "timeZoneInIso8601Format is ", result )
--
--		assert( result == "-0800",
--			"timeZone was wrong" )
--	end
--end

--------------------------------------------------------------------------------

function KSDateTest.testLocalizedDiffInAllComponents( test )

	local negative, years, months, days, hours, minutes, seconds
		= KSDate.localizedDiffInAllComponents( "2012-01-01T09:12:34", "2011-12-29T08:11:33" )

	KSTestUtils.assertEquals( test, "time difference was wrong" )(
							  true,     0,     0,      3,    1,     1,       1 )(
							  negative, years, months, days, hours, minutes, seconds )

	-- Sorry for funky formatting, but Lua lint complains about ambiguous syntax
	-- if open (s are on their own lines where I'd want to put them. :-(
	
end

--------------------------------------------------------------------------------

function KSDateTest.testPoliteDate( test )

	-- FIXME_WINRT:

	if WFPlatformInfo.isWindows then
		debug.print( "testPoliteDate not working on Win RT" )
		return
	end

	-- FIXME [scouten 2013-02-12]: May not be able to run this test on Mac and
	-- even on non-US iOS devices. This test fails when the system date formats
	-- don't match the defaults provided on US English devices.
	
	if WFPlatformInfo.isMac then
		debug.print( "testPoliteDate doesn't work on customized Macs" )
		return
	end

	local result = KSDate.politeDate( 2012, 10, 7, 'short' )

	KSTestUtils.assertEquals( test, "polite date was wrong" )( "10/7/12" )( result )

	result = KSDate.politeDateFromTimestamp( "2012-12-07T14:51:29", 'medium', 'medium' )

	-- NOTE [epoley 2012-12-19]: For whatever reason, iOS 6 gives the
	-- result below with a comma after the year, but earlier iOS
	-- versions and Mac do not. Use a regex rather than a simple
	-- string equality test.
	
	result = result:gsub( "2012,", "2012" )
	
	KSTestUtils.assertEquals( test, "polite date was wrong" )( "Dec 7, 2012 2:51:29 PM" )( result )

end

--------------------------------------------------------------------------------

function KSDateTest.testFormatDateTime( test )

	local utcTime = KSDate.formatDateTimeAsUTC( 319659344 )
	local localTime = KSDate.formatDateTimeAsLocal( 319659344 )

	KSTestUtils.assertEquals( test, "formatDateTimeAsUTC was wrong" )( "2011-02-17T18:15:44" )( utcTime )
	
	-- This test does not work in Minnesota:
	-- KSTestUtils.assertEquals( test, "formatDateTimeAsLocal was wrong" )( "2011-02-17T10:15:44" )( localTime )

end

--------------------------------------------------------------------------------

return KSDateTest
