--[[----------------------------------------------------------------------------

KSimportJpegServiceMetrics.lua
Andrew Rahn
Mar 1, 2013

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

FIXME [erwright 2011-05-5]: future work on import: we need to revisit the overall
import structure for handling assets from the asset library, data straight from
the camera sensor, and filesystem files on the desktop.  Are file paths and
camera sensor data two distinct import paths?  Or are file paths just a high-level
layer on top of a core data-import component?

------------------------------------------------------------------------------]]

local WFRequestDispatcher = require "WFRequestDispatcher"

--==============================================================================

local processedQueueEntries = 0
local totalQueueEntries = 0

-- A list of callers and what percent they have responded to.
-- Only respond to every 10 percent or so...
local printImportMetricsCallers = {}

--------------------------------------------------------------------------------

local function adjustQueueEntryCounts( numProcessed, numTotal )

	processedQueueEntries = processedQueueEntries + numProcessed
	totalQueueEntries = totalQueueEntries + numTotal

end

--------------------------------------------------------------------------------

local function getNumProcessedQueueEntries()
	return processedQueueEntries
end

--------------------------------------------------------------------------------

local function getNumTotalQueueEntries()
	return totalQueueEntries
end

--------------------------------------------------------------------------------

local function resetQueueEntryCounts()
	processedQueueEntries = 0
	totalQueueEntries = 0
end

--------------------------------------------------------------------------------

-- Faciltate import metrics
local function shouldPrintMetrics ( caller , timeConsumed )


    -- Only return true every 10 percent

    if  totalQueueEntries == 0 or processedQueueEntries == 0 then
        return false
    end

    local percentDone = ( processedQueueEntries/totalQueueEntries )

    local percentage = math.floor( percentDone * 10 )/10
    
    --debug.print( "caller =  ", caller )

    local callerEntry =  printImportMetricsCallers[ caller ]

    if callerEntry == nil then
         printImportMetricsCallers[ caller ] = 0
    end

    local currentCallerPercent =  printImportMetricsCallers[ caller ]

    -- Check to see if we have a new percentage increment...

    if  currentCallerPercent ~= percentage or timeConsumed > 7.0 then
        debug.print( " percentage of import done =  ", percentage)
        printImportMetricsCallers[ caller ] = percentage
        return true
    else
        return false
    end
end

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessageDo {

    message = 'shouldPrintImportMetricsP',
    simpleResponses = true, -- No need for reactive handling

    handler = function( caller, timeConsumed )
        return shouldPrintMetrics( caller, timeConsumed )
    end,

}

--==============================================================================

return {
	adjustQueueEntryCounts = adjustQueueEntryCounts,
	getNumProcessedQueueEntries = getNumProcessedQueueEntries,
	getNumTotalQueueEntries = getNumTotalQueueEntries,
	resetQueueEntryCounts = resetQueueEntryCounts,
}
