--[[----------------------------------------------------------------------------

KSImportJpegServiceTest.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author aksherry

------------------------------------------------------------------------------]]

local WFWorkerThread = require 'WFWorkerThread'

local KSPathUtils = require 'KSPathUtils'
local KSFileUtils = require 'KSFileUtils'

local importJpegService = WFWorkerThread( "KSImportJpegServiceEPInit.lua" )

-- FIXME [jandrew 2012-05-15]: We need to uncomment this once this namespace
-- has been implemented on WinRT.
local WFPlatformInfo = require 'WFPlatformInfo'

local KSImportJpegService = require 'KSImportJpegService'

local child = KSPathUtils.child

--------------------------------------------------------------------------------

local KSImportJpegServiceTest = {}

local TEST_WAIT_TIME = 2 -- normally 2, but increase while debugging

local TEST_CATALOG_ID = 'be8d866121d24a748ce42b052dddc567'


--------------------------------------------------------------------------------
-- silence unused variables warnings

KSImportJpegServiceTest.TEST_WAIT_TIME = TEST_WAIT_TIME
KSImportJpegServiceTest.TEST_CATALOG_ID = TEST_CATALOG_ID
KSImportJpegServiceTest.importJpegService = importJpegService
KSImportJpegServiceTest.KSPathUtils = KSPathUtils

--------------------------------------------------------------------------------

local toBeClosed
	-- set of objects that should be closed when test is finished
	-- valid after setup and until tearDown()
function KSImportJpegServiceTest.setUp()
	toBeClosed = {}
end

--------------------------------------------------------------------------------

function KSImportJpegServiceTest.tearDown()

	for obj in pairs( toBeClosed ) do
		obj:close()
	end
	toBeClosed = nil
	
end

--------------------------------------------------------------------------------

function KSImportJpegServiceTest.testImportJpeg( )
-- FIXME [aksherry 2011-04-05]: We need to implement this once we have support
-- for test data in the build.  As part of the test, include image files with
-- different aspect ratios to verify they get scaled correctly.

	if WFPlatformInfo.isiOS then return end

	if WFPlatformInfo.isWinRT then return end
		
	local testDataDir = KSPathUtils.getStandardFilePath( 'testdata' )
	local importTestDataDir = child( testDataDir, "import" )
	local filename = "importTestImage";
	local jpegExtension = ".jpg"
	local imagePath = child( importTestDataDir, filename .. jpegExtension )
	
	local actualSourcePath, originalData, previewData, thumbnailData, highDpiThumbData, metadata, proxyData =
		KSImportJpegService.analyzeImageData( imagePath, nil )
	
	toBeClosed[ originalData ] = 1
	
	local tempDir = KSPathUtils.getStandardFilePath( 'temp' )
	local outputDir = child( tempDir, "KSImportJpegServiceTest_output" )
	local outputPath = child( outputDir, filename )
	
	if not KSFileUtils.exists( outputPath ) then
		KSFileUtils.mkdir( outputPath )
	end
	
	debug.print( outputPath )
	
	local file = assert( io.open( outputPath .. "_preview" .. jpegExtension, "w" ) )
    assert( file:write( tostring( previewData ) ) )
    assert( file:close( ) )
	
	file = assert( io.open( outputPath .. "_thumbnail" .. jpegExtension, "w" ) )
    assert( file:write( tostring( thumbnailData ) ) )
    assert( file:close( ) )
	
	file = assert( io.open( outputPath .. "_proxy" .. ".dng", "w" ) )
    assert( file:write( tostring( proxyData ) ) )
    assert( file:close( ) )

	file = assert( io.open( outputPath .. "_highdpithumb" .. jpegExtension, "w" ) )
    assert( file:write( tostring( highDpiThumbData ) ) )
    assert( file:close( ) )

	assert( actualSourcePath == imagePath )
	
	local KSTableArchiving = require 'KSTableArchiving'
	debug.print( "Property table:\n", KSTableArchiving.simpleTableToChunkString( metadata ) )
	
end

--------------------------------------------------------------------------------
--[[
function KSImportJpegServiceTest.testEPImportFile( test )

	-- FIXME [erwright 2011-04-05]: import only works on Mac currently.  To fix this, we need to
	-- publish testdata to the simulator/device.
	
	if not WFPlatformInfo.isMac then return end
	
	-- FIXME [erwright 2011-04-06]: feedback from scouten is to rewrite this using WFAsyncTestCase
	
	local testDataFolder = KSPathUtils.getStandardFilePath( "testdata" )
	local testFile = testDataFolder .. "/import/importTestImage.jpg"
	local requestE = importJpegService:requestE( 'importFile', TEST_CATALOG_ID, 'testUser', testFile )
	
	local importWasSuccessful = false
	
	requestE:onDataE( function( filePath, success ) 
	
		debug.print( string.format( "testImportFileE: onDataE( %s, %s )", filePath, tostring( success ) ) )
		importWasSuccessful = success

		-- FIXME [erwright 2011-04-07]: read the data back from OzDocStore and make sure we inserted
		-- what we expected
	end )
	
	requestE:onErrorE( function( errorMessage )
	
		debug.print( "testEPImportFile: onErrorE: " .. errorMessage )
		
	end )
	
	requestE:onDoneE( function()

		debug.print( "testEPImportFile: onDoneE" )

		if importWasSuccessful then
			test:notifyTestDidSucceed()
		else
			debug.print( "testEPImportFolder: got a done message without first receiving an error message" )
			test:notifyTestDidFail()
		end

	end )

	test:waitForTestToFinish( TEST_WAIT_TIME )

end

--------------------------------------------------------------------------------

function KSImportJpegServiceTest.testEPImportFolder( test )

	-- FIXME [erwright 2011-04-05]: import only works on Mac currently.  To fix this, we need to
	-- publish testdata to the simulator/device.

	if not WFPlatformInfo.isMac then return end

	-- FIXME [erwright 2011-04-06]: feedback from scouten is to rewrite this using WFAsyncTestCase

	local testDataFolder = KSPathUtils.getStandardFilePath( "testdata" )
	local testFolder = testDataFolder .. "/import"

	local requestE = importJpegService:requestE( 'importFolder', TEST_CATALOG_ID, 'testUser', testFolder )
	
	local importWasSuccessful = true
	local callbacks = 0
	
	requestE:onDataE( function( filePath, success )

		debug.print( "onDataE called" )
		callbacks = callbacks + 1
		importWasSuccessful = importWasSuccessful and success
	
	end )
	
	requestE:onErrorE( function( errorMessage )
	
		-- this error message is not expected (yet)
		debug.print( "testEPImportFolder: did not expect to receive this error: " .. errorMessage	)
		test:notifyTestDidFail()
		
	end )
	
	requestE:onDoneE( function()

		if importWasSuccessful and callbacks == 1 then
			test:notifyTestDidSucceed()
		else
			debug.print( string.format( "testEPImportFolder: failed to import folder, callbacks = %d, status = %s",
										callbacks,
										tostring(importWasSuccessful) ) )
			test:notifyTestDidFail()
		end

	end )

	test:waitForTestToFinish( TEST_WAIT_TIME )
	
end
--]]
--------------------------------------------------------------------------------
if WFPlatformInfo.isWindows or WFPlatformInfo.isAndroid then
	return KSImportJpegServiceTest
else
	return KSImportJpegServiceTest and {}
end
	
	-- FIXME [scouten 2011-06-30]: don't have a way to create a test user right now.
