--[[----------------------------------------------------------------------------

KSImportMetricsTest.lua

@author ppaurav 2012-09-04

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- services.http_client
local OzUser = require 'OzUser'

	-- wichitafoundation.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'

local OzAccountService = require 'OzAccountService'

local KSPrefs = require 'KSPrefs'

local KSOAuth2Session = require 'KSOAuth2Session'

local KSIdGenerator = require 'KSIdGenerator'

local KSJsonDecoder = require 'KSJsonDecoder'

local KSPathUtils = require 'KSPathUtils'

local WFPlatformInfo = require 'WFPlatformInfo'

local WFWorkerThread = require 'WFWorkerThread'

local importJpegService = WFWorkerThread( "KSImportJpegServiceEPInit.lua" )

local WFDiagnosticLog = require 'WFDiagnosticLog'

local KSData  = require 'KSData'

local KSPromises = require 'KSPromises'

local WFTestMetrics = require 'WFTestMetrics'

local KSJsonEncoder = require 'KSJsonEncoder'

--============================================================================--

local KSImportMetricsTest = {}

--------------------------------------------------------------------------------

local ozUserId = 'ad36762b4f2f3a1f1758914a62425e5c'
	-- user id for ppaurav+importmetrics@adobetest.com on oznext. 

local ozUser
local oAuthSession = KSOAuth2Session
local testdataOnTestMachine = "/DontTouchThisFolder/ImportMetricsTestData"
local testOutputOnTestMachine = "/DontTouchThisFolder/TestAutomationOutput"
local testMetrics

--------------------------------------------------------------------------------

local function cleanupTestAccountP()

	local catalogsToBeDeleted = {}

	local function deleteCatalog( arg )
		return ozUser:deleteCatalogP( arg.id )
	end

	return ozUser:catalogsP()
		:mapDataP( function( status, result )

			for _, catalogInfo in ipairs( result ) do

				if not string.find( catalogInfo.name, "DontTouchThisCatalog" ) then
					table.insert( catalogsToBeDeleted, catalogInfo )
				end

			end

			if next( catalogsToBeDeleted ) then

				debug.print( string.format( "Deleting %d old catalogs from "
											.. "previous test runs", #catalogsToBeDeleted ) )
				return KSPromises.allP( deleteCatalog, catalogsToBeDeleted )

			else

				return KSPromises.resolvedPromiseP( 'data', true )

			end

		end )

end

--------------------------------------------------------------------------------

function KSImportMetricsTest.setUp()

	oAuthSession = KSOAuth2Session( ozUserId )
	ozUser = OzUser.make( ozUserId )
	OzAccountService.setCurrentUser( ozUserId )	
	
end

--------------------------------------------------------------------------------

function KSImportMetricsTest.tearDown()
	
	oAuthSession:close()
	oAuthSession = nil
	
	ozUser:close()
	ozUser = nil
	
	testMetrics = nil
	
end

--------------------------------------------------------------------------------

function KSImportMetricsTest.test200Images_testPerformance( test )

	-- This test case verifies structure of diagnostic log plus some basic validation of few fields for
	-- this particular import of 200 images  
	local catalogId = KSIdGenerator.uuidInLowercaseHex()
	local accesstoken
	local refreshtoken
	local isLogEnabled
	local jsonData
	local clientId
	local clientSecret
	
	testMetrics = WFTestMetrics.populateTestMetrics( test )
		
	WFAsyncTestCase( test, 600 )
	
		:waitFor( function() 
		
			local Bablu = require 'Bablu'
			return Bablu:getImsClientIdAndSecretP()
			
		end )
		
		:whenResolvedDo( function( status, id, secret )
		
			clientId = id 
			clientSecret  = secret
			
		end )
		
		:waitFor( function() 
		
			local postUrl = 'https://ims-na1-qa1.adobelogin.com/ims/login/code'
			local userAuthParams = 'username=ppaurav%2Bimportmetrics@adobetest.com&password=adobetest'
			local postParamsFormatStr = 'client_id=%s&redirect_uri=air-auth://carousel&locale=en_US&scope=openid,AdobeID&client_secret=%s&%s'
			local postParams = string.format( postParamsFormatStr, clientId, clientSecret, userAuthParams )
			
			return ozUser:getAccessTokenRefreshTokenFromImsP( postUrl, postParams )
			
		end )
		
		:whenResolvedDo( function( status, refresh_token, access_token )

			assert( status == 'data', "getAccessTokenRefreshTokenFromImsP failed to retreive tokens from IMS" )
			refreshtoken  = refresh_token
			accesstoken = access_token
		
		end )
		
		:waitFor( function() 
		
			return oAuthSession:storeTokensP( refreshtoken, accesstoken )
		
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor ( function()
		
			return cleanupTestAccountP()
			
		end )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
			
			local catalogArgs = {
				name = 'testImportMetrics_SmallImport',
				id = catalogId,
			}
	
			 return ozUser:createCatalogP( catalogArgs )
			
		end )
		
		:continueWhenValueMatches( 'data', ':ignore:', ':ignore:', catalogId, true )
		
		:waitFor( function()
		
			isLogEnabled = KSPrefs[ 'WFTroubleshooting_automationOutputEnabled' ]
			if ( not isLogEnabled ) then 
				KSPrefs[ 'WFTroubleshooting_automationOutputEnabled' ] = not isLogEnabled
			end
			return 1
			
		end )
		
		:waitFor( function()
			
			local picturesFolder = KSPathUtils.getStandardFilePath( "pictures" )
			local testPhotosFolder = picturesFolder .. testdataOnTestMachine .. "/200Images"
			return importJpegService:requestP( 'importFolder', catalogId, ozUserId, testPhotosFolder )

		end )
	
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			return WFDiagnosticLog.generateHtmlReportP {
					appName = 'Revel',
					emailTarget = 'ppaurav+importmetrics@adobetest.com',
				}
				
		end )
		
		:whenResolvedDo( function( status, reportHtml, automationOutputOrNil ) 		
			
			-- data is collected, lets set it false.
			KSPrefs[ 'WFTroubleshooting_automationOutputEnabled' ] = false
			
			local automationResultPath			
			local pictures = KSPathUtils.getStandardFilePath( 'pictures' )	
			local outputFolder = pictures .. testOutputOnTestMachine
		
			local time = os.date( "%Y-%m-%d-%H-%M-%S" )
			local filename = string.format( "200ImagesOutput_%s.txt", time )	
			
			if automationOutputOrNil then
				local luaToJson = KSJsonEncoder.prettyPrint( automationOutputOrNil )
				jsonData = KSData.make( luaToJson )
				automationResultPath = KSData.saveToUniquelyNamedFile( jsonData, KSPathUtils.child( outputFolder, filename ) )		
			end
			debug.print( "Your automation output file is at " .. automationResultPath ) 
						
		end )
		
		:waitFor( function()
		
			-- data to be verified is already in in jsonData variable, lets delete the catalog first
			-- if test fails in validation of data, this catalog will not be hanging 
			-- around the account.
			return ozUser:deleteCatalogP( catalogId )
		
		end ) 
		
		:continueWhenValueMatches( 'data', true, ':ignore:' )
		
		:waitFor( function()
		
			-- Validate json output
			local result = KSJsonDecoder.decodeWebSafeJson( jsonData )
			local importmetrics = result.ImportMetrics
			local metricsFound = false
			
			for i, v in pairs( importmetrics ) do 
				
				if string.find( i, "Import started at mach" ) and v.InProgress == false and v.TotalEnqueued == 224 then 
					local metrics = v
					metricsFound = true
					-- 1. verify ResultCounts table
					local resultCounts = metrics.ResultCounts
					assert( resultCounts.canceled == 0, "Incorrect Number of canceled images, actual result is " .. resultCounts.canceled .. " and expected result was 0" )
					assert( resultCounts.duplicate == 3, "Incorrect Number of duplicate images, actual result is " .. resultCounts.duplicate .. " and expected result was 3" )
					assert( resultCounts.error == 3, "Incorrect Number of error images, actual result is " .. resultCounts.error .. " and expected result was 3" )
					-- since import is finished and promise is resolved , I guess pending will always be 0 for this test case.
					assert( resultCounts.pending == 0, "Incorrect Number of pending images, actual result is " .. resultCounts.pending .. " and expected result was 0" )
					assert( resultCounts.success == 218, "Incorrect Number of success images, actual result is " .. resultCounts.success .. " and expected result was 218" )
					
					-- 2. verify error table 
					local errorArray = resultCounts.errorDetails
					assert( #errorArray == 3 , "Incorrect Number of error messages in array, actual result is " .. #errorArray .. " and expected result was 3" ) 
					local errMsg = errorArray[ 1 ] 
					assert( errMsg[ "message" ] == "invalid file path/extension", "Wrong error messages, actual message is " .. errMsg[ "message" ] .. " and expected result was " .. "invalid file path/extension" )
					
 					-- 3. verify TotalTimings table ( total time taken by different operations/function )
					local totalTimings = metrics.TotalTimings
					assert( type( totalTimings[ "verify file size" ] ) == 'number' and  totalTimings[ "verify file size" ] > 0, "Something wrong with verify file size metric in TotalTiming table" )
					assert( type( totalTimings[ "makeAnalysisRequestP" ] ) == 'number' and totalTimings[ "makeAnalysisRequestP" ] > 0, "Something went wrong with makeAnalysisRequestP metric in TotalTiming table" )
					assert( type( totalTimings[ "hasPhotoBySha256P" ] ) == 'number' and totalTimings[ "hasPhotoBySha256P" ] > 0, "Something went wrong with hasPhotoBySha256P metric in TotalTiming table" )
					assert( type( totalTimings[ "addPhotoP" ] ) == 'number' and totalTimings[ "addPhotoP" ] > 0, "Something went wrong with hasPhotoBySha256P metric in TotalTiming table" )
					
					-- 4. test checkpoints data
					local checkPoints = metrics.tenthsCheckpointData
					local nCPoints = 0
			
					for _ in pairs( checkPoints ) do 
						nCPoints = nCPoints + 1
					end 
					assert( nCPoints == 10, "Incorrect Number of checkpoints, actual result is " .. nCPoints .. " and expected result was 10" )
					
					-- take a random checkpoint table and verify different fields
					local nthChkPnt = checkPoints[ "Checkpoint 1" ]
					assert( type( nthChkPnt[ "TotalTime" ] ) == 'number' and nthChkPnt[ "TotalTime" ] > 0, "Something went wrong with TotalTime metric in checkpoint table" )
					assert( type( nthChkPnt[ "TotalMegapixels" ] ) == 'number' and nthChkPnt[ "TotalMegapixels" ] > 0, "Something went wrong with TotalMegapixels metric in checkpoint table" )
					assert( type( nthChkPnt[ "TotalFileSize" ] ) == 'number' and nthChkPnt[ "TotalFileSize" ] > 0, "Something went wrong with TotalFileSize metric in checkpoint table" )
					assert( type( nthChkPnt[ "TotalCount" ] ) == 'number' and nthChkPnt[ "TotalCount" ] > 0, "Something went wrong with TotalCount metric in checkpoint table" )
					assert( type( nthChkPnt[ "TimeComparedToFirstCheckpoint" ] ) == 'number' and nthChkPnt[ "TimeComparedToFirstCheckpoint" ] > 0, "Something went wrong with TimeComparedToFirstCheckpoint metric in checkpoint table" )
					
					-- total count should be same for any two checkpoints 
					local fifthChkPnt = checkPoints[ "Checkpoint 5" ]
					local count5 = fifthChkPnt.TotalCount
					assert( nthChkPnt[ "TotalCount" ] == count5, "Each checkpoint should have same numbers of files" ) 
					
					-- 5. test threadStats table 
					assert( type( nthChkPnt[ "threadStats" ] ) == 'table', "expected type of threadStats is table type but actual was " .. type( nthChkPnt[ "threadStats" ] ) )
					local threadStatsTable = nthChkPnt[ "threadStats" ]
					local nThreadStats = 0
					for _ in pairs( threadStatsTable ) do 
						nThreadStats = nThreadStats + 1
					end 
					assert( nThreadStats > 1 , "threadStats table should have more than 1 item" )
					
					-- pick any thread data and check its fields
					local nthThread = threadStatsTable[ 1 ] 
					assert( type( nthThread[ "threadName" ] ) == 'string' and nthThread[ "threadName" ] ~= nil, "threadName should be of string type and non nil" )
					assert( type( nthThread[ "sizeAfterGC" ] ) == 'number' and nthThread[ "sizeAfterGC" ] > 0, "sizeAfterGC should be of number type and non nil" )
					assert( type( nthThread[ "peakSize" ] ) == 'number' and nthThread[ "peakSize" ] > 0, "peakSize should be of number type and non nil" )
					assert( type( nthThread[ "currentSize" ] ) == 'number' and nthThread[ "currentSize" ] > 0, "currentSize should be of number type and non nil" )				
					
					-- 6. finally , total time taken to import 200+ images
					local totalTime = metrics.TotalTime
					local totalEnq = metrics.TotalEnqueued
					
					testMetrics[ "wfp_execution_time" ] = totalTime
					testMetrics[ "wfp_other_data" ] = metrics
					WFTestMetrics.writeTestMetrics( testMetrics )
					
					assert( totalEnq == 224, "actual totalEnqueued is " .. totalEnq .. "but expected was 224"  )
					--assert( totalTime < 400, "total time taken to import 224 images exceeded 400s earlier it used to be in range of 150-250, actual time taken is " .. totalTime )
					--assert( totalTime > 150, "total time taken to import 224 images was less than 150s, This is not a red flag and dont really want to fail test case but thats the only way I can let you know that performance is improved, actual time taken is " .. totalTime )

				end -- end of if 
				
			 end -- end of for 
			if not metricsFound then
				assert( false, "There were no import metrics in output" ) 
			end 
			return 1
			
		end )
		
end

--------------------------------------------------------------------------------

function KSImportMetricsTest.test1000Images_testPerformance( test )
 
	local catalogId = KSIdGenerator.uuidInLowercaseHex()
	local accesstoken
	local refreshtoken
	local isLogEnabled
	local jsonData
	local clientId
	local clientSecret
	
	-- generating test metrics, to be used by dashboard
	testMetrics = WFTestMetrics.populateTestMetrics( test )
	
	WFAsyncTestCase( test, 4000 )
	
		:waitFor( function() 
		
			local Bablu = require 'Bablu'
			return Bablu:getImsClientIdAndSecretP()
			
		end )
		
		:whenResolvedDo( function( status, id, secret )
		
			clientId = id 
			clientSecret  = secret
			
		end )
		
		:waitFor( function() --step 
		
			local postUrl = 'https://ims-na1-qa1.adobelogin.com/ims/login/code'
			local userAuthParams = 'username=ppaurav%2Bimportmetrics@adobetest.com&password=adobetest'
			local postParamsFormatStr = 'client_id=%s&redirect_uri=air-auth://carousel&locale=en_US&scope=openid,AdobeID&client_secret=%s&%s'
			local postParams = string.format( postParamsFormatStr, clientId, clientSecret, userAuthParams )
			
			return ozUser:getAccessTokenRefreshTokenFromImsP( postUrl, postParams )
			
		end )
		
		:whenResolvedDo( function( status, refresh_token, access_token )

			assert( status == 'data', "getAccessTokenRefreshTokenFromImsP failed to retreive tokens from IMS" )
			refreshtoken  = refresh_token
			accesstoken = access_token
		
		end )
		
		:waitFor( function()	--step 
		
			return oAuthSession:storeTokensP( refreshtoken, accesstoken )
		
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	-- step 
			
			local catalogArgs = {
				name = 'testImportMetrics_largeImport',
				id = catalogId,
			}
	
			 return ozUser:createCatalogP( catalogArgs )
			
		end )
		
		:continueWhenValueMatches( 'data', ':ignore:', ':ignore:', catalogId, true )
		
		:waitFor( function()	--step 
		
			isLogEnabled = KSPrefs[ 'WFTroubleshooting_automationOutputEnabled' ]
			if ( not isLogEnabled ) then 
				KSPrefs[ 'WFTroubleshooting_automationOutputEnabled' ] = not isLogEnabled
			end
			return 1
			
		end )
		
		:waitFor( function()	--step 5
			
			local picturesFolder = KSPathUtils.getStandardFilePath( "pictures" )
			local testPhotosFolder = picturesFolder .. testdataOnTestMachine .. "/1000Images"
			return importJpegService:requestP( 'importFolder', catalogId, ozUserId, testPhotosFolder )

		end )
	
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	--step 6
		
			return WFDiagnosticLog.generateHtmlReportP {
					appName = 'Revel',
					emailTarget = 'ppaurav+importmetrics@adobetest.com',
				}
				
		end )
		
		:whenResolvedDo( function( status, reportHtml, automationOutputOrNil ) 		
			
			-- data is collected, lets set it false.
			KSPrefs[ 'WFTroubleshooting_automationOutputEnabled' ] = false
			
			local automationResultPath			
			local pictures = KSPathUtils.getStandardFilePath( 'pictures' )		
			local outputFolder = pictures .. testOutputOnTestMachine
	
			local time = os.date( "%Y-%m-%d-%H-%M-%S" )
			local filename = string.format( "1000ImagesOutput_%s.txt", time )	
			
			if automationOutputOrNil then
				local luaToJson = KSJsonEncoder.prettyPrint( automationOutputOrNil )
				jsonData = KSData.make( luaToJson )
				automationResultPath = KSData.saveToUniquelyNamedFile( jsonData, KSPathUtils.child( outputFolder, filename ) )		
			end
			debug.print( "Your automation output file is at " .. automationResultPath ) 
						
		end )
		
		:waitFor( function()
		
			-- data to be verified is already in in jsonData variable, lets delete the catalog first
			-- if test fails in validation of data, this catalog will not be hanging 
			-- around the account.
			return ozUser:deleteCatalogP( catalogId )
		
		end ) 
		
		:continueWhenValueMatches( 'data', true, ':ignore:' )
		
		:waitFor( function()	--step 
		
			-- Validate json output
			local result = KSJsonDecoder.decodeWebSafeJson( jsonData )
			local importmetrics = result.ImportMetrics
			local metricsFound = false
			for i, v in pairs( importmetrics ) do 
				
				if string.find( i, "Import started at mach" ) and v.InProgress == false and v.TotalEnqueued == 1000 then 
					metricsFound = true
					local metrics = v
					-- verify TotalTimings table ( total time taken by different operations/function )
					local totalTimings = metrics.TotalTimings
					print( "VerifyFileSize function took ", totalTimings[ "verify file size" ] ) 
					print( "makeAnalysisRequestP function took ", totalTimings[ "makeAnalysisRequestP" ] ) 
					print( "hasPhotoBySha256P function took ", totalTimings[ "hasPhotoBySha256P" ] ) 
					print( "addPhotoP function took ", totalTimings[ "addPhotoP" ] ) 

					-- total time taken to import 1000 images
					local totalTime = metrics.TotalTime
					print( "Total time to import 1000 images ", totalTime ) 
					--assert( totalTime < 1500, "total time taken to import 1000 images exceeded 1500, earlier it used to be in range of 800-1200s, actual number is " .. totalTime )
					--assert( totalTime > 800, "total time taken to import 1000 images was less than 800s, This is not a red flag and dont really want to fail test case but thats the only way I can let you know performance is improved, actual time taken is " .. totalTime )

					testMetrics[ "wfp_execution_time" ] = totalTime
					testMetrics[ "wfp_other_data" ] = metrics
					WFTestMetrics.writeTestMetrics( testMetrics )
	
				end 
				
			 end
			if not metricsFound then
				assert( false, "There were no import metrics in output" ) 
			end 
			return 1
			
		end )
		
end

--------------------------------------------------------------------------------

if WFPlatformInfo.isMac then 
	return KSImportMetricsTest
else 
	return {}
end 
