--[[----------------------------------------------------------------------------

KSImportUtils.lua

@author arahn 2012-10-24

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local WFArguments = require 'WFArguments'

--==============================================================================

local string_match = string.match
local string_lower = string.lower

--------------------------------------------------------------------------------

local kJpegContentType = "image/jpeg"
local kPngContentType = "image/png"

local kValidFilePathExtensions = {
	jpg = kJpegContentType,
	jpeg = kJpegContentType,
	jpe = kJpegContentType,
	jif = kJpegContentType,
	jfif = kJpegContentType,
	jfi = kJpegContentType,
	png = kPngContentType
}

local kValidContentTypes = {
	[ kJpegContentType ] = true,
	[ kPngContentType ] = true,
}

for k, contentType in pairs( kValidFilePathExtensions ) do
	assert( kValidContentTypes[ contentType ], "All file extensions' content-types must be valid" )
end

--------------------------------------------------------------------------------

local function makeLateRequire( packageName )

	WFArguments.check( "string" )
	local package = nil
	
	return function()
		if package == nil then
			package = require( packageName )
		end
		return package
	end

end

--------------------------------------------------------------------------------

local function contentTypeForFilePath( filePath )

	WFArguments.check( "string" )
	local pathExtension = string_match( filePath, "%.([%a%d]*)$" )
	
	pathExtension = pathExtension and string_lower( pathExtension )
	
	return pathExtension and kValidFilePathExtensions[ pathExtension ]

end

--------------------------------------------------------------------------------

local function isValidFilePath( filePath )

	WFArguments.check( "string" )
	return not not contentTypeForFilePath( filePath )

end

--------------------------------------------------------------------------------

local function isSupportedContentType( contentType )
	
	WFArguments.check( "?string" )
	if not contentType then	
		return false
	end
	local strippedOfProperties = contentType:gsub( ";.*", "" )
	return not not kValidContentTypes[ strippedOfProperties ]

end

--------------------------------------------------------------------------------

return {
	makeLateRequire = makeLateRequire,
	contentTypeForFilePath = contentTypeForFilePath,
	isValidFilePath = isValidFilePath,
	isSupportedContentType = isSupportedContentType,
}