--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local KSImportUtilsTest = {}

--==============================================================================

function KSImportUtilsTest.testMakeLateRequire()

	local KSImportUtils = require 'KSImportUtils'
	assert( KSImportUtils, "Namespace" )
	
	local makeLateRequire = KSImportUtils.makeLateRequire
	assert( makeLateRequire, "makeLateRequire is fn" )
	
	local lateSelf = makeLateRequire 'KSImportUtils'
	assert( lateSelf, "late require makes fn" )
	
	assert( lateSelf() == KSImportUtils, "Works once..." )
	assert( lateSelf() == KSImportUtils, "Works twice..." )
	
end

--------------------------------------------------------------------------------

function KSImportUtilsTest.testIsSupportedContentType()

	local KSImportUtils = require 'KSImportUtils'
	assert( KSImportUtils, "Namespace" )
	
	local isSupportedContentType = KSImportUtils.isSupportedContentType
	assert( isSupportedContentType, "has isSupportedContentType fn" )
	
	assert( isSupportedContentType( "image/jpeg" ), "support jpeg" )
	assert( isSupportedContentType( "image/jpeg;" ), "support jpeg; just for Matt" )
	assert( isSupportedContentType( "image/jpeg;foo=bar" ), "support jpeg; params" )

	assert( not isSupportedContentType( "andy/fake;not=a;real=contentType" ), "dont support non image/fake" )
	assert( not isSupportedContentType( ), "dont support nothin" )
	
end

--------------------------------------------------------------------------------

function KSImportUtilsTest.testIsValidFilePath()

	local KSImportUtils = require 'KSImportUtils'
	assert( KSImportUtils, "Namespace" )
	
	local isValidFilePath = KSImportUtils.isValidFilePath
	assert( isValidFilePath, "has contentTypeForFilePath fn" )
	
	assert( isValidFilePath( "filename.jpg" ), "basic functionality" )
	assert( not isValidFilePath( "imagejpeg.not" ), "basic neg functionality" )
	
end

--------------------------------------------------------------------------------

function KSImportUtilsTest.testContentTypeForFilePath()

	local KSImportUtils = require 'KSImportUtils'
	assert( KSImportUtils, "Namespace" )
	
	local contentTypeForFilePath = KSImportUtils.contentTypeForFilePath
	assert( contentTypeForFilePath, "has contentTypeForFilePath fn" )
	
	assert( contentTypeForFilePath( "filename.jpg" ) == 'image/jpeg', "support jpg" )
	assert( contentTypeForFilePath( "filename.png.jpg" ) == 'image/jpeg', "support jpg" )
	assert( contentTypeForFilePath( ".jpg" ) == 'image/jpeg', "support jpg" )
	assert( contentTypeForFilePath( "f dup\0 filename\r\n.jpg" ) == 'image/jpeg', "support jpg" )
	assert( contentTypeForFilePath( "nife.jpeg.andy" ) == nil, "support jpg" )
	
end

--==============================================================================

return KSImportUtilsTest