--[[----------------------------------------------------------------------------

KSJsonEncoder.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

Preconfigured Lua to JSON encoders for common cases. Write directly to the
lua-yajl interface (or add to this namespace) if the common cases aren't
suitable.

@author scouten 2011-03-08

------------------------------------------------------------------------------]]

	-- third_party.yajl
local yajl = require 'yajl'


--============================================================================--
-- @mark KSJsonEncoder namespace

--[[!

	\class KSJsonEncoder

	\brief This namespace provides translation utilities between JSON-encoded
		REST request contents and the Lua structures used by the LUA API calls.
		
		These functions are typically used as the translation functions for
		\c mapB calls.

]]--

local KSJsonEncoder = {}

--------------------------------------------------------------------------------
--[[!
     \brief Translates a Lua-object string into the equivalent JSON data structure.

     \param luaObject The Lua-object string.
 
     \return The JSON string.
]]--

function KSJsonEncoder.encodeWebSafeJson( --[[string]] luaObject )

	local result = yajl.to_string( luaObject )
	
	return "while (1) {}\n" .. result
	
end

--------------------------------------------------------------------------------

--[[!
     \brief Translates a Lua-object string into the equivalent JSON data structure,
     	adding whitespace for readability.

     \param luaObject The Lua-object string.
 
     \return The JSON string.
]]--

function KSJsonEncoder.prettyPrint( --[[string]] luaObject )

	return yajl.to_string( luaObject, { indent = "\t" } )

end

--------------------------------------------------------------------------------

--[[!
     \brief Translates a Lua-object string into the equivalent JSON data structure
     	that is specifically suitable for Oz upload opeerations.

     \param luaObject The Lua-object string.
 
     \return The JSON string.
]]--

function KSJsonEncoder.encodeOzUploadJson( --[[string]] luaObject )

	return yajl.to_string( luaObject )
	
end

--------------------------------------------------------------------------------

return KSJsonEncoder
