--[[----------------------------------------------------------------------------

KSKeychainService.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

This namespace provides secure storage for passwords, access tokens, and similar
information that is especially important to the application. It may be accessed
from any thread.

This API is a minimalist wrapper over Mac and iOS Keychain Services.

--------------------------------------------------------------------------------

@author scouten 2011-08-02

------------------------------------------------------------------------------]]

	-- wichitafoundation.core.threading
local WFWorkerThread = require 'WFWorkerThread'


--------------------------------------------------------------------------------

local keychainManagerEP = WFWorkerThread( "KSKeychainServiceImp.lua" )


--============================================================================--

-- @mark KSKeychainService namespace

--[[!
	\class	KSKeychainService
	\brief This namespace provides secure storage for passwords, access tokens,
		and similar information. It can be accessed from any thread.

]]--

local KSKeychainService = {}

--------------------------------------------------------------------------------
--[[!
	Reads a single value from the keychain.

	\param key (string) The key for the desired value.
	
	\return A promise object that resolves to the value.
]]--

function KSKeychainService.readP( --[[string]] key )

	assert( type( key ) == 'string', "KSKeychainService.read: key must be a string" )

	return keychainManagerEP:requestP( 'read', key )

end

--------------------------------------------------------------------------------
--[[!
	Writes a single value into the keychain.
	
	\param key (string) The key for the new value.
	\param value (string) The new value.
	
	\return A promise object that signals completion, This can be ignored.
]]--

function KSKeychainService.writeP( --[[string]] key, --[[string]] value )

	-- FIXME [scouten 2011-08-26]: Rename this to write() once we have killed off
	-- all instances of the old event-stream based write().

	assert( type( key ) == 'string', "KSKeychainService.write: key must be a string" )
	assert( type( value ) == 'string', "KSKeychainService.write: value must be a string" )

	local resultP = keychainManagerEP:requestP( 'write', key, value )
	
	resultP:runUntilDone()
	
	return resultP

end

--------------------------------------------------------------------------------
--[[!
	Deletes a value from the keychain.
	
	\param key (string) The key for the valuet to delete.
	
	\return A promise object that signals completion, This can be ignored.
]]--

function KSKeychainService.deleteP( --[[string]] key )

	assert( type( key ) == 'string', "KSKeychainService.delete: key must be a string" )

	local resultP = keychainManagerEP:requestP( 'delete', key )
	
	resultP:runUntilDone()
	
	return resultP

end

--------------------------------------------------------------------------------

return KSKeychainService
