--[[----------------------------------------------------------------------------

KSKeychainServiceTest.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

This namespace provides secure storage for passwords, access tokens, and similar
information that is especially important to the application. It may be accessed
from any thread.

This API is a minimalist wrapper over Mac and iOS Keychain Services.

--------------------------------------------------------------------------------

@author scouten 2011-08-02

------------------------------------------------------------------------------]]

	-- wichitafoundation.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'
local WFPlatformInfo = require 'WFPlatformInfo'

	-- services.keychain_store
local KSKeychainService = require 'KSKeychainService'


--============================================================================--

local KSKeychainServiceTest = {}

--------------------------------------------------------------------------------

function KSKeychainServiceTest.testDeleteReadWrite( test )

if WFPlatformInfo.isAndroid then
	-- FIXME [scouten 2011-08-02]: Disabling this test because it appears prone
	-- to intermittent failure if Keychain Manager gets grumpy. This might be
	-- because the app signature doesn't match the one that was used to create
	-- a prior keychain entry (i.e. the app was rebuilt).

	WFAsyncTestCase( test, 4 )

		:waitFor( function()	-- step 1
			
			return KSKeychainService.deleteP( "test_key" )
			
		end )
		
		:waitFor( function()	-- step 2
			
			return KSKeychainService.readP( "test_key" )
			
		end )
		:continueWhenValueMatches( 'data', nil )
		
		:waitFor( function()	-- step 3
		
			return KSKeychainService.writeP( "test_key", "foo" )
		
		end )
		
		:waitFor( function()	-- step 4
			
			return KSKeychainService.readP( "test_key" )
			
		end )
		:continueWhenValueMatches( 'data', "foo" )
		
		:waitFor( function()	-- step 5
			
			return KSKeychainService.deleteP( "test_key" )
			
		end )
		
		:waitFor( function()	-- step 6
			
			return KSKeychainService.readP( "test_key" )
			
		end )
		:continueWhenValueMatches( 'data', nil )
end

end

--------------------------------------------------------------------------------

-- FIXME [scouten 2011-07-26]: Should also test various error conditions.

--------------------------------------------------------------------------------

return KSKeychainServiceTest
