--[[----------------------------------------------------------------------------

KSLoggingTest.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2010 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author epoley 2010-09-15

@reviewed epoley 2010-10-28
@reviewed scouten 2010-10-28
@reviewed mhamburg 2010-11-02
@reviewed erwright 2010-11-02

------------------------------------------------------------------------------]]

local KSLoggingTest = {}

local KSLogging = require 'KSLogging'

--------------------------------------------------------------------------------

function KSLoggingTest.tearDown()
	
	KSLogging.removeAllLogs()
	
end

--------------------------------------------------------------------------------
-- A basic example, including setting the log level to filter noise

function KSLoggingTest.testBasicLogger()
	
	KSLogging.log( "debug" ):info( "information" )
	KSLogging.log( "debug" ):warn( "warning" )
	KSLogging.log( "debug" ):error( "error!" )
	
	KSLogging.log( "debug" ):setLevel( KSLogging.ERROR )
	KSLogging.log( "debug" ):warn( "more warning" )
	KSLogging.log( "debug" ):error( "OH NO!" )
	
end

--------------------------------------------------------------------------------
-- An example of logging to two different logs

function KSLoggingTest.testTwoLoggers()
	
	KSLogging.log( "debug" ):info( "information" )
	KSLogging.log( "debug" ):warn( "warning" )
	KSLogging.log( "debug" ):error( "error!" )
	
	KSLogging.makeDefaultLog( "mylog" )
	KSLogging.log( "mylog" ):info( "foobar v.%d", 2 )
	
	KSLogging.log( "debug" ):setLevel( KSLogging.ERROR )
	KSLogging.log( "debug" ):warn( "more warning" )
	KSLogging.log( "debug" ):error( "OH NO!" )
	
end

--------------------------------------------------------------------------------

function KSLoggingTest.testConcise()
	
	local log = KSLogging.log( "debug" )
	log:info( "You can save yourself some typing" )

end

--------------------------------------------------------------------------------
-- An example of making a log that logs using a custom function
-- Useful for "electronic communication"

function KSLoggingTest.testCustomLogFunction()
	
	local spoilTheFun = function( self, level, message )
				debug.print( level, string.gsub( message, "damn", "darn" ) )
				return true
			  end
			  
	KSLogging.makeLog( "PG-rated log", spoilTheFun )
	KSLogging.log( "PG-rated log" ):error( "damn, it failed" )

end

--------------------------------------------------------------------------------
-- Testing Ellie's string formatting extensions

function KSLoggingTest.testStringFormatting()
	
	KSLogging.log( "debug" ):debug( "debugging stuff..." )
	KSLogging.log( "debug" ):info( "information: pi = %.5f", math.pi )
	KSLogging.log( "debug" ):error( "error!" )
	KSLogging.log( "debug" ):warn( "warning: drinking more than %d cups of %s a day causes caffeine dependency", 2, "coffee" )
	KSLogging.log( "debug" ):warn( "foo" )
	KSLogging.log( "debug" ):warn( "This percent sign is not a format code: % nor is this worse case: %s" )
	KSLogging.log( "debug" ):warn( setmetatable( {}, {
			__tostring = function() error "Die hard" end
		} ) )
	KSLogging.log( "debug" ):fatal( 42 )
	
end

--------------------------------------------------------------------------------
-- Testing the logging of tables, which was originally broken in LuaLogger
-- Currently the default behavior (via LuaLogger) is to print tables
-- with the keys in alphabetical order.

function KSLoggingTest.testTableLogging()
	
	local mytable = {
		name = "nice rect",
		x = 2,
		y = 10,
		width = 100,
		height = 200,
		color = { r = 1, g = 0, b = 0, a = .5 }
	}
	KSLogging.log( "debug" ):debug( mytable )
	
end

--------------------------------------------------------------------------------
-- Testing the logging of tables with both string and numeric keys.

function KSLoggingTest.testTableLoggingWithNumericKeys()
	
	local mytable = {
		name = "nice rect",
		x = 2,
		y = 10,
		width = 100,
		height = 200,
		color = { r = 1, g = 0, b = 0, a = .5 }
	}
	mytable[ 2 ] = { name = "bar" }
	mytable[ 1 ] = { name = "foo" }
	
	-- Tables as keys blow up...
	
	--local tableKey = { foo = "baz" }
	--mytable[ tableKey ] = "what?"
		
	KSLogging.log( "debug" ):debug( mytable )
	
end

--[[

[epoley 9/15/10]: I promise that I have run these and they work, but lacking a
better way to test log file creation at the moment, I am commenting them out
so as to not create stray files when the unit tests are run.

--------------------------------------------------------------------------------

function KSLoggingTest.testFileLog()
	
	KSLogging.makeDefaultLogWithFile( "myfilelog", "foobar.log" )
	KSLogging.log( "myfilelog" ):debug( "I love debugging" )
	KSLogging.log( "myfilelog" ):error( "There was an error!" )
	
end

--------------------------------------------------------------------------------

function KSLoggingTest.testDatedFileLog()
	
	KSLogging.makeDatedLogWithFile( "datedfilelog", "foobar_ %s.log", "%Y-%m-%d" )
	KSLogging.log( "datedfilelog" ):debug( "I love debugging" )
	KSLogging.log( "datedfilelog" ):error( "There was an error!" )
	
end

]]

--------------------------------------------------------------------------------

return KSLoggingTest
