--[[----------------------------------------------------------------------------

KSLuaUtils.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2010 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author scouten 2010-10-01 (based on earlier work by Mark)

@reviewed mhamburg 2010-11-02
@reviewed erwright 2010-11-02
@reviewed epoley 2010-11-03
@reviewed scouten 2011-03-01
@reviewed mhamburg 2011-03-14

-- GRUNGE [mhamburg 2010-11-02]: Hmmm... No unit tests.

------------------------------------------------------------------------------]]

local KSLogging = require 'KSLogging'
local KSLuaErrors = require 'KSLuaErrors'

local WFDebugTraceback_traceback = require 'WFDebugTraceback'.traceback

--------------------------------------------------------------------------------

local debugLog = KSLogging.log 'debug'
local string_format = string.format

local debug = debug
local debug_print = debug.print

--------------------------------------------------------------------------------

local KSLuaUtils = {}

--------------------------------------------------------------------------------
-- Test the value to see whether it is true. If it is, just return it. If not,
-- then signal an error with a message constructed by running through string.format.
-- This has the benefit of avoiding constructing the error message until we
-- actually need it.

function KSLuaUtils.assertf( value, msg, ... )

	if value then
		return value
	end
	
	error( string_format( msg, ... ), 0 )

end

--------------------------------------------------------------------------------

-- A version of assert with format and traceback support.

function KSLuaUtils.assertft( value, msg, ... )

	if value then
		return value
	end
	
	if select( '#', ... ) then
		msg = string_format( msg, ... )
	end
	
	debug_print( "Error: " .. msg )
	debug_print( WFDebugTraceback_traceback { "KSReactive.lua", "KSLuaUtils.lua", "WFTimer.lua", "in function 'pcall'", "KSPcallLoops.lua", "KSReactiveTimer.lua" } )
	error( msg, 2 )
	
end

--------------------------------------------------------------------------------
-- Implementation for the colon message send sugar. Useful if you need to send
-- a message that changes at runtime.

function KSLuaUtils.send( obj, msg, ... )

	return obj[ msg ]( obj, ... )

end

local send = KSLuaUtils.send

--------------------------------------------------------------------------------
-- Second half of dpcall. See below. Spits an error out to debug.print and then
-- proceeds as normal.

local function dpcall_finish( ident, success, ... )

	if not success and not KSLuaErrors.isCancelError( ( ... ) ) then
		debugLog:error( string_format( "Error in '%s': %s", tostring( ident ), tostring( ... ) ) )
	end

	return success, ...
	
end

--------------------------------------------------------------------------------
-- The same as pcall, but in the event of a failure, it logs a message to that
-- effect before returning.

function KSLuaUtils.dpcall( ident, fn, ... )

	if not fn then
		local error = string_format( "No function for '%s'", tostring( ident ) )
		debugLog:error( error )
		return false, error
	end

	return dpcall_finish( ident, pcall( fn, ... ) )

end

--------------------------------------------------------------------------------

local dpcall = KSLuaUtils.dpcall

--------------------------------------------------------------------------------
-- A protected version of KSLuaUtils.send which wraps the send operation in a
-- pcall.

function KSLuaUtils.psend( obj, msg, ... )

	return dpcall( msg, send, obj, msg, ... )

end

--------------------------------------------------------------------------------

return KSLuaUtils
