--[[----------------------------------------------------------------------------

KSMemoryCacheTest.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author erwright 2011-02-03

@author epoley 2011-03-08

------------------------------------------------------------------------------]]

local KSMemoryCache = require 'KSMemoryCache'

--------------------------------------------------------------------------------

local KSMemoryCacheTest = { }

--------------------------------------------------------------------------------

function KSMemoryCacheTest.testCacheExpiration()

	local simpleCache = KSMemoryCache( 3 )
	assert( simpleCache.maxEntries == 3 )

	simpleCache:add( "foo", { x = 1 } )
	simpleCache:add( "bar", { x = 2 } )
	simpleCache:add( "baz", { x = 3 } )
	
	assert( simpleCache.count == 3, "simpleCache:count == 3" )
	
	simpleCache:add( "bleen", { x = 4 } )

	assert( simpleCache.count == 3, "simpleCache:count == 3" )
	
	collectgarbage()
	collectgarbage()
	collectgarbage()
	
	assert( not simpleCache:getData("foo") )
	assert( simpleCache:getData("bar") )
	assert( simpleCache:getData("baz") )
	assert( simpleCache:getData("bleen") )

end

--------------------------------------------------------------------------------

function KSMemoryCacheTest.testAdjustMaximumEntries()

	local simpleCache = KSMemoryCache( 3 )
	assert( simpleCache.maxEntries == 3 )

	simpleCache:add( "foo", { x = 1 } )
	simpleCache:add( "bar", { x = 2 } )
	simpleCache:add( "baz", { x = 3 } )
	
	assert( simpleCache.count == 3, "simpleCache:count == 3" )
	
	simpleCache:add( "bleen", { x = 4 } )

	assert( simpleCache.count == 3, "simpleCache:count == 3" )
	
	simpleCache:adjustMaximumEntries( 5 )
	
	assert( simpleCache.count == 3, "simpleCache:count == 3" )
	
	simpleCache:add( "bang", { x = 5 } )
	simpleCache:add( "boom", { x = 6 } )
	
	assert( simpleCache.count == 5, "simpleCache:count == 5" )
	
	assert( simpleCache:getData( "bang" ) )
	assert( simpleCache:getData( "boom" ) )
	
	collectgarbage()
	collectgarbage()
	collectgarbage()
	
	assert( not simpleCache:getData( "foo" ) )
	assert( simpleCache:getData( "bar" ) )
	assert( simpleCache:getData( "baz" ) )
	assert( simpleCache:getData( "bleen" ) )
	
	simpleCache:adjustMaximumEntries( 3 )
	
	collectgarbage()
	collectgarbage()
	collectgarbage()
	
	assert( simpleCache:getData( "bar" ) )
	assert( simpleCache:getData( "bleen" ) )
	assert( simpleCache:getData( "baz" ) )
	
	assert( not simpleCache:getData( "foo" ) )
	assert( not simpleCache:getData( "bang" ) )
	assert( not simpleCache:getData( "boom" ) )
	
	local success = pcall( function() simpleCache:adjustMaximumEntries( 1 ) end )
	
	assert( not success )

end

--------------------------------------------------------------------------------

function KSMemoryCacheTest.testRemoveFirst()

	local simpleCache = KSMemoryCache( 3 )
	assert( simpleCache.maxEntries == 3 )

	simpleCache:add( "bar", { x = 1 } )
	simpleCache:add( "baz", { x = 2 } )
	
	simpleCache:remove( "baz" )
	simpleCache:remove( "bar" )

	simpleCache:add( "foo", { x = 3 } )
	simpleCache:remove( "foo" )

	simpleCache:add( "bar", { x = 1 } )
	simpleCache:remove( "bar" )

	simpleCache:add( "bar", { x = 1 } )
	simpleCache:add( "baz", { x = 2 } )

	simpleCache:remove( "baz" )
	simpleCache:add( "baz", { x = 2 } )

end

--------------------------------------------------------------------------------

function KSMemoryCacheTest.testStringData()

	local simpleCache = KSMemoryCache( 3 )
	
	simpleCache:add( "foo", "bar" )
	
	assert( simpleCache:getData( "foo" ) == "bar" )
	
end

--------------------------------------------------------------------------------

function KSMemoryCacheTest.testUnsupportedDataTypes()
	
	local simpleCache = KSMemoryCache( 3 )
	
	local success = pcall( function() simpleCache:add( "number", 1 ) end )
	assert( not success )
	
	success = pcall( function() simpleCache:add( "boolean", true ) end )
	assert( not success )
	
	success = pcall( function() simpleCache:add( "nil", nil ) end )
	assert( not success )
	
	success = pcall( function() simpleCache:add( "function", function() end ) end )
	assert( not success )

end

--------------------------------------------------------------------------------

return KSMemoryCacheTest
