--[[----------------------------------------------------------------------------

KSPathUtils.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author arahn 2013-01-03

------------------------------------------------------------------------------]]

	-- core.os
local KSPathUtilsImp = require 'KSPathUtilsImp'

--============================================================================--

-- @mark KSPathUtils namespace

--[[!

\class KSPathUtils

\brief This namespace provides cross-platform path-manipulation utilities.

]]--


local KSPathUtils = {}

--------------------------------------------------------------------------------
--[[!
	Retrieves a standard system-defined path.

	\param name (string) The name of the path to find. One of these values:
	<ul><li>"app" (current application install folder)</li>
	        <li>"appBuiltInPlugins" (folder for built-in plug-ins: see [ NSBundle builtInPlugInsPath ] )</li>
		<li>"adobeAppData" (Adobe application support data)</li>
		<li>"home" (user's home folder)</li>
		<li>"caches"</li>
		<li>"carouselDocuments" (the package in the pictures folder for revel documents.
			For example, <code>~/Pictures/Adobe Revel/Adobe Revel Library.rvl</code> in Mac OS. </li>
		<li>"carouselCaches" (caches subfolder specific to Adobe Revel)</li>
		<li>"pictures" (user pictures)</li>
		<li>"desktop"</li>
		<li>"applications"</li>
		<li>"temp"</li>
		<li>"testdata" (path to dir in app package for test data)</li>
		<li>"Resources" (application resources)</li>
		<li>"TranslatedStrings" (dictionary <em>file</em> for locale-specific translated strings)</li>
	  </ul>

	\return (string) The file path.
]]--

function KSPathUtils.getStandardFilePath( --[[string]] name )
	-- WFArguments.check( "string" )
	-- This wrapper exists only for documentation purposes; real implementation
	-- comes from native code (see "Real implementation" below).
end

--------------------------------------------------------------------------------
--[[!
	Concatenates a folder name to a base path to form a new system path.

	\param base (string) The base file path.
	\param name (string) The name of the contained folder.
	
	\return (string) The complete path.
]]--

function KSPathUtils.child( --[[string]] base, --[[string]] name )
	-- WFArguments.check( "string", "string" )
	-- This wrapper exists only for documentation purposes; real implementation
	-- comes from native code (see "Real implementation" below).
end

--------------------------------------------------------------------------------
--[[!
	Strips the leaf file-name part from a full path.
	
	\param path (string) The path string.
	
	\return (string) The path of the folder containing the file.
]]--

function KSPathUtils.parent( --[[string]] path )
	-- WFArguments.check( "string" )
	-- This wrapper exists only for documentation purposes; real implementation
	-- comes from native code (see "Real implementation" below).
end

--------------------------------------------------------------------------------
--[[!
	Retrieves the leaf file-name part of a full path.

	\param path (string) The path string.
	
	\return (string) The file name.
]]--

function KSPathUtils.leafName( --[[string]] path )
	-- WFArguments.check( "string" )
	-- This wrapper exists only for documentation purposes; real implementation
	-- comes from native code (see "Real implementation" below).
end

--==============================================================================
-- @mark The real implementation
--==============================================================================

KSPathUtils = KSPathUtilsImp

--==============================================================================

return KSPathUtils
