--[[----------------------------------------------------------------------------

KSPrefs.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2010 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

WARNING [epoley 2010-10-29]: Warning: Cocoa preferences do not behave as expected
in the simulator if you quit with command-Q (or equivalent) instead of properly
terminating the app by clicking the back button. As far as I can tell, there
is no such issue on the device.

--------------------------------------------------------------------------------

@author scouten 2010-10-13

@reviewed epoley 2010-11-02
@reviewed erwright 2010-03-07
@reviewed scouten 2011-03-02
@reviewed epoley 2011-03-10

------------------------------------------------------------------------------]]
	
	-- wichitafoundation.core.reactive
local KSReactive -- = require 'KSReactive' -- lazy loaded

	-- wichitafoundation.core.utils
local WFCloseSupport = require 'WFCloseSupport'

	-- wichitafoundation.os.prefs
local KSPrefsImp = require 'KSPrefsImp' -- platform core API



--============================================================================--

-- @mark Preference behaviors

local preferenceBehaviors = setmetatable( {}, { __mode = 'kv' } )

local didStartWatchingPrefs = false

--------------------------------------------------------------------------------

local function installSystemNotificationHook()

	assert( not didStartWatchingPrefs )
	didStartWatchingPrefs = true

	KSPrefsImp.startWatchingPrefs()
	
	require 'KSSystemNotifications'.notificationStreamE( 'KSPrefsImp_userDefaultsDidChange' )
		:onDataE( function()
	
			-- As far as I know, we don't have a way to know *which* preference
			-- was changed, so we just iterate through all prefs and refresh.
			-- I wish there was a faster way.
		
			for k, sourceB in pairs( preferenceBehaviors ) do
			
				if WFCloseSupport.hasBeenOrWillBeClosed( sourceB ) then
					preferenceBehaviors[ k ] = nil
				else
					sourceB:set( KSPrefsImp.getValue( k ) )
				end
			
			end
		
		end )

end

--------------------------------------------------------------------------------

local function prefB( k )

	local resultB = WFCloseSupport.fetchCached( preferenceBehaviors, k )
	
	if not resultB then

		if not didStartWatchingPrefs then
			installSystemNotificationHook()
		end

		KSReactive = KSReactive or require 'KSReactive'

		resultB = KSReactive.variableB( KSPrefsImp.getValue( k ) )
		preferenceBehaviors[ k ] = resultB
		
	end
	
	return resultB:relayB()

end


--============================================================================--

-- @mark One-off access to prefs values (classic interface)

--------------------------------------------------------------------------------

local prefsMetatable = {}

--------------------------------------------------------------------------------

function prefsMetatable.__index( t, k )

	if type( k ) ~= 'string' then
		error( "KSPrefs: preference keys must be string", 2 )
	end
	
	if k == 'prefB' then
		return prefB
	end

	return KSPrefsImp.getValue( k )

end

--------------------------------------------------------------------------------

function prefsMetatable.__newindex( t, k, v )

	if type( k ) ~= 'string' then
		error( "KSPrefs: preference keys must be string", 2 )
	end
	
	if k == 'prefB' then
		error( "KSPrefs: can not set prefB", 2 )
	end

	local existingValue = KSPrefsImp.getValue( k )
	
	if existingValue ~= v then
		KSPrefsImp.setValue( k, v )
	end

	local localPrefB = WFCloseSupport.fetchCached( preferenceBehaviors, k )
	
	if localPrefB then
		localPrefB:set( v )
	end

end

--------------------------------------------------------------------------------

return setmetatable( {}, prefsMetatable )
