--[[----------------------------------------------------------------------------

KSPrefsImpLua.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2010 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

WARNING [matthew 2013-03-01]: Win32 doesn't have an application specific key/value
preference store (other than the registry) so keep a table that gets written out to
local app data storage.


--------------------------------------------------------------------------------

@author matthewj 2013-03-01

------------------------------------------------------------------------------]]

	-- wichitafoundation.core.os
local KSPathUtils = require 'KSPathUtils'
local KSFileUtils = require 'KSFileUtils'

	-- wichitafoundation.core.table
local KSTableArchiving = require 'KSTableArchiving'

--============================================================================--
--[[!
    \brief Implementation for KSPrefs on Win32 and other platforms that don't have an API for
    a native app specific key/value storage area. Since Win32 doesn't have such a thing, we
    will serialize a prefs cache in the local app data and manage the prefs in Lua.  This lua file
    is loaded by the native code into its own universe to keep access to the prefs thread safe.
]]--

--------------------------------------------------------------------------------

local prefsTable = false

local prefsDirectory = KSPathUtils.child( KSPathUtils.getStandardFilePath( "adobeAppData" ), "Preferences" )
KSFileUtils.mkdir( prefsDirectory )

local prefsFilename = KSPathUtils.child( prefsDirectory, "preferences.wfprefs" )


--------------------------------------------------------------------------------

local function loadPreferences( )
	
	local file = io.open( prefsFilename, "r" )
	
	if file then
		local prefsString = file:read( "*a" )
		file:close()
		
		local loadfunc = loadstring( 'return ' .. prefsString )
		
		local ok, ret = pcall( loadfunc )
		
		return ok and ret or {}
	end
	return {}
end

--------------------------------------------------------------------------------

local function savePreferences( )
	
	local prefsString = KSTableArchiving.simpleTableToChunkString( prefsTable or {} )
	
	local file = io.open( prefsFilename, "w+" )
	
	if file then
		file:write( prefsString )
	
		file:close()
	end
end

--------------------------------------------------------------------------------

prefsTable = loadPreferences() or {}

local KSPrefs = {}


--------------------------------------------------------------------------------

function KSPrefs.fromNativeCodeGetValue( key )

	return prefsTable[ key ]

end

--------------------------------------------------------------------------------

function KSPrefs.fromNativeCodeSetValue( key, value )
	
	local valueType = type( value )
	
	if valueType == "string" or valueType == "boolean" or
		valueType == "number" or valueType == "nil" then
		
		prefsTable[ key ] = value

		savePreferences()
		
	else
	
		error( "prefs values must be scalar types only (boolean, number, string)" )
	
	end
end

--------------------------------------------------------------------------------

return KSPrefs

