--[[----------------------------------------------------------------------------

KSPrivateKeys.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author mhamburg 2011-05-19

------------------------------------------------------------------------------]]

--[[

	KSPrivateKeys: Support for private keys
	
	Private keys are tables intended to be used as private keys in other
	tables. They can't be synthesized but they can be extracted via enumeration
	so they aren't perfectly secure.
	
	__index and __newindex implementations that guard against reading or
	assigning unknown keys should generally allow private keys because they
	will be used for attaching foreign data.
	
	function KSPrivateKeys.makePrivateKey( name ) -> private key
		-- Make a private key with the given name. name should be a string.
		
	function KSPrivateKeys.isPrivate( value ) -> boolean
		-- Returns true iff value is a private key.
	
]]

--============================================================================--

local KSPrivateKeys = { }

--------------------------------------------------------------------------------

local setmetatable = setmetatable
local assert = assert
local type = type
local string_format = string.format

--------------------------------------------------------------------------------

local privateKeyToName = setmetatable( {}, { __mode = 'kv' } )
	-- A set of all of the private keys. The values are the names.

--------------------------------------------------------------------------------

local privateKeyMT = {
	__metatable = '< Private Key Metatable >',
	__tostring = function( value )
		return privateKeyToName[ value ] or '< Error: Bad private key >'
	end

}

--------------------------------------------------------------------------------

function KSPrivateKeys.makePrivateKey( name )

	assert( type( name ) == 'string', "string expected for name" )
	
	local result = setmetatable( { }, privateKeyMT )
	
	privateKeyToName[ result ] = string_format( '< Private: %s >', name )
	
	return result

end

--------------------------------------------------------------------------------

function KSPrivateKeys.isPrivateKey( value )

	return not not privateKeyToName[ value ]

end

--------------------------------------------------------------------------------

return KSPrivateKeys
