--[[----------------------------------------------------------------------------

KSPrivateKeysTest.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author mhamburg 2011-05-19

------------------------------------------------------------------------------]]

local T = { }

--------------------------------------------------------------------------------

function T.testMakePrivateKey()

	local KSPrivateKeys = require 'KSPrivateKeys'
	
	local pk = KSPrivateKeys.makePrivateKey( "Foo" )
	
	local pks = tostring( pk )
	
	assert( string.find( pks, "Foo" ), "Private key should be able to get to the name" )

end

--------------------------------------------------------------------------------

function T.testDuplicatePrivateKeys()

	local KSPrivateKeys = require 'KSPrivateKeys'
	
	local pk1 = KSPrivateKeys.makePrivateKey( "Foo" )
	
	local pk2 = KSPrivateKeys.makePrivateKey( "Foo" )
	
	assert( pk1 ~= pk2, "Making two private keys with the same name should still make two keys" )

end

--------------------------------------------------------------------------------

function T.testIsPrivateKey()

	local KSPrivateKeys = require 'KSPrivateKeys'
	
	local pk = KSPrivateKeys.makePrivateKey( "Foo" )
	
	assert( KSPrivateKeys.isPrivateKey( pk ), "Private keys should report as private keys." )
	
	local others = { 1, "Baz", { }, true, false }
	
	for _, v in ipairs( others ) do
		assert( not KSPrivateKeys.isPrivateKey( v ), type( v ) .. " should not be reported as a private key" )
	end
	
	assert( not KSPrivateKeys.isPrivateKey( nil ), "nil should not be reported as a private key" )
	
	assert( not KSPrivateKeys.isPrivateKey(), "none should not be reported as a private key" )

end

--------------------------------------------------------------------------------

return T
