--[[----------------------------------------------------------------------------

KSReactiveBindingTest.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2010 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author mhamburg 2011-02-15

@reviewed erwright 2011-03-10

------------------------------------------------------------------------------]]

local KSReactiveBindingTest = { }

--------------------------------------------------------------------------------

local assert = assert

--------------------------------------------------------------------------------

function KSReactiveBindingTest.testWrapSetField()

	local KSReactiveBinding = require 'KSReactiveBinding'
	
	local KSReactive = require 'KSReactive'
	
	local KSPromises = require 'KSPromises'
	
	local WFCloseSupport = require 'WFCloseSupport'
	
	local eventA = KSReactive.sourceE()
	
	local behaviorB = KSReactive.variableB( "B1" )
	
	local eventC = KSReactive.sourceE()
	
	local function setField( t, k, v )
		t.values[ k ] = v
	end
	
	local wrappedSetField = KSReactiveBinding.wrapSetFieldForReactiveBinding( setField )
	
		-- FIXME [scouten 2011-12-16]: This direct use of WFCloseSupport methods is now deprecated.
		-- This is the only remaining reference to it, aside from WFCloseSupport's unit test.
		-- Can we convert to making this a proper class using WFCloseSupport.addCloseSupportMethods?
	local t1 = { values = { }, addCloseTarget = WFCloseSupport.addCloseTargetMethod, close = WFCloseSupport.closeMethod }
	
	wrappedSetField( t1, "foo", "_" )
	
	assert( t1.values.foo == "_", "Raw set needs to work." )
	
	wrappedSetField( t1, "foo", eventA:relayE() )
	
	assert( t1.values.foo == "_", "Binding an event stream shouldn't change things." )
	
	eventA:post( "A1" )
	
	assert( t1.values.foo == "A1", "Binding should have set us up for receiving events." )
	
	eventA:post( "A2" )
	
	assert( t1.values.foo == "A2", "Binding should have set us up for receiving events. (2)" )
	
	wrappedSetField( t1, "foo", behaviorB:relayB() )
	
	assert( t1.values.foo == "B1", "Binding to a behavior should have changed the value." )
	
	eventA:post( "A3" )
	
	assert( t1.values.foo == "B1", "Binding to a behavior should have broken the old binding link." )
	
	behaviorB:set( "B2" )
	
	assert( t1.values.foo == "B2", "Changing the behavior should update the value." )
	
	wrappedSetField( t1, "foo", nil )
	
	assert( t1.values.foo == nil, "Using a raw set should have adjusted the value." )
	
	behaviorB:set( "B3" )
	
	assert( t1.values.foo == nil, "Using a raw set should have broken the link to the behavior." )
	
	setmetatable( t1, { __newindex = wrappedSetField } )
	
	t1.foo = "X"
	
	assert( t1.values.foo == "X", "Wrapped function should work as __newindex metamethod." )
	
	t1.foo = eventC
	
	eventC:post( "C1" )
	
	assert( t1.values.foo == "C1", "We should be able to bind via assignment." )
	
	local pX, pXResolver = KSPromises.promiseP()
	
	t1.foo = pX

	assert( t1.values.foo == "C1", "Binding to an unresolved promise should not change the value." )
	
	assert( WFCloseSupport.hasBeenOrWillBeClosed( eventC ), "Changing the binding should have closed the source for the binding" )
	
	pXResolver( 'data', "X1" )
	
	assert( t1.values.foo == "X1", "Resolved promise should have updated the binding." )
	
	local pY, pYResolver = KSPromises.promiseP()
	
	pY:runUntilDone() -- Run it to silence the limbo warnings
	
	pYResolver( 'data', "Y1" )
	
	t1.foo = pY

	assert( t1.values.foo == "Y1", "Binding to a resolved promise should change the value." )
	
	local pZ, pZResolver = KSPromises.promiseP()
	
	t1.foo = pZ
	
	local pW, pWResolver = KSPromises.promiseP()
	
	t1.foo = pW
	
	pZResolver( 'data', "Z1" )
	
	assert( t1.values.foo == "Y1", "We should not update the binding after we've rebound." )
	
	pWResolver( 'data', "W1" )
	
	assert( t1.values.foo == "W1", "The binding change on the promise should pick up the resolution." )

end

--------------------------------------------------------------------------------

return KSReactiveBindingTest
