--[[----------------------------------------------------------------------------

KSStringUtilsTest.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author nicholas 2012-03-30


------------------------------------------------------------------------------]]

local KSStringUtilsTest = {}

--------------------------------------------------------------------------------

local KSStringUtils = require 'KSStringUtils'

local WFPlatformInfo = require 'WFPlatformInfo'
--------------------------------------------------------------------------------


function KSStringUtilsTest.testi18n_collate()
	
	assert( KSStringUtils.localizedStringCompare( "Abc", "Def" ) < 0, "'Abc' not < 'Def'" )
	assert( KSStringUtils.localizedStringCompare( "Abc", "Abc" ) == 0, "'Abc' != 'Abc'" )
	assert( KSStringUtils.localizedStringCompare( "Def", "Abc" ) > 0, "'Def' not > 'Abc'" )
	
	assert( KSStringUtils.localizedStringCompare( "A", "N" ) < 0, "1: 'A' not < 'N'" )
	assert( KSStringUtils.localizedStringCompare( "N", "n" ) < 0, "2: 'N' not < 'n'" )
	assert( KSStringUtils.localizedStringCompare( "n", "N1" ) < 0, "3: 'n' not < 'N1'" )
	assert( KSStringUtils.localizedStringCompare( "N1", "n1" ) < 0, "4: 'N1' not < 'n1'" )
	assert( KSStringUtils.localizedStringCompare( "n1", "N2" ) < 0, "5: 'n1' not < 'N2'" )
	assert( KSStringUtils.localizedStringCompare( "N2", "n2" ) < 0, "6: 'N2' not < 'n2'" )
		
	if not WFPlatformInfo.isAndroid and not WFPlatformInfo.isWindows then
		assert( KSStringUtils.localizedStringCompare( "n2", "N10" ) < 0, "7: 'n2' not < 'N10'" )
	end
		
	assert( KSStringUtils.localizedStringCompare( "N10", "n10" ) < 0, "8: 'N10' not < 'n10'" )
	assert( KSStringUtils.localizedStringCompare( "n10", "\195\145" ) < 0, "9: 'n10' not < '\195\145'" )
	assert( KSStringUtils.localizedStringCompare( "\195\145", "\195\1451" ) < 0, "10: '\195\145' not < '\195\1451'" )
	assert( KSStringUtils.localizedStringCompare( "\195\1451", "\195\1452" ) < 0, "11: '\195\1451' not < '\195\1452'" )
		
	if not WFPlatformInfo.isAndroid and not WFPlatformInfo.isWindows then
		assert( KSStringUtils.localizedStringCompare( "\195\1452", "\195\14510" ) < 0, "12: '\195\1452' not < '\195\14510'" )
		assert( KSStringUtils.localizedStringCompare( "\195\14510", "Z" ) < 0, "13: '\195\14510' not < 'Z'" )
	end
	
end

--------------------------------------------------------------------------------

function KSStringUtilsTest.testi18n_localized_standard_compare()
	
	assert( KSStringUtils.localizedStandardStringCompare( "Abc", "Def" ) < 0, "'Abc' not < 'Def'" )
	assert( KSStringUtils.localizedStandardStringCompare( "Abc", "Abc" ) == 0, "'Abc' != 'Abc'" )
	assert( KSStringUtils.localizedStandardStringCompare( "Def", "Abc" ) > 0, "'Def' not > 'Abc'" )

	assert( KSStringUtils.localizedStandardStringCompare( "A", "n" ) < 0, "1: 'A' not < 'n'" )
	
	-- FIXME_WINRT:  This exception can be eliminated if the parameters are reversed for the case sensitive compare (see Android).
	if not WFPlatformInfo.isWindows then
		assert( KSStringUtils.localizedStandardStringCompare( "n", "N" ) < 0, "2: 'n' not < 'N'" )
	end
	
	assert( KSStringUtils.localizedStandardStringCompare( "N", "\195\145" ) < 0, "3: 'N' not < '\195\145'" )

	if not WFPlatformInfo.isAndroid and not WFPlatformInfo.isWindows then
		assert( KSStringUtils.localizedStandardStringCompare( "\195\145", "n1" ) < 0, "4: '\195\145' not < 'n1'" )
	end

	-- FIXME_WINRT:  This exception can be eliminated if the parameters are reversed for the case sensitive compare (see Android).
	if not WFPlatformInfo.isWindows then
		assert( KSStringUtils.localizedStandardStringCompare( "n1", "N1" ) < 0, "5: 'n1' not < 'N1'" )
	end

	assert( KSStringUtils.localizedStandardStringCompare( "N1", "\195\1451" ) < 0, "6: 'N1' not < '\195\1451'" )

	if not WFPlatformInfo.isAndroid and not WFPlatformInfo.isWindows then
		assert( KSStringUtils.localizedStandardStringCompare( "\195\1451", "n2" ) < 0, "7: '\195\1451' not < 'n2'" )
	end

	-- FIXME_WINRT:  This exception can be eliminated if the parameters are reversed for the case sensitive compare (see Android).
	if not WFPlatformInfo.isWindows then
		assert( KSStringUtils.localizedStandardStringCompare( "n2", "N2" ) < 0, "8: 'n2' not < 'N2'" )
	end

	assert( KSStringUtils.localizedStandardStringCompare( "N2", "\195\1452" ) < 0, "9: 'N2' not < '\195\1452'" )

	if not WFPlatformInfo.isAndroid and not WFPlatformInfo.isWindows then
		assert( KSStringUtils.localizedStandardStringCompare( "\195\1452", "n10" ) < 0, "10: '\195\1452' not < 'n10'" )
	end

	-- FIXME_WINRT:  This exception can be eliminated if the parameters are reversed for the case sensitive compare (see Android).
	if not WFPlatformInfo.isWindows then
		assert( KSStringUtils.localizedStandardStringCompare( "n10", "N10" ) < 0, "11: 'n10' not < 'N10'" )
	end

	assert( KSStringUtils.localizedStandardStringCompare( "N10", "\195\14510" ) < 0, "12: 'N10' not < '\195\14510'" )

	if not WFPlatformInfo.isAndroid and not WFPlatformInfo.isWindows then
		assert( KSStringUtils.localizedStandardStringCompare( "\195\14510", "Z" ) < 0, "13: '\195\14510' not < 'Z'" )
	end
	
end

--------------------------------------------------------------------------------

local kUpperADoubleDot = "\195\132"
local kLowerADoubleDot = "\195\164"


function KSStringUtilsTest.testLower()
	
	local function t( input, expected )
		local result = KSStringUtils.lower( input )
		local success = result == expected
		if not success then
			assert( success, "'" .. input .. "' not lowered" )
		end
	end
	
	t( "ABC", 'abc' )
	t( "AbC", 'abc' )
	t( "  AbC  ", '  abc  ' )
	t( "", '' )
	
	t( kUpperADoubleDot, kLowerADoubleDot )
	t( kLowerADoubleDot, kLowerADoubleDot )

end

--------------------------------------------------------------------------------


function KSStringUtilsTest.testUpper()

	local function t( input, expected )
		local result = KSStringUtils.upper( input )
		local success = result == expected
		if not success then
			assert( success, "'" .. input .. "' not lowered" )
		end
	end
	
	t( "ABC", 'ABC' )
	t( "AbC", 'ABC' )
	t( "  AbC  ", '  ABC  ' )
	
	t( kUpperADoubleDot, kUpperADoubleDot )
	t( kLowerADoubleDot, kUpperADoubleDot )

end

--------------------------------------------------------------------------------


function KSStringUtilsTest.testTrim()

	assert( KSStringUtils.trim( "ABC" ) == 'ABC', "'ABC' not trimmed" )
	assert( KSStringUtils.trim( "  AbC" ) == 'AbC', "'  AbC' not trimmed" )
	assert( KSStringUtils.trim( "AbC  " ) == 'AbC', "'AbC  ' not trimmed" )
	assert( KSStringUtils.trim( "   AbC  " ) == 'AbC', "'   AbC  ' not trimmed" )

end

--------------------------------------------------------------------------------

function KSStringUtilsTest.testBase64()

	local KSData = require "KSData"
	
	local input = KSData.make( "Revel\n" )
	local encoded = KSStringUtils.encodeBase64( input )
	assert( encoded == 'UmV2ZWwK', "'Revel' not base64 encoded" )
	
	input = "V2ljaGl0YSBGb3VuZGF0aW9uCg=="
	local decodedString = KSStringUtils.decodeBase64AsString( input )
	assert( decodedString == 'Wichita Foundation\n', "'Wichita Foundation' not base64 decoded" )
	local decoded = KSStringUtils.decodeBase64( input )
	local shaSum = decoded:digest( "SHA1" )
	assert( shaSum == '2ab9cb1ad8ad25cd2ba1448da3cb705b44a44c84', "'Revel' not base64 decoded" )

end

--------------------------------------------------------------------------------

function KSStringUtilsTest.testConvertToHex()

	local KSData = require "KSData"
	
	local input = KSData.make( "TestKSData" )
	local hexOutput = KSStringUtils.convertToHex( input )
	assert( hexOutput == '546573744b5344617461', "'TestKSData' not converted to hex." )
	
	input = "TestStringData"
	local stringHexOutput = KSStringUtils.convertToHex( input )
	assert( stringHexOutput == '54657374537472696e6744617461', "'TestStringData' not converted to hex." )

end

--------------------------------------------------------------------------------


return KSStringUtilsTest
