--[[----------------------------------------------------------------------------

KSUndoManagerTest.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author erwright 2011-02-05

@reviewed mhamburg 2011-03-11

------------------------------------------------------------------------------]]

local KSUndoManager = require 'KSUndoManager'

--------------------------------------------------------------------------------

local KSUndoManagerTest = { }

--------------------------------------------------------------------------------

function KSUndoManagerTest.testTransaction()

	local success = pcall( KSUndoManager.beginTransaction )
	assert( not success, "beginTransaction didn't throw on empty name" )

	success = pcall( KSUndoManager.beginTransaction, 1 )
	assert( not success, "beginTransaction didn't throw on numeric name" )
	
	local x = 5
	
	KSUndoManager:beginTransaction( "foo" )
	assert( KSUndoManager:isTransactionActive() )
	
	local undo = function() x = x - 1 end
	local redo = function() x = x + 1 end
	redo()
	KSUndoManager:addUndoRedoFunctions( undo, redo )
	KSUndoManager:commit()
	
	assert( x == 6, "x wasn't incremented" )
	assert( KSUndoManager:canUndoB():valueNow() )
	assert( not KSUndoManager:canRedoB():valueNow() )
	
	KSUndoManager:undo()
	
	assert( x == 5, "x wasn't decremented" )
	assert( not KSUndoManager:canUndoB():valueNow() )
	assert( KSUndoManager:canRedoB():valueNow() )

	KSUndoManager:redo()
	
	assert( x == 6, "x wasn't incremented" )
	assert( KSUndoManager:canUndoB():valueNow() )
	assert( not KSUndoManager:canRedoB():valueNow() )

end

--------------------------------------------------------------------------------

return KSUndoManagerTest


