--[[----------------------------------------------------------------------------

OzBackupFlagAgentManager.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

This namespace coordinates the backup flag agents for all catalogs.

Suggest we cycle back and remove this code in summer or fall of 2012, on the
assumption that most people will have upgraded through one or more versions that
is running it by then.

------------------------------------------------------------------------------]]

	-- core.logging
local KSLogging = require 'KSLogging'

	-- wichitafoundation.core.lua.threading
local WFRequestDispatcher = require 'WFRequestDispatcher'

	-- libs.reactive
local KSReactive = require 'KSReactive'

	-- oz.backup_flag_agent
local OzBackupFlagAgent = require 'OzBackupFlagAgent'


--------------------------------------------------------------------------------

local backupFlagLog = KSLogging.log 'OzBackupFlagAgentLog'
--	:setLevel( KSLogging.DEBUG )


--============================================================================--

-- @mark OzBackupFlagAgentManager

local OzBackupFlagAgentManager = {}

--------------------------------------------------------------------------------

local activeBackupFlagAgentBehaviors = {}
	-- key: (string) catalogId
	-- value: (B< nil or OzBackupFlagAgent >) backup flag agent for this catalog

	-- NOTE: This is intentionally not a weak map. Once referenced, the
	-- behaviors stay present indefinitely so we can do mapB and mapSwitchB
	-- on them. We do clear out the agent itself.
	
--------------------------------------------------------------------------------

local function backupFlagAgentBehaviorForCatalogId( catalogId )

	local resultB = activeBackupFlagAgentBehaviors[ assert( catalogId ) ]
	
	if not resultB then
	
		resultB = KSReactive.variableB( nil )
		activeBackupFlagAgentBehaviors[ catalogId ] = resultB
	
	end
	
	return resultB

end


--============================================================================--

-- @mark backup flag agent management

--------------------------------------------------------------------------------

local WFPlatformInfo = require 'WFPlatformInfo'

--------------------------------------------------------------------------------

local function startBackupFlagAgentForCatalogId( catalogId )
	
	if not WFPlatformInfo.isiOS then return end
	
	assert( type( catalogId ) == 'string',
		"startBackupFlagAgent: catalogId was not a string" )
	
	local backupFlagAgentB = backupFlagAgentBehaviorForCatalogId( catalogId )
	
	if not backupFlagAgentB:valueNow() then

		backupFlagLog:debug( "Creating OzBackupFlagAgent for catalog ", catalogId )

		backupFlagAgentB:set( OzBackupFlagAgent( catalogId ) )

	end
	
end

--------------------------------------------------------------------------------

function OzBackupFlagAgentManager.installMessageHandlers()

	----------------------------------------------------------------------------
	
	WFRequestDispatcher.onMessageNoCancel( 'startBackupFlagAgent',
										   startBackupFlagAgentForCatalogId )
	
	----------------------------------------------------------------------------
	
	WFRequestDispatcher.onMessageNoCancel( 'stopBackupFlagAgent', function( catalogId )
	
		assert( type( catalogId ) == 'string',
			"stopBackupFlagAgent: catalogId was not a string" )
		
		local backupFlagAgentB = backupFlagAgentBehaviorForCatalogId( catalogId )
		
		local backupFlagAgent = backupFlagAgentB:valueNow()
		
		if backupFlagAgent then

			backupFlagLog:debug( "Closing OzBackupFlagAgent for catalog ", catalogId )

			backupFlagAgentB:set( nil )
				-- Do this first, so behaviors that are mapped from
				-- backupFlagAgentB don't also get closed.

			backupFlagAgent:close()

		end
		
	end )

	----------------------------------------------------------------------------

	WFRequestDispatcher.onMessage( 'backupFlagEnabledB', function( catalogId )

		return backupFlagAgentBehaviorForCatalogId( catalogId ):toBooleanB()
		
	end )
	
end

--------------------------------------------------------------------------------

return OzBackupFlagAgentManager
