--[[----------------------------------------------------------------------------

OzCatAbuse.lua

@author arahn 2012-08-30

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local KSJsonDecoder = require "KSJsonDecoder"
local OzCatalogUrls = require "OzCatalogUrls"
local KSTableUtils = require "KSTableUtils"
local OzCatHelperFns = require "OzCatHelperFns"

local WFOzRenditionServerUtils = require 'WFOzRenditionServerUtils'


--============================================================================--

local ozLinkNameForRenditionName = assert( WFOzRenditionServerUtils.ozLinkNameForRenditionName )

--------------------------------------------------------------------------------

-- @mark OzCatalogService class

local Methods = {
}

--------------------------------------------------------------------------------

--[[!
 	\brief Retrieves a Vader abuse report for a photo and rendition in this catalog that
	the application can show.
	
	\param photoId The unique ID of the photo.
	\param rendition The rendition, one of 'thumbnail' or 'preview'.

	\return <code>P< string ></code>  A Promise object that resolves to the URL of
		the abuse report.
]]--

function Methods:abuseReportingUrlP( --[[ string ]] photoId, --[[ string ]] rendition )

	if type( photoId ) ~= 'string' or #photoId == 0 then
		error( "OzCatalogService:abuseReportingUrlP photoId must be a non-empty string", 2 )
	end

	if type( rendition ) ~= 'string' or #rendition == 0 then
		error( "OzCatalogService:abuseReportingUrlP rendition must be a non-empty string", 2 )
	end
	
	if rendition ~= 'thumbnail' and rendition ~= 'preview' then
		error( "OzCatalogService:abuseReportingUrlP rendition did not specify a recognized rendition type", 2 )
	end
				
	return self._docStore:docValueOnceP( 'asset', photoId )
	:mapP( function ( status, data )
	
		if status == 'error' then
			return 'error', data
		end
		
		local relUrl = KSTableUtils.derefOpt( data,
											  '_serverProvidedMetadata',
											  'links',
											  ozLinkNameForRenditionName[ rendition ],
											  'href' )
		
		if not relUrl then
			return 'error', 'OzCatalogService:abuseReportingUrlP Href for this rendition was not found in the payload'
		end
		
		local abuseReportUrl	
		local completeUrl = OzCatHelperFns.checkForRelativeUrlAndAppendBase( relUrl, OzCatalogUrls.baseServerUrl )
		
		abuseReportUrl = string.format( "%s/abuse_report", completeUrl )
				
		local requestP = self._ozUser:getP( abuseReportUrl )
			:mapDataP( KSJsonDecoder.decodeWebSafeJson_stripNullP )
			:mapDataP( function( status, data )
				
			if data and data.links then
				local link = data.links['/rels/abuse_report'].href
				return 'data', link
			else
				return 'error', 'OzCatalogService:abuseReportingUrlP Could not fetch the abuse reporting URL for this asset'
			end
			
		end )
		
		requestP:runUntilDone()
		
		return requestP

	end )
	
end

--============================================================================--

return {
	AbuseMethods = Methods,
}
