--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local WFRequestDispatcher = require 'WFRequestDispatcher'

local OzCatComments = require 'OzCatComments'
local WFActivity = require 'WFActivity'
local WFDigest = require 'WFDigest'

--==============================================================================

local kCommentFields = {
	'assetId', 'author_id AS userId', 'created AS activityDate', 'comment.docId', 'coreInfo.importedBy AS photoOwnerId'
}

--------------------------------------------------------------------------------

local reformatActivity = function( activityType, needUsers, photoId, userId, serverDate )
	needUsers[ userId ] = needUsers[ userId ] or {}
	local userEntry = {
		id = userId,
	}
	table.insert( needUsers[ userId ], userEntry )

	return {
		type = activityType,
		bookmark = serverDate,
		id = WFDigest.MD5.digest( activityType .. photoId .. userId .. serverDate ),
		date = serverDate,
		users = {
			userEntry,
		},
		photos = {
			photoId,
		},
	}
end

--------------------------------------------------------------------------------

local function processActivity(optLimit, yourUserId, favorites, newAssets, newComments, newAlbumAssets)

	local combined = {}
	local needUsers = {}

	for idx, comment in ipairs( newComments ) do
		local commentId = comment[ 4 ]
		local assetOwnerId = comment[ 5 ]
		local formatted = OzCatComments.formatComment( comment, 1 + #kCommentFields )

		comment = reformatActivity( 'new comment', needUsers, unpack( comment, 1, 3 ) )
		comment.comment = {
			body = formatted.body,
			id = commentId,
		}
		comment.yourPhotos = assetOwnerId == yourUserId

		combined[ #combined + 1 ] = comment
	end
	
	for idx, activity in ipairs( newAssets ) do
		combined[ #combined + 1 ] = reformatActivity( 'new asset', needUsers, unpack( activity, 1, 3 ) )
	end
	
	for idx, activity in ipairs( favorites ) do
		local assetOwnerId = activity[ 4 ]
		local formatted = reformatActivity( 'new favorite', needUsers, unpack( activity, 1, 3 ) )
		formatted.yourPhotos = assetOwnerId == yourUserId
		combined[ #combined + 1 ] = formatted
	end
	
	for idx, activity in ipairs( newAlbumAssets ) do
		local newActivity = reformatActivity( 'new asset in album', needUsers, unpack( activity, 1, 3 ) )
		local album = {
			id = activity[ 4 ],
			name = activity[ 5 ],
		}
		newActivity.album = album
		combined[ #combined + 1 ] = newActivity
	end
	
	table.sort( combined, function( a, b )
		return a.date > b.date
	end )

	combined = WFActivity.rollUpActivity( combined, optLimit )
	
	return combined, not not (needUsers and next( needUsers ) )
	
end

--------------------------------------------------------------------------------

local function initMessageProcessor()
	WFRequestDispatcher.onMessage( 'processActivity', function( ... )
		return processActivity( ... )
	end )
end

--==============================================================================

return {
	kCommentFields = kCommentFields,
	initMessageProcessor = initMessageProcessor,
}