--[[----------------------------------------------------------------------------

OzCatActivityTest.lua

4/8/2013
arahn

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local WFOzSyncTestUtils = require 'WFOzSyncTestUtils'
local WFAsyncTestCase = require 'WFAsyncTestCase'
local WFPlatformInfo = require 'WFPlatformInfo'
local KSJsonEncoder = require 'KSJsonEncoder'
local OzCatalogTestUtils = require 'OzCatalogTestUtils'

local TEST_WAIT_TIME = 2

if WFPlatformInfo.isAndroid then
	TEST_WAIT_TIME = 6		-- PERFORMANCE_ANDROID
end

--==============================================================================
-- @mark setup
--==============================================================================

local TestSuite = {}

local OzCatalogService, WFIndexStoreSchema

local T = {}
local docStore, ozUser, catalog

--------------------------------------------------------------------------------

local kAllison = string.rep( 'a', 32 )
local kBarney = string.rep( 'b', 32 )
local kChloe = string.rep( 'c', 32 )
local kDick = string.rep( 'd', 32 )

local kPhoto1 = string.rep( '1', 32 )
local kPhoto2 = string.rep( '2', 32 )

local kCatalogId = string.rep( '0', 32 )

local kCommentId = string.rep( 'c0', 16 )
local kCommentBody = string.rep( "hello ", 12 )
local kAlbumId = string.rep( 'a1bb', 8 )
local kAlbumName = "My Album"
local kAlbumAssetId = string.rep( 'a1ba55e4', 4 )

--------------------------------------------------------------------------------

local kUsers = {
	[ kAllison ] = {
		first_name = 'Alli',
		last_name = 'Ansi',
		full_name = 'Allison Ansi',
	},
	[ kBarney ] = {
		first_name = 'Barney',
		last_name = 'Barnes',
		full_name = 'Barney B Barnes',
	},
	[ kChloe ] = {
		first_name = 'Chloe',
		last_name = 'Chism',
		full_name = 'Chloe Chism-Chisholm',
	},
	[ kDick ] = {
		first_name = 'Dick',
		last_name = 'Phillip',
		full_name = 'Richard K Phillip',
	},
}

--------------------------------------------------------------------------------

local curTime = 0
local function newDate()
	curTime = curTime + 1
	local KSDate = require 'KSDate'
	return KSDate.timeInIso8601Format( curTime * 3600 )
end

--------------------------------------------------------------------------------

local function newAssetDoc( overlay )
	local result = {
		width = 1,
		height =1,
		_serverProvidedMetadata = {
			created = newDate(),
			links = {
				['/rels/rendition_type/144h'] = {
					href = 'somewhere',
				}
			}
		}
	}
	for k,v in pairs( overlay ) do
		result [ k ] = v
	end
	return result
end

--------------------------------------------------------------------------------

local function makeOzUser( id )
	local src = kUsers[ id ]
	local oz = {
		_id = id,
		first_name = src.first_name,
		last_name = src.last_name,
		full_name = src.full_name,
	}
	return oz
end

--------------------------------------------------------------------------------

local function makeWFUser( id )
	local oz = kUsers[ id ]
	local wf = {
		id = id,
		firstName = oz.first_name,
		lastName = oz.last_name,
		fullName = oz.full_name,
	}
	return wf
end

--==============================================================================

function TestSuite.setUp()
	
	curTime = 0
	
		-- Utterly fake oz user.  Vends canned responses to GETs and stores
		-- of what was PUT for later analysis.
	ozUser = WFOzSyncTestUtils.makeFakeOzUser( T, makeOzUser( kAllison ) )
	
	docStore, catalog = OzCatalogTestUtils.buildMocksForTestingCatalogService( T, ozUser, kCatalogId, { [ 'OzCatActivityImp.lua' ] = true } )
	
	T.userGetPutResponse[ 'https://api.oznext.com/v1.0/catalogs/' .. kCatalogId .. '/users' ] = {
		'data',
		KSJsonEncoder.encodeWebSafeJson {
			makeOzUser( kAllison ),
			makeOzUser( kBarney ),
			makeOzUser( kDick ),
			makeOzUser( kChloe ),
		},
	}

end

--------------------------------------------------------------------------------

function TestSuite.tearDown()
	catalog:close()
	docStore, ozUser, catalog, OzCatalogService, WFIndexStoreSchema = nil
	T = {}
end

--==============================================================================

function TestSuite.test_activityB_newFavorites( test )
	
	docStore:setServerValueP(
		'asset',
		kPhoto1,
		newAssetDoc {
			importSource = {
				importedBy = kAllison,
					-- Allison is the catalog owner
			},
			favorites = {
				[ kAllison ] = newDate(),
				[ kBarney ] = newDate(),
				[ kChloe ] = newDate(),
				[ kDick ] = newDate(),
			},
		}
	)
	
	docStore:setServerValueP(
		'asset',
		kPhoto2,
		newAssetDoc {
			importSource = {
				importedBy = kBarney,
			},
			favorites = {
				[ kAllison ] = newDate(),
				[ kBarney ] = newDate(),
				[ kChloe ] = newDate(),
				[ kDick ] = newDate(),
			},
		}
	)
	
	return WFAsyncTestCase( test, TEST_WAIT_TIME )

		:waitFor( catalog:activityB():mapB( function( a )
			return a
		end ) )
		:continueWhenValueMatches( {
			{
				andMaybeMore = false,
				bookmark = "2001-01-01T10:00:00.000Z",
				date = "2001-01-01T10:00:00.000Z",
				id = "ce695feb12726773d9b945e6c823e2ea",
				photos = {
					kPhoto2,
				},
				type = "new asset",
				users = {
					makeWFUser( kBarney ),
				},
			},
			{
				andMaybeMore = false,
				bookmark = "2001-01-01T09:00:00.000Z",
				date = "2001-01-01T09:00:00.000Z",
				id = "2a4c3bd0ed79687a55695bbafec224ce",
				photos = {
					kPhoto2,
				},
				type = "new favorite",
				users = {
					makeWFUser( kBarney ),
					makeWFUser( kChloe ),
					makeWFUser( kDick ),
				},
				yourPhotos = false,
			},
			{
				andMaybeMore = false,
				bookmark = "2001-01-01T04:00:00.000Z",
				date = "2001-01-01T04:00:00.000Z",
				id = "1db3ce0e94142d9fa666295bf9088a50",
				photos = {
					kPhoto1,
				},
				type = "new favorite",
				users = {
					makeWFUser( kBarney ),
					makeWFUser( kChloe ),
					makeWFUser( kDick ),
				},
				yourPhotos = true,
			},
		}
 	)
		

end

--------------------------------------------------------------------------------

function TestSuite.test_activityB_newComments( test )
	
	docStore:setServerValueP(
		'asset',
		kPhoto1,
		newAssetDoc {
			importSource = {
				importedBy = kAllison,
			},
			favorites = {
			},
		}
	)
	
	docStore:setServerValueP(
		'comment',
		kCommentId,
		{
			created = newDate(),
			asset = {
				_id = kPhoto1,
			},
			body = kCommentBody,
			author = makeOzUser( kChloe ),
		}
	)
	
	return WFAsyncTestCase( test, TEST_WAIT_TIME )

		:waitFor( catalog:activityB():mapB( function( a )
			return a
		end ) )
		:continueWhenValueMatches( {
			{
				andMaybeMore = false,
				bookmark = "2001-01-01T02:00:00.000Z",
				comment = {
					body = kCommentBody,
					id = kCommentId,
				},
				date = "2001-01-01T02:00:00.000Z",
				id = "6f057e2cc00d440034265c0e53d01a41",
				photos = {
					kPhoto1,
				},
				type = "new comment",
				users = {
					makeWFUser( kChloe ),
				},
				yourPhotos = true,
			},
		}
 	)
		

end

--------------------------------------------------------------------------------

function TestSuite.test_activityB_newAssetInAlbum( test )
	
	docStore:setServerValueP(
		'asset',
		kPhoto1,
		newAssetDoc {
			importSource = {
				importedBy = kAllison,
			},
			favorites = {
			},
		}
	)
	
	docStore:setServerValueP(
		'album',
		kAlbumId,
		{
			updated = newDate(),
			name = kAlbumName,
		}
	)
	
	docStore:setServerValueP(
		'album_asset',
		kAlbumAssetId,
		{
			updated = newDate(),
			album = {
				_id = kAlbumId,
			},
			asset = {
				_id = kPhoto1,
			},
			added_by = "unittest::extras::" .. kBarney,
		}
	)
	
	return WFAsyncTestCase( test, TEST_WAIT_TIME )

		:waitFor( catalog:activityB():mapB( function( a )
			return a
		end ) )
		:continueWhenValueMatches( {
			{
				andMaybeMore = false,
				album = {
					id = kAlbumId,
					name = "My Album",
				},
				bookmark = "2001-01-01T03:00:00.000Z",
				date = "2001-01-01T03:00:00.000Z",
				id = "6e8aa9133c4643286409223aa75aeae7",
				photos = {
					kPhoto1,
				},
				type = "new asset in album",
				users = {
					makeWFUser( kBarney ),
				},
			},
		}
 	)
		

end

--==============================================================================

return TestSuite