--[[----------------------------------------------------------------------------

OzCatCache.lua

@author arahn 2012-08-30

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local KSLogging = require "KSLogging"
local WFPlatformInfo = require "WFPlatformInfo"
local WFWorkerThread = require "WFWorkerThread"


--============================================================================--

local BabluEP = WFWorkerThread( "BabluImp.lua" )

-- @mark OzCatalogService class

local OzCatCacheMethods = {}

--------------------------------------------------------------------------------

local purgeLog = KSLogging.log 'OzCatalogService :: purge'
	--	:setLevel( KSLogging.DEBUG )

--------------------------------------------------------------------------------

-- Most of the implementation for this is down in WFOzPurgeAgent.

--------------------------------------------------------------------------------
--[[!
	\brief Manages the automatic backup flags for this catalog. Automatic backup
	should be enabled for any catalog that is visible in the UI.
	
	\param shouldCheckBackupFlags (boolean) The new value, \c true to enable
		automatic backup, \c false to disable checking.
]]--

function OzCatCacheMethods:setBackupFlagEnabled( --[[ bool ]] shouldCheckBackupFlags )

	assert( type( shouldCheckBackupFlags ) == 'boolean' )

	purgeLog:debug( "Setting backup flag agent for catalog %s to %s",
					self._catalogId, shouldCheckBackupFlags )

	BabluEP:requestP( shouldCheckBackupFlags and 'startBackupFlagAgent' or 'stopBackupFlagAgent',
					  self._catalogId )
		:runUntilDone()

end

--------------------------------------------------------------------------------
--[[!
	\brief Enables/disables active cache management for this catalog. Active cache management
		should be enabled for any catalog that is visible in the UI.
	
	\param shouldPurge (boolean) The new value, \c true to enable active cache
		management, \c false to disable.
]]--

function OzCatCacheMethods:setPurgeEnabled( --[[ bool ]] shouldPurge )

	assert( type( shouldPurge ) == 'boolean' )

	purgeLog:debug( "Setting purgeEnabled for catalog %s to %s",
					self._catalogId, shouldPurge )

	BabluEP:requestP( shouldPurge and 'startPurgeAgent' or 'stopPurgeAgent',
					  self._catalogId )
		:runUntilDone()

end

--------------------------------------------------------------------------------
--[[!
	\brief Enables or disables downloading of originals for this catalog. Should only be
	enabled for Mac OS. This setting can also be toggled using a UI element.
	
	\param shouldDownload (boolean) The new value, \c true to enable downloading,
		\c false to disable.
	
]]--

function OzCatCacheMethods:setDownloadOriginalsEnabled( --[[ bool ]] shouldDownload )

	-- Only allow download originals for MAC

	if not WFPlatformInfo.isMac then return end

	if type( shouldDownload ) ~= 'boolean' then
		debug.print( "setDownloadOriginalsEnabled unexpected value ", shouldDownload )
	end

	assert( type( shouldDownload ) == 'boolean' )

	BabluEP:requestP( shouldDownload and 'startOriginalsAgent' or 'stopOriginalsAgent',
					  self._catalogId,
					  self._ozUserId )
		:runUntilDone()

end

--==============================================================================

return {
	CacheMethods = OzCatCacheMethods,
}
