--[[----------------------------------------------------------------------------

OzCatDiagnosticLog.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- wichitafoundation.core.diagnostic_log
local WFDiagnosticLog = require 'WFDiagnosticLog'

	-- wichitafoundation.core.reactive
local KSReactivePromiseBridge = require 'KSReactivePromiseBridge'

	-- wichitafoundation.core.string
local KSStringUtils = require 'KSStringUtils'

	-- wichitafoundation.oz_client.catalog
local OzCatHelperFns = require 'OzCatHelperFns'


--============================================================================--

local OzCatHelperFns_hasDownloadedSuccessfullyB = assert( OzCatHelperFns.hasDownloadedSuccessfullyB )

--------------------------------------------------------------------------------

-- @mark OzCatalogService class

local DiagnosticLogMethods = {}

--------------------------------------------------------------------------------
--[[!
	\brief Writes information about this catalog to the diagnostic log.

]]--
function DiagnosticLogMethods:sendDiagnosticLogData()

	-- FIXME_AUTO_CLOSE: Do I need the relay here?

	local nameP = KSReactivePromiseBridge.behaviorToPromise( self:nameB():relayForAutoCloseB(),
		function( value )
			return not not value
		end )
	
	nameP:whenResolvedDo( function( _, name )
	
		local syncDisabledBy = self._syncDisabledBy
		if syncDisabledBy then
			syncDisabledBy = KSStringUtils.split( self._syncDisabledBy, '\n' )
		end
	
		WFDiagnosticLog.addSectionToLog {

			name = "Catalog: " .. KSStringUtils.escapeHtml( name ),
			id = 'OzCatalogService-' .. self._catalogId,
			weight = 200,

			body = {
			
				catalogUrls = WFDiagnosticLog.shallowCopyOfSimpleValues( require 'OzCatalogUrls' ),

				catalogId = self._catalogId,
				ozUserId = self._ozUserId,
				catalogName = name,
				catalogInfo = self:catalogInfoP():runUntilDoneP(),

				catalogOwnerStatus = self:catalogOwnerStatusB(),
				catalogOwnerId = self:catalogOwnerIdP():runUntilDoneP(),
				
				catalogOwnerInfo = self:catalogOwnerUserP()
					:mapDataP( function( _, ozUser )
						return ozUser:userInfoP()
					end )
					:runUntilDoneP(),
				
				canAccess = self:canAccessB(),
				
				catalogUsers = self:catalogUsersP():runUntilDoneP(),
				collaboratorQuota = self:collaboratorQuotaB(),
				
				originalsFolderDocuments = self._originalsFolderDocuments,
				originalsFolderCaches = self._originalsFolderCaches,
				
				assetCount = self:getAssetCountB(),
				
				originalsFolderDocuments = self._originalsFolderDocuments,
				originalsFolderCaches = self._originalsFolderCaches,
			
				hasCompletedInitialSync = self:hasCompletedInitialSyncB(),
				hasDownloadedSuccessfully = OzCatHelperFns_hasDownloadedSuccessfullyB( self._catalogId ),
				uploadError = self:raw_uploadErrorB(),
				downloadError = self:raw_downloadErrorB(),
				
				syncEnabled = self:syncEnabledB(),
				syncDisabledBy = syncDisabledBy or nil,
				
				photosWithConflict = self._indexStore:queryP(
					[[SELECT count( rowid ) FROM ozUploadStatus WHERE ozUploadStatus == '_conflict']] )
						:runUntilDoneP(),
					
				selectedPhotoIds = self:getSelectedPhotosP():runUntilDoneP(),
				selectedPhotosCount = self:selectedPhotosCountP():runUntilDoneP(),
				
				albumNamesSorted = self:albumsDateSortedB(),
				smartAlbums = self:smartAlbumsB(),
				
				nPhotosAdded = self._nPhotosAdded,
				
				recentReachabilityStatusUpdates = self._ozHostReachabilityLog,

			}
		
		}
	
	end ):runUntilDone()
			
end

--==============================================================================

return {
	DiagnosticLogMethods = DiagnosticLogMethods,
}
