--[[----------------------------------------------------------------------------

OzCatPhotoInfo.lua

@author scouten 2013-01-16

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- wichitafoundation.core.reactive
local KSReactive = require 'KSReactive'

	-- wichitafoundation.core.utils
local WFArguments = require 'WFArguments'


--------------------------------------------------------------------------------

local check = assert( WFArguments.check )


--============================================================================--

-- @mark OzCatalogService class

local OzCatPhotoInfoMethods = {}

--------------------------------------------------------------------------------
--[[!
	\brief Retrieves (as a Behavior) information about a specific photo in
		this catalog.

	\param photoId (optional, string) The unique asset ID. May be \c nil,
		in which case the result behavior will always return \c nil.
	
	\return <code>B< table ></code> A Behavior object that resolves to
		the photo information.
	
	The photo information is a large table whose contents are specified on the
	Revel project web site at http://wichita.corp.adobe.com/2011/09/23/spec-for-asset-data-for-carousel-version-1-x/
]]--

function OzCatPhotoInfoMethods:photoInfoB( --[[optional, string]] photoId )

	check( 'OzCatalogServiceImp', '?string' )

	if photoId == nil then
		return KSReactive.constantB( nil )
	else
		return self._docStore:docValueB( 'asset', photoId )
	end

end

--------------------------------------------------------------------------------
--[[!
	\brief Retrieves (as a Promise) information about a specific photo in
		this catalog.

	\param photoId (string) The unique asset ID. May not be \c nil.
	
	\return <code>P< table ></code> A Promise object that resolves to
		the photo information.
		
	The photo information is a large table whose contents are specified on the
	Revel project web site at http://wichita.corp.adobe.com/2011/09/23/spec-for-asset-data-for-carousel-version-1-x/
]]--

function OzCatPhotoInfoMethods:photoInfoP( --[[string]] photoId )

	check( 'OzCatalogServiceImp', 'string' )

	return self._docStore:docValueOnceP( 'asset', photoId )

end

--------------------------------------------------------------------------------

return {
	PhotoInfoMethods = OzCatPhotoInfoMethods,
}
