--[[----------------------------------------------------------------------------

OzCatUsers.lua

@author arahn 2012-08-30

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

	-- wichitafoundation.core.reactive
local KSReactive = require 'KSReactive'
local KSReactivePromiseBridge = require 'KSReactivePromiseBridge'

	-- wichitafoundation.core.string
local KSStringUtils = require 'KSStringUtils'

	-- wichitafouncation.oz_client.account
local OzUser = require 'OzUser'

	-- wichitafoundation.oz_client.catalog
local OzCatalogUrls = require 'OzCatalogUrls'

	-- wichitafoundation.web.json
local KSJsonDecoder = require 'KSJsonDecoder'
local KSJsonEncoder = require 'KSJsonEncoder'


--============================================================================--

-- @mark OzCatalogService class

local OzUserMethods = {}

--------------------------------------------------------------------------------

--[[!
	\brief Retrieves (as a Behavior)  all users who are collaborators in this catalog.
	((is that right??))
	
	\return <code>B< table ></code> A Behavior object whose value
		is the list of user IDs.  ((is that right??))
]]--

function OzUserMethods:catalogUsersB()

	if not self._catalogUsersB then

		self._catalogUsersB = self._docStore:fieldValueB( 'catalog', self._catalogId, 'catalogUsers' )

		self:addCloseTarget( self._catalogUsersB )

	end

	return self._catalogUsersB:relayB()

end

--------------------------------------------------------------------------------
--[[!
	\brief Reports (as a Behavior) the number of collaborators left in this
	catalog owner's quota.
	
	\return <code>B< nil | number ></code> A Behavior object whose value
		is the number of collaborators.	Initial value is nil.
]]--

function OzUserMethods:collaboratorQuotaB()

	if not self._collaboratorQuotaB then

		self._collaboratorQuotaB = self._docStore:fieldValueB( 'catalog', self._catalogId,
		'resources/users' )

		self:addCloseTarget( self._collaboratorQuotaB )

	end

	return self._collaboratorQuotaB:relayB()

end

--------------------------------------------------------------------------------

local function fetchCatalogOwnerIdP( self )

	return self._docStore:docValueOnceP( 'catalog', self._catalogId )
		:mapDataP( function( _, docValue )
		
			local ownerUrn = docValue and docValue.owner and docValue.owner.id
			if ownerUrn then
				local ownerId = ownerUrn:gsub( 'urn:carousel:account:', '' )
				return 'data', ownerId
			else
				return 'error', "owner/id missing from catalog data"
			end
	
	end )
	
end

--------------------------------------------------------------------------------

--[[!
	\brief Retrieves (as a Promise) the ID of the user who owns this catalog.
	((is that right??))
	
	\return <code>P< string ></code> A Promise object that resolves to
		the owner's user ID.
]]--

function OzUserMethods:catalogOwnerIdP()

	if not self._catalogOwnerIdP then

		self._catalogOwnerIdP = fetchCatalogOwnerIdP( self )
			:mapP( function( status, message )
				if status == 'error' and message == 'owner/id missing from catalog data' then
					return self:syncNowP():mapDataP( function()
						return fetchCatalogOwnerIdP( self )
					end )
				end
			
			end )

	end

	return self._catalogOwnerIdP

end

--------------------------------------------------------------------------------

--[[!
	\brief Retrieves (as a Behavior)  the ID of the user who owns this catalog.
	((is that right??))
	
	\return <code>B< string ></code> A Behavior object whose value is
		the owner's user ID.
]]--

function OzUserMethods:catalogOwnerIdB()

	return KSReactivePromiseBridge.promiseToBehavior( self:catalogOwnerIdP(), nil )

end

--------------------------------------------------------------------------------

--[[!
	\brief Retrieves (as a Behavior) the user who owns this catalog. ((is that right??))
	
	\return <code>B< OzUser ></code> A Behavior object whose value is
		the owner's Oz User object. ((??))
]]--

function OzUserMethods:catalogOwnerUserB()

	return self:catalogOwnerIdB()
		:rawMapB( function( status, userId )
		
			if status == 'error' then
				return 'data', nil
			else
				return 'data', ( userId and OzUser.make( userId ) or nil )
			end
		
		end )

end

--------------------------------------------------------------------------------

--[[!
	\brief Retrieves (as a Promise) the user who owns this catalog. ((is that right??))
	
	\return <code>P< OzUser ></code> A Promise object that resolves to
		the owner's Oz User object. ((??))
]]--
function OzUserMethods:catalogOwnerUserP()

	return self:catalogOwnerIdP()
		:mapDataP( function( _, userId )
		
			return 'data', userId and OzUser.make( userId ) or nil
		
		end )

end

--------------------------------------------------------------------------------

--[[!
	\brief Retrieves (as a Promise) information about the user who owns this catalog.
	((is that right??))
	
	\return <code>P< ?? ></code> A Promise object that resolves to ((??))
]]--

function OzUserMethods:catalogInfoP()

	return self._docStore:docValueOnceP( 'catalog', self._catalogId )

end

--------------------------------------------------------------------------------
--[[!
 	\brief Reports (as a Behavior)  whether the owner of this catalog is a
 	Freemium or trial user.

	\return <code>B< boolean|nil ></code> A Behavior object whose value is
		\c true if the catalog owner is a freemium or trial user, or
		nil if the owner is not found. ((is this right?))
--]]

function OzUserMethods:isCatalogOwnerFreemiumOrTrialUserB()

	return self:catalogOwnerUserB()
		:mapSwitchB( function( user )
		
			if user then
				return user:isFreemiumOrTrialUserB()
			else
				return KSReactive.constantB( nil )
			end

		end )

end

--------------------------------------------------------------------------------
--[[!
 	\brief Reports (as a Promise) whether the owner of this catalog is a Freemium or trial user.

	\return <code>P< boolean ></code> A Promise object that resolves to
		\c true if the catalog owner is a Freemium or trial user. ((is this right?))
--]]

function OzUserMethods:isCatalogOwnerFreemiumOrTrialUserP()

	return self:catalogOwnerUserP()
		:mapDataP( function( _, user )
			return user:isFreemiumOrTrialUserP()
		end )

end

--------------------------------------------------------------------------------
--[[!
 	\brief Reports (as a Behavior)  whether the owner of this catalog is a metered
 	Freemium user.
 	

	\return <code>B< boolean|nil ></code> A Behavior object whose value is
		\c true if the catalog owner is subject to metered Freemium limits, or
		nil if the owner is not found. ((is this right?))
--]]

function OzUserMethods:isCatalogOwnerMeteredFreemiumUserB()

	return self:catalogOwnerUserB()
		:mapSwitchB( function( user )
		
			if user then
				return user:isMeteredFreemiumUserB()
			else
				return KSReactive.constantB( nil )
			end

		end )

end

--------------------------------------------------------------------------------
--[[!
 	\brief Reports (as a Promise) whether the owner of this catalog is a metered Freemium user.

	\return <code>P< boolean ></code> A Promise object that resolves to
		\c true if the catalog owner is subject to metered Freemium limits.
		((is this right?))
--]]

function OzUserMethods:isCatalogOwnerMeteredFreemiumUserP()

	return self:catalogOwnerUserP()
		:mapDataP( function( _, user )
			return user:isMeteredFreemiumUserP()
		end )

end

--------------------------------------------------------------------------------
--[[!
	\brief Retrieves (as a Behavior) the maximum number of photos the catalog owner
	is allowed to upload in a period.
	
	\return <code>B< number|false|nil ></code> A Behavior object whose value is
	 the maximum allowed number of uploads, or \c false if uploads are unlimited,
	 or nil if the upload limit is not yet known.
]]--

function OzUserMethods:uploadLimitB()

	return self:catalogOwnerUserB()
		:mapSwitchB( function( user )
		
			if user then
				return user:uploadLimitB()
			else
				return KSReactive.constantB( nil )
			end

		end )

end

--------------------------------------------------------------------------------
--[[!
	\brief Retrieves (as a Promise) the maximum number of photos the catalog owner
	is allowed to upload in a period.
	
	\return <code>P< number|false ></code> A Promise object that resolves to
	 the maximum allowed number of uploads, or \c false if uploads are unlimited.
 ]]--

function OzUserMethods:uploadLimitP()

	return self:catalogOwnerUserP()
		:mapDataP( function( _, user )
			return user:uploadLimitP()
		end )

end

--------------------------------------------------------------------------------
--[[!
	\brief Retrieves (as a Behavior) the number of photos the catalog owner and collaborators
	have uploaded so far in the current period.
	
	\return <code>B< number|false|nil ></code> A Behavior object whose value is
	 the current number of uploads, or \c false if uploads are unlimited,
	 or nil if the number of uploads is not yet known.
]]--

function OzUserMethods:uploadUsageB()

	return self:catalogOwnerUserB()
		:mapSwitchB( function( user )
		
			if user then
				return user:uploadUsageB()
			else
				return KSReactive.constantB( nil )
			end

		end )

end

--------------------------------------------------------------------------------
--[[!
	\brief Retrieves (as a Promise) the number of photos the catalog owner and collaborators
	have uploaded so far in the current period.
	
	\return <code>P< number|false ></code> A Promise object that resolves to
	 the current number of uploads, or \c false if uploads are unlimited.
]]--

function OzUserMethods:uploadUsageP()

	return self:catalogOwnerUserP()
		:mapDataP( function( _, user )
			return user:uploadUsageP()
		end )

end

--------------------------------------------------------------------------------

--[[!
 	\brief Reports (as a Promise) whether the owner of this catalog is subject to upload limits,
 	and if so, the difference between the limit and the current number of uploads.

	\return <code>P< number|false ></code> A Promise object that resolves to
		 the greater of <code>limit - usage</code> or 0, or \c false
		 if uploads are unlimited.
--]]

function OzUserMethods:availableUploadsP()

	return self:catalogOwnerUserP()
		:mapDataP( function( _, user )
			return user:availableUploadsP()
		end )

end

--------------------------------------------------------------------------------
--[[!
	\brief Retrieves (as a Promise) all users who are collaborators in this catalog.
	((is this right?))

 	\return <code>P< array ></code> A Promise object that resolves to
	 the list of user IDs.	((is this right? array?))
]]--


function OzUserMethods:catalogUsersP()

	local catalogUsersUrl = string.format( '%s/%s/users', OzCatalogUrls.catalogsUrl,
						self._catalogId )
	
	return self._ozUser:getP( catalogUsersUrl )
		:mapDataP( function ( status, data, headers )
			
			local catalogUsers = KSJsonDecoder.decodeWebSafeJson( data )
			return 'data', catalogUsers
			
		end )

end

--------------------------------------------------------------------------------
--[[!
 	\brief Retrieves (as a Behavior) the account status for the owner of this catalog.
	
	\return <code>B< string|nil ></code> A Behavior object whose value is
		the account status. Initial value is nil.
]]--

function OzUserMethods:catalogOwnerStatusB()
	
	return self:catalogOwnerUserB()
		:mapSwitchB( function( user )
		
			if user then
				return user:accountStatusB()
			else
				return KSReactive.constantB( nil )
			end

		end )

end

--------------------------------------------------------------------------------

local function trim( s )

	return s:gsub("^%s*(.-)%s*$", "%1")

end

--------------------------------------------------------------------------------

-- checks if the loggedInOzUser (OzUser instance) and removeUser ( Oz User Id or email)
-- are the same

local function areEqual( loggedInOzUser, removeUser )

	-- removeUser could be an email or user id, so we need to compare both
	-- and if either matches it is the same user

	local email = KSStringUtils.trim( loggedInOzUser:emailB():valueNow() )

	return loggedInOzUser:id() == removeUser or email == removeUser

end

--------------------------------------------------------------------------------
--[[!
	
	Invites a user to be a collaborator in this catalog.
	
	\param invitedUsers (array) The email addresses for users to invite. 

	\return A Promise object that resolves to \c true if the request
		succeeded; it can be safely ignored.
]]--

function OzUserMethods:inviteUsersP( --[[table]] invitedUsers, --[[boolean]] sendInvite )

	if type( invitedUsers ) ~= 'table' or #invitedUsers == 0 then
		error( "OzCatalogService:inviteUsersP emails must be a non-empty table", 2 )
	end

	local resultP = self:catalogOwnerIdP()
		:mapDataP( function( status, catalogOwner )

			if self._ozUserId ~= catalogOwner then

				return 'error', 'OzUserMethods:inviteUserP Only the catalog owner can invite other users'

			end
		
			local catalogUrl = string.format( '%s/%s', OzCatalogUrls.catalogsUrl,
				self._catalogId )

			-- table of users

			local usersTable = {}
			
			for i = 1, #invitedUsers do
				-- prepend with "+" to signal addition of user
				usersTable[ i ] = "+mailto:" .. KSStringUtils.trim( invitedUsers[ i ] )
			end
			
			--local usersData = KSJsonEncoder.encodeOzUploadJson { users = usersTable }

			-- TORNADO_HACK
			-- add support for suppressing invite emails
			local usersData = {}
			
			if sendInvite then
				usersData = KSJsonEncoder.encodeOzUploadJson { users = usersTable }
			else
				usersData = KSJsonEncoder.encodeOzUploadJson { users = usersTable, suppress_notification = true }
			end
			-- END TORNADO HACK
			
			local postArgs = {
				requestHeaders = {
					{ header = 'Content-Type', value = 'application/json' },
					},
				requestMethod = 'POST',
			}

			return self._ozUser:putP( catalogUrl, usersData, postArgs )
				:mapP( function( status, data, headers )
					if status == 'data' then
						return 'data', true
					elseif status == 'error' then
						local result, errors = pcall( KSJsonDecoder.decodeWebSafeJson, data )
						if result and errors then
							return 'error', errors.code, errors.description
						else
							return 'error', false
						end
					end
				end )

		end )

	resultP:runUntilDone()

	return resultP

end


--------------------------------------------------------------------------------
--[[!
	\brief Invites a user to be a collaborator in this catalog.
	
	\param invitedUser (string) The email address for the user to invite.

	\return A Promise object that resolves to \c true if the request
		succeeded; it can be safely ignored.
]]--

function OzUserMethods:inviteUserP( --[[string]] invitedUser )

	if type( invitedUser ) ~= 'string' or #trim( invitedUser ) == 0 then
		error( "OzCatalogService:inviteUserP email must be a non-empty string", 2 )
	end

	-- Oz doesn't like whitespace before or after the email characters.
	-- It is done here so every client doesn't need to do the work.
	local trimmedInvitedUser = KSStringUtils.trim( invitedUser )

	local resultP = self:catalogOwnerIdP()
		:mapDataP( function( status, catalogOwner )

			if self._ozUserId ~= catalogOwner then

				return 'error', 'OzUserMethods:inviteUserP Only the catalog owner can invite other users'

			end
		
			local catalogUrl = string.format( '%s/%s', OzCatalogUrls.catalogsUrl,
				self._catalogId )

			-- table of users

			local usersTable = {}
			-- prepend with "+" to signal addition of user
			table.insert( usersTable, "+mailto:" .. trimmedInvitedUser )

			local usersData = KSJsonEncoder.encodeOzUploadJson { users = usersTable  }

			local postArgs = {
				requestHeaders = {
					{ header = 'Content-Type', value = 'application/json' },
					},
				requestMethod = 'POST',
			}

			return self._ozUser:putP( catalogUrl, usersData, postArgs )
				:mapP( function( status, data, headers )
					if status == 'data' then
						return 'data', true
					elseif status == 'error' then
						local result, errors = pcall( KSJsonDecoder.decodeWebSafeJson, data )
						if result and errors then
							return 'error', errors.code, errors.description
						else
							return 'error', false
						end
					end
				end )

		end )

	resultP:runUntilDone()

	return resultP

end

--------------------------------------------------------------------------------
-- FIXME [scouten 2011-09-22]: Document new function later.
--[[!
	\brief TBD ((what does it do?))

	\param removeUser (string) The email address for the user to remove.
	\param reload (boolean) Should the shared catalogs be reloaded after the user is removed; usually false or nil.

	\return A Promise object that resolves to \c true if the request
		succeeded; it can be safely ignored.
]]--

function OzUserMethods:removeUserP_knownNonOwner( --[[string]] removeUser, --[[boolean]] reload )
	
	local outerResultP = self:catalogUsersP():mapDataP( function ( _, users )
	
		-- Oz doesn't like whitespace before or after the email characters.
		-- It is done here so every client doesn't need to do the work.
		local trimmedRemoveUser = KSStringUtils.trim( removeUser )
			
		-- First check if this user is in the current list.
		-- if not return an error.
		local userExists = false
		for i = 1, #users do
			if KSStringUtils.trim( users[ i ].email ) == trimmedRemoveUser then
				userExists = true
				break
			end
		end
		
		if not userExists then
			return 'error', 'UserNotPresent'
		end
		
		local catalogUrl = string.format( '%s/%s', OzCatalogUrls.catalogsUrl,
			self._catalogId )
	
		-- table of users
	
		local usersTable = {}
	
		-- prepend with "-" to signal removal of user
	
		table.insert( usersTable, "-mailto:" .. trimmedRemoveUser )
	
		local usersData = KSJsonEncoder.encodeOzUploadJson { users = usersTable  }
	
		local postArgs = {
			requestHeaders = {
				{ header = 'Content-Type', value = 'application/json' },
			},
			requestMethod = 'POST',
		}

		local resultP = self._ozUser:putP( catalogUrl, usersData, postArgs )
				:mapP( function( status, data, headers )
					if status == 'data' then
						return 'data', true
					elseif status == 'error' then
						local result, errors = pcall( KSJsonDecoder.decodeWebSafeJson, data )
						if result and errors then
							return 'error', errors.code, errors.description
						else
							return 'error', false
						end
					end
				end )
	
		resultP:whenResolvedDo( function()

			-- if this was an attempt to leave a carousel, delete the local assets now (so that
			-- canAccessB() fires rignt away to alert the app that this carousel is no lnoger
			-- available), and also reload the shared catalogs list.

			if areEqual( self._ozUser, trimmedRemoveUser ) then
				self:deleteLocalCatalogAndAssetsP():runUntilDone()
				self._ozUser:requestSharedCatalogReload()
			end
			
			if reload then
				self._ozUser:requestSharedCatalogReload()
			end

		end ):runUntilDone()

		return resultP

	end )
	
	return outerResultP
	
end

--------------------------------------------------------------------------------
--[[!
 	\brief Removes a user from the list of collaborators in this catalog.

	\param removeUser (string) The email address for the user to remove.
	\param reload (boolean) Should the shared catalogs be reloaded after the user is removed; usually false or nil.

	\return A Promise object that resolves to \c true if the request
		succeeded; it can be safely ignored.
]]--

function OzUserMethods:removeUserP( --[[string]] removeUser, --[[boolean]] reload )

	if type( removeUser ) ~= 'string' or #trim( removeUser ) == 0 then
		error( "OzCatalogService:removeUserP email must be a non-empty string", 2 )
	end

	local resultP = self:catalogOwnerIdP()
		:mapDataP( function( _, catalogOwner )
	
			-- if the current user is not the catalog owner and is trying to remove
			-- another user, return error. If he is removing himself, allow.
	
			if self._ozUserId ~= catalogOwner then
		
				if not areEqual( self._ozUser, removeUser ) then
	
					return 'error', 'OzUserMethods:removeUserP Only the catalog owner can remove other users'
				end

			else

				-- if the current user ( also the catalog owner ) is trying to remove himself,
				-- return error
	
				if areEqual( self._ozUser, removeUser ) then
	
					return 'error', 'OzUserMethods:removeUserP The catalog owner cannot be removed from the catalog'
	
				end
			end
	
			return self:removeUserP_knownNonOwner( removeUser, reload )
			
		end )
	
	resultP:runUntilDone()
	
	return resultP

end

--------------------------------------------------------------------------------
--[[!
 	\brief Renames this catalog.
	\param newName (string) The new name.

	\return A Promise object that resolves to \c true if the request
		succeeded; it can be safely ignored.
]]--

function OzUserMethods:renameCatalogP( --[[string]] newName )

	if type( newName ) ~= 'string' or #trim( newName ) == 0 then
		return 'error', 'OzCatalogService:renameCatalogP name must be a non-empty string'
	end

	-- check if the current user is also the owner for this catalog, and if not error out

	local resultP = self:catalogOwnerIdP()
		:mapP( function ( status, owner )

			if owner ~= self._ozUserId then
	
				error( "OzCatalogService:renameCatalogP only catalog owners can rename their catalogs", 2 )
	
			end
	
			local catalogUrl = string.format( '%s/%s', OzCatalogUrls.catalogsUrl,
			self._catalogId )
	
			local newCatalogName = KSJsonEncoder.encodeOzUploadJson { name = newName }
	
			local postArgs = {
				requestHeaders = {
				{ header = 'Content-Type', value = 'application/json' },
				},
				requestMethod = 'POST',
			}
			
			return self._ozUser:putP( catalogUrl, newCatalogName, postArgs )
				:mapDataP( function()
					return 'data', true
				end )

		end )

	resultP:runUntilDone()
	
	return resultP

end

--------------------------------------------------------------------------------
--[[!
	\brief Starts watching for status updates from the owner of this catalog.
]]--

function OzUserMethods:watchCatalogOwnerUserStatus()

	if self._ownerUserBeingWatched then return end
	
	self:catalogOwnerUserP()
		:whenResolvedDo( function( status, ownerUser )
		
			if status == 'data' and ownerUser:id() ~= self._ozUser:id() then
				self._catalogOwnerUser = ownerUser
				ownerUser:startWatchingForCatalogUpdates( 0 )
				self:addCloseTarget( ownerUser )
			end
		
		end )
		:attachTo( self )

	self._ownerUserBeingWatched = true

end

--------------------------------------------------------------------------------

return {
	UserMethods = OzUserMethods,
}
