--[[----------------------------------------------------------------------------

OzCatalogIndexDefinitionsTest.lua

@author scouten 2013-02-12

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- wichitafoundation.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'

	-- wichitafoundation.oz_client.catalog
local OzCatalogIndexDefinitions = require 'OzCatalogIndexDefinitions'.test_indexStoreSchema

	-- wichitafoundation.oz_client.doc_store
local OzDocStore = require 'OzDocStore'

	-- wichitafoundation.oz_client.index_store
local WFIndexStoreMapServer = require 'WFIndexStoreMapServer'
local WFIndexStoreQueryServer = require 'WFIndexStoreQueryServer'

local WFPlatformInfo = require 'WFPlatformInfo'


--============================================================================--

-- @mark OzCatalogIndexDefinitionsTest

local OzCatalogIndexDefinitionsTest = {}

--------------------------------------------------------------------------------

local docStore, indexStoreDb, mapServer, queryServer

local TEST_TIMEOUT = 2

--------------------------------------------------------------------------------

function OzCatalogIndexDefinitionsTest.setUp()

	docStore = OzDocStore ':memory:'
	
	indexStoreDb = OzCatalogIndexDefinitions:openOrCreateDatabaseAtPath( ':memory:' )

	mapServer = WFIndexStoreMapServer( docStore, indexStoreDb, OzCatalogIndexDefinitions )
	
	queryServer = WFIndexStoreQueryServer( mapServer,
										   OzCatalogIndexDefinitions:tableNames() )

end

--------------------------------------------------------------------------------

function OzCatalogIndexDefinitionsTest.tearDown()

	if docStore then
		docStore:close()
		docStore = nil
	end
	
	if indexStoreDb then
		indexStoreDb:close()
		indexStoreDb = nil
	end

	if mapServer then
		mapServer:close()
		mapServer = nil
	end
	
	if queryServer then
		queryServer:close()
		queryServer = nil
	end
	
end

--------------------------------------------------------------------------------

-- @mark coreInfo table

--------------------------------------------------------------------------------

function OzCatalogIndexDefinitionsTest.testCoreInfo_ignoresPhotoWithoutThumbnail( test )

	WFAsyncTestCase( test, TEST_TIMEOUT )

		-- Add one "photo" to the doc store.
		
		:waitFor( function()
			
			return docStore:applyOverlayToDocumentP {
				docType = 'asset',
				docId = '651d079c86bd4514aa0ea10874515af2',
				overlay = {
					width = 30,
					height = 20,
					captureDate = '2012-08-15T15:22:29',
				},
			}
		
		end )

		-- Query result should be empty because this photo did not
		-- contain a thumbnail reference.

		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT docId, aspectRatio, captureDate, displayDate, timestamp,
						 copyTimestamp, thumbnailMD5, eventName, eventNameLC,
						 favoritedCount
					FROM coreInfo]]
			}
		end )

		:continueWhenValueMatches( 'data', {} )
		
end

--------------------------------------------------------------------------------

function OzCatalogIndexDefinitionsTest.testCoreInfo_acceptsPhotoWithLocalThumbnail( test )

	WFAsyncTestCase( test, TEST_TIMEOUT )

		-- Add one "photo" to the doc store.
		
		:waitFor( function()
			
			return docStore:applyOverlayToDocumentP {
				docType = 'asset',
				docId = '651d079c86bd4514aa0ea10874515af2',
				overlay = {
					_localOnly = {
						thumbnail = {
							epoch = 1,
							md5 = ( WFPlatformInfo.isTornado and '' or 'b772c4c900bae7e699468a56aa1fa194' ),
							referenceHeight = 128,
							referenceWidth = 96,
							renditionId = '5ec0b75370254587a3080c3558a6ad29',
							revId = 'd16e715e79da4c1b9144e369bb80e173',
							size = 5564,
						},
					},
					width = 96*4,
					height = 128*4,
					captureDate = '2012-08-15T15:22:29',
				},
			}
		
		end )
		
		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT docId, aspectRatio, captureDate, displayDate, timestamp,
						 copyTimestamp, thumbnailMD5, eventName, eventNameLC,
						 favoritedCount
					FROM coreInfo]]
			}
		end )

		:continueWhenValueMatches( 'data',
			{
				{
					'651d079c86bd4514aa0ea10874515af2', -- docId
					0.75, -- aspectRatio (taken from referenceHeight/Width)
					'2012-08-15T15:22:29', -- captureDate
					'2012-08-15', -- displayDate
					nil, -- timestamp
					nil, -- copyTimestamp
					( WFPlatformInfo.isTornado and '' or 'b772c4c900bae7e699468a56aa1fa194' ), -- thumbnailMD5
					'', -- eventName
					'', -- eventNameLC
					0, -- favoritedCount
				}
			}
		)
		
end

--------------------------------------------------------------------------------

function OzCatalogIndexDefinitionsTest.testCoreInfo_acceptsPhotoWithThumbnailOnOz( test )

	WFAsyncTestCase( test, TEST_TIMEOUT )

		-- Add one "photo" to the doc store.
		
		:waitFor( function()
			
			return docStore:setServerValueP( 'asset', '651d079c86bd4514aa0ea10874515af2',
				{
					_serverProvidedMetadata = {
						created = '2012-09-19T17:46:23.744702Z',
						links = {
							[ '/rels/rendition_type/144h' ] = {
								href = '/v1.0/catalogs/cb38211be46c43cf93fa81d0874c0912/assets/ee85f51507a84085a937273066dfb094/revisions/d16e715e79da4c1b9144e369bb80e173/renditions/5ec0b75370254587a3080c3558a6ad29',
							},
						},
						subtype = 'jpeg',
						type = 'asset',
						updated = '2012-09-19T17:46:27.278744Z',
					},
					captureDate = '2012-09-19T10:20:41',
					changedAtTime = '2012-09-19T10:46:37.254',
					height = 2400,
					importSource = {
						fileName = 'IMG_0191.JPG',
						fileSize = 1369123,
						importTimestamp = '2012-09-19T10:46:37.059',
						importedBy = '5cf9b3c832dd43f884a62fa7664cd926',
						importedOnDevice = "Eric's iPad 3",
						sha256 = 'cfa2508beabd49f6175aec956ec553e76fd657ea24f28576d6e18b0511bcc3df',
					},
					originalCaptureDate = '2012-09-19T10:20:41',
					version = 1,
					width = 3600,
				}
			)
		
		end )
		
		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT docId, aspectRatio, captureDate, displayDate, timestamp,
						 copyTimestamp, thumbnailMD5, eventName, eventNameLC,
						 favoritedCount
					FROM coreInfo]]
			}
		end )

		:continueWhenValueMatches( 'data',
			{
				{
					'651d079c86bd4514aa0ea10874515af2', -- docId
					1.5, -- aspectRatio
					'2012-09-19T10:20:41', -- captureDate
					'2012-09-19', -- displayDate
					'2012-09-19T10:46:37.059', -- timestamp
					nil, -- copyTimestamp
					nil, -- thumbnailMD5
					'', -- eventName
					'', -- eventNameLC
					0, -- favoritedCount
				}
			}
		)
		
end

--------------------------------------------------------------------------------

function OzCatalogIndexDefinitionsTest.testCoreInfo_prefersCropDimensions( test )

	WFAsyncTestCase( test, TEST_TIMEOUT )

		-- Add one "photo" to the doc store.
		
		:waitFor( function()
			
			return docStore:setServerValueP( 'asset', '651d079c86bd4514aa0ea10874515af2',
				{
					_serverProvidedMetadata = {
						created = '2012-09-19T17:46:23.744702Z',
						links = {
							[ '/rels/rendition_type/144h' ] = {
								href = '/v1.0/catalogs/cb38211be46c43cf93fa81d0874c0912/assets/ee85f51507a84085a937273066dfb094/revisions/d16e715e79da4c1b9144e369bb80e173/renditions/5ec0b75370254587a3080c3558a6ad29',
							},
						},
						subtype = 'jpeg',
						type = 'asset',
						updated = '2012-09-19T17:46:27.278744Z',
					},
					captureDate = '2012-09-19T10:20:41',
					changedAtTime = '2012-09-19T10:46:37.254',
					develop = {
						croppedHeight = 1200,
						croppedWidth = 1500,
						userOrientation = 1,
						xmpBase = [[<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="Adobe XMP Core 5.5-c002 1.148022, 2012/07/15-18:06:45 (debug)"> <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"> <rdf:Description rdf:about="" xmlns:crs="http://ns.adobe.com/camera-raw-settings/1.0/" crs:Version="7.3" crs:ProcessVersion="6.7" crs:ConvertToGrayscale="False" crs:LensProfileSetup="LensDefaults" crs:HasSettings="True" crs:CropTop="0.106875" crs:CropLeft="0.319922" crs:CropBottom="0.8425" crs:CropRight="0.951953" crs:CropAngle="-6.659619" crs:CropConstrainToWarp="0" crs:HasCrop="True" crs:AlreadyApplied="False"/> </rdf:RDF> </x:xmpmeta>]],
						xmpLook = '',
					},
					height = 1600,
					importSource = {
						fileName = 'IMG_0191.JPG',
						fileSize = 1369123,
						importTimestamp = '2012-09-19T10:46:37.059',
						importedBy = '5cf9b3c832dd43f884a62fa7664cd926',
						importedOnDevice = "Eric's iPad 3",
						sha256 = 'cfa2508beabd49f6175aec956ec553e76fd657ea24f28576d6e18b0511bcc3df',
					},
					originalCaptureDate = '2012-09-19T10:20:41',
					version = 1,
					width = 2560,
				}
			)
		
		end )
		
		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT docId, aspectRatio, captureDate, displayDate, timestamp,
						 copyTimestamp, thumbnailMD5, eventName, eventNameLC,
						 favoritedCount
					FROM coreInfo]]
			}
		end )

		:continueWhenValueMatches( 'data',
			{
				{
					'651d079c86bd4514aa0ea10874515af2', -- docId
					1.25, -- aspectRatio
					'2012-09-19T10:20:41', -- captureDate
					'2012-09-19', -- displayDate
					'2012-09-19T10:46:37.059', -- timestamp
					nil, -- copyTimestamp
					nil, -- thumbnailMD5
					'', -- eventName
					'', -- eventNameLC
					0, -- favoritedCount
				}
			}
		)
		
end

--------------------------------------------------------------------------------

function OzCatalogIndexDefinitionsTest.testCoreInfo_copyTimestamp( test )

	WFAsyncTestCase( test, TEST_TIMEOUT )

		-- Add one "photo" to the doc store.
		
		:waitFor( function()
			
			return docStore:setServerValueP( 'asset', '651d079c86bd4514aa0ea10874515af2',
				{
					_serverProvidedMetadata = {
						created = '2012-09-19T17:46:23.744702Z',
						links = {
							[ '/rels/rendition_type/144h' ] = {
								href = '/v1.0/catalogs/cb38211be46c43cf93fa81d0874c0912/assets/ee85f51507a84085a937273066dfb094/revisions/d16e715e79da4c1b9144e369bb80e173/renditions/5ec0b75370254587a3080c3558a6ad29',
							},
						},
						subtype = 'jpeg',
						type = 'asset',
						updated = '2012-09-19T17:46:27.278744Z',
					},
					captureDate = '2012-09-19T10:20:41',
					copied_from_href = '/v1.0/catalogs/f230590f036d4c63b8416a256f112adc/assets/b6320d830fbd4028b66c39c1acb0b0f6/revisions/b596312a3bb979f6e92a280d19d5dbf6',
					changedAtTime = '2012-09-19T10:46:37.254',
					height = 1600,
					importSource = {
						copyTimestamp = '2013-01-21T09:29:38.392:2013-02-12T23:55:12.412Z:X',
						fileName = 'IMG_0191.JPG',
						fileSize = 1369123,
						importTimestamp = '2012-09-19T10:46:37.059',
						importedBy = '5cf9b3c832dd43f884a62fa7664cd926',
						importedOnDevice = "Eric's iPad 3",
						sha256 = 'cfa2508beabd49f6175aec956ec553e76fd657ea24f28576d6e18b0511bcc3df',
					},
					originalCaptureDate = '2012-09-19T10:20:41',
					version = 1,
					width = 2560,
				}
			)
		
		end )
		
		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT docId, aspectRatio, captureDate, displayDate, timestamp,
						 copyTimestamp, thumbnailMD5, eventName, eventNameLC,
						 favoritedCount
					FROM coreInfo]]
			}
		end )

		:continueWhenValueMatches( 'data',
			{
				{
					'651d079c86bd4514aa0ea10874515af2', -- docId
					1.6, -- aspectRatio
					'2012-09-19T10:20:41', -- captureDate
					'2012-09-19', -- displayDate
					'2012-09-19T10:46:37.059', -- timestamp
					'2013-01-21T09:29:38.392:2013-02-12T23:55:12.412Z:X', -- copyTimestamp
					nil, -- thumbnailMD5
					'', -- eventName
					'', -- eventNameLC
					0, -- favoritedCount
				}
			}
		)
		
end

--------------------------------------------------------------------------------

function OzCatalogIndexDefinitionsTest.testCoreInfo_eventName( test )

	WFAsyncTestCase( test, TEST_TIMEOUT )

		-- Add one "photo" to the doc store.
		
		:waitFor( function()
			
			return docStore:setServerValueP( 'asset', '651d079c86bd4514aa0ea10874515af2',
				{
					_serverProvidedMetadata = {
						created = '2012-09-19T17:46:23.744702Z',
						links = {
							[ '/rels/rendition_type/144h' ] = {
								href = '/v1.0/catalogs/cb38211be46c43cf93fa81d0874c0912/assets/ee85f51507a84085a937273066dfb094/revisions/d16e715e79da4c1b9144e369bb80e173/renditions/5ec0b75370254587a3080c3558a6ad29',
							},
						},
						subtype = 'jpeg',
						type = 'asset',
						updated = '2012-09-19T17:46:27.278744Z',
					},
					captureDate = '2012-09-19T10:20:41',
					changedAtTime = '2012-09-19T10:46:37.254',
					eventName = 'South America',
					height = 1600,
					importSource = {
						fileName = 'IMG_0191.JPG',
						fileSize = 1369123,
						importTimestamp = '2012-09-19T10:46:37.059',
						importedBy = '5cf9b3c832dd43f884a62fa7664cd926',
						importedOnDevice = "Eric's iPad 3",
						sha256 = 'cfa2508beabd49f6175aec956ec553e76fd657ea24f28576d6e18b0511bcc3df',
					},
					originalCaptureDate = '2012-09-19T10:20:41',
					version = 1,
					width = 2560,
				}
			)
		
		end )
		
		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT docId, aspectRatio, captureDate, displayDate, timestamp,
						 copyTimestamp, thumbnailMD5, eventName, eventNameLC,
						 favoritedCount
					FROM coreInfo]]
			}
		end )

		:continueWhenValueMatches( 'data',
			{
				{
					'651d079c86bd4514aa0ea10874515af2', -- docId
					1.6, -- aspectRatio
					'2012-09-19T10:20:41', -- captureDate
					'2012-09-19', -- displayDate
					'2012-09-19T10:46:37.059', -- timestamp
					nil, -- copyTimestamp
					nil, -- thumbnailMD5
					'South America', -- eventName
					'south america', -- eventNameLC
					0, -- favoritedCount
				}
			}
		)
		
end

--------------------------------------------------------------------------------

function OzCatalogIndexDefinitionsTest.testCoreInfo_favoritedCount( test )

	WFAsyncTestCase( test, TEST_TIMEOUT )

		-- Add one "photo" to the doc store.
		
		:waitFor( function()
			
			return docStore:setServerValueP( 'asset', '651d079c86bd4514aa0ea10874515af2',
				{
					_serverProvidedMetadata = {
						created = '2012-09-19T17:46:23.744702Z',
						links = {
							[ '/rels/rendition_type/144h' ] = {
								href = '/v1.0/catalogs/cb38211be46c43cf93fa81d0874c0912/assets/ee85f51507a84085a937273066dfb094/revisions/d16e715e79da4c1b9144e369bb80e173/renditions/5ec0b75370254587a3080c3558a6ad29',
							},
						},
						subtype = 'jpeg',
						type = 'asset',
						updated = '2012-09-19T17:46:27.278744Z',
					},
					captureDate = '2012-09-19T10:20:41',
					changedAtTime = '2012-09-19T10:46:37.254',
					favorites = {
						[ '5cf9b3c832dd43f884a62fa7664cd926' ] = true,
					},
					height = 1600,
					importSource = {
						fileName = 'IMG_0191.JPG',
						fileSize = 1369123,
						importTimestamp = '2012-09-19T10:46:37.059',
						importedBy = '5cf9b3c832dd43f884a62fa7664cd926',
						importedOnDevice = "Eric's iPad 3",
						sha256 = 'cfa2508beabd49f6175aec956ec553e76fd657ea24f28576d6e18b0511bcc3df',
					},
					originalCaptureDate = '2012-09-19T10:20:41',
					version = 1,
					width = 2560,
				}
			)
		
		end )
		
		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT docId, aspectRatio, captureDate, displayDate, timestamp,
						 copyTimestamp, thumbnailMD5, eventName, eventNameLC,
						 favoritedCount
					FROM coreInfo]]
			}
		end )

		:continueWhenValueMatches( 'data',
			{
				{
					'651d079c86bd4514aa0ea10874515af2', -- docId
					1.6, -- aspectRatio
					'2012-09-19T10:20:41', -- captureDate
					'2012-09-19', -- displayDate
					'2012-09-19T10:46:37.059', -- timestamp
					nil, -- copyTimestamp
					nil, -- thumbnailMD5
					'', -- eventName
					'', -- eventNameLC
					1, -- favoritedCount
				}
			}
		)

		-- Add another favorite.
		
		:waitFor( function()
			
			return docStore:applyOverlayToDocumentP {
				docType = 'asset',
				docId = '651d079c86bd4514aa0ea10874515af2',
				overlay = {
					favorites = {
						[ 'd10d909b0b6d406ea6d9f96b402e9dca' ] = true,
					},
				},
			}
		
		end )

		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT docId, aspectRatio, captureDate, displayDate, timestamp,
						 copyTimestamp, thumbnailMD5, eventName, eventNameLC,
						 favoritedCount
					FROM coreInfo]]
			}
		end )

		:continueWhenValueMatches( 'data',
			{
				{
					'651d079c86bd4514aa0ea10874515af2', -- docId
					1.6, -- aspectRatio
					'2012-09-19T10:20:41', -- captureDate
					'2012-09-19', -- displayDate
					'2012-09-19T10:46:37.059', -- timestamp
					nil, -- copyTimestamp
					nil, -- thumbnailMD5
					'', -- eventName
					'', -- eventNameLC
					2, -- favoritedCount
				}
			}
		)

		-- Clear that favorite.
		
		:waitFor( function()
			
			return docStore:applyOverlayToDocumentP {
				docType = 'asset',
				docId = '651d079c86bd4514aa0ea10874515af2',
				overlay = {
					favorites = {
						[ 'd10d909b0b6d406ea6d9f96b402e9dca' ] = { _delete = true },
					},
				},
			}
		
		end )

		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT docId, aspectRatio, captureDate, displayDate, timestamp,
						 copyTimestamp, thumbnailMD5, eventName, eventNameLC,
						 favoritedCount
					FROM coreInfo]]
			}
		end )

		:continueWhenValueMatches( 'data',
			{
				{
					'651d079c86bd4514aa0ea10874515af2', -- docId
					1.6, -- aspectRatio
					'2012-09-19T10:20:41', -- captureDate
					'2012-09-19', -- displayDate
					'2012-09-19T10:46:37.059', -- timestamp
					nil, -- copyTimestamp
					nil, -- thumbnailMD5
					'', -- eventName
					'', -- eventNameLC
					1, -- favoritedCount
				}
			}
		)

end

--------------------------------------------------------------------------------

function OzCatalogIndexDefinitionsTest.testCoreInfo_ignoresDeletedPhoto( test )

	WFAsyncTestCase( test, TEST_TIMEOUT )

		-- Add one "photo" to the doc store.
		
		:waitFor( function()
			
			return docStore:applyOverlayToDocumentP {
				docType = 'asset',
				docId = '651d079c86bd4514aa0ea10874515af2',
				overlay = {
					_localOnly = {
						thumbnail = {
							epoch = 1,
							md5 = 'b772c4c900bae7e699468a56aa1fa194',
							referenceHeight = 128,
							referenceWidth = 96,
							renditionId = '5ec0b75370254587a3080c3558a6ad29',
							revId = 'd16e715e79da4c1b9144e369bb80e173',
							size = 5564,
						},
					},
					width = 96*4,
					height = 128*4,
					captureDate = '2012-08-15T15:22:29',
				},
			}
		
		end )
		
		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT docId, aspectRatio FROM coreInfo]]
			}
		end )

		:continueWhenValueMatches( 'data',
			{
				{ '651d079c86bd4514aa0ea10874515af2', 0.75 }
			}
		)
		
		-- Now delete that photo.
		
		:waitFor( function()
			
			return docStore:applyOverlayToDocumentP {
				docType = 'asset',
				docId = '651d079c86bd4514aa0ea10874515af2',
				overlay = { _deleted = true },
			}
		
		end )
		
		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT docId, aspectRatio FROM coreInfo]]
			}
		end )

		:continueWhenValueMatches( 'data', {} )
		
end

--------------------------------------------------------------------------------

function OzCatalogIndexDefinitionsTest.testAlbumAsset_ignoresDeletedAlbumAsset( test )

	WFAsyncTestCase( test, TEST_TIMEOUT )

		-- Add one "photo" to the doc store.
		
		:waitFor( function()
			
			return docStore:applyOverlayToDocumentP {
				docType = 'asset',
				docId = '651d079c86bd4514aa0ea10874515af2',
				overlay = {
					favorites = {
						tarderSauce = true,
					},
				},
			}
		
		end )
		
		-- Add an 'album_asset' to the docStore
		
		:waitFor( function() 
			
			return docStore:applyOverlayToDocumentP {
				docType = 'album_asset',
				docId = '089422de03444fd29753e96b2e47f88b',
				overlay = {
					album = {
						_id = '7c3ca81508d841b28f57ed83893c6ade',
					},
					asset = {
						_id = 'ee69c819bf5f49c88d51449254f0d0b5',
					},
					updated = '2012-09-01',
					added_by = 'tarderSauce',
				}
			}
			
		end )
		
		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT assetId, userId, albumId, updateTimeStamp FROM album_asset]]
			}
		end )

		:continueWhenValueMatches( 'data',
			{
				{
					"651d079c86bd4514aa0ea10874515af2",
					[3] = "favorites",
				},
				{
					"651d079c86bd4514aa0ea10874515af2",
					[3] = "favoritestarderSauce",
				},
				{
					"ee69c819bf5f49c88d51449254f0d0b5",
					"tarderSauce",
					"7c3ca81508d841b28f57ed83893c6ade",
					"2012-09-01",
				},
			}
		)
		
		-- Now delete the album_asset.
		
		:waitFor( function()
			
			return docStore:applyOverlayToDocumentP {
				docType = 'album_asset',
				docId = '089422de03444fd29753e96b2e47f88b',
				overlay = { _deleted = true },
			}
		
		end )
		
		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT assetId, userId, albumId, updateTimeStamp FROM album_asset]]
			}
		end )

		:continueWhenValueMatches( 'data', 
			{ 
				{
					"651d079c86bd4514aa0ea10874515af2",
					[3] = "favorites",
				},
				{
					"651d079c86bd4514aa0ea10874515af2",
					[3] = "favoritestarderSauce",
				},			
			}	
		)

end

--------------------------------------------------------------------------------

function OzCatalogIndexDefinitionsTest.testAlbumAsset_favorites_ignoresDeletedPhoto( test )

	WFAsyncTestCase( test, TEST_TIMEOUT )

		-- Add one "photo" to the doc store.
		
		:waitFor( function()
			
			return docStore:applyOverlayToDocumentP {
				docType = 'asset',
				docId = '651d079c86bd4514aa0ea10874515af2',
				overlay = {
					favorites = {
						tarderSauce = true,
					},
				},
			}
		
		end )
		
		-- Add an 'album_asset' to the docStore
		
		:waitFor( function() 
			
			return docStore:applyOverlayToDocumentP {
				docType = 'album_asset',
				docId = '089422de03444fd29753e96b2e47f88b',
				overlay = {
					album = {
						_id = '7c3ca81508d841b28f57ed83893c6ade',
					},
					asset = {
						_id = 'ee69c819bf5f49c88d51449254f0d0b5',
					},
					updated = '2012-09-01',
					added_by = 'tarderSauce',
				}
			}
			
		end )
		
		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT assetId, userId, albumId, updateTimeStamp FROM album_asset]]
			}
		end )

		:continueWhenValueMatches( 'data',
			{
				{
					"651d079c86bd4514aa0ea10874515af2",
					[3] = "favorites",
				},
				{
					"651d079c86bd4514aa0ea10874515af2",
					[3] = "favoritestarderSauce",
				},
				{
					"ee69c819bf5f49c88d51449254f0d0b5",
					"tarderSauce",
					"7c3ca81508d841b28f57ed83893c6ade",
					"2012-09-01",
				},
			}
		)
		
		-- Now delete the photo.
		
		:waitFor( function()
			
			return docStore:applyOverlayToDocumentP {
				docType = 'asset',
				docId = '651d079c86bd4514aa0ea10874515af2',
				overlay = { _deleted = true },
			}
		
		end )
		
		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT assetId, userId, albumId, updateTimeStamp FROM album_asset]]
			}
		end )

		:continueWhenValueMatches( 'data', 
			{ 
				{
					"ee69c819bf5f49c88d51449254f0d0b5",
					"tarderSauce",
					"7c3ca81508d841b28f57ed83893c6ade",
					"2012-09-01",
				},			
			}	
		)

	
end

--------------------------------------------------------------------------------

function OzCatalogIndexDefinitionsTest.testAlbumAsset_favorites_ignoresEmptyTable( test )

	WFAsyncTestCase( test, TEST_TIMEOUT )

		-- Add one "photo" to the doc store.
		
		:waitFor( function()
			
			return docStore:applyOverlayToDocumentP {
				docType = 'asset',
				docId = '651d079c86bd4514aa0ea10874515af2',
				overlay = {
					favorites = {
					},
				},
			}
		
		end )
		
		:waitFor( function()
			return queryServer:queryP{ sql = 
				[[SELECT assetId, userId, albumId, updateTimeStamp FROM album_asset]]
			}
		end )

		:continueWhenValueMatches( 'data',
			{ }
		)

end

--------------------------------------------------------------------------------

return OzCatalogIndexDefinitionsTest
