--[[----------------------------------------------------------------------------

OzCatalogServiceTest.lua

$Date:
$Author:

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author ppaurav 2012-03-14

------------------------------------------------------------------------------]]
--FIXME [ppaurav 2012-08-15]: cleaning up the account after running all test cases
-- can't do this in tearDown as it does not support async code. I guess I need to do
-- cleanup in setup before running next round of tests.
-- Meanwhile if you are here to debug test case failures, Please login into Revel
-- App with ppaurav@adobetest.com & adobetest credentials and check if there are
-- any extra catalogs except DontTouchThisCatalog. if there are delete them.

	-- wichitafoundation.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'

	-- services.http_client
local OzUser = require 'OzUser'

local OzCatalogService = require 'OzCatalogService'

local OzAccountService = require 'OzAccountService'

local KSOAuth2Session = require 'KSOAuth2Session'

local KSIdGenerator = require 'KSIdGenerator'

local KSDate = require 'KSDate'

local KSData = require 'KSData'

local KSTestUtils = require 'KSTestUtils'

local testFails = KSTestUtils.testFails

local assertFails = KSTestUtils.assertFails

local KSPromises = require 'KSPromises'

local WFPlatformInfo = require 'WFPlatformInfo'

local Bablu = require 'Bablu'

local OzCatalogTestUtils = require "OzCatalogTestUtils"
local KSReactivePromiseBridge = require "KSReactivePromiseBridge"

	-- wichitafoundation.os.prefs
local KSPrefs = require 'KSPrefs'
KSPrefs.currentUserId = nil

--============================================================================--

local OzCatalogServiceTest = {}

--------------------------------------------------------------------------------

local ozUserId = '38c82a0119464e25a76b602f6e55c5e5'
	-- user id for ppaurav@adobetest.com on oznext.

local TEST_WAIT_TIME = 60

local ozUser
local oAuthSession = KSOAuth2Session
local catalogServiceTest
local catalogServiceDefault
	-- catalog Id for this catalog 'f05068bb097d4d878492bb7ceec80c7c'
	-- Using this catalog for rootCollectionInfoB and other getter APIs
	-- becasue of frequent distructive changes in catalogServiceTest's
	-- collections by other APIs

local catalogIdTest
local catalogIdDefault = 'f05068bb097d4d878492bb7ceec80c7c' -- catalog name DontTouchThisCatalog
local testAlbumId = '1dcbce7d33104c8e9442dd2de697c32c' -- DontTouchThisAlbum
local setupP

--[[
	collectionIds for OzcatalogServiceTest can be referred from here
	if required by any of the test cases
collections = {
		collections = {
		{
			displayDate = "2012-04-13",
			eventNameFirst = false,
			id = "date	2012-04-13",
			lcname = "2012-04-13",
			name = "2012-04-13",
		},
		{
			displayDate = "2012-04-10",
			eventNameFirst = false,
			id = "date	2012-04-10",
			lcname = "2012-04-10",
			name = "2012-04-10",
		},
		{
			displayDate = "2004-11-02",
			eventNameFirst = false,
			id = "date	2004-11-02",
			lcname = "2004-11-02",
			name = "2004-11-02",
		},
	},

	},
]]

--------------------------------------------------------------------------------

local function getFirstExpectedCatalogUser()

	return
		{
			_id = "38c82a0119464e25a76b602f6e55c5e5",
			email = "ppaurav@adobetest.com",
			first_name = "P",
			full_name = "P P",
			id = "urn:carousel:account:38c82a0119464e25a76b602f6e55c5e5",
			last_name = "P",
			limits = {
				catalog_users = "ZEROED",
				catalogs = "ZEROED",
			},
			links = {
				["self"] = {
					href = "/v1.0/accounts/38c82a0119464e25a76b602f6e55c5e5",
				},
			},
			role = "owner",
			status = "trial",
			usage = {
				uploads = "ZEROED",
			},
		}

end

--------------------------------------------------------------------------------

local zeroVariableNumbersMapBFn =  function( t, ... )
	
	if t then
		t[ 1 ].limits.catalog_users = type( t[ 1 ].limits.catalog_users ) == 'number' and "ZEROED"
		t[ 1 ].limits.catalogs = type( t[ 1 ].limits.catalogs ) == 'number' and "ZEROED"
		t[ 1 ].usage.uploads = type( t[ 1 ].usage.uploads ) == 'number' and "ZEROED"
	end
	
	return t, ...

end

--------------------------------------------------------------------------------

local zeroVariableNumbersMapPFn =  function( status, ... )
	return status, zeroVariableNumbersMapBFn( ... )
end

--------------------------------------------------------------------------------

-- SPAM this was copied wholesale from OzUserTest
local function cleanupTestAccountP()

	local catalogsToBeDeleted = {}

	local function deleteCatalog( arg )
		return ozUser:deleteCatalogP( arg.id )
	end

	return ozUser:catalogsP()
		:mapDataP( function( status, result )

			for _, catalogInfo in ipairs( result ) do

				if not string.find( catalogInfo.name, "DontTouchThisCatalog" ) then
					table.insert( catalogsToBeDeleted, catalogInfo )
				end

			end

			if next( catalogsToBeDeleted ) then

				debug.print( string.format( "Deleting %d old catalogs from "
											.. "previous test runs", #catalogsToBeDeleted ) )
				return KSPromises.allP( deleteCatalog, catalogsToBeDeleted )

			else

				return KSPromises.resolvedPromiseP( 'data', true )

			end

		end )

end

--------------------------------------------------------------------------------

local function syncWaitingForRealtime( catalogService )

	return function()
			-- With the removal of the realtime flag to our oz sync, even an explicit
			-- call to syncNow might miss a new change.  So we wait 5 seconds to make
			-- sure the change will show up in the stream.
			local WFTimer = require "WFTimer"
			return WFTimer.afterDelayP( 5 )
			
			:mapDataP( function()
			
				catalogService:setSyncEnabled( true )
				return catalogService:syncNowP()

			end )
	end
	
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.setUpClass()

	oAuthSession = KSOAuth2Session( ozUserId )
	ozUser = OzUser.make( ozUserId )
	KSPrefs.currentUserId = ozUserId
	OzAccountService.setCurrentUser( ozUserId )
	
	local prefsFlush = KSIdGenerator.uuidInLowercaseHex()
	
	-- POST paramateres
	-- client_id, client_secret as already configured
	-- redirect_uri = 'air-auth://carousel'
	-- locale = en_US
	-- scope = openid,AdobeID
	-- username = 'ppaurav@adobetest.com'
	-- password ='adobetest'
	
	local postUrl = 'https://ims-na1-qa1.adobelogin.com/ims/login/code'
	local userAuthParams = 'username=ppaurav@adobetest.com&password=adobetest'
	
	local postParamsFormatStr = 'client_id=%s&redirect_uri=air-auth://carousel&locale=en_US&scope=openid,AdobeID&client_secret=%s&%s'
	
	catalogServiceDefault = OzCatalogService( catalogIdDefault, ozUser )
	catalogIdTest = KSIdGenerator.uuidInLowercaseHex()
	catalogServiceTest = OzCatalogService( catalogIdTest, ozUser )
	local flushB = KSPrefs.prefB( "flushPrefsForOzCatalogServiceTest" )

	setupP = require 'WFWorkerThread'( "WFRequestTestHandler.lua" ):requestP( "setPref", "flushPrefsForOzCatalogServiceTest", prefsFlush )
	:mapDataP( function()
		return KSReactivePromiseBridge.behaviorToPromise(
			flushB:relayForAutoCloseB(), -- FIXME_AUTO_CLOSE: Do I need the relay here?
			function( value )
				return value == prefsFlush
			end
		)
	end )
	:mapDataP( function()
		return Bablu:getImsClientIdAndSecretP()
	end )
	:mapDataP( function( status, clientId, secret )
	
		local postParams = string.format( postParamsFormatStr, clientId, secret, userAuthParams )
		return ozUser:getAccessTokenRefreshTokenFromImsP( postUrl, postParams )
	
	end )
	:mapDataP( function( status, refresh_token, access_token )

		return oAuthSession:storeTokensP( refresh_token, access_token )
		
	end )
	:mapDataP( function()

		return cleanupTestAccountP()
	end )
	:mapDataP( function()

		return ozUser:createCatalogP {
			name = 'OzCatalogServiceTest',
			id = catalogIdTest,
		}
	end )
	:mapDataP( function()

		return OzCatalogTestUtils.importPhotosP( catalogServiceTest, {
			{ captureDate = "2004-11-02T11:10:09" },
			{ captureDate = "2012-05-02T08:07:06" },
			{ captureDate = "2001-02-03T04:05:06" },
		} )
	end )
	:runUntilDoneP()
	

end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.tearDownClass()

	--delete the catalogs created in setup
	ozUser:deleteCatalogP( catalogIdTest )
	
	catalogServiceTest:close()
	catalogServiceTest = nil
	
	catalogServiceDefault:close()
	catalogServiceDefault = nil
	
	oAuthSession:close()
	oAuthSession = nil
	
	ozUser:close()
	ozUser = nil
	
end

--------------------------------------------------------------------------------

-- @mark view modes

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testGetAssetCountB_testNightly( test )
	
	-- use default catalog since assetCount can be changed by deletion
	local countB = catalogServiceDefault:getAssetCountB()
		
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( syncWaitingForRealtime( catalogServiceDefault ) )

		:continueWhenValueMatches( 'data', true )
		
		:waitFor( countB )
		
		:continueWhenValueMatches( 3 )
		:waitFor( function()
		
			catalogServiceDefault:setSyncEnabled( false )
			return 1
			
		end )
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testRootCollectionInfoIsEmptyB_NonEmpty_testNightly( test )

	-- using default catalog since can't make sure if newly created test catalog is ready to use
	local isEmptyB = catalogServiceDefault:rootCollectionInfoIsEmptyB( 'date', nil )
		
	WFAsyncTestCase( test, TEST_WAIT_TIME )

		:waitFor( setupP )
		:waitFor( isEmptyB )
		
		:continueWhenValueMatches( false )

end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testRootCollectionInfoIsEmptyB_Empty_testNightly( test )

	local catalogIdTemp = KSIdGenerator.uuidInLowercaseHex()
	local tempCatalogService
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( setupP )
		:waitFor( function() --step2
			
			return ozUser:createCatalogP( { name = 'testRootCollectionInfoIsEmptyB_Empty', id = catalogIdTemp, } )
			
		end )
		
		:continueWhenValueMatches( 'data', ':ignore:', ':ignore:', catalogIdTemp, true )
		
		:waitFor( function()  --step3
		
				tempCatalogService = OzCatalogService( catalogIdTemp, ozUser )
				return 1
		end )
		
		:waitFor( function()  --step4

			return tempCatalogService:rootCollectionInfoIsEmptyB( 'date', nil )
			
		end )
				
		:continueWhenValueMatches( true )
		
		:waitFor( function()	-- delete the test catalog after test
			
			tempCatalogService:close()
			tempCatalogService = nil
			return ozUser:deleteCatalogP( catalogIdTemp )
			
		end )
		
		:continueWhenValueMatches( 'data', true, ':ignore:' )
	
end

--------------------------------------------------------------------------------

-- FIXME [ppaurav 2012-08-22]: Enable this test when watson #3316012 is fixed
function OzCatalogServiceTest.DISABLEDtestCollectionInfoB_testNightly( test )

	local collectionId = 'date\tRoadTrip2\t2004-11-02\t2004-11-02\troadtrip2'  -- collection from default catalog
	
	-- SPAM [arahn 2012-09-04]: Before this test can be re-enabled it will be necessary
	-- to provide the correct viewGrouping parameter as the second argument to this
	-- function call.
	local collectionInfoB = catalogServiceDefault:collectionInfoB( collectionId )
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		:waitFor( collectionInfoB )
		
		:continueWhenValueMatches( {
			assetCount = 1,
			displayDate = "2004-11-02",
			endDate = "2004-11-02",
			eventName = "RoadTrip2",
			eventNameFirst = false,
			favoritedAssets = 0,
			name = "2004-11-02",
			photos = {
				{
					aspectRatio = 4/3,
					captureDate = "2004-11-02T11:51:49",
					favoritedCount = 0,
					id = "8114a17e899c4c2fa3934aa2dda834da",
					thumbnailMD5 = ( WFPlatformInfo.isTornado and "" or "a87d7c84c3f953b407ff3e246d77735b" ),
				},
			},
			startDate = "2004-11-02",

		} )
					
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testRootCollectionInfoB_testNightly( test )
	local expectedData = {
		collections = {
			{
				displayDate = "2012-05-02",
				eventNameFirst = false,
				id = "date\t2012-05-02",
				lcname = "2012-05-02",
				name = "2012-05-02",
			},
			{
				endDate = "2004-11-02",
				eventName = "RoadTrip2",
				eventNameFirst = false,
				id = "date\tRoadTrip2\t2004-11-02\t2004-11-02\troadtrip2",
				lcname = "2004-11-02",
				name = "2004-11-02",
				startDate = "2004-11-02",
			},
		},
	}

	local collectionInfoB = catalogServiceDefault:rootCollectionInfoB( 'date', nil )
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( setupP )
		
		:waitFor( syncWaitingForRealtime( catalogServiceDefault ) )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( collectionInfoB )
		
		:continueWhenValueMatches( expectedData )
		
		:waitFor( function()
		
			catalogServiceDefault:setSyncEnabled( false )
			return 1
			
		end )
		
end

--------------------------------------------------------------------------------

-- @mark collaborators

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testPhotoInfoB_nil_testNightly( test )
	
	local photoInfoB
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )

		:waitFor( setupP )
		:waitFor( syncWaitingForRealtime( catalogServiceDefault ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			photoInfoB = catalogServiceDefault:photoInfoB( nil )
			return photoInfoB
		
		end )
		:continueWhenValueMatches( nil )

		-- This behavior should not change from nil to anything else.
		-- Crude test for that.

		:waitFor( 1.5 )

		:waitFor( function()
			return photoInfoB
		end )
		:continueWhenValueMatches( nil )
		
		:waitFor( function()
		
			catalogServiceDefault:setSyncEnabled( false )
			return 1
			
		end )
	
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testPhotoInfoB_testNightly( test )
	
	--photoIds are fixed in default catalog DontTouchThisCatalog
	local photoId = '8114a17e899c4c2fa3934aa2dda834da'	-- canon2004.jpg from RoadTrip2 event
		
	WFAsyncTestCase( test, TEST_WAIT_TIME )

		:waitFor( setupP )
		:waitFor( syncWaitingForRealtime( catalogServiceDefault ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			return catalogServiceDefault:photoInfoB( photoId )
				:mapB( function( photoInfo )
				
					return photoInfo
						and type( photoInfo ) == 'table'
						and photoInfo.eventName
					
				end )
		
		end )
		:continueWhenValueMatches( 'RoadTrip2' )
		
		:waitFor( function()
		
			catalogServiceDefault:setSyncEnabled( false )
			return 1
			
		end )
	
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testPhotoInfoP_testNightly( test )
	
	--photoIds are fixed in default catalog DontTouchThisCatalog
	local photoId = '8114a17e899c4c2fa3934aa2dda834da'	-- canon2004.jpg from RoadTrip2 event
		
	WFAsyncTestCase( test, TEST_WAIT_TIME )

		:waitFor( setupP )
		:waitFor( syncWaitingForRealtime( catalogServiceDefault ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			return catalogServiceDefault:photoInfoP( photoId )
				:mapDataP( function( _, photoInfo )
				
					assert( photoInfo.eventName == 'RoadTrip2', "Incorrect photo event name" )
					return 'data', true
					
				end )
		
		end )
		
		:waitFor( function()
		
			catalogServiceDefault:setSyncEnabled( false )
			return 1
			
		end )
	
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testCatalogInfoP_testNightly( test )
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( setupP )
		:waitFor( syncWaitingForRealtime( catalogServiceDefault ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			return catalogServiceDefault:catalogInfoP()
		
		end )
		
		:whenResolvedDo( function( status, catalogInfo )

			if status == 'data' then
				assert( catalogInfo._docId == catalogIdDefault, "catalogInfoP returned Incorrect _docId" )
				assert( catalogInfo._docType == 'catalog', "catalogInfoP returned Incorrect _docType" )
				assert( catalogInfo.name == 'DontTouchThisCatalog', "catalogInfoP returned Incorrect catalog name" )
			else
				assert( false, "catalogInfoP returned an error" )
			end
			
		end )
		:waitFor( function()
		
			catalogServiceDefault:setSyncEnabled( false )
			return 1
			
		end )

end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testRenameEventP_testNightly( test )
	
	local newName = 'NewEventName'
	--local oldEventName = 'null'
	local photoIds = {}
	local collectionId = 'date\t2004-11-02'
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( setupP )
		:waitFor( syncWaitingForRealtime( catalogServiceTest ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	--step5
		
			return catalogServiceTest:collectionInfoP( 'date', nil, collectionId )
			
		end )
		:whenResolvedDo( function( status, collection )

			assert( status == 'data', "collectionInfoP resolved in error" )
			local photos = collection.photos
			assert( #photos ~= 0, "No photos in collection" )
			for i = 1, #photos do
				photoIds[ i ] = photos[ i ].id
			end
			
		end )
		
		:waitFor( function()	--step6  get the current eventname
		
			return catalogServiceTest:photoInfoP( photoIds[ 1 ] )
	
		end )
		:whenResolvedDo( function( status, photoInfo )
		
			assert( status == 'data', "photoInfoP resolved in error" )
			--oldEventName = photoInfo.eventName

		end )
		:waitFor( function()	--step7
		
				return catalogServiceTest:renameEventP( newName, photoIds )
			
		end )
		
		:continueWhenValueMatches( 'data', ':ignore:' ) -- it returns revision id. Is that acceptable ??
		
		:waitFor( syncWaitingForRealtime( catalogServiceTest ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	-- step Make sure event was renamed
		
			return catalogServiceTest:photoInfoP( photoIds[ 1 ] )
	
		end )
		
		:whenResolvedDo( function( status, photoInfo )
		
			assert( photoInfo.eventName == newName, "renameEventP didnt rename the event name" )
			
		end )
	
		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )
end

--------------------------------------------------------------------------------

--FIXME [ppaurav 2012-09-27]: Enable this test case when watson #3337646 is fixed

function OzCatalogServiceTest.testRenameCatalogP_AlreadyUsedName_testNightly( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( function()
		
			catalogServiceTest:watchCatalogOwnerUserStatus()
		
			return syncWaitingForRealtime( catalogServiceTest )()
			
		end )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
							
				return catalogServiceTest:renameCatalogP( 'ppaurav' )
			
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	--leaving it in original state
							
				return catalogServiceTest:renameCatalogP( 'OzCatalogServiceTest' )
			
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )

end

--------------------------------------------------------------------------------

--FIXME [ppaurav 2012-09-27]: Enable this test case when watson #3337646 is fixed
function OzCatalogServiceTest.testRenameCatalogP_testNightly( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( setupP )
				
		:waitFor( function()		--step3
							
				return catalogServiceTest:renameCatalogP( 'testRenameCatalogP' )
			
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()		--step
		
			catalogServiceTest:watchCatalogOwnerUserStatus()
		
			return syncWaitingForRealtime( catalogServiceTest )()
			
		end )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	--step7
		
			return catalogServiceTest:catalogInfoP()
		
		end )
		
		:whenResolvedDo( function( status, catalogInfo )

			if status == 'data' then
				assert( catalogInfo._docId == catalogIdTest, "catalogInfoP returned Incorrect _docId" )
				assert( catalogInfo.name == 'testRenameCatalogP', "Incorrect catalog name, looks like renameCatalogP didn't change the name" )
			else
				assert( false, "catalogInfoP returned an error" )
			end
			
		end )
		
		:waitFor( function()	--leaving it in original state
							
				return catalogServiceTest:renameCatalogP( 'OzCatalogServiceTest' )
			
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( syncWaitingForRealtime( catalogServiceTest ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			return catalogServiceTest:catalogInfoP()
		
		end )
		
		:whenResolvedDo( function( status, catalogInfo )

			if status == 'data' then
				assert( catalogInfo._docId == catalogIdTest, "catalogInfoP returned Incorrect _docId" )
				assert( catalogInfo.name == 'OzCatalogServiceTest', "Incorrect catalog name, looks like renameCatalogP didn't change the name" )
			else
				assert( false, "catalogInfoP returned an error" )
			end
			
		end )
		
		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )
		
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testInviteAndRemoveUserP_testNightly( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		:waitFor( function()
		
			return ozUser:clearCaches():refreshP()
			
		end )
		:continueWhenValueMatches( 'data', ':ignore:', ':ignore:' )
		
		:waitFor( function()	-- step 3
		
			catalogServiceTest:watchCatalogOwnerUserStatus()
		
			return syncWaitingForRealtime( catalogServiceTest )()
			
		end )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()

			return catalogServiceTest:inviteUserP( 'ppaurav+5@adobetest.com' )
		
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			return catalogServiceTest:catalogUsersP():mapDataP( zeroVariableNumbersMapPFn )
			
		end )
		:continueWhenValueMatches( 'data',  {
			getFirstExpectedCatalogUser(),
			{
				email = "ppaurav+5@adobetest.com",
				role = "invitee",
			},
		} )
		
		:waitFor( function()

			return catalogServiceTest:removeUserP( 'ppaurav+5@adobetest.com' )
		
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			return catalogServiceTest:catalogUsersP():mapDataP( zeroVariableNumbersMapPFn )
			
		end )
		
		:continueWhenValueMatches( 'data',  {
			getFirstExpectedCatalogUser()
		} )
		
		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )
end

--------------------------------------------------------------------------------

-- from the client app point of view , catalog owner cant invite
-- himslef. He gets duplicate email error then how come removal
-- will be possible
--FIXME [ppaurav 2012-09-27]: Enable this test case when watson #3337646 is fixed
function OzCatalogServiceTest.testRemoveUserP_Yourself_testNightly( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( setupP )
		
		:waitFor( function()
		
			catalogServiceTest:watchCatalogOwnerUserStatus()
		
			return syncWaitingForRealtime( catalogServiceTest )()
			
		end )
		:continueWhenValueMatches( 'data', true )

		:waitFor( function()
							
				return catalogServiceTest:removeUserP( 'ppaurav@adobetest.com' )
			
		end )
		
		:continueWhenValueMatches( 'error', "OzUserMethods:removeUserP The catalog owner cannot be removed from the catalog" )
		
		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )
		
end

--------------------------------------------------------------------------------

--FIXME [ppaurav 2012-09-27]: Enable this test case when watson #3337646 is fixed
function OzCatalogServiceTest.testRemoveUserP_Unknown_testNightly( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		:waitFor( function()
		
			catalogServiceTest:watchCatalogOwnerUserStatus()
		
			return syncWaitingForRealtime( catalogServiceTest )()
			
		end )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
							
				return catalogServiceTest:removeUserP( 'unknown@adobetest.com' )
			
		end )
		
		:continueWhenValueMatches( 'error', "UserNotPresent" )
		
		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )

end

--------------------------------------------------------------------------------

--FIXME [ppaurav 2012-09-27]: Enable this test case when watson #3337646 is fixed
function OzCatalogServiceTest.testInviteUserP_InvalidAddress_testNightly( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( setupP )
		
		:waitFor( function()
		
			catalogServiceTest:watchCatalogOwnerUserStatus()
		
			return syncWaitingForRealtime( catalogServiceTest )()
			
		end )
		:continueWhenValueMatches( 'data', true )

		:waitFor( function()
							
				return catalogServiceTest:inviteUserP( 'ppauravadobetest.com' )
			
		end )
		
		:continueWhenValueMatches( 'error', 1005, "Input Validation Error" )
				
		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )

end

--------------------------------------------------------------------------------

--FIXME [ppaurav 2012-09-27]: Enable this test case when watson #3337646 is fixed
function OzCatalogServiceTest.testInviteUserP_Yourself_testNightly( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( function()
		
			catalogServiceTest:watchCatalogOwnerUserStatus()
		
			return syncWaitingForRealtime( catalogServiceTest )()
			
		end )
		:continueWhenValueMatches( 'data', true )

		:waitFor( function()
							
				return catalogServiceTest:inviteUserP( 'ppaurav@adobetest.com' )
			
		end )
		
		:continueWhenValueMatches( 'error', 1005, "Input Validation Error" )

		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )

end


--------------------------------------------------------------------------------

--FIXME [ppaurav 2012-09-27]: Enable this test case when watson #3337646 is fixed
function OzCatalogServiceTest.testInviteUserP_Max_testNightly( test )

	local quota = 50 -- whenever collaborator quota is changed this testcase needs to be changed
	local userName = 'ppaurav+'
	local emailSuffix = '@adobetest.com'
	local emailAddress = {}
	-- create array of email ids
	for i = 1, quota do
		emailAddress[ i ]  = string.format( "%s%s%s", userName, i, emailSuffix )
	end
	
	WFAsyncTestCase( test, 2000000 )
	
		:waitFor( setupP )
		
		:waitFor( function()
		
			catalogServiceTest:watchCatalogOwnerUserStatus()
		
			return syncWaitingForRealtime( catalogServiceTest )()
			
		end )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	--step 3
		
			local function inviteUser( email )
				return catalogServiceTest:inviteUserP( email )
			end
			
			local resultP = KSPromises.allP( inviteUser, emailAddress )
			return resultP
			
		end )
				
		:continueWhenValueMatches ( 'data', true )
		
		:waitFor( function()	--step4
			
			--after inviting 50 users , 51th invite should result in error
			local lastEmail = string.format( "%s%s%s", userName, quota + 1, emailSuffix )
			return catalogServiceTest:inviteUserP( lastEmail )
			
		end )
		
		:continueWhenValueMatches( 'error', 1014, 'Too many users' )
		
		:waitFor( function()	--step 5
		
			local function removeUser( email )
				return catalogServiceTest:removeUserP( email )
			end
			
			local resultP = KSPromises.allP( removeUser, emailAddress )
			return resultP
			
		end )
				
		:continueWhenValueMatches ( 'data', true )
		
		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )


end

--------------------------------------------------------------------------------
--FIXME [ppaurav 2012-09-27]: Enable this test when #3435740 is fixed

if WFPlatformInfo.isMac then

-- TODO [arahn 2013-05-14]: This test only passes once in a while; sometimes
-- a previous test changes a precondition causing the owner status to never
-- resolve to the expected value.

function OzCatalogServiceTest.disabled_testCatalogOwnerStatusB_testNightly( test )

	--local userStatusB = catalogServiceDefault:catalogOwnerStatusB()

	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( setupP )
		:waitFor( syncWaitingForRealtime( catalogServiceDefault ) )
		:continueWhenValueMatches( 'data', true )

		:waitFor( function()
		
			return catalogServiceDefault:catalogOwnerStatusB()
			
		end )
		
		:continueWhenValueMatches( 'trial' )
		
		:waitFor( function()
		
			catalogServiceDefault:setSyncEnabled( false )
			return 1
			
		end )

end

end
--------------------------------------------------------------------------------

function OzCatalogServiceTest.testCollaboratorQuotaB_testNightly( test )

	local quotaB = catalogServiceDefault:collaboratorQuotaB()

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		:waitFor( syncWaitingForRealtime( catalogServiceDefault ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( quotaB )
		
		:continueWhenValueMatches( 50, true )
		
		:waitFor( function()
		
			catalogServiceDefault:setSyncEnabled( false )
			return 1
			
		end )
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testCatalogOwnerIdP_testNightly( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )

		:waitFor( setupP )
		:waitFor( syncWaitingForRealtime( catalogServiceDefault ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()

			return catalogServiceDefault:catalogOwnerIdP()
		
		end )
		
		:continueWhenValueMatches( 'data', ozUserId )
		
		:waitFor( function()
		
			catalogServiceDefault:setSyncEnabled( false )
			return 1
			
		end )
		
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testCatalogUsersB_testNightly( test )

	local usersB = catalogServiceDefault:catalogUsersB()
		
	WFAsyncTestCase( test, TEST_WAIT_TIME )

		:waitFor( setupP )
		
		:waitFor( syncWaitingForRealtime( catalogServiceDefault ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( usersB:mapB( zeroVariableNumbersMapBFn ) )
		
		:continueWhenValueMatches( { getFirstExpectedCatalogUser() } , true )
		
		:waitFor( function()
		
			catalogServiceDefault:setSyncEnabled( false )
			return 1
			
		end )

end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testCatalogUsersP_testNightly( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )

		:waitFor( setupP )
		:waitFor( syncWaitingForRealtime( catalogServiceDefault ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			return catalogServiceDefault:catalogUsersP():mapDataP( zeroVariableNumbersMapPFn )
	
		end )
		
		:continueWhenValueMatches( 'data', { getFirstExpectedCatalogUser() } )
		
		:waitFor( function()
		
			catalogServiceDefault:setSyncEnabled( false )
			return 1
			
		end )

end

--------------------------------------------------------------------------------

-- @mark abuse reporting

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAbuseReportingUrlP_testNightly( test )
	
	local photoId = '61f4e3b00da545429097c9426b773d47'	-- sunrise.jpg from dontTouchThisCatalog catalog

	WFAsyncTestCase( test, TEST_WAIT_TIME )

		:waitFor( setupP )
		:waitFor( syncWaitingForRealtime( catalogServiceDefault ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()

			return catalogServiceDefault:abuseReportingUrlP( photoId, 'thumbnail' )
		
		end )
		
		:continueWhenValueMatches( 'data', "http://www.ponysaurus.com/abuse_reports?type=revel&adobe-id=0B7677224F610F5C0A490D16&asset-id=/v1.0/catalogs/f05068bb097d4d878492bb7ceec80c7c/assets/61f4e3b00da545429097c9426b773d47/revisions/5badb998f0b2445f99b8ebd2417db510/renditions/e817a6c9799e49f4b7f3a2b5d14592e9&asset-href=/v1.0/catalogs/f05068bb097d4d878492bb7ceec80c7c/assets/61f4e3b00da545429097c9426b773d47/revisions/5badb998f0b2445f99b8ebd2417db510/renditions/e817a6c9799e49f4b7f3a2b5d14592e9" )
		
		:waitFor( function()

			return catalogServiceDefault:abuseReportingUrlP( photoId, 'preview' )
		
		end )
		
		:continueWhenValueMatches( 'data', "http://www.ponysaurus.com/abuse_reports?type=revel&adobe-id=0B7677224F610F5C0A490D16&asset-id=/v1.0/catalogs/f05068bb097d4d878492bb7ceec80c7c/assets/61f4e3b00da545429097c9426b773d47/revisions/5badb998f0b2445f99b8ebd2417db510/renditions/ebc9f8caa5764340a7f57854b7771bf6&asset-href=/v1.0/catalogs/f05068bb097d4d878492bb7ceec80c7c/assets/61f4e3b00da545429097c9426b773d47/revisions/5badb998f0b2445f99b8ebd2417db510/renditions/ebc9f8caa5764340a7f57854b7771bf6" )
		
		:waitFor( function()
		
			catalogServiceDefault:setSyncEnabled( false )
			return 1
			
		end )
		
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAbuseReportingUrlP_InvalidRenditionType_testNightly( test )
	
	local photoId = '61f4e3b00da545429097c9426b773d47'
	
	assertFails( testFails( function ()
		
		catalogServiceDefault:abuseReportingUrlP( photoId, 'invalid' )
		
	end ) , "OzCatalogService:abuseReportingUrlP rendition did not specify a recognized rendition type" )

end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAbuseReportingUrlP_NilPhotoId_testNightly( test )
	
	assertFails( testFails( function ()
		
		catalogServiceDefault:abuseReportingUrlP( nil, 'preview' )
		
	end ) , "OzCatalogService:abuseReportingUrlP photoId must be a non-empty string" )

end

--------------------------------------------------------------------------------

-- @mark originals

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testOriginalDataP_testNightly( test )

	local photoId = '8114a17e899c4c2fa3934aa2dda834da'	-- canon2004.jpg from RoadTrip2 event from DontTouchThisCatalog
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )

		:waitFor( setupP )
		:waitFor( syncWaitingForRealtime( catalogServiceDefault ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()

			return catalogServiceDefault:originalDataP( photoId )
		
		end )
		:whenResolvedDo( function( status, data )
			
			assert( status == 'data', "originalDataP resolved in error" )
			assert( KSData.length( data ) == 25885, "Incorrect length of original data" )
			assert( KSData.isData( data ), "data returned by originalDataP is not KSData" )
			
		end )
		:waitFor( function()
		
			catalogServiceDefault:setSyncEnabled( false )
			return 1
			
		end )
		
end

--------------------------------------------------------------------------------

-- @mark date correction

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testCorrectCaptureDateP_testNightly( test )

	local photoIds = {}
	local collectionId = 'date\t2004-11-02'
	local oldCaptureDate
	local newCaptureDate
		
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( syncWaitingForRealtime( catalogServiceTest ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			return catalogServiceTest:collectionInfoP( 'date', nil, collectionId )
			
		end )
		
		:whenResolvedDo( function( status, collection )

			assert( status == 'data', "collectionInfoP resolved in error" )
			local photos = collection.photos
			assert( #photos ~= 0, "No photos in collection" )
			for i = 1, #photos do
				photoIds[ i ] = photos[ i ].id
			end
			
		end )
		
		:waitFor( function() 	-- first getting the original capture date
		
			return catalogServiceTest:photoInfoP( photoIds[ 1 ] )
			
		end )
		
		:whenResolvedDo( function( status, photo )
		
			oldCaptureDate = photo.captureDate
			newCaptureDate = KSDate.addTimeInSeconds( oldCaptureDate , 3600 )
			
		end )
		
		:waitFor( function()
		
			return catalogServiceTest:correctCaptureDateP( { photoIds[ 1 ], }, newCaptureDate )
		
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	-- make sure new captureDate is correct
		
			return catalogServiceTest:photoInfoP( photoIds[ 1 ] )
			
		end )
		
		:whenResolvedDo( function( status, photo )
		
			assert( status == 'data', "photoInfoP resolved in errror" )
			assert( photo.captureDate == newCaptureDate, "correctCaptureDateP didnt change capture date" )
			
		end )
		
		:waitFor( function()	-- change it to original captureDate
		
			return catalogServiceTest:correctCaptureDateP( { photoIds[ 1 ] }, oldCaptureDate )
		
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )
	
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testCorrectCaptureDateForTrackP_testNightly( test )
	
	local photoIds = {}
	local collectionId = 'date\t2004-11-02'
	local oldCaptureDate
	local newCaptureDate
		
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( syncWaitingForRealtime( catalogServiceTest ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	--step5
		
			return catalogServiceTest:collectionInfoP( 'date', nil, collectionId )
			
		end )
		
		:whenResolvedDo( function( status, collection )
			
			assert( status == 'data', "collectionInfoP resolved in error" )
			local photos = collection.photos
			assert( #photos ~= 0, "No photos in collection" )
			
			for i = 1, #photos do
				photoIds[ i ] = photos[ i ].id
			end

			oldCaptureDate = photos[ 1 ].captureDate
			assert( oldCaptureDate, "No capture date in photos" )
			newCaptureDate = KSDate.addTimeInSeconds( oldCaptureDate , 3600 )

		end )
		
		:waitFor( function()	--step6
		
			assert( oldCaptureDate, "Old capture date is nill" )
			return catalogServiceTest:correctCaptureDateForTrackP( photoIds, oldCaptureDate, newCaptureDate )
		
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( syncWaitingForRealtime( catalogServiceTest ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	--step9 make sure new captureDate is correct
		
			return catalogServiceTest:photoInfoP( photoIds[ 1 ] )
			
		end )
		
		:whenResolvedDo( function( status, photo )
		
			assert( status == 'data', "photoInfoP resolved in errror" )
			assert( photo, "photoInfoP returned null" )
			assert( photo.captureDate == newCaptureDate, "correctCaptureDateForTrackP didnt change capture date" )
			
		end )
		
		:waitFor( function()	-- step10 change it to original captureDate
		
			return catalogServiceTest:correctCaptureDateForTrackP( photoIds, newCaptureDate, oldCaptureDate )
		
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )
		
end

--------------------------------------------------------------------------------

-- @mark delete photos

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testDeletePhotoP_InvalidPhotoId_testNightly( test )

	local photoId = '2176b5a00ce54eba9ddf0f27f9380923'
		
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( function()
		
			return catalogServiceTest:deletePhotoP( photoId )
		
		end )
		
		:continueWhenValueMatches( 'error', 'No such photo' )
		
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testZDeletePhotoP_testNightly( test )
	
	local photoIds = {}
	local collectionId  = 'date\t2012-05-02'
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( syncWaitingForRealtime( catalogServiceTest ) )
		:continueWhenValueMatches( 'data', true )

		:waitFor( function()
		
			return catalogServiceTest:collectionInfoP( 'date', nil, collectionId )
			
		end )
		:whenResolvedDo( function( status, collection )
			
			assert( status == 'data', "collectionInfoP resolved in error" )
			local photos = collection.photos
			assert( #photos ~= 0, "no photos in collection" )
			for i = 1, #photos do
				photoIds[ i ] = photos[ i ].id
			end
			
		end )
		
		:waitFor( function()
		
			return catalogServiceTest:deletePhotoP( photoIds[ 1 ] )
		
		end )
		:continueWhenValueMatches( 'data', ':ignore:' )
		
		:waitFor( function()
		
			return catalogServiceTest:photoInfoP( photoIds[ 1 ] )
			
		end )
		
		:whenResolvedDo( function( status, photo )
		
			assert( status == 'data', "photoInfoP returned error" )
			assert( photo._deleted == true, "Looks like photo was not deleted" )
			
		end )
		
		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )
	
end

--------------------------------------------------------------------------------

-- @mark selection

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testSelectedPhotosCountP_testNightly( test )
	
	local photoIds = {}
	local collectionId  = 'date\t2004-11-02'
		
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( syncWaitingForRealtime( catalogServiceTest ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	--step3
		
			return catalogServiceTest:collectionInfoP( 'date', nil, collectionId )
			
		end )
		:whenResolvedDo( function( status, collection )
			
			assert( status == 'data', "collectionInfoP resolved in error" )
			local photos = collection.photos
			for i = 1, #photos do
				photoIds[ i ] = photos[ i ].id
			end
			
		end )
		
		:waitFor( function()	-- step4  Initially it will be zero
		
			return catalogServiceTest:selectedPhotosCountP()
	
		end )
		
		:continueWhenValueMatches( 'data', 0 )
		
		:waitFor( function() -- step5 select some photos
		
			return catalogServiceTest:setSelectedPhotosP( photoIds )
	
		end )
		
		:waitFor( function()	--step6
			
			return catalogServiceTest:selectedPhotosCountP()
	
		end )
		
		:whenResolvedDo( function( status, n )
		
			assert( status == 'data', "selectedPhotosCountP resolved in error" )
			assert( n == #photoIds, "wrong number of selected photos" )
			
		end )
		
		:waitFor( function() -- leave the photos in original state
		
			return catalogServiceTest:clearSelectedPhotosP( photoIds )
	
		end )
		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )
		
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testGetSelectedPhotosP_testNightly( test )

	local photoIds = {}
	local collectionId  = 'date\t2004-11-02'
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( syncWaitingForRealtime( catalogServiceTest ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			return catalogServiceTest:collectionInfoP( 'date', nil, collectionId )
			
		end )
		:whenResolvedDo( function( status, collection )
			
			assert( status == 'data', "collectionInfoP resolved in error" )
			local photos = collection.photos
			for i = 1, #photos do
				photoIds[ i ] = photos[ i ].id
			end
			
		end )
		
		:waitFor( function() -- Initially it should be blank
		
			return catalogServiceTest:getSelectedPhotosP()
	
		end )
		
		:continueWhenValueMatches( 'data', {} )
		
		:waitFor( function() -- select some photos
		
			return catalogServiceTest:setSelectedPhotosP( photoIds )
	
		end )
		
		:waitFor( function()
		
			return catalogServiceTest:getSelectedPhotosP()
	
		end )
		
		:continueWhenValueMatches( 'data', photoIds )
		
		:waitFor( function() -- leave the photos in original state
		
			return catalogServiceTest:clearSelectedPhotosP( photoIds )
		
		end )
		
		:waitFor( function()
		
			return catalogServiceTest:getSelectedPhotosP()
	
		end )
		
		:continueWhenValueMatches( 'data', {} )
		
		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )
		
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testSetSelectedPhotosP_testNightly( test )

	local photoIds = {}
	local collectionId  = 'date\t2004-11-02'
		
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( syncWaitingForRealtime( catalogServiceTest ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			return catalogServiceTest:collectionInfoP( 'date', nil, collectionId )
			
		end )
		
		:whenResolvedDo( function( status, collection )
			
			assert( status == 'data', "collectionInfoP resolved in error" )
			local photos = collection.photos
			for i = 1, #photos do
				photoIds[ i ] = photos[ i ].id
			end
			
		end )
		
		:waitFor( function() --step
		
			return catalogServiceTest:setSelectedPhotosP( photoIds )
	
		end )
		
		:waitFor( function() --step
		
			return catalogServiceTest:getSelectedPhotosP()
	
		end )
		
		:continueWhenValueMatches( 'data', photoIds )
		
		:waitFor( function() --step leave the photos in original state
		
			return catalogServiceTest:clearSelectedPhotosP( photoIds )
	
		end )
		
		:waitFor( function() --step
		
			return catalogServiceTest:getSelectedPhotosP()
	
		end )
		
		:continueWhenValueMatches( 'data', {} )
		
		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )

end

--------------------------------------------------------------------------------

-- @mark Albums

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_CreateAlbumP_DeleteAlbumP_testNightly( test )
	
	local albumId
	local albumName = 'testAlbums_CreateAlbumP_DeleteAlbumP'

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( function() 	--step3
		
			return catalogServiceTest:createAlbumP( albumName )
	
		end )
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				albumId = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		
		:waitFor( function()	--step5
		
			--check if newly created album is in list
			return catalogServiceTest:albumsP()
			
		end )
		:whenResolvedDo( function( status, albumList )
			
			assert( status == 'data', "albumsP resolved in error" )
			local found = false
			for i, value in ipairs( albumList ) do
				if value[ 'id' ] == albumId and value[ 'name' ] == albumName then
					found = true
					break
				end
			end
			assert( found, "Album created in step3 not found in albumsP() list" )
			
		end )
		
	  	:waitFor( function()	--step6
			
			-- delete newly created album
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId )
		
	 	end )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function() 	--step8
		
			--make sure after deletion, list gets updated
			return catalogServiceTest:albumsP()
			
		end )
		
		:whenResolvedDo( function( status, albumList )
			
			assert( status == 'data', "albumsP resolved in error" )
			local deleted = true
			for i, value in ipairs( albumList ) do
				if value[ 'id' ] == albumId and value[ 'name' ] == albumName then
					deleted = false
					break
				end
			end
			assert( deleted, "Album deleted in step7 still in albumsP() list" )
			
		end )

end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_CreateAlbumP_IgnoreSecondArgument_testNightly( test )
	
	local albumId
	local albumName = 'testAlbums_CreateAlbumP_IgnoreSecondArgument'
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( function() --step3
		
			return catalogServiceTest:createAlbumP( albumName )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		
		:waitFor( function() --step5
		
			-- when second argument is not passed, by default createAlbumP
			-- checks for uniqueness so it should throw an error
			return catalogServiceTest:createAlbumP( albumName )
	
		end )
		
		:continueWhenValueMatches( 'error', 'NameExists' )
		
		:waitFor( function()	--step6
		
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId )
		
	 	end )
	
		:continueWhenValueMatches( 'data', true )

end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_CreateAlbumP_BypassUniquenessCheck_True_testNightly( test )
	
	local albumId1
	local albumId2
	local albumName = 'testAlbums_CreateAlbumP_BypassUniquenessCheck_True'
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( function() --step3
		
			return catalogServiceTest:createAlbumP( albumName )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId1 = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		
		:waitFor( function() --step4
		
			return catalogServiceTest:createAlbumP( albumName, true )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId2 = id
			else
				assert( false, "createAlbumP resolved in error while creating duplicate album" )
			end
	
		end )
		
		:waitFor( function() --step6
		
			-- make sure albumList has duplicate album names
			return catalogServiceTest:albumsP()
			
		end )
		
		:whenResolvedDo( function( status, albumList )
			
			assert( status == 'data', "albumsP resolved in error" )
			local duplicate = false
			local name1
			local name2
			for i, value in ipairs( albumList ) do
				if value[ 'id' ] == albumId1 then
					name1 = value[ 'name' ]
				end
				if value[ 'id' ] == albumId2 then
					name2 = value[ 'name' ]
				end
				if name1 == name2 then
					duplicate = true
				end
			end
			assert( duplicate, "No duplicate Albums found in albumsP() list" )
			
		end )
				
		:waitFor( function() --step6
			
			assert( albumId1, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId1 )
		
	 	end )
	
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
			
			assert( albumId1, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId2 )
		
	 	end )
	
		:continueWhenValueMatches( 'data', true )

end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_CreateAlbumP_BypassUniquenessCheck_False_testNightly( test )
	
	local albumId1
	local albumName = 'testAlbums_CreateAlbumP_BypassUniquenessCheck_False'
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( function() --step3
		
			return catalogServiceTest:createAlbumP( albumName )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId1 = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		
		:waitFor( function() --step5
		
			return catalogServiceTest:createAlbumP( albumName, false )
	
		end )
		
		:continueWhenValueMatches( 'error', 'NameExists' )
		
		:waitFor( function()	--step6
		
			assert( albumId1, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId1 )
		
	 	end )
	
		:continueWhenValueMatches( 'data', true )

end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_DeleteAlbumP_DoesNotExists_testNightly( test )
	
	local albumId = 'a176b5a00ce54eba9ddf0f27f9380923'

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( function()
		
			return catalogServiceTest:deleteAlbumP( albumId )
	
		end )
		
		:continueWhenValueMatches( 'data', true )
		
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_RenameAlbumP_BypassUniquenessCheck_True_testNightly( test )
	
	local albumId1
	local albumId2
	local albumName1 = 'testAlbums_RenameAlbumP_BypassUniquenessCheck_true'
	local albumName2 = 'testAlbums_RenameAlbumP_BypassUniquenessCheck_true_Duplicate'
	local newName = albumName2 --renaming albumId1 and new name will ne albumId2's name
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		:waitFor( function() --step3
		
			return catalogServiceTest:createAlbumP( albumName1 )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId1 = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		
		:waitFor( function() 	--step4
		
			return catalogServiceTest:createAlbumP( albumName2 )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
			  	albumId2 = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		
		:waitFor( function() --step5
		
			assert( albumId1, "AlbumId is nil" )
			return catalogServiceTest:renameAlbumP( albumId1, newName, true )
	
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function() --step7
		
			-- make sure albumList has duplicate album names
			return catalogServiceTest:albumsP()
			
		end )
		
		:whenResolvedDo( function( status, albumList )
			
			assert( status == 'data', "albumsP resolved in error" )
			local duplicate = false
			local name1
			local name2
			for i, value in ipairs( albumList ) do
				if value[ 'id' ] == albumId1 then
					name1 = value[ 'name' ]
				end
				if value[ 'id' ] == albumId2 then
					name2 = value[ 'name' ]
				end
				if name1 == name2 then
					duplicate = true
				end
			end
			assert( duplicate, "No duplicate Albums found in albumsP() list" )
			
		end )

		:waitFor( function()
			
			assert( albumId1, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId1 )
		
	 	end )
	
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
			
			assert( albumId2, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId2 )
		
	 	end )
	
		:continueWhenValueMatches( 'data', true )

end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_RenameAlbumP_BypassUniquenessCheck_False_testNightly( test )
	
	local albumId1
	local albumId2
	local albumName1 = 'testAlbums_RenameAlbumP_BypassUniquenessCheck_false'
	local albumName2 = 'testAlbums_RenameAlbumP_BypassUniquenessCheck_false_Duplicate'
	local newName = albumName2 --renaming albumId1 and new name will ne albumId2's name
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		:waitFor( function() --step3
		
			return catalogServiceTest:createAlbumP( albumName1 )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
			  	albumId1 = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		
		:waitFor( function() 	--step4
		
			return catalogServiceTest:createAlbumP( albumName2 )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId2 = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		
		:waitFor( function() 	--step6
		
			assert( albumId1, "AlbumId1 is nil" )
			return catalogServiceTest:renameAlbumP( albumId1, newName, false )
	
		end )
		
		:continueWhenValueMatches( 'error', 'NameExists' )

		:waitFor( function()	--step7
			
			assert( albumId1, "AlbumId1 is nil" )
			return catalogServiceTest:deleteAlbumP( albumId1 )
		
	 	end )
	
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	--step8
			
			assert( albumId2, "AlbumId2 is nil" )
			return catalogServiceTest:deleteAlbumP( albumId2 )
		
	 	end )
	
		:continueWhenValueMatches( 'data', true )

end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_RenameAlbumP_Default_testNightly( test )
	
	local albumId
	local albumName = 'testAlbums_RenameAlbumP_Default'
	local newName = 'testAlbums_RenameAlbumPRenamed'
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		:waitFor( function() --step3
		
			return catalogServiceTest:createAlbumP( albumName )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		
		:waitFor( function()	--step5
			
			return catalogServiceTest:albumsP()
			
		end )
		
		:whenResolvedDo( function( status, albumList )
			
			assert( status == 'data', "albumsP resolved in error" )
			local found = false

			for i, value in ipairs( albumList ) do
				if value[ 'id' ] == albumId and value[ 'name' ] == albumName then
					found = true
					break
				end
			end
			assert( found, "testAlbums_RenameAlbumP_Default album not found in albumsP() list" )
			
		end )
		
		:waitFor( function() 	--step6
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:renameAlbumP( albumId, newName )
	
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function() 	--step8
		
			return catalogServiceTest:albumsP()
			
		end )
		
		:whenResolvedDo( function( status, albumList )
			
			assert( status == 'data', "albumsP resolved in error" )
			local renamed = false

			for i, value in ipairs( albumList ) do
				if value[ 'id' ] == albumId and value[ 'name' ] == newName then
					renamed = true
					break
				end
			end
			assert( renamed, "Album name not changed" )
			
		end )
		
		:waitFor( function()	--step9
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId )
		
	 	end )
	
		:continueWhenValueMatches( 'data', true )

end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_TouchAlbumP_testNightly( test )
	
	local albumId
	local albumName = 'testAlbums_TouchAlbumP'

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		:waitFor( function() --step3
		
			return catalogServiceTest:createAlbumP( albumName )
	
		end )
		
		:whenResolvedDo( function( status, id )

			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		:waitFor( function() --step4
		
			assert( albumId, "AlbumId returned by createAlbumP is nil" )
			return catalogServiceTest:touchAlbumP( albumId )
	
		end )
		
		:continueWhenValueMatches( 'data', true )
		--fixme check new timestamp in the albumList
		:waitFor( function()
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId )
		
	 	end )
	
		:continueWhenValueMatches( 'data', true )

end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_AlbumNameForIdB_testNightly( test )
	
	local albumId
	local albumName = 'testAlbums_AlbumNameForIdB'
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		:waitFor( function() --step3
		
			return catalogServiceTest:createAlbumP( albumName )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		:waitFor( function()
		
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:albumNameForIdB( albumId )
			
		end )
				
		:continueWhenValueMatches( albumName )
		
		:waitFor( function()	--step5
		
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:renameAlbumP( albumId, 'NewName' )
			
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:albumNameForIdB( albumId )
			
		end )
				
		:continueWhenValueMatches( 'NewName' )
		
		:waitFor( function() --step7
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId )
		
	 	end )
	
		:continueWhenValueMatches( 'data', true )


end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_AlbumsDateSortedB_testNightly( test )
	
	local albums = catalogServiceDefault:albumsDateSortedB()
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( syncWaitingForRealtime( catalogServiceDefault ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( albums )	--step2
		
		:continueWhenValueMatches( {
			{
				id = "1dcbce7d33104c8e9442dd2de697c32c",
				name = "DontTouchThisAlbum",
				updateTimeStamp = "2012-06-05T23:19:55.587095Z",
			},
		} )
		
		:waitFor( function()
		
			catalogServiceDefault:setSyncEnabled( false )
			return 1
			
		end )
		
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_SmartAlbumsB_testNightly( test )
	
	local albums = catalogServiceDefault:smartAlbumsB()
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( syncWaitingForRealtime( catalogServiceDefault ) )
		:continueWhenValueMatches( 'data', true )

		:waitFor( albums )	--step
		
		:continueWhenValueMatches( {
			{
				id = "favorites38c82a0119464e25a76b602f6e55c5e5",
				name = "My Favorites",
			},
		} )
		:waitFor( function()
		
			catalogServiceDefault:setSyncEnabled( false )
			return 1
			
		end )
		
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_ShareAlbumP_UnshareAlbumP_testNightly( test )
	
	local albumId
	local albumName = 'testAlbums_ShareAlbumP_UnshareAlbumP'

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( function() --step3
		
			return catalogServiceTest:createAlbumP( albumName )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		:waitFor( function()	--step4
		
			assert( albumId, "AlbumId returned by createAlbumP is nil" )
			return catalogServiceTest:shareAlbumP( albumId )
			
		end )
		
		:whenResolvedDo( function( status, spaceId )
		
			assert( status == 'data', "shareAlbumP resolved in error" )
			assert( spaceId, "spaceId returned by shareAlbumP is nil" )
			
		end )
		
		:waitFor( function()	--step5
		
			assert( albumId, "AlbumId returned by createAlbumP is nil" )
			return catalogServiceTest:isAlbumSharedP( albumId )
			
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	--step6
		
			assert( albumId, "AlbumId returned by createAlbumP is nil" )
			return catalogServiceTest:unshareAlbumP( albumId )
			
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	--step7
		
			assert( albumId, "AlbumId returned by createAlbumP is nil" )
			return catalogServiceTest:isAlbumSharedP( albumId )
			
		end )
		
		:continueWhenValueMatches( 'data', false )
		
		:waitFor( function()	--step8
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId )
		
	 	end )
	
		:continueWhenValueMatches( 'data', true )
		
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_IsAlbumSharedB_Unshared_testNightly( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		:waitFor( function() --step3
		
			return catalogServiceDefault:isAlbumSharedB( testAlbumId )
	
		end )
		
		:continueWhenValueMatches( false )
		
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_IsAlbumSharedB_Shared_testNightly( test )
	
	local albumId
	local albumName = 'testAlbums_IsAlbumSharedB_shared'

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
		
		:waitFor( function() --step3
		
			return catalogServiceTest:createAlbumP( albumName )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		:waitFor( function() --step4
		
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:isAlbumSharedB( albumId )
	
		end )
		
		:continueWhenValueMatches( false )
		
		:waitFor( function()	--step5
		
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:shareAlbumP( albumId )
			
		end )
		
		:continueWhenValueMatches( 'data', ':ignore:' )
	
		:waitFor( function() --step6
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:isAlbumSharedB( albumId )
	
		end )
		
		:continueWhenValueMatches( true )
		
		:waitFor( function()
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId )
		
	 	end )
	
		:continueWhenValueMatches( 'data', true )
		
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_SharedAlbumUrlP_testNightly( test )

	local albumId
	local shareUrl
	local spaceId
	
	WFAsyncTestCase( test, 120 )
	
		:waitFor( setupP )
				
		:waitFor( function()
		
			return catalogServiceTest:createAlbumP( 'testAlbums_SharedAlbumUrlP' )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		
		:waitFor( syncWaitingForRealtime( catalogServiceTest ) )
		:continueWhenValueMatches( 'data', true )
		:waitFor( function()
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:sharedAlbumUrlP( albumId )
	
		end )
		
		:continueWhenValueMatches( 'data', nil )
		
		:waitFor( function()
		
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:shareAlbumP( albumId )
			
		end )
		
		:whenResolvedDo( function( status, space )
			
			assert( status == 'data', "shareAlbumP resolved in error" )
			assert( space ~= nil , "spaceId returned by shareAlbumP is nil" )
			spaceId = space
			shareUrl = "http://ponysaurus.com/shares/" .. spaceId
			
		end )
		:waitFor( syncWaitingForRealtime( catalogServiceTest ) )
		:continueWhenValueMatches( 'data', true )
		:waitFor( function()
			
			assert( albumId, "AlbumId is nil" )
			--debug.print("URLP*****", shareUrl )
			return catalogServiceTest:sharedAlbumUrlP( albumId, true )
	
		end )
		:whenResolvedDo( function( status, url )
		
			assert( status == 'data', "sharedAlbumUrlP resolved in error" )
			assert( url , "Url returned by sharedAlbumUrlP is nil" )
			assert( url == shareUrl , "share Url didnt match with expected share Url" )
			
		end )
		:waitFor( function()
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId )
	
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )

end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_SharedAlbumUrlB_testNightly( test )
	
	local albumId
	local shareUrl

	WFAsyncTestCase( test, 120 )
	
		:waitFor( setupP )
				
		:waitFor( function() --step3
		
			return catalogServiceTest:createAlbumP( 'testAlbums_SharedAlbumUrlB' )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		
		:waitFor( syncWaitingForRealtime( catalogServiceTest ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function() --step5
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:sharedAlbumUrlB( albumId )
	
		end )
		
		:continueWhenValueMatches( nil )
		
		:waitFor( function()	--step6
		
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:shareAlbumP( albumId )
			
		end )
		
		:whenResolvedDo( function( status, space )
			
			assert( status == 'data', "shareAlbumP resolved in error" )
			assert( space ~= nil , "spaceId returned by shareAlbumP is nil" )
			shareUrl = "www.ponysaurus.com/shares/"..space
			
		end )
		
		:waitFor( syncWaitingForRealtime( catalogServiceTest ) )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function() --step8
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:sharedAlbumUrlB( albumId )
	
		end )
		
		:continueWhenValueMatches( shareUrl )
		:waitFor( function() --step9
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId )
	
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	-- step
		
			catalogServiceTest:setSyncEnabled( false )
			return 1
			
		end )
end

--------------------------------------------------------------------------------


function OzCatalogServiceTest.testAlbums_InviteUsersToAlbumP_RemoveUsersFromAlbumP_testNightly( test )
	
	local albumId
	local emails = {
		'ppaurav+test1@adobetest.com',
		'ppaurav+test2@adobetest.com',
	}
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
				
		:waitFor( function() --step3
		
			return catalogServiceTest:createAlbumP( 'testAlbums_InviteUsersToAlbumP_RemoveUsersFromAlbumP' )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		
		:waitFor( function() --step5
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:inviteUsersToAlbumP( albumId, emails )
	
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	--step7
		
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:removeUsersFromAlbumP( albumId, emails )
			
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function() --step8
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId )
	
		end )
		
		:continueWhenValueMatches( 'data', true )
		
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_SetGetRequireLoginForAlbumP_testNightly( test )
	
	local albumId
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
				
		:waitFor( function() --step3
		
			return catalogServiceTest:createAlbumP( 'testAlbums_setRequireLoginForAlbumP' )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		
		:waitFor( function() --step5
			
			--initially when album is not shared getRequireLoginForAlbumB
			--should return ....
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:getRequireLoginForAlbumB( albumId )
	
		end )
		
		:continueWhenValueMatches( nil ) -- Confirm with Neha nil or false

		:waitFor( function() --step6
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:setRequireLoginForAlbumP( albumId, true )
	
		end )
		
		:continueWhenValueMatches( 'error', 'AlbumNotShared' )
		
		:waitFor( function() --step7
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:shareAlbumP( albumId )
	
		end )
		
		:continueWhenValueMatches( 'data', ':ignore:' )
		
		:waitFor( function() --step 9
			
			-- after album is shared getRequireLoginForAlbumB
			--should return false
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:getRequireLoginForAlbumB( albumId )
	
		end )
		
		:continueWhenValueMatches( false )
		
		:waitFor( function() --step9
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:setRequireLoginForAlbumP( albumId, true )
	
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function() --step10
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:getRequireLoginForAlbumB( albumId )
	
		end )
		
		:continueWhenValueMatches( true )
		
		:waitFor( function() --step11
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:setRequireLoginForAlbumP( albumId, false )
	
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function() --step12
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:getRequireLoginForAlbumB( albumId )
	
		end )
		
		:continueWhenValueMatches( false )
		
		:waitFor( function() --step13
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId )
	
		end )
		
		:continueWhenValueMatches( 'data', true )
		
end

--------------------------------------------------------------------------------

function OzCatalogServiceTest.testAlbums_SetGetAllowDownloadsForAlbumP_testNightly( test )
	
	local albumId
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( setupP )
				
		:waitFor( function() --step3
		
			return catalogServiceTest:createAlbumP( 'testAlbums_SetGetAllowDownloadsForAlbumP' )
	
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id, "AlbumId returned by createAlbumP is nil" )
				albumId = id
			else
				assert( false, "createAlbumP resolved in error" )
			end
	
		end )
		
		:waitFor( function() --step5
			
			--initially when album is not shared getAllowDownloadsForAlbumB
			--should return ....
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:getAllowDownloadsForAlbumB( albumId )
	
		end )
		
		:continueWhenValueMatches( nil ) -- Confirm with Neha nil or false

		:waitFor( function() --step6
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:setAllowDownloadsForAlbumP( albumId, true )
	
		end )
		
		:continueWhenValueMatches( 'error', 'AlbumNotShared' )
		
		:waitFor( function() --step7
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:shareAlbumP( albumId )
	
		end )
		
		:continueWhenValueMatches( 'data', ':ignore:' )
		
		:waitFor( function() --step9
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:setAllowDownloadsForAlbumP( albumId, true )
	
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function() --step10
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:getAllowDownloadsForAlbumB( albumId )
	
		end )
		
		:continueWhenValueMatches( true )
		
		:waitFor( function() --step11
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:setAllowDownloadsForAlbumP( albumId, false )
	
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function() --step12
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:getAllowDownloadsForAlbumB( albumId )
	
		end )
		
		:continueWhenValueMatches( false )
		
		:waitFor( function() --step13
			
			assert( albumId, "AlbumId is nil" )
			return catalogServiceTest:deleteAlbumP( albumId )
	
		end )
		
		:continueWhenValueMatches( 'data', true )
		

end

--------------------------------------------------------------------------------

--FIXME [ppaurav 2012-09-21]: They never worked on winrt. There is a trello
-- card for this https://trello.com/card/fix-oz-test-cases-on-winrt/501f82ffc119f0ee5061ee07/96
if WFPlatformInfo.isWinRT then
	return {}
else
	return OzCatalogServiceTest
end
