--[[----------------------------------------------------------------------------

OzCatalogUrls.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

Centralized repository for Oz URLs so we can change it as needed for new
milestone server deployments.

--------------------------------------------------------------------------------

@author scouten 2011-05-18

------------------------------------------------------------------------------]]

local KSPrefs = require 'KSPrefs'
local KSPathUtils = require "KSPathUtils"
local WFPlatformInfo = require 'WFPlatformInfo'

local DISTRIBUTION = _G.DISTRIBUTION

--------------------------------------------------------------------------------

local urls = {}

--------------------------------------------------------------------------------

local function setUrlsForOverrideUrl( url )

	urls.baseServerUrl = url
	urls.baseAdminServerUrl = string.gsub( url, "api.", "admin." )
	urls.catalogsUrl = string.format( '%s/v1.0/catalogs', url )
	urls.receiptsUrl = string.format( '%s/v1.0/receipts', url )

end

--------------------------------------------------------------------------------

if WFPlatformInfo.isAndroid then

	local ozUrl = KSPathUtils.getOzServerUrl()
	setUrlsForOverrideUrl( ozUrl )

else

	if DISTRIBUTION then

		setUrlsForOverrideUrl( 'https://api.adbecrsl.com' )

	else
	
		local function urlForPref( url )
			if url then
				return url
			else
				return 'https://api.oznext.com'
			end
		end
	
		urls._overridePrefB = KSPrefs.prefB 'ozServerOverrideUrl':mapB( function( url )
			local baseUrl = urlForPref( url )
			--debug.print( "OzCatalogUrls is using Oz server url:", url )
			setUrlsForOverrideUrl( baseUrl )
		end )
	
	end
end

--------------------------------------------------------------------------------

return urls
