--[[----------------------------------------------------------------------------

File: OzFilePaths.lua

Date: 10/23/2012
Author: Andrew Rahn

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local KSPathUtils = require "KSPathUtils"
local WFArguments = require "WFArguments"

local WFArguments_check = WFArguments.check
local KSPathUtils_child = KSPathUtils.child
local KSPathUtils_getStandardFilePath = KSPathUtils.getStandardFilePath
--==============================================================================

local OzFilePaths = {}

--------------------------------------------------------------------------------

--[[!
	\brief construct a file path for the standard originalFolderDocuments path, based
	on a catalog ID
	
	\param catalogId the GUID id for the catalog
	\return (string) the file path
--]]
function OzFilePaths.getStandardOriginalFolderDocumentsPath( catalogId )

	WFArguments_check( "string" )
	
	local documentsDir = KSPathUtils_getStandardFilePath 'carouselDocuments'
	
	local originalsFolderDocuments = KSPathUtils_child( documentsDir, catalogId )
	originalsFolderDocuments = KSPathUtils_child( originalsFolderDocuments, "Originals" )
	
	return originalsFolderDocuments
	
end

--------------------------------------------------------------------------------

--[[!
	\brief construct a file path for the standard originalFolderCaches path, based
	on a catalog ID
	
	\param catalogId the GUID id for the catalog
	\return (string) the file path
--]]
function OzFilePaths.getStandardOriginalFolderCachesPath( catalogId )

	WFArguments_check( "string" )
	
	local cachesDir = KSPathUtils.getStandardFilePath 'carouselCaches'
	
	local originalsFolderCaches = KSPathUtils.child( cachesDir, catalogId )
	originalsFolderCaches = KSPathUtils.child( originalsFolderCaches, "Originals" )
	
	return originalsFolderCaches
	
end

--------------------------------------------------------------------------------

--[[!
	\brief construct a file path for an original based on a originalsFolderDocuments
	path and a capture date in format like this: 2012-10-08T14:15:44
	
	\param originalsFolderDocuments Absolute path to the originalFolderDocuments folder
	\param captureDate Date of the original asset that you're trying to find a home for
]]--

function OzFilePaths.originalsFolderForDate( originalsFolderDocuments, captureDate )

	WFArguments_check( "string", "string" )

	captureDate = captureDate:sub( 1, 10 )

	local folder = KSPathUtils_child( originalsFolderDocuments, captureDate:sub( 1, 4 ) ) -- yyyy
	folder = KSPathUtils_child( folder, captureDate:sub( 1, 7 ) ) -- yyyy-mm
	folder = KSPathUtils_child( folder, captureDate ) -- yyyy-mm-dd
	
	return folder

end

--==============================================================================

return OzFilePaths
