--[[----------------------------------------------------------------------------

OzOriginalsAgentManager.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

This namespace coordinates the originals download agents for all catalogs

------------------------------------------------------------------------------]]

	-- wichitafoundation.core.lua.threading
local WFRequestDispatcher = require 'WFRequestDispatcher'

	-- libs.reactive
local KSReactive = require 'KSReactive'

	-- oz.originals_agent
local OzOriginalsAgent = require 'OzOriginalsAgent'


--============================================================================--

-- @mark OzOriginalsAgentManager

local OzOriginalsAgentManager = {}

--------------------------------------------------------------------------------

local activeOriginalsAgentBehaviors = {}
	-- key: (string) catalogId
	-- value: (B< nil or OzOriginalsAgent >) originals agent for this catalog

	-- NOTE: This is intentionally not a weak map. Once referenced, the
	-- behaviors stay present indefinitely so we can do mapB and mapSwitchB
	-- on them. We do clear out the agent itself.

--------------------------------------------------------------------------------

local function originalsAgentBehaviorForCatalogId( catalogId )

	local resultB = activeOriginalsAgentBehaviors[ assert( catalogId ) ]
	
	if not resultB then
	
		resultB = KSReactive.variableB( nil )
		activeOriginalsAgentBehaviors[ catalogId ] = resultB
	
	end
	
	return resultB

end


--============================================================================--

-- @mark originals agent management

--------------------------------------------------------------------------------

local function startOriginalsAgentForCatalogId( catalogId, userId )
			
	assert( type( catalogId ) == 'string',
		"startOriginalsAgent: catalogId was not a string" )
		
	assert( type( userId ) == 'string',
		"startOriginalsAgent: userId was not a string" )
	
	local originalsAgentB = originalsAgentBehaviorForCatalogId( catalogId )
	
	if not originalsAgentB:valueNow() then

		--debug.print( "Creating OzOriginalsAgent for catalog ",
		--				catalogId )

		originalsAgentB:set( OzOriginalsAgent( catalogId, userId ) )

	end
	
end

--------------------------------------------------------------------------------

function OzOriginalsAgentManager.installMessageHandlers()

	----------------------------------------------------------------------------
	
	WFRequestDispatcher.onMessageNoCancel( 'startOriginalsAgent',
										   startOriginalsAgentForCatalogId )
	
	----------------------------------------------------------------------------
	
	WFRequestDispatcher.onMessageNoCancel( 'stopOriginalsAgent',
		function( catalogId )
		
			assert( type( catalogId ) == 'string',
				"stopOriginalsAgent: catalogId was not a string" )
			
			local originalsAgentB = originalsAgentBehaviorForCatalogId( catalogId )
			
			local originalsAgent = originalsAgentB:valueNow()
			
			if originalsAgent then

				--debug.print( "Closing OzOriginalsAgent for catalog ",
				--				catalogId )

				originalsAgentB:set( nil )

				originalsAgent:close()

			end
			
		end )

	----------------------------------------------------------------------------

	WFRequestDispatcher.onMessage( 'originalsEnabledB', function( catalogId )

		return originalsAgentBehaviorForCatalogId( catalogId ):toBooleanB()
		
	end )
		
end

--------------------------------------------------------------------------------

return OzOriginalsAgentManager
