--[[----------------------------------------------------------------------------

OzSyncAgentTest.lua

$Date:
$Author:

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author ppaurav 2012-03-14

------------------------------------------------------------------------------]]

	-- services.http_client
local OzUser = require 'OzUser'

	-- wichitafoundation.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'

local OzCatalogService = require 'OzCatalogService'

local OzAccountService = require 'OzAccountService'

local OzCatalogUrls = require 'OzCatalogUrls'

local KSOAuth2Session = require 'KSOAuth2Session'

local KSIdGenerator = require 'KSIdGenerator'

local KSJsonEncoder = require 'KSJsonEncoder'

local KSDate = require 'KSDate'

local WFPlatformInfo = require 'WFPlatformInfo'

local Bablu = require 'Bablu'

local KSPromises = require 'KSPromises'


local OzCatalogTestUtils = require "OzCatalogTestUtils"
local KSReactivePromiseBridge = require "KSReactivePromiseBridge"
local WFTimer = require 'WFTimer'


-- wichitafoundation.os.prefs
local KSPrefs = require 'KSPrefs'
KSPrefs.currentUserId = nil

--============================================================================--

local OzSyncAgentTest = {}

--------------------------------------------------------------------------------

local ozUserId = '4a0983eb2bae4a98bac70e15617cf952'
	-- user id for ppaurav+syncagent@adobetest.com on oznext.

local TEST_WAIT_TIME = 600
local SYNC_WAIT_TIME = WFPlatformInfo.isAndroid and 15 or 5

local ozUser
local oAuthSession = KSOAuth2Session
local catalogService
local catalogId
local setupP

--------------------------------------------------------------------------------

local function syncWaitingForRealtime( catalogService )

	return function()
			-- With the removal of the realtime flag to our oz sync, even an explicit
			-- call to syncNow might miss a new change.  So we wait 5 seconds to make
			-- sure the change will show up in the stream.
			return WFTimer.afterDelayP( SYNC_WAIT_TIME )
			
			:mapDataP( function()
			
				catalogService:setSyncEnabled( true )
				return catalogService:syncNowP()
				
			end )
	end
	
end

--------------------------------------------------------------------------------

local function cleanupTestAccountP()

	local catalogsToBeDeleted = {}

	local function deleteCatalog( arg )
		return ozUser:deleteCatalogP( arg.id )
	end

	return ozUser:catalogsP()
		:mapDataP( function( status, result )

			for _, catalogInfo in ipairs( result ) do

				if not string.find( catalogInfo.name, "DontTouchThisCatalog" ) then
					table.insert( catalogsToBeDeleted, catalogInfo )
				end

			end

			if next( catalogsToBeDeleted ) then

				debug.print( string.format( "Deleting %d old catalogs from "
											.. "previous test runs", #catalogsToBeDeleted ) )
				return KSPromises.allP( deleteCatalog, catalogsToBeDeleted )

			else

				return KSPromises.resolvedPromiseP( 'data', true )

			end
			
		end )

end

--------------------------------------------------------------------------------

function OzSyncAgentTest.setUpClass()

	oAuthSession = KSOAuth2Session( ozUserId )
	ozUser = OzUser.make( ozUserId )
	KSPrefs.currentUserId = ozUserId
	OzAccountService.setCurrentUser( ozUserId )
	
	local prefsFlush = KSIdGenerator.uuidInLowercaseHex()
	
	-- POST paramateres
	-- client_id, client_secret as already configured
	-- redirect_uri = 'air-auth://carousel'
	-- locale = en_US
	-- scope = openid,AdobeID
	-- username = 'ppaurav+syncagent@adobetest.com'
	-- password ='adobetest'
	
	local postUrl = 'https://ims-na1-qa1.adobelogin.com/ims/login/code'
	local userAuthParams = 'username=ppaurav%2Bsyncagent%40adobetest.com&password=adobetest'
	
	local postParamsFormatStr = 'client_id=%s&redirect_uri=air-auth://carousel&locale=en_US&scope=openid,AdobeID&client_secret=%s&%s'
	
	local flushB = KSPrefs.prefB( "flushPrefsForOzCatalogServiceTest" )
	catalogId = KSIdGenerator.uuidInLowercaseHex()
	catalogService = OzCatalogService( catalogId, ozUser )
	
	setupP = require 'WFWorkerThread'( "WFRequestTestHandler.lua" ):requestP( "setPref", "flushPrefsForOzCatalogServiceTest", prefsFlush )
	:mapDataP( function()
		return KSReactivePromiseBridge.behaviorToPromise(
			flushB:relayForAutoCloseB(), -- FIXME_AUTO_CLOSE: Do I need the relay here?
			function( value )
				return value == prefsFlush
			end
		)
	end )
	:mapDataP( function()
		return Bablu:getImsClientIdAndSecretP()
	end )
	:mapDataP( function( status, clientId, secret )
	
		local postParams = string.format( postParamsFormatStr, clientId, secret, userAuthParams )
		return ozUser:getAccessTokenRefreshTokenFromImsP( postUrl, postParams )
	
	end )
	:mapDataP( function( status, refresh_token, access_token )

		return oAuthSession:storeTokensP( refresh_token, access_token )
		
	end )
	:mapDataP( function()

		return cleanupTestAccountP()
		
	end )
	:mapDataP( function()
	
		return ozUser:createCatalogP {
			name = 'OzSyncAgentTest',
			id = catalogId,
		}
		
	end )
	:mapDataP( function()

		return OzCatalogTestUtils.importPhotosP( catalogService, {
			{ captureDate = "2004-11-02T11:10:09" },
			{ captureDate = "2012-05-02T08:07:06" },
			{ captureDate = "2001-02-03T04:05:06" },
		} )
	end )
	:runUntilDoneP()
	
end

--------------------------------------------------------------------------------

function OzSyncAgentTest.tearDownClass()
	
	catalogService:close()
	catalogService = nil
	
	oAuthSession:close()
	oAuthSession = nil
	
	ozUser:close()
	ozUser = nil
	
end

--------------------------------------------------------------------------------

-- @download

--------------------------------------------------------------------------------

function OzSyncAgentTest.testRenameEvent_testNightly( test )

	local photoIds = {}
	local photoInfo
	local collectionId = 'date\t2004-11-02'
	local photoId
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( setupP )

		:waitFor( syncWaitingForRealtime( catalogService ) )			-- Make sure the catalog is synced before we begin.
		
		:continueWhenValueMatches( 'data', true )

		:waitFor( function()
		
			return catalogService:collectionInfoP( 'date', nil, collectionId )
			
		end )
		:whenResolvedDo( function( status, collection )

			assert( status == 'data', "collectionInfoP resolved in error" )
			local photos = collection.photos
			assert( #photos ~= 0, "No photos in collection" )
			for i = 1, #photos do
				photoIds[ i ] = photos[ i ].id
			end
			photoId = photoIds[ 1 ]
			
		end )
			
		:waitFor( function()

			return catalogService:photoInfoP( photoId )
			
		end )
		
		:whenResolvedDo( function( status, photo )
			
			photoInfo = photo
		
		end )
		
		:waitFor( function()

			-- construct a PUT URL
			local catalogUrl = string.format( '%s/%s', OzCatalogUrls.catalogsUrl, catalogId )
			local docUrl = string.format( '%s/assets/%s', catalogUrl, photoId )
			local putUrl = string.format( '%s/revisions/%s', docUrl, KSIdGenerator.uuidInLowercaseHex() )
			
			-- Need parent revision Id
			local parentRevLink = photoInfo._serverProvidedMetadata
					  and photoInfo._serverProvidedMetadata.links
					  and photoInfo._serverProvidedMetadata.links[ "/rels/latest_revision" ]
					  and photoInfo._serverProvidedMetadata.links[ "/rels/latest_revision" ].href
			local parentRevId = parentRevLink and parentRevLink:sub( -32 )
			
			-- construct a upload content
			local deltaForUpload = {
           	 	eventName = 'NewEventName',
				eventTimestamp = KSDate.timeInIso8601Format( nil, false ),
       		 }

			local uploadContent = {
				parent_revision_id = parentRevId,
				user_id = ozUserId,
				payload = deltaForUpload,
			}
			local jsonUpload = KSJsonEncoder.encodeOzUploadJson( uploadContent )
			
			local putArgs = {
				requestHeaders = {
					{ header = 'Content-Type', value = 'application/json' },
					{ header = 'Accept', value = 'application/json' },
				},
			}
			return ozUser:putP( putUrl, jsonUpload, putArgs )
		
		end )
		
		:whenResolvedDo( function( status, data, headers )

			if status == 'data' then
				assert ( headers and headers.status_code == '201', "status code should be 201" .. " but it was " .. headers.status_code )
			else
				
				assert( false, "failed to rename event name on Oz server" )
			end
		
		end )

		:waitFor( syncWaitingForRealtime( catalogService ) )

		:waitFor( function()
			
			return catalogService:photoInfoP( photoId )
			
		end )
		
		:whenResolvedDo( function( status, photo )

			assert( status == 'data', "photoInfoP returned error" )
			assert( photo.eventName == 'NewEventName', "Event name not yet synced")
			
		end )
		
		:waitFor( function()
		
			catalogService:setSyncEnabled( false )
			return 1
			
		end )

end

--------------------------------------------------------------------------------

function OzSyncAgentTest.testFavoritePhoto_testNightly( test )

	local photoIds = {}
	local photoInfo
	local collectionId = 'date\t2012-05-02'
	local photoId
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( setupP )

		:waitFor( syncWaitingForRealtime( catalogService ) )			-- Make sure the catalog is synced before we begin.
		
		:continueWhenValueMatches( 'data', true )

		:waitFor( function()
		
			return catalogService:collectionInfoP( 'date', nil, collectionId )
			
		end )
		:whenResolvedDo( function( status, collection )

			assert( status == 'data', "collectionInfoP resolved in error" )
			local photos = collection.photos
			assert( #photos ~= 0, "No photos in collection" )
			for i = 1, #photos do
				photoIds[ i ] = photos[ i ].id
			end
			photoId = photoIds[ 1 ]
			
		end )
			
		:waitFor( function()
		
			return catalogService:photoInfoP( photoId )
			
		end )
		
		:whenResolvedDo( function( status, photo )
			
			photoInfo = photo
		
		end )
		
		:waitFor( function()

			-- construct a PUT URL
			local catalogUrl = string.format( '%s/%s', OzCatalogUrls.catalogsUrl, catalogId )
			local docUrl = string.format( '%s/assets/%s', catalogUrl, photoId )
			local putUrl = string.format( '%s/revisions/%s', docUrl, KSIdGenerator.uuidInLowercaseHex() )
			
			-- Need parent revision Id
			local parentRevLink = photoInfo._serverProvidedMetadata
					  and photoInfo._serverProvidedMetadata.links
					  and photoInfo._serverProvidedMetadata.links[ "/rels/latest_revision" ]
					  and photoInfo._serverProvidedMetadata.links[ "/rels/latest_revision" ].href
			local parentRevId = parentRevLink and parentRevLink:sub( -32 )
			
			-- construct a upload content
			local favorites = {}
			favorites[ ozUserId ] = true
			local deltaForUpload = {}
			deltaForUpload[ 'favorites' ] = favorites
			local uploadContent = {
				parent_revision_id = parentRevId,
				user_id = ozUserId,
				payload = deltaForUpload,
			}
			local jsonUpload = KSJsonEncoder.encodeOzUploadJson( uploadContent )
			
			local putArgs = {
				requestHeaders = {
					{ header = 'Content-Type', value = 'application/json' },
					{ header = 'Accept', value = 'application/json' },
				},
			}
			return ozUser:putP( putUrl, jsonUpload, putArgs )
		
		end )
		
		:whenResolvedDo( function( status, data, headers )

			if status == 'data' then
				assert ( headers and headers.status_code == '201', "status code should be 201" )
			else
				
				--debug.print( "headers ", headers )
				assert( false, "Oz call failed to favorite the photo, status code is " .. headers.status_code )
			end
		
		end )

		:waitFor( syncWaitingForRealtime( catalogService ) )

		:waitFor( function()
			
			return catalogService:photoInfoP( photoId )
			
		end )
		
		:whenResolvedDo( function( status, photo )
		
			assert( status == 'data', "photoInfoP returned error" )
			local fav = photo.favorites
			assert( fav[ ozUserId ] == true, "favorite not yet synced")
			
		end )
		
		:waitFor( function()
		
			catalogService:setSyncEnabled( false )
			return 1
			
		end )

end

--------------------------------------------------------------------------------

function OzSyncAgentTest.testZDeletePhoto_testNightly( test )

	local photoId
	local collectionId = 'date\t2001-02-03'
	local photoIds = {}
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( setupP ) --step1

		:waitFor( syncWaitingForRealtime( catalogService ) )			-- Make sure the catalog is synced before we begin.
		:continueWhenValueMatches( 'data', true )

		:waitFor( function()
		
			return catalogService:collectionInfoP( 'date', nil, collectionId )
			
		end )
		:whenResolvedDo( function( status, collection )

			assert( status == 'data', "collectionInfoP resolved in error" )
			local photos = collection.photos
			assert( #photos ~= 0, "No photos in collection" )
			for i = 1, #photos do
				photoIds[ i ] = photos[ i ].id
			end
			photoId = photoIds[ 1 ]
			
		end )
	
		:waitFor( function() --step3

			local catalogUrl = string.format( '%s/%s', OzCatalogUrls.catalogsUrl, catalogId )
			local deleteUrl = string.format( '%s/assets?asset_ids=%s', catalogUrl, photoId )
			return ozUser:getP( deleteUrl, { requestMethod = 'DELETE' } )
		
		end )
		
		:whenResolvedDo( function( status, data, headers )
			
			if status == 'data' then
				assert ( headers and headers.status_code == '204', "status code should be 204" )
			else
				assert( false, "failed to delete photo from Oz server" )
			end
		
		end )

		:waitFor( syncWaitingForRealtime( catalogService ) )
	
		:waitFor( function()	--step6
			
			return catalogService:photoInfoP( photoId )
			
		end )
		
		:whenResolvedDo( function( status, photo )
		
			assert( status == 'data', "photoInfoP returned error" )
			assert( photo._deleted == true, "Looks like photo was not deleted" )
			
		end )
		
		:waitFor( function() --step7
		
			catalogService:setSyncEnabled( false )
			return 1
			
		end )

end

--------------------------------------------------------------------------------

function OzSyncAgentTest.testAlbumSync_CreateAlbum_testNightly( test )

	local albumId = KSIdGenerator.uuidInLowercaseHex()
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( setupP )	--step1

		:waitFor( syncWaitingForRealtime( catalogService ) )			-- Make sure the catalog is synced before we begin.
		:continueWhenValueMatches( 'data', true )

		:waitFor( function()  -- Create an album on Oz
		
			-- PUT URL
			local catalogUrl = string.format( '%s/%s', OzCatalogUrls.catalogsUrl, catalogId )
			local putUrl = string.format( '%s/albums/%s', catalogUrl, albumId )
			
			-- Upload content
			local uploadContent = {
           	 	name = 'testAlbumSync_CreateAlbum',
       		 }

			local jsonUpload = KSJsonEncoder.encodeOzUploadJson( uploadContent )
			
			local postArgs = {
				requestHeaders = {
					{ header = 'Content-Type', value = 'application/json' },
					{ header = 'Accept', value = 'application/json' },
				},
			}
			return ozUser:putP( putUrl, jsonUpload, postArgs )
			
		end )
		
		:whenResolvedDo( function( status, data, headers )
			
			if status == 'data' then
				assert ( headers and headers.status_code == '201', "status code should be 201" )
			else
				assert( false, "failed to create album on Oz." )
			end
		
		end )
		
		:waitFor( syncWaitingForRealtime( catalogService ) )

		:waitFor( function()	-- step 4
			
			return catalogService:albumsP()
			
		end )
		
		:whenResolvedDo( function( status, albums )

			local albumName
			for i = 1, # albums do
				if albums[ i ].id == albumId then
					albumName = albums[ i ].name
					break
				end
			end
			assert( status == 'data', "albumsP returned error" )
			assert( albumName == 'testAlbumSync_CreateAlbum', "Newly created Album not synced yet" )
			
		end )
		
		:waitFor( function() -- Delete the album
		
			return catalogService:deleteAlbumP( albumId )
			
		end )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	-- step 6
		
			catalogService:setSyncEnabled( false )
			return 1
			
		end )

end

--------------------------------------------------------------------------------

function OzSyncAgentTest.testAlbumSync_DeleteAlbum_testNightly( test )

	local albumId
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( setupP )	--step1

		:waitFor( syncWaitingForRealtime( catalogService ) )			-- Make sure the catalog is synced before we begin.
		:continueWhenValueMatches( 'data', true )
	
		:waitFor( function()  --step2
		
			return catalogService:createAlbumP( 'testAlbumSync_DeleteAlbum', true )
			
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id , "Album Id is nil" )
				albumId = id
			else
				assert( false, "createAlbumP failed to create album." )
			end
		
		end )

		:waitFor( function()	-- step 3
			
			return catalogService:albumsP()
			
		end )
		
		:whenResolvedDo( function( status, albums )

			local albumName
			for i = 1, # albums do
				if albums[ i ].id == albumId then
					albumName = albums[ i ].name
					break
				end
			end
			assert( status == 'data', "albumsP returned error" )
			assert( albumName == 'testAlbumSync_DeleteAlbum', "Newly created Album not in albumsP yet" )
			
		end )
		
		:waitFor( syncWaitingForRealtime( catalogService ) )
		
		:waitFor( function() -- Delete the album on Oz
			
			assert( albumId, "Album Id is nil" )
			-- DELETE URL
			local catalogUrl = string.format( '%s/%s', OzCatalogUrls.catalogsUrl, catalogId )
			local delUrl = string.format( '%s/albums/%s', catalogUrl, albumId )
		
			local httpOptions = {
				requestMethod = 'DELETE',
				returnHttpErrorsAsData = true,
			}
	
			local deleteP = ozUser:getP( delUrl, httpOptions )
			return deleteP
			
		end )
		:whenResolvedDo( function ( status, data, headers )
		
			if status == 'data' then
				assert ( headers and headers.status_code == '204', "expected status code was 204 but it is  ".. headers.status_code )
			else
				assert( false, "failed to delete album on Oz." )
			end
			
		end )
		
		:waitFor( syncWaitingForRealtime( catalogService ) )

		:waitFor( function()	-- step 7
			
			return catalogService:albumsP()
			
		end )
		
		:whenResolvedDo( function( status, albums ) -- Check that Album shouldn't exists in albumsP ist
			
			assert( status == 'data', "albumsP returned error" )
			for i = 1, #albums do
				if albums[ i ].id == albumId then
					assert( false, "deleted album still exists in album list returned by albumsP" )
					break
				end
			end
			
		end )
		
		:waitFor( function()	-- step 7
		
		 	catalogService:setSyncEnabled( false )
			return 1
			
		end )

end

--------------------------------------------------------------------------------

function OzSyncAgentTest.testAlbumSync_RenameAlbum_testNightly( test )

	local albumId
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( setupP )	--step1

		:waitFor( syncWaitingForRealtime( catalogService ) )			-- Make sure the catalog is synced before we begin.
		:continueWhenValueMatches( 'data', true )

		:waitFor( function()  --step2
		
			return catalogService:createAlbumP( 'testAlbumSync_RenameAlbum', true )
			
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id , "Album Id is nil" )
				albumId = id
			else
				assert( false, "createAlbumP failed to create album." )
			end
		
		end )
		
		:waitFor( function()	-- step 4
			
			return catalogService:albumsP()
			
		end )
		
		:whenResolvedDo( function( status, albums )

			local albumName
			for i = 1, # albums do
				if albums[ i ].id == albumId then
					albumName = albums[ i ].name
					break
				end
			end
			assert( status == 'data', "albumsP returned error" )
			assert( albumName == 'testAlbumSync_RenameAlbum', "Newly created Album not synced yet" )
			
		end )
		
		:waitFor( syncWaitingForRealtime( catalogService ) )
		
		:waitFor( function() -- Rename the album on Oz
			
			assert( albumId , "Album Id is nil" )
			
			-- POST URL
			local catalogUrl = string.format( '%s/%s', OzCatalogUrls.catalogsUrl, catalogId )
			local postUrl = string.format( '%s/albums/%s', catalogUrl, albumId )
			
			-- Upload content
			local uploadContent = {
           	 	name = 'Renamed Album',
       		 }

			local jsonUpload = KSJsonEncoder.encodeOzUploadJson( uploadContent )
			
			local postArgs = {
				requestHeaders = {
					{ header = 'Content-Type', value = 'application/json' },
					{ header = 'Accept', value = 'application/json' },
				},
				requestMethod = 'POST',
			}
			debug.print( "postUrl*** " ,  postUrl)
			debug.print( "jsonUpload*** " ,  jsonUpload)
			debug.print( "postArgs*** " ,  postArgs)
			
			return ozUser:putP( postUrl, jsonUpload, postArgs )
				
		end )
		
		:whenResolvedDo( function ( status, data, headers )
		
			if status == 'data' then
				assert ( headers and headers.status_code == '204', "status code should be 204" )
			else
				assert( false, "failed to rename album on Oz." )
			end
			
		end )
		
		:waitFor( syncWaitingForRealtime( catalogService ) )

		:waitFor( function()	-- step 8
			
			return catalogService:albumsP()
			
		end )
		
		:whenResolvedDo( function( status, albums ) -- Check that Album shouldn't exists in albumsP ist
			
			local albumName
			for i = 1, # albums do
				if albums[ i ].id == albumId then
					albumName = albums[ i ].name
					break
				end
			end
			assert( status == 'data', "albumsP returned error" )
			assert( albumName == 'Renamed Album', "Album Name not synced yet")
			
		end )
		
		:waitFor( function() -- Delete the album
		
			return catalogService:deleteAlbumP( albumId )
			
		end )
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()	-- step 10
		
			 catalogService:setSyncEnabled( false )
			return 1
			
		end )

end

--------------------------------------------------------------------------------

function OzSyncAgentTest.testAlbumSync_AllowDownloads_testNightly( test )

	local albumId
	local spaceId
	
	WFAsyncTestCase( test, TEST_WAIT_TIME * 2 )

		:waitFor( setupP )

		:waitFor( syncWaitingForRealtime( catalogService ) )			-- Make sure the catalog is synced before we begin.
		:continueWhenValueMatches( 'data', true )

		:waitFor( function()
		
			return catalogService:createAlbumP( 'testAlbumSync_AllowDownloads', true )
			
		end )
		
		:whenResolvedDo( function( status, id )
			
			if status == 'data' then
				assert( id , "Album Id is nil" )
				albumId = id
			else
				assert( false, "createAlbumP failed to create album." )
			end
		
		end )

		:waitFor( syncWaitingForRealtime( catalogService ) )

		:waitFor( function()
		
			return catalogService:shareAlbumP( albumId )
			
		end )
		
		:whenResolvedDo( function( status, spaceid )
			
			assert( status == 'data', "shareAlbumP resolved in error" )
			assert( spaceid, "spaceId returned by shareAlbumP is nil" )
			spaceId = spaceid
			
		end )
		
		:waitFor( function()
			
			assert( albumId, "Album Id is nil" )
			return catalogService:getAllowDownloadsForAlbumB( albumId )
			
		end )
		
		:continueWhenValueMatches( true )
		
		:waitFor( syncWaitingForRealtime( catalogService ) )
		
		:waitFor( function()
			
			assert( spaceId , "Space Id is nil" )
			
			-- POST URL
			local catalogUrl = string.format( '%s/%s', OzCatalogUrls.catalogsUrl, catalogId )
			local postUrl = string.format( '%s/spaces/%s', catalogUrl, spaceId )
			
			-- Upload content
			local uploadContent = {}
			uploadContent[ 'download' ] = false
           	
			local jsonUpload = KSJsonEncoder.encodeOzUploadJson( uploadContent )
			
			local postArgs = {
				requestHeaders = {
					{ header = 'Content-Type', value = 'application/json' },
					{ header = 'Accept', value = 'application/json' },
				},
				requestMethod = 'POST',
			}

			return ozUser:putP( postUrl, jsonUpload, postArgs )
				
		end )
		
		:whenResolvedDo( function ( status, data, headers )
		
			if status == 'data' then
				assert ( headers and headers.status_code == '204', "status code should be 204" )
			else
				assert( false, "failed to update download orginial value on Oz." )
			end
			
		end )
		
		:waitFor( syncWaitingForRealtime( catalogService ) )

		:waitFor( function()
			
			return catalogService:getAllowDownloadsForAlbumB( albumId )
			
		end )
		
		:continueWhenValueMatches( false )
				
		:waitFor( function()
		
			return catalogService:deleteAlbumP( albumId )
			
		end )
		
		:continueWhenValueMatches( 'data', true )
		
		:waitFor( function()
		
			 catalogService:setSyncEnabled( false )
			return 1
			
		end )

end

--------------------------------------------------------------------------------

--FIXME [ppaurav 2012-09-21]: They never worked on winrt. There is a trello
-- card for this https://trello.com/card/fix-oz-test-cases-on-winrt/501f82ffc119f0ee5061ee07/96
if WFPlatformInfo.isWinRT then
	return {}
else
	return OzSyncAgentTest
end
