--[[----------------------------------------------------------------------------

OzUserInfoCacheIndexDefinitions.lua

@author scouten 2011-07-13

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- core.logging
local KSLogging = require 'KSLogging'

	-- wichitafoundation.oz_client.index_store
local WFIndexStoreImp = require 'WFIndexStoreImp'
local WFIndexStoreSchema = require 'WFIndexStoreSchema'


--============================================================================--

local indexStoreSchema = WFIndexStoreSchema()

--------------------------------------------------------------------------------

local uploadStatusLog = KSLogging.log 'OzUserInfoCacheIndexDefinitions::uploadStatus'
	:setLevel( KSLogging.DEBUG )

--------------------------------------------------------------------------------

indexStoreSchema:addTable {

	-- Warning [ arahn 11/19/2012 ]: this table definition must match the one defined by OzCatalogIndexDefinitions
	-- because some queries operate on both indexes, for example processUploadQueueP()'s query
	-- in WFOzSyncCycle.
	sql = [[CREATE TABLE ozUploadStatus( ozUploadStatus, docType, ozUser, captureDate, albumId, assetId, uploadOrder NOT NULL, changeCounter, freemiumUploadEmbargoUntil, uploadWave )]],

	map = function( docValue, emit )

		if docValue._docType == 'receipt' then
			
			local uploadStatus = docValue._ozMetadataUploadStatus or 'notUploaded'
	
			if uploadStatus == 'uploaded' then return end
		
			uploadStatusLog:debug( "RECEIPT %s >> uploadStatus = %s", docValue, uploadStatus )
			
			emit( uploadStatus, docValue._docType, docValue.userId, nil, nil, nil, 1, nil, nil )
		
		end

	end,

}

indexStoreSchema:addIndex
	[[CREATE INDEX ozUploadStatus_status ON ozUploadStatus(
					ozUploadStatus, ozUser, captureDate, validRev ASC )]]

--------------------------------------------------------------------------------

indexStoreSchema:addTable {

	sql = [[CREATE TABLE pendingReceiptNotifications(
				userId, receiptStatus, ozErrorCode, externalStoreErrorCode )]],
	
	map = function( docValue, emit )
	
		if docValue._docType == 'receipt'
			and docValue.receiptStatus -- values are "accepted" or "invalid"
			and not docValue.receiptStatusAcknowledgedByUI
		then
		
			emit( docValue.userId,
					docValue.receiptStatus,
					docValue.ozErrorCode,
					docValue.externalStoreErrorCode )
		
		end
	
	end,
	
}

indexStoreSchema:addIndex
	[[CREATE INDEX pendingReceiptNotifications_byUser ON pendingReceiptNotifications(
					userId, receiptStatus )]]

--------------------------------------------------------------------------------

local initMessageProcessor = function( ... )
	WFIndexStoreImp.start( indexStoreSchema, ... )
end

--==============================================================================

return {
	initMessageProcessor = initMessageProcessor,
}
