--[[----------------------------------------------------------------------------

OzUserProgressTest.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local WFAsyncTestCase = require 'WFAsyncTestCase'
local KSPromises = require 'KSPromises'
local OzUserTestUtils = require 'OzUserTestUtils'

local OzUser
local T
local putData

local TestSuite = {}

--============================================================================--

local function buildMocks()

	return
	{
		OzUserTestUtils.buildMocks(),
				
		KSOAuth2Session = function( userId )
			return {
				getP = function( self, url, optArgs )
					return KSPromises.resolvedPromiseP( unpack( T[ url ] or { 'error', 'unexpected get: ' .. url } ) )
				end,
				putP = function( self, url, data, optArgs )
					putData[ url ] = data
					return KSPromises.resolvedPromiseP( 'data', true )
				end,
			}
		end
	}
	
end

--==============================================================================
-- @mark setup
--==============================================================================

local kUserId = string.rep( 'a', 32 )
local kCatalogId = string.rep( 'ca7a1091', 4 )

function TestSuite.setUpClass()

	OzUserTestUtils.setUp( buildMocks() )

	OzUser = require 'OzUser'

end

--------------------------------------------------------------------------------

function TestSuite.tearDownClass()

	OzUserTestUtils.tearDown( buildMocks() )
	OzUser = nil
	
end

--------------------------------------------------------------------------------

function TestSuite.test_downloadBookmarkP( test )
	
	T = {}
	putData = {}
	local user = OzUser.make( kUserId )
	
	WFAsyncTestCase( test )
		:waitFor( function()
			return user._userInfoCache:setServerValueP( 'user', kUserId, {
				settings = {
					revel = {
						download_progress = {
							[ kCatalogId ] = 10,
						},
					}
				},
			} )
		end )
		:waitFor( function()
			return user:setDownloadBookmarkP( kCatalogId, 5, false )
		end )
		:waitFor( function()
			return user:getDownloadBookmarkP( kCatalogId )
		end )
		:continueWhenValueMatches( 'data', 10, 5 )
		:waitFor( function()
			return user:setDownloadBookmarkP( kCatalogId, 15, true )
		end )
		:waitFor( function()
			return user:getDownloadBookmarkP( kCatalogId )
		end )
		:continueWhenValueMatches( 'data', 15, 15 )
		:waitFor( function()
			return putData[ 'https://api.oznext.com/v1.0/accounts/' .. kUserId .. '/settings' ]
		end )
		:continueWhenValueMatches( 'data', [[{"revel":{"download_progress":{"]] .. kCatalogId .. [[":15}}}]] )
end

--------------------------------------------------------------------------------

function TestSuite.testPendingUploadCountP( test )
	
	T = {}
	putData = {}
	local user = OzUser.make( kUserId )
	
	WFAsyncTestCase( test )
		:waitFor( function()
			return user:addToPendingUploadCountP( kCatalogId, 2 )
		end )
		:waitFor( function()
			return user:addToPendingUploadCountP( kCatalogId, 3 )
		end )
		:waitFor( function()
			return user:getPendingUploadCountB( kCatalogId )
		end )
		:continueWhenValueMatches( 5 )
		:waitFor( function()
			return user:resetPendingUploadCountP( kCatalogId )
		end )
		:waitFor( function()
			return user:getPendingUploadCountB( kCatalogId )
		end )
		:continueWhenValueMatches( 0 )
end

--------------------------------------------------------------------------------

return TestSuite