--[[----------------------------------------------------------------------------

OzUserTestUtils.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local WFOzSyncTestUtils = require 'WFOzSyncTestUtils'

local OzCatalogTestUtils = require 'OzCatalogTestUtils'

local OzUserTestUtils = {}


--============================================================================--

function OzUserTestUtils.buildMocks()

	local docStore, indexStore
	local WFIndexStoreSchema
	return {

		WFWorkerThread = OzCatalogTestUtils.makeWorkerThreadMock(),
		OzDocStore = function( catalogId )
			docStore = WFOzSyncTestUtils.makeFakeDocStore()
			local WFIndexStoreSchema = require 'WFIndexStoreSchema'
			return WFIndexStoreSchema and docStore
		end,

		WFIndexStore = function( docStore, fname )
			-- Build a fake indexStore.  It's powered by a real sql db in memory
			-- that uses the real schema and mapping fns.
			indexStore = docStore:raw_test_buildMockIndexStore( fname )
			return indexStore
		end,

		OzUserInfoCacheIndexDefinitions = false,

		-- Fake implementation which uses OzCatalogIndexDefinitions to build out the proper
		-- schema and mapping functions for our in-memory indexStore
		WFIndexStoreSchema = function( ... )
			if not WFIndexStoreSchema then
				WFIndexStoreSchema = docStore:raw_test_buildMockWFIndexStoreSchema()( ... )
			end
			return WFIndexStoreSchema
		end,
	}

end

--------------------------------------------------------------------------------

local function loadUnload( ns, setOrClear )

	local LOADED = debug.getregistry()[ "_LOADED" ]

	for k, namespace in pairs( ns ) do
		if type( k ) == 'number' then
			loadUnload( namespace, setOrClear )
		else
			LOADED[ k ] = setOrClear and namespace
		end
	end
	
	return LOADED
	
end

--------------------------------------------------------------------------------

function OzUserTestUtils.setUp( ns )

	local LOADED = loadUnload( ns, true )

	LOADED.OzUser = nil

end

--------------------------------------------------------------------------------

function OzUserTestUtils.tearDown( ns )

	local LOADED = loadUnload( ns )

	LOADED.OzUser = nil

end

--------------------------------------------------------------------------------

return OzUserTestUtils