--[[----------------------------------------------------------------------------

$File: //wichita/tornado/releases/1.0/wichita/re_ios/src/scripts/REEventProcessorScript.lua $

$Date: 2013/04/16 $
$Author: tdonahue $

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author tdonahue 2012-06-20

------------------------------------------------------------------------------]]

local OzCatalogService = require 'OzCatalogService'

local REAppLuaInterface = require 'REAppLuaInterface'

local OzAccountService = require 'OzAccountService'

local KSCommandManager = require 'KSCommandManager'

local KSStringUtils = require 'KSStringUtils'

local KSFileUtils = require 'KSFileUtils'

local KSReactive = require 'KSReactive'

local KSData = require 'KSData'

local Bablu = require 'Bablu'

local WFWorkerThread = require 'WFWorkerThread'
local importJpegService = WFWorkerThread( "KSImportJpegServiceEPInit.lua" )

local WFRequestDispatcher = require 'WFRequestDispatcher'

local WFCloseSupport = require 'WFCloseSupport'
local WFCloseSupport_hasBeenOrWillBeClosed = assert( WFCloseSupport.hasBeenOrWillBeClosed )

local REViewDefinition = require 'REViewDefinition'
local GroupPixPin = require 'GroupPixPin'

local WFCommandManagerImp = require 'WFCommandManagerImp'

local WFArguments = require 'WFArguments'

local KSOAuth2Session = require 'KSOAuth2Session'

local OzUser = require 'OzUser'

local WFAnalyticsService = require 'WFAnalyticsService'

local KSPrefs = require 'KSPrefs'

--------------------------------------------------------------------------------

local reInstance 
local groupPixPin

local importMonitorE

local importStatus = {}

local executeCheckConnectionB = KSReactive.constantB( true )

local developDetails = {}


--------------------------------------------------------------------------------

-- This call initializes the REViewDefinition class.
REAppLuaInterface.registerInitFunction( function( )

    debug.print( "REAppLuaInterface.registerInitFunction called" )

    if not reInstance
        or WFCloseSupport_hasBeenOrWillBeClosed( reInstance )
    then 
        reInstance = REViewDefinition( )
    end
        
    if not groupPixPin
        or WFCloseSupport_hasBeenOrWillBeClosed( groupPixPin )
    then 
        groupPixPin = GroupPixPin( )
    end
        
end )


--------------------------------------------------------------------------------

local function canExecuteIfUserIsLoggedInB()

    return OzAccountService.currentUserB():closingMapB( function( ozUser ) return ozUser ~= nil end )

end

--------------------------------------------------------------------------------
-- Message entry point for WFCommandManager 
--------------------------------------------------------------------------------

WFCommandManagerImp.registerCommandHandler( KSCommandManager.executeCommand )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessageNoCancel( 'pauseImport', function()

    importJpegService:requestP( 'pauseImport' ):runUntilDone()

end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessageNoCancel( 'resumeImport', function()

    importJpegService:requestP( 'resumeImport' ):runUntilDone()

end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessageNoCancel( 'startImport', 
    function( fileOrFolderOrKSDataOrArrayOfFiles, catalogId, albumId, optFromInAppCamera, savedImagePath, xmpBase, xmpLook, orientation, croppedWidth, croppedHeight)
    
    --    WFArguments.check('string', 'string', 'string', 'string' )

    assert( reInstance and not WFCloseSupport_hasBeenOrWillBeClosed( reInstance ) )

    --[[ Save the details so that we can get the correct details in the 
        monitor callback, if not then the monitor onData will keep on getting updated
        Which will lead to wrong savedImagePath used for wrong original file Bug # 3657106
    ]]--
    
    local developInfo = {}
    developInfo['savedImagePath'] = savedImagePath
    local test = developInfo['savedImagePath']
    developInfo['xmpBase'] = xmpBase
    developInfo['xmpLook'] = xmpLook
    developInfo['orientation'] = orientation
    developInfo['croppedWidth'] = croppedWidth
    developInfo['croppedHeight'] = croppedHeight
    
    developDetails[savedImagePath] = developInfo
    
    local currentUser = OzAccountService.currentUserB():valueNow()
    local currentUserId = currentUser and currentUser:id()

    local optAddToAlbumId = albumId
    local importRequestP
    
    local result = {}
    result[ "eventId" ] = catalogId
                 
    if not importMonitorE then
        importMonitorE = importJpegService:requestE( 'getStatusCallback' )
    end
    
    local prefToSetForAssetAdd = "AssetIdSaveLocationKey"
    importMonitorE:onDataE( function( status, filePath, importCounts, randomId )
        if randomId == nil or importStatus[randomId] == nil then
            importStatus[randomId or ''] = 'done'
                
            if status == 'duplicateAddedToAlbum' then
                debug.print("status = ", status)
                debug.print("filePath = ", filePath)
            end

            local result = {}
            result['eventId'] = catalogId

            if status == 'failedLowDiskSpace' then
                debug.print("importMonitorE:onDataE")
                result['error'] = 'failedLowDiskSpace'
                REAppLuaInterface.performResultCallback( "lowDiskSpaceAlert", {} )
                REAppLuaInterface.performResultCallback( "importDone", result )
            end

            if status == 'UsageLimitExceeded' then
                debug.print('Usage limit hit')
                
                
                result['error'] =  'UsageLimitExceeded';
                REAppLuaInterface.performResultCallback( "importDone", result )
                
            end
            
            if status == 'imported' then
                debug.print('Successfully imported')
                
                REAppLuaInterface.performResultCallback( "queuedForImport", result )
                
                local assetid = KSPrefs[prefToSetForAssetAdd];
                
                if assetid ~= nil then
                    result['assetId'] = assetid;
                    -- Now get the relevant data
                    local developInfoForPath = developDetails[randomId]
                    local savedImagePath = developInfoForPath['savedImagePath']
                    local xmpBase = developInfoForPath['xmpBase']
                    local xmpLook = developInfoForPath['xmpLook']
                    local orientation = developInfoForPath['orientation']
                    local croppedWidth = developInfoForPath['croppedWidth']
                    local croppedHeight = developInfoForPath['croppedHeight']
                    debug.print('Start development')
                    local developSessionP = reInstance:developP(assetid, catalogId, savedImagePath, xmpBase, xmpLook, orientation, croppedWidth, croppedHeight)
                 end
            end
            
            if status == 'failed' then
                result['error'] = 'Failed'
                REAppLuaInterface.performResultCallback( "importDone", result )
            end

        end
        
        
    end )

    if type( fileOrFolderOrKSDataOrArrayOfFiles ) == 'table' then
    
        local firstEntry = fileOrFolderOrKSDataOrArrayOfFiles[ 1 ] 
        
        if firstEntry then
            -- If the first entry is an assets library URL, assume the rest are
            if string.find( firstEntry, "assets-library://", nil, true ) then
                importRequestP = importJpegService:requestP( 'importAssets', catalogId, currentUserId,
                    fileOrFolderOrKSDataOrArrayOfFiles, nil, optAddToAlbumId )
            else
                
                importRequestP = importJpegService:requestP( 'importFilesAndFolders', catalogId, currentUserId,
                    fileOrFolderOrKSDataOrArrayOfFiles, optAddToAlbumId )

            end
            
        else
            debug.print( "Empty list of files, not starting import" )
            return
        end
    
    elseif KSData.isData( fileOrFolderOrKSDataOrArrayOfFiles ) then
        
        importRequestP = importJpegService:requestP( 'importData', catalogId, currentUserId, fileOrFolderOrKSDataOrArrayOfFiles, nil, nil, optFromInAppCamera, optAddToAlbumId )
        
        importRequestP:whenResolvedDo( function()
            fileOrFolderOrKSDataOrArrayOfFiles:close()
        end ):runUntilDone()
            
    elseif string.find( fileOrFolderOrKSDataOrArrayOfFiles, "assets-library://", nil, true ) then

        -- FIXME [erwright 2011-06-07]: since we're using a custom image picker (ELCImagePicker), we should be
        -- able to tweak the code to pass along the group the image came from, so we don't have to fabricate
        -- the group name here.

        importRequestP = importJpegService:requestP( 'importAsset', catalogId, currentUserId,
            fileOrFolderOrKSDataOrArrayOfFiles, nil, optAddToAlbumId )
            
    else

        local pathExists = KSFileUtils.exists( fileOrFolderOrKSDataOrArrayOfFiles )
    
        if not pathExists then
        
            debug.print( "Import couldn't start - path doesn't exist: " .. fileOrFolderOrKSDataOrArrayOfFiles )
            result[ "error" ] = "path does not exist"
            REAppLuaInterface.performResultCallback( "importDone", result )
            return
            
        end
        
        if pathExists == 'file' then
        
            importRequestP = importJpegService:requestP( 'importFile', catalogId, currentUserId,
                fileOrFolderOrKSDataOrArrayOfFiles, optAddToAlbumId , prefToSetForAssetAdd, savedImagePath)
    
        else

            importRequestP = importJpegService:requestP( 'importFolder', catalogId, currentUserId, 
                fileOrFolderOrKSDataOrArrayOfFiles, optAddToAlbumId )
            
        end

    end
   
    importRequestP:whenResolvedDo( 

        function( status, data )
   
            if status == 'data' then
                debug.print( 'imported photo = ', data )
               -- reInstance:canPostComments( data )
            elseif status == 'error' then
                debug.print(data)
                result[ "error" ] = data
                if data == 'LowDiskSpace' then
                    REAppLuaInterface.performResultCallback( "lowDiskSpaceAlert", {} )
                end
            end
            
        end 
        
    ):runUntilDone()

end )


--------------------------------------------------------------------------------
-- Global command registration
--------------------------------------------------------------------------------


KSCommandManager.registerCommand( 'setWantBigThumbs',
    
    function( wantBigThumbs )
        WFArguments.check( 'boolean' )
        reInstance:setWantsBigThumbs( wantBigThumbs )
    end,
    
    -- canExecture behavior
    nil

)

KSCommandManager.registerCommand( 'logInSucceeded',

-- execute function
function( accessToken, refreshToken )

    debug.print('Login succeeded')
    local loginResultP = OzAccountService.storeNewImsCredentialsP( accessToken, refreshToken )

    loginResultP:whenResolvedDo( function( status, message )
        if status == 'error' then
            debug.print( "ERROR storing new IMS credentials userLoginFailed" )
            REAppLuaInterface.performResultCallback( "userLoginFailed", {} )
        end
    end):runUntilDone()

end,

-- canExecute behavior
nil
)


--------------------------------------------------------------------------------
    
KSCommandManager.registerCommand( 'simulateForcedLogout', 

    -- execute function
    function()

        local ozUser = OzAccountService.currentUserB():valueNow()
        ozUser:simulateForcedLogout()

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
    
KSCommandManager.registerCommand( 'logout', 

    -- execute function
    function()

        local ozUser = OzAccountService.currentUserB():valueNow()

         reInstance:logout( ozUser )
        --WFAnalyticsService.send( "Revel Event: Log Out", "Application" )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
    
KSCommandManager.registerCommand( 'reIsUserLoggedIn', 

    -- execute function
    function( )
        reInstance:checkLogin()
    end,
    
    -- canExecute behavior
    executeCheckConnectionB
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reSharing',

    -- execute function

    function( isSharing )

        WFArguments.check('boolean' )

        reInstance:setSharingB( isSharing )

    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reMonitorAsset',

    -- execute function

    function( eventId, assetId )

        WFArguments.check('string', 'string' )
        
        local ozUser = OzAccountService.currentUserB():valueNow()
        
        reInstance:monitorAsset( ozUser, eventId, assetId )

    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)
--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reSelectedEvent',

    -- execute function

    function( eventId )

        -- null eventId is valid, it clears the current event.

        local ozUser = OzAccountService.currentUserB():valueNow()
        reInstance:setCurrentCatalog( ozUser, eventId )

    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

------------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reGetAlbumCover',

    -- execute function

    function( albumId )

        WFArguments.check( 'string' )

        reInstance:getAlbumCover(albumId)

    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reSelectedAlbum',

    -- execute function

    function( albumId )

        WFArguments.check('string' )

        reInstance:getUserAlbumColections( albumId )

    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
    
KSCommandManager.registerCommand( 'reMonitorCatalog', 

    -- execute function
    function( currentCatalogID, sortOrder )
        local ozUser = OzAccountService.currentUserB():valueNow()
        reInstance:monitorCatalog( ozUser, currentCatalogID, sortOrder )
    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)


--------------------------------------------------------------------------------
    
KSCommandManager.registerCommand( 'reChangeCatalogPollInterval', 

    -- execute function
    function( numSeconds )
        local ozUser = OzAccountService.currentUserB():valueNow()
        reInstance:changeCatalogPollInterval( ozUser, numSeconds )
    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)


--------------------------------------------------------------------------------
    
KSCommandManager.registerCommand( 'reSetSyncHalt', 

    -- execute function
    function( halt )
        reInstance:setSyncHalt( halt )
    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)


--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reSetLiked',

    -- execute function
    function( eventId, photoId, liked )

        assert( eventId )
        assert( photoId )
        reInstance:setLiked( OzAccountService.currentUserB():valueNow(), eventId, photoId, liked )

    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reLuaGarbageCollect',

    -- execute function
    function( )
        reInstance:garbageCollect( )
    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)


--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reClearFavoritesBehaviors',

    -- execute function
    function(  )
        reInstance:clearFavoriteBehaviors(  )
    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reMemoryWarning',

    -- execute function
    function( warning )
        reInstance:memoryWarning( warning )
    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reCloseThumbnailBehavior',
            
    -- execute function
    function( photoId )
        assert( photoId )
        reInstance:closeThumbnailBehavior( photoId )
    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reGetImage',

    -- execute function
    function( eventId, photoId, renditionName ) -- renditionName: 'preview', 'thumbnail' or 'thumbnail2x'
        reInstance:getImage( OzAccountService.currentUserB():valueNow(), eventId, photoId, renditionName )
    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reCancelImage',

    -- execute function
    function( photoId, preview )
        reInstance:cancelImage( photoId, preview and 'preview' or 'thumbnail2x' )
    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reGetThumbnail',

    -- execute function
    function( eventId, albumId, photoId, thumbnailMD5 )
       -- debug.print("Request thumbnails 3")
        reInstance:getThumbnail( OzAccountService.currentUserB():valueNow(), eventId, albumId, photoId, thumbnailMD5 )
    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reGetPreview',

    -- execute function
    function( eventId, photoId )
        reInstance:getPreview( OzAccountService.currentUserB():valueNow(), eventId, photoId )
    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reGetOriginal',

    -- execute function
    function( eventId, photoId )
        reInstance:getOriginal( OzAccountService.currentUserB():valueNow(), eventId, photoId )
    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reCancelOriginal',

    -- execute function
    function( photoId )
        reInstance:cancelOriginal( photoId )
    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'getDefaultCatalogId',

    -- execute function
    function()
        return KSReactive.constantB( nil )
    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reUpdateOmnitureSettingOnStartUp',
    -- execute function
    function( setting )
        reInstance:updateOmnitureSettingOnStartUp( setting )
    end,

    nil
)

--------------------------------------------------------------------------------
KSCommandManager.registerCommand( 'reUpdateOmnitureSetting',

    -- execute function
    function( setting )
        local ozUser = OzAccountService.currentUserB():valueNow()
        reInstance:updateOmnitureSetting( ozUser, setting )
    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)
    

--------------------------------------------------------------------------------
KSCommandManager.registerCommand( 'reSetupTestAccount',
  
     function()

        local ozUserId = '94f0fadf1632b447dbc187f2a96e6410'
        local userLoggedInB = KSReactive.variableB( false )
        local oAuthSession = KSOAuth2Session( ozUserId )
        local ozUser = OzUser.make( ozUserId  )
        
        local postUrl = 'https://ims-na1-qa1.adobelogin.com/ims/login/code'
        local postParams = 'client_id=RevelMacTest1&redirect_uri=air-auth://carousel&locale=en_US&scope=openid,AdobeID&client_secret=12d5de23-e638-41f5-815e-04a10a5e13e1&username=rseeler%2B111@adobetest.com&password=adobetest'  
    
        local storeTokenP = ozUser:getAccessTokenRefreshTokenFromImsP( postUrl, postParams )
        :mapDataP( function( status, refresh_token, access_token )
    
            return oAuthSession:storeTokensP( refresh_token, access_token )
            
        end )
        
        storeTokenP:whenResolvedDo( function( status )
    
            oAuthSession:close()
            
            if status == 'data' then
                userLoggedInB:set( true )

                OzAccountService.setCurrentUser( ozUserId  )

                local result = {}
                result[ "ozUserId" ] = ozUserId
                REAppLuaInterface.performResultCallback( "accountSetupDone", result )
                
            end

        end ):runUntilDone()

    end,

   nil
)

local newEventB
--------------------------------------------------------------------------------
-- @mark reCreateEvent
KSCommandManager.registerCommand( 'reCreateEvent',

    -- execute function
    function( eventName, resultCallback, startDate, endDate, broadcast, broadcastType, broadcastAutoAcceptJoin, optEmailInviteList )
        debug.print("reCreateEvent call")

        local ozUser = OzAccountService.currentUserB():valueNow()
        
        newEventB = reInstance:newEventB()

        debug.print("OzAccountService.currentUserB() call")
        local emailAddresses = nil

        if ( optEmailInviteList and (string.len( optEmailInviteList ) > 0 ) ) then
            emailAddresses = KSStringUtils.split(optEmailInviteList, ",")
        end
        
        local catalogData = {   
                name = eventName, 
                subtype = 'grouppix', 
                payload = {
                    [ 'start_date' ] = startDate,
                    [ 'end_date' ] = endDate,
                    [ 'X-Event-Broadcast' ] = broadcast,
                    [ 'X-Event-BroadcastType' ] = broadcastType,
                    [ 'X-Event-BroadcastAutoJoin' ] = broadcastAutoAcceptJoin
                },
                users = emailAddresses
            }
                            
        local resultP = ozUser:createCatalogP( catalogData )
            :mapDataP( function(  status, location, headers, eventId, inviteUsersSucceeded )
                
                local catalog = OzCatalogService( eventId, ozUser)
                catalog:setSyncEnabled( true )
                
                local albumId = false
                
                return catalog:createAlbumP(eventName, true )
                    :mapDataP( function( status, data )
                                            
                        albumId = data
                        -- set the default album id in the catalog metadata
                        reInstance:setDefaultAlbum( ozUser, catalog, eventId, albumId )
                        
                        newEventB:onDataB( function( shouldFire )
                             if shouldFire then
                                local result = {}
                                result[ "eventName" ] = eventName
                                result[ "eventId" ] = eventId
                                result[ "defaultAlbumId" ] = albumId
                                if ( optEmailInviteList and (string.len( optEmailInviteList ) > 0 ) ) then
                                    result[ "invitees" ] = optEmailInviteList
                                end
                                REAppLuaInterface.performResultCallback( "updateDefaultAlbum", result )
                                

                                reInstance:closeNewEventB()
                                newEventB = nil

                                return 'data', albumId
                            end
                        end )
                    end )
            end)
        
        resultP:whenResolvedDo( function ( status, data ) 
            if status == 'error' then
            
                -- send an error response back for the default album. yes. weird.
                local result = {}
                result[ "eventName" ] = eventName
                result[ "error" ] = data
                REAppLuaInterface.performResultCallback( "updateDefaultAlbum", result )
                

            end
        end 
        
        ):runUntilDone()
        
        --WFAnalyticsService.send( "Revel Event: Create Event ", "Organizing" )
            
        end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reSetDefaultAlbum
-- This command deletes the event or removes the user from the event, depending on
-- who owns the event.
KSCommandManager.registerCommand( 'reSetDefaultAlbum', 

    -- execute function
    function( eventId, albumId )
        local ozUser = OzAccountService.currentUserB():valueNow()
        reInstance:setDefaultAlbum( ozUser, eventId, albumId )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reRemoveEvent
-- This command deletes the event or removes the user from the event, depending on
-- who owns the event.
KSCommandManager.registerCommand( 'reRemoveEvent', 

    -- execute function
    function( eventId, pinCode)
        local ozUser = OzAccountService.currentUserB():valueNow()

        reInstance:removeEvent( ozUser, eventId, pinCode )
        
        --WFAnalyticsService.send( "Revel Event: Remove Event ", "Organizing" )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reCloseCurrentAlbum
-- This command closes the current album.
KSCommandManager.registerCommand( 'reCloseCurrentAlbum', 

    -- execute function
    function(  )
        reInstance:closeCurrentCatalog( )
        --WFAnalyticsService.send( "Revel Event: Remove Event ", "Organizing" )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reClearEventCache
KSCommandManager.registerCommand( 'reClearEventCache', 

    -- execute function
    function( eventId )
        Bablu:purgeAllEligible( eventId )
    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)


--------------------------------------------------------------------------------
-- @mark reRenameEvent
KSCommandManager.registerCommand( 'reRenameEvent', 

    -- execute function
    function( eventId, newEventName, defaultAlbumId )
    
        local ozUser = OzAccountService.currentUserB():valueNow()
        
        reInstance:renameEvent( ozUser, eventId, newEventName, defaultAlbumId )
        
        --WFAnalyticsService.send( "Revel Event: Rename Event ", "Organizing" )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reAddEventInvitees
KSCommandManager.registerCommand( 'reAddEventInvitees', 

    -- execute function
    function( eventId, emailInviteList, sendInvite )

        local ozUser = OzAccountService.currentUserB():valueNow()
        
        reInstance:addEventInvitees( ozUser, eventId, emailInviteList, sendInvite )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reRemoveEventInvitee
KSCommandManager.registerCommand( 'reRemoveEventInvitee', 

    -- execute function
    function( eventId, emailOfUserToRemove )

        local ozUser = OzAccountService.currentUserB():valueNow()
        
        reInstance:removeEventInvitee( ozUser, eventId, emailOfUserToRemove )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reUpdateEventDates
KSCommandManager.registerCommand( 'reUpdateEventDates', 

    -- execute function
    function( eventId, startDate, endDate ) 

        local ozUser = OzAccountService.currentUserB():valueNow()

        reInstance:updateEventDates( ozUser, eventId, startDate, endDate )
        
      --  WFAnalyticsService.send( "Revel Event: Update Event Dates ", "Organizing" )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reUpdateBroadcastParams
KSCommandManager.registerCommand( 'reUpdateBroadcastParams', 

    -- execute function
    function( eventId, broadcast, broadcastType, broadcastAutoJoin )    

        local ozUser = OzAccountService.currentUserB():valueNow()

        reInstance:updateBroadcastParams( ozUser, eventId, broadcast, broadcastType, broadcastAutoJoin )
        
    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reAcceptEventInvite
KSCommandManager.registerCommand( 'reAcceptEventInvite', 

    -- execute function
    function( eventId )

        local ozUser = OzAccountService.currentUserB():valueNow()
        ozUser:acceptCatalogInviteP( eventId ):mapDataP( function ( status, data )
            ozUser:requestSharedCatalogReload()
        end )
        :runUntilDone()

      --  WFAnalyticsService.send( "Revel Event: Accept Event Invite ", "Organizing" )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reCreatePinCode
KSCommandManager.registerCommand( 'reCreatePinCode', 

    -- execute function
    function( eventId )

        local ozUser = OzAccountService.currentUserB():valueNow()
        
        reInstance:setCurrentCatalog( ozUser, eventId )
        local catalog = reInstance._currentCatalog
        
        groupPixPin:createPinForCatalog( ozUser, catalog )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reGetPinCode
KSCommandManager.registerCommand( 'reGetPinCode', 

    -- execute function
    function( pinCode )

        local ozUser = OzAccountService.currentUserB():valueNow()
        
        groupPixPin:getPin( ozUser, pinCode )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reAddUserByPinCode
KSCommandManager.registerCommand( 'reAddUserByPinCode', 

    -- execute function
    function( pinCode, userId )

        local ozUser = OzAccountService.currentUserB():valueNow()
                
        groupPixPin:addUserByPin( ozUser, pinCode, userId )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reChangePinState
KSCommandManager.registerCommand( 'reChangePinState', 

    -- execute function
    function( eventId, pinCode, pinState )

        local ozUser = OzAccountService.currentUserB():valueNow()
        reInstance:setCurrentCatalog( ozUser, eventId )
        local catalog = reInstance._currentCatalog

        groupPixPin:changePinState( ozUser, catalog, pinCode, pinState )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reDeletePinCode
KSCommandManager.registerCommand( 'reDeletePinCode', 

    -- execute function
    function( eventId, pinCode )

        local ozUser = OzAccountService.currentUserB():valueNow()
        
        reInstance:setCurrentCatalog( ozUser, eventId )
        local catalog = reInstance._currentCatalog

        groupPixPin:deletePinForCatalog( ozUser, catalog, pinCode )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reIsSharedAlbum
-- This command sets album to be shared. Return values come back in updateSharedAlbum
KSCommandManager.registerCommand( 'reIsSharedAlbum', 

    -- execute function
    function( eventId, albumId, checkServer )
        local ozUser = OzAccountService.currentUserB():valueNow()
        reInstance:isSharedAlbum( ozUser, eventId, albumId, checkServer )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reShareAlbum
-- This command sets album to be shared. Return values come back in updateSharedAlbum
KSCommandManager.registerCommand( 'reShareAlbum', 

    -- execute function
    function( eventId, albumId )
        local ozUser = OzAccountService.currentUserB():valueNow()
        reInstance:shareAlbum( ozUser, eventId, albumId )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reUnShareAlbum
-- This command sets album to NOT be shared. Return values come back in updateSharedAlbum
KSCommandManager.registerCommand( 'reUnShareAlbum', 

    -- execute function
    function( eventId, albumId )
        local ozUser = OzAccountService.currentUserB():valueNow()
        reInstance:unShareAlbum( ozUser, eventId, albumId )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reSharedAlbumAllowDownloads
-- This command sets whether or not downloads are allowed from the shared album
KSCommandManager.registerCommand( 'reSharedAlbumAllowDownloads', 

    -- execute function
    function( eventId, albumId, allowsDownloads )
        local ozUser = OzAccountService.currentUserB():valueNow()
        reInstance:sharedAlbumAllowDownloads( ozUser, eventId, albumId, allowsDownloads )

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'recordAutoImportForCameraRollExport',
            -- execute function
    function( assetUrl, sha256 )
        reInstance:recordAutoImportForCameraRollExport( assetUrl, sha256 )      
    end
)

--------------------------------------------------------------------------------
-- @mark reStartImport
KSCommandManager.registerCommand( 'reStartImport',

    -- execute function
    function( fileOrKSData, catalogId, albumId, savedImagePath, xmpBase, xmpLook, orientation, croppedWidth, croppedHeight )
 
        WFWorkerThread( ':self:' ):requestP( 'startImport', fileOrKSData, catalogId, albumId, nil, savedImagePath, xmpBase, xmpLook, orientation, croppedWidth, croppedHeight)
            :runUntilDone()

        -- TODO: Add call to 'sendAnalyticsEvent' command.

    end,
    
    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'createRevision',
    function(catalogId, photoId, savedImagePath, xmpBase, xmpLook, orientation, croppedWidth, croppedHeight)
            local developSessionP = reInstance:developP(photoId, catalogId, savedImagePath, xmpBase, xmpLook, orientation, croppedWidth, croppedHeight)

            developSessionP:whenResolvedDo(
                function(status, message)
                    local developResult = {}
                    developResult["eventId"] = catalogId
                    developResult['assetId'] = photoId
                    if status == 'error' then
                        developResult["error"] = "Asset revision creation failed"
                    end
                    developResult["update"] = true;
                    REAppLuaInterface.performResultCallback( "importDone", developResult )

                end
                ):runUntilDone()

        end,

        canExecuteIfUserIsLoggedInB()
    )

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'rePauseImport',

    -- execute function
    function( )

        WFWorkerThread( ':self:' ):requestP( 'pauseImport' ):runUntilDone()

    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reResumeImport',

    -- execute function
    function( )

        WFWorkerThread( ':self:' ):requestP( 'resumeImport' ):runUntilDone()

    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reDeleteAsset
KSCommandManager.registerCommand( 'reDeleteAsset',

    -- execute function
    function( currentCatalogID, assetID  )
        reInstance:deleteAsset( OzAccountService.currentUserB():valueNow(), currentCatalogID, assetID )

      --  WFAnalyticsService.send( "Revel Event: Delete Photo", "Organizing" )

    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

-- @mark reDeleteAsset
KSCommandManager.registerCommand( 'reDeleteAssets',

    -- execute function
    function( currentCatalogID, assetIDs  )
        reInstance:deleteAssets( OzAccountService.currentUserB():valueNow(), currentCatalogID, assetIDs )

      --  WFAnalyticsService.send( "Revel Event: Delete Photo", "Organizing" )

    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reReportAbuse',

    -- execute function
    function( currentCatalogID, assetID  )
        reInstance:reportAbuse( OzAccountService.currentUserB():valueNow(), currentCatalogID, assetID )

     --  WFAnalyticsService.send( "Revel Event: Report Abuse", "Organizing" )

    end,

    -- canExecute behavior
   canExecuteIfUserIsLoggedInB()

)

--------------------------------------------------------------------------------
-- @mark reChangeSort
KSCommandManager.registerCommand( 'reChangeSort',

    -- execute function
    function( sortOrder, userID  )
        reInstance:changeSortOrder( sortOrder, userID  )
    end,

    -- canExecute behavior
    canExecuteIfUserIsLoggedInB()

)

--------------------------------------------------------------------------------
-- @mark reDeleteCommentForAssetId
KSCommandManager.registerCommand( 'reDeleteCommentForAssetId',
        
    function( catalogId, albumId, assetId, commentId )
         -- required arguments
        assert( catalogId )
        assert( commentId )
        -- assetId and albumId are not used.
        
        reInstance:deleteComment(  OzAccountService.currentUserB():valueNow(), catalogId, commentId  )

     --   WFAnalyticsService.send( "Revel Event: Delete Comment ", "Organizing" )


      end,
                

        -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reDeleteAllCommentForAssetId
KSCommandManager.registerCommand( 'reDeleteAllCommentForAssetId',
       
     function( catalogId, assetId )

        -- required arguments
        assert( catalogId )
        assert( assetId )
            reInstance:deleteAllComment( OzAccountService.currentUserB():valueNow(), catalogId, assetId )

         --   WFAnalyticsService.send( "Revel Event: Delete All Comments ", "Organizing" )

        end,

        -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reAddCommentForAssetId
KSCommandManager.registerCommand( 'reAddCommentForAssetId',
        
    function( catalogId, assetId, comment )
    
        -- required arguments
        assert( catalogId )
    
        assert( assetId )
        assert( comment )
        reInstance:addComment( OzAccountService.currentUserB():valueNow(), catalogId,  assetId, comment )
    
         --   WFAnalyticsService.send( "Revel Event: Add Comment ", "Organizing" )
    
        end,
        
        -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reGetCommentListForAssetId
KSCommandManager.registerCommand( 'reGetCommentListForAssetId',
    function( catalogId, albumId, assetId )
        -- required arguments
        assert( catalogId )
        assert( assetId )
        -- albumId is not used
        
        reInstance:getCommentList( OzAccountService.currentUserB():valueNow(), catalogId, assetId )
    
    -- analytics
    -- WFAnalyticsService.send( "Actions: Get Caption for Asset ID", "Editing" );
    
    end,
    
        -- canExecute behavior
    canExecuteIfUserIsLoggedInB() 
)

--------------------------------------------------------------------------------

-- @mark reGetCommentListForAssetId
KSCommandManager.registerCommand( 'reCloseCommentListForAssetId',
    function( catalogId, albumId, assetId )
        -- required arguments
        assert( catalogId )
        assert( assetId )
        -- albumId is not used.
        
        reInstance:closeCommentList( OzAccountService.currentUserB():valueNow(), catalogId, assetId )
    
    -- analytics
    -- WFAnalyticsService.send( "Actions: Get Caption for Asset ID", "Editing" );
    
    end,
    
        -- canExecute behavior
    canExecuteIfUserIsLoggedInB() 
)

--------------------------------------------------------------------------------
-- @mark reIsMyFavoriteForAssetId
KSCommandManager.registerCommand( 'reIsMyFavoriteForAssetId',
    function( catalogId, assetId )
    
        -- required arguments
        assert( catalogId )
        assert( assetId )
        
        reInstance:getMyFavoriteForAssetId( OzAccountService.currentUserB():valueNow(), catalogId, assetId )
    
    end,
    
        -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reAllFavoritesForAssetId
KSCommandManager.registerCommand( 'reAllFavoritesForAssetId',
    function( catalogId, assetId )
    
        -- required arguments
        assert( catalogId )
        assert( assetId )
        
        reInstance:getAllFavoritesForAssetId( OzAccountService.currentUserB():valueNow(), catalogId, assetId )
    
    end,
    
        -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------

KSCommandManager.registerCommand( 'reCloseFavoritesForAssetId',
    function( catalogId, assetId )
    
        -- required arguments
        assert( catalogId )
        assert( assetId )
        
        reInstance:closeFavoritesForAssetId( OzAccountService.currentUserB():valueNow(), catalogId, assetId )
    
    end,
    
        -- canExecute behavior
    canExecuteIfUserIsLoggedInB()
)

--------------------------------------------------------------------------------
-- @mark reHostIsReachableBehavior
KSCommandManager.registerCommand( 'reHostIsReachableBehavior',
            
    -- execute function
    function(  )
        debug.print("reHostIsReachableBehavior called")
        reInstance:hostIsReachable()
    end,
    
    -- canExecute behavior
    executeCheckConnectionB
)

--------------------------------------------------------------------------------
-- @mark reGenerateDiagnosticReport
KSCommandManager.registerCommand( 'reGenerateDiagnosticReport',
            
    -- execute function
    function(  )
        reInstance:generateDiagnosticReport()
    end,
    
    -- canExecute behavior
    executeCheckConnectionB
)

--------------------------------------------------------------------------------
-- @mark    sendAnalyticsEvent command

-- A wrapper around WFAnalyticsService.send for use by native code.

KSCommandManager.registerCommand( 'sendAnalyticsEvent',

    -- execute function
    WFAnalyticsService.sendFromNative

    -- canExecute behavior: always available

)

--------------------------------------------------------------------------------
-- @mark reGetIMSAuthInfo
KSCommandManager.registerCommand( 'reGetIMSAuthInfo',

    -- execute function
    function(  )
        reInstance:getImsAuthInfo()
    end,

    -- canExecute behavior
    executeCheckConnectionB
)

--------------------------------------------------------------------------------
-- @mark reRegisterDeviceToken
KSCommandManager.registerCommand( 'reRegisterDeviceToken',

    -- execute function
    function( token )
        reInstance:registerDeviceToken( token )
    end

    -- canExecute behavior: always available
)

--------------------------------------------------------------------------------
-- @mark reDeleteDeviceToken
KSCommandManager.registerCommand( 'reDeleteDeviceToken',

    -- execute function
    function( eventId )

        reInstance:deleteDeviceToken( OzAccountService.currentUserB():valueNow() )

    end

    -- canExecute behavior
)


