--[[----------------------------------------------------------------------------

$File: //wichita/tornado/releases/1.0/wichita/re_ios/src/scripts/REEventProcessorScriptTest.lua $

$Date: 2013/04/09 $
$Author: tdonahue $

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author rseeler 2012-08-14

------------------------------------------------------------------------------]]

local KSCommandManager = require 'KSCommandManager'

local KSOAuth2Session = require 'KSOAuth2Session'

local OzAccountService = require 'OzAccountService'

local WFAsyncTestCase = require 'WFAsyncTestCase'

local OzUser = require 'OzUser'

local KSReactive = require 'KSReactive'

local KSTestUtils = require 'KSTestUtils'
local testFails = KSTestUtils.testFails
local assertFails = KSTestUtils.assertFails

local WFPlatformInfo = require 'WFPlatformInfo'

local T = { }

local ozUserId = '94f0fadf1632b447dbc187f2a96e6410'
	-- user id for rseeler+111@adobetest.com on oznext.

local TEST_WAIT_TIME = 20

local ozUser = nil
local testEventId = nil		-- When creating a new event, put it's ID here.  It will be auto-deleted when the test finishes.
local userLoggedInB = false	-- Don't let your test proceed until this value is true

--------------------------------------------------------------------------------

function T.setUpClass()

	-- Log in the test user account...
	-- See OzCatalogServiceTest.lua for other setup routines, if needed.

	userLoggedInB = KSReactive.variableB( false )
	local oAuthSession = KSOAuth2Session( ozUserId )
	ozUser = OzUser.make( ozUserId )
	
	local postUrl = 'https://ims-na1-qa1.adobelogin.com/ims/login/code'
	local postParams = 'client_id=RevelMacTest1&redirect_uri=air-auth://carousel&locale=en_US&scope=openid,AdobeID&client_secret=12d5de23-e638-41f5-815e-04a10a5e13e1&username=rseeler%2B111@adobetest.com&password=adobetest'	

	local storeTokenP = ozUser:getAccessTokenRefreshTokenFromImsP( postUrl, postParams )
	:mapDataP( function( status, refresh_token, access_token )

		return oAuthSession:storeTokensP( refresh_token, access_token )
		
	end )
	
	storeTokenP:whenResolvedDo( function( status )

		oAuthSession:close()
		
		if status == 'data' then
			userLoggedInB:set( true )
		end
		
	end ):runUntilDone()

	OzAccountService.setCurrentUser( ozUserId )
	
end

--------------------------------------------------------------------------------

function T.tearDownClass()

	-- Be sure to remove any created Events when done...
	if ozUser and testEventId then
		ozUser:deleteCatalogP( testEventId ):runUntilDone()
	end

	ozUser:close()
	ozUser = nil
	
end

--------------------------------------------------------------------------------

-- TODO: The '_testNightly' suffix keeps this test from being run in the pre-flight tests.
-- Currently, this test causes problems when run in "Wichita Self Tests" app, but works ok in
-- "Tornado Self Tests".  Once the foundation team comes up with a way to exclude certain directories of
-- lua files from the test runner, we can remove the '_testNightly' and have
-- this test run in Tornado's self tests application.  See Guy for status on this.

function T.testCreateEvent_testNightly( test )

	if WFPlatformInfo.isWinRT then
		return
	end
	
	local REEventProcessorScript = require 'REEventProcessorScript'

	local createEventCommandCompletedB = KSReactive.variableB( false )

	WFAsyncTestCase( test, TEST_WAIT_TIME )

		:waitFor( userLoggedInB )
		
		:continueWhenValueMatches( true )

		:waitFor( function()

			KSCommandManager.executeCommand( 'reCreateEvent', "TestEventName", function( properties )

				testEventId = properties.eventId
				createEventCommandCompletedB:set( testEventId ~= nil )
				assert( testEventId ~= nil, "Command:reCreateEvent failed to generate Event." )
				
			end, 0, 0 )
			
			return createEventCommandCompletedB
		end )
		
		:continueWhenValueMatches( true )

end

--------------------------------------------------------------------------------

if WFPlatformInfo.isMac then 
	return T
else 
	return T and {}
end 
