--[[----------------------------------------------------------------------------

$File: //wichita/tornado/releases/1.0/wichita/re_ios/src/scripts/REViewDefinition.lua $

$Date: 2013/04/10 $
$Author: mscarafo $

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author tdonahue 2012-08-02

------------------------------------------------------------------------------]]


local OzCatalogService = require 'OzCatalogService'

local OzAccountService = require 'OzAccountService'

local WFPlatformInfo = require 'WFPlatformInfo'

local KSReactive = require 'KSReactive'

local KSClass = require 'KSClass'

local KSStringUtils = require 'KSStringUtils'

local REAppLuaInterface = require 'REAppLuaInterface'

local WFReachability = require 'WFReachability'

local OzCatalogUrls = require 'OzCatalogUrls'

local WFAnalyticsService = require 'WFAnalyticsService'

local KSPrefs = require 'KSPrefs'

local Bablu = require 'Bablu'

local KSJsonEncoder = require 'KSJsonEncoder'

local WFArguments = require "WFArguments"

local WFOzRenditionServer = require 'WFOzRenditionServer'

local KSLogging = require 'KSLogging'

local WFCloseSupport = require 'WFCloseSupport'

local KSDate = require 'KSDate'

local WFWorkerThread = require 'WFWorkerThread'

local KSJsonDecoder = require 'KSJsonDecoder'

local GroupPixPin = require 'GroupPixPin'

local importJpegService = WFWorkerThread( "KSImportJpegServiceEPInit.lua" )

local reScriptLog = KSLogging.log 'REScript'
-- :setLevel( KSLogging.DEBUG )

local reThumbLog = KSLogging.log 'REThumbs'
-- :setLevel( KSLogging.DEBUG )

local OzDevelopSession = require 'OzDevelopSession'

local KSImportJpegService = require 'KSImportJpegService'
local KSPromises = require 'KSPromises'

local KSTableUtils = require 'KSTableUtils'
local WFOzContentTypes = require 'WFOzContentTypes'

--------------------------------------------------------------------------------

local wantBigThumbnails = WFPlatformInfo.isHighDPIDevice
local REScriptArg = 'REViewDefinition'
local groupPixPin = GroupPixPin( )

local derefOpt = assert( KSTableUtils.derefOpt )

--------------------------------------------------------------------------------

local methods = {
	_monitorUserInfoB = false,
	_initialSyncB = false,
	_statB = false,
	_monitorUserAuthStatusB = false,
	_monitorUserCatalogsB = false,
	_monitorUserAlbumsB = false,
	_monitorCatalogsSharedWithUserB = false,
	_favoriteBehaviors = {},
	_allFavoritesBehaviours = {},
    _thumbnailBehaviors = {},
    _imageBehaviors = {},
    _originalImageBehaviors = {},
    _getCommentListBehaviors = {},
    _getAlbumCoverBehaviors = {},
	_currentCatalog = false,
	_currentRenditionServer = false,
	_currentCatalogId = false,
	_currentCollectionB = false,
	_currentCatalogSyncStateB = false,
    _pipAlertShown = false,
    _dataSource = false,
    _newEventB = false,
    _ozIsReachableB = false, 
	_catalogUserLimitB = false,
    _memoryWarning = false,
	_forceUpgradeB = false,
	_assetCommentCountB = false,
	_assetLikedCountB = false,
	_collaboratorPhotoCountB = false,
	_assetTotalCountB = false,
	_assetChangedB = false,
	_filterByUserId = false,
	_useUserIdSubFiltering = not WFPlatformInfo.isAndroid,
	_idleTimerDisabledB = false,
	_catalogAccessRequestB = false,
	_imsAuthUrlFailed = false,
	_failedUserListCatalog = false,
	_sharingB = false,
	_serviceAvailableB = false,
	_serviceAvailableOnStartupB = false,
	_defaultEventB = false,
	_deleteEventFlag = false,
	_currentAlbumCollectionB = false,
	_deviceToken = false,
	_prevOriginalLeft = 0,

}


--------------------------------------------------------------------------------
-- Give the device token to WF to give to Oz.
-- We received the device token from the OS.
--------------------------------------------------------------------------------
local function registerDeviceToken( --[[OzUser]] ozUser, --[[string]] deviceToken )

    local resultP = ozUser:registerDeviceTokenP( WFPlatformInfo.getAppIdentifier(), 'production', deviceToken )

    resultP:whenResolvedDo( function( status, data )

        local result = {}

        if status == 'error' then
            result["result"] = "0"
        else
            result["result"] = "1"
        end

        REAppLuaInterface.performResultCallback( "deviceTokenRegistration", result )

    end )
    :runUntilDone()

end

--------------------------------------------------------------------------------

function methods:__init( nativeViewController )
	
	--debug.print( "REViewDefinition init" )
	self._defaultEventB = KSReactive.variableB( false )
    self:hostIsReachable()
	self:watchCurrentUser()	
	self._sharingB = KSReactive.variableB( false )

end

----------------

local function analysis_finish( sourceKSData, success, ... )

	-- If we have data that came through, it will have been reconstructed as
	-- a new KSData object by the analysis code.

	if sourceKSData then
		sourceKSData:close()
	end

	if success then
		return KSPromises.resolvedPromiseP( 'data', ... )
	else
		return KSPromises.resolvedPromiseP( 'error', ... )
	end

end

--------------------------------------------------------------------------------

local function makeAnalysisRequestP( filePath, optKSData )

	return analysis_finish( optKSData, pcall( KSImportJpegService.analyzeImageData, filePath, optKSData ) )

end
----------------------

function methods:developP(photoId, eventId, filePath, xmpBase, xmpLook, orientation, croppedWidth, croppedHeight)
	local ozUser = OzAccountService.currentUserB():valueNow()
	local catalogService = OzCatalogService( eventId, ozUser)
	local anotherService = self._currentCatalog
	local developSession = OzDevelopSession(catalogService, photoId)
	
	
	local getThumbDataP
	getThumbDataP = makeAnalysisRequestP( filePath, nil )

	getThumbDataP = getThumbDataP:mapDataP( 
		function( status, _analyzedFilePath_, originalKSData, previewKSData, thumbKSData, highDpiThumbKSData, analysisInfo, maybeProxy, creationDate, originalFilenameFromALOrNil )
			
			if analysisInfo then

				local developP
			
				local changes = {}
				if orientation then
					changes.userOrientation = orientation
				else
					changes.userOrientation = 1		
				end
				changes.preview = previewKSData
				changes.thumbnail = thumbKSData
				changes.highDpiThumbnail = highDpiThumbKSData
				changes.xmpBase = xmpBase
				changes.xmpLook = xmpLook
				changes.croppedWidth = croppedWidth
				changes.croppedHeight = croppedHeight
			
				developP = developSession:saveAndCloseP(changes)
				
				return developP
				
			else
				return 'error', 'analysisFailed'
			end
			
		end )
		
		return getThumbDataP
	
end

--[[
	newEventB Becuse of a race condition I'm seeing in the unit test, I am 
	adding this behavior to the Create New Event work flow.
	It is because I am seeing 'updateDefaultAlbum' fire before 'updateEventList'.
	Now we create this behavior in 'reCreateEvent' command and fire it after 'updateEventList'
--]]
--------------------------------------------------------------------------------
function methods:setWantsBigThumbs(bigThumbs)

	wantBigThumbnails = bigThumbs
	
	debug.print( "using big thumbs = ", wantBigThumbnails )
	
end

--------------------------------------------------------------------------------
function methods:newEventB()

	self._newEventB = KSReactive.variableB( false )
	return self._newEventB
	
end

--------------------------------------------------------------------------------
function methods:closeNewEventB()

	if self._newEventB then
		self._newEventB:close()
		self._newEventB = false

	end
end

--------------------------------------------------------------------------------
function methods:setSharingB(value)
	self._sharingB:set(value)
end
--------------------------------------------------------------------------------
function methods:getMemoryStatus()
    return self._memoryWarning
end
--------------------------------------------------------------------------------
function methods:memoryWarning(warning)
    --debug.print("set memory warning")
    self._memoryWarning = warning
    self:clearCurrentBehaviors()
    
    self:garbageCollect()
end

--------------------------------------------------------------------------------

function methods:setSyncHalt( halt )
	Bablu:setOzAutoSyncEnabled( not halt )
	if self._currentCatalog then
		self._currentCatalog:setSyncEnabled( not halt )
	end
end

--------------------------------------------------------------------------------

function methods:closeMonitors( )

	self._monitorUserCatalogsB:close()
	self._monitorUserCatalogsB = false
	
	self._monitorCatalogsSharedWithUserB:close()
	self._monitorCatalogsSharedWithUserB = false
end

--------------------------------------------------------------------------------
function methods:checkForDefaultEvent(ozUser)

	local haveDefaultEvent = false
	local eventName
	
	self._defaultEventB:set(false)
	
	local resultP = ozUser:catalogsP( true ):whenResolvedDo( function( status, catalogList)
		
		if status ~= 'error' then
			for i = 1, #catalogList do
				-- make it easier to reference
				local catalog = catalogList[ i ]
				local subtype = catalog.subtype
				
				if subtype ~= 'grouppix' then
					haveDefaultEvent = true
				end
			end -- for loop

			if haveDefaultEvent == false then
				
				local resultP = ozUser:userInfoP():whenResolvedDo( function( status, userInfo)
					if status ~= 'error' then
					
						eventName = userInfo.first_name .. '\'s '.. 'Library'

						local catalogData = { 	
							name = eventName, 
						}

						local createCatalogP = ozUser:createCatalogP( catalogData ):whenResolvedDo( function( status, data) 
							self._defaultEventB:set(true)
						end)
																
						createCatalogP:runUntilDone()
				end
			end )
			
			resultP:runUntilDone()
			else
				self._defaultEventB:set(true)
			end -- No default event
			
		end -- if no error
		
	end )
	
	resultP:runUntilDone()
	
end

--------------------------------------------------------------------------------
function methods:watchCurrentUser()

    OzAccountService.currentUserB():onDataB( function( ozUser )

		if ozUser then

			self:isServiceAvailableOnStartUp(ozUser)

			self:checkForDefaultEvent(ozUser)

			ozUser:startWatchingForCatalogUpdates( 60 )
			
			self:monitorForceUpgrade( ozUser )
			self:monitorUserInfo( ozUser )
			self:monitorUserCatalogs( ozUser )
			self:monitorCatalogsSharedWithUser( ozUser )
			
            -- Register the device token, if there is one.
            -- We received a device token while the user was not logged in.
            -- OR, the user logged out and logged back in again.
            if self._deviceToken ~= false then
                registerDeviceToken( ozUser, self._deviceToken )
            end
			
			-- send back the user id
			-- we will monitor the rest of the user info
			-- with another behavior
			local result = {}
			result[ "userId" ] = ozUser:id()
			REAppLuaInterface.performResultCallback( "updateUserState", result )
			
			groupPixPin:getPinFormat( ozUser )
            
            -- Oz's list products API allows us to specify the location/language for push notifications
            self:listProducts( ozUser )

		else
					
			-- send an empty user id to indicate logged out state
			local result = {}
			result[ "userId" ] = ""
			REAppLuaInterface.performResultCallback( "updateUserState", result )

			self:getImsAuthInfo()
		end
	end )
end

--------------------------------------------------------------------------------
function methods:isServiceAvailableOnStartUp(ozUser)

	if self._serviceAvailableOnStartupB then
		self._serviceAvailableOnStartupB:close()
	end
	
	self._serviceAvailableOnStartupB = ozUser:isServiceAvailableB()
	
	self._serviceAvailableOnStartupB:onDataB(function(data)
			-- no service available
			if data == false then
				local result = {}
				result[ "serviceAvailable" ] = '0'
				REAppLuaInterface.performResultCallback( "updateServiceAvailable", result )
			end
	end)
	
end
--------------------------------------------------------------------------------

function methods:getImsAuthInfo( )

	local resultP = OzAccountService.getImsAuthUrlP()
		:mapP( function( status, imsAuthUrl, imsBaseUrl )

			if status == 'data' then
				-- we are not logged in, but we can get  IMS server info, send that info back
				
				return Bablu:getImsClientIdAndSecretP()
					:mapDataP( function( status, client_id, client_secret )
						
						self._imsAuthUrlFailed = false
						local result = {}
						result[ "userId" ] = ""
						result[ "imsAuthUrl" ] = imsAuthUrl
						result[ "imsBaseUrl" ] = imsBaseUrl
						result[ "imsClientId" ] = client_id
						result[ "imsClientSecret" ] = client_secret
						
						REAppLuaInterface.performResultCallback( "updateIMSAuthInfo", result )
						
					end)
			else
			 	-- error condition
				-- 'error', "error message"
				self._imsAuthUrlFailed = true
				local result = {}
				result[ "userId" ] = ""
				-- second arg will be the error message, imsAuthUrl in this case
				result[ "error" ] = imsAuthUrl
				REAppLuaInterface.performResultCallback( "updateIMSAuthInfo", result )
				
				return status, imsAuthUrl
			end
			
		end )
		
		resultP:runUntilDone()

end

--------------------------------------------------------------------------------

function methods:monitorForceUpgrade( ozUser )

	if self._forceUpgradeB then
		self._forceUpgradeB:close()
	end
	
	self._forceUpgradeB = false

	if not ozUser then return end

	self._forceUpgradeB = ozUser:forceUpgradeB()
			
	self._forceUpgradeB:onDataB( function ( data )
--		debug.print("forceUpgradeB fired ")
		if data then
			local result = {}
			REAppLuaInterface.performResultCallback( "forceUpgradeAlert", result )
--		else
--			debug.print( "Force upgrade is false" )
		end
	end )

	ozUser:addCloseTarget( self._forceUpgradeB )

end

--------------------------------------------------------------------------------

-- Observe changes to the user and update the native proxy model
function methods:monitorUserInfo( ozUser )

	if self._monitorUserInfoB then
		self._monitorUserInfoB:close()
	end
		
	self._monitorUserInfoB = false

	if not ozUser then return end

	self._monitorUserAuthStatusB = ozUser:hasValidConnectionToOzB()
	self._monitorUserAuthStatusB:onDataB( function( isValid )
	
		if isValid then return end
		
		-- We were unable to refresh the user's auth and refresh tokens,
		-- so tell the user that we need them to re-enter their credentials,
		-- then take them to the login screen.
		local result = {}
		result[ "userId" ] = ozUser:id()
		REAppLuaInterface.performResultCallback( "userAuthStatusInvalid", result )
		
	end )

	ozUser:addCloseTarget( self._monitorUserAuthStatusB )
	
	local function userInfoChanged( fullName, email, accountStatus, catalogQuota, ozUser )
		-- send over the contributor and event/catalog limits

		ozUser:isCurrentUserB():onDataB( function( isCurrentUser )

			if isCurrentUser then
				ozUser:catalogLimitP():whenResolvedDo( function( status, catalogLimit)
					
						-- for correct key names, see RevelUser.m
						local result = {}		
						result[ "userId" ] = ozUser:id()
						result[ "fullName" ] = fullName
						result[ "email" ] = email
						result[ "accountState" ] = accountStatus
						
						if catalogQuota ~= nil then
							result[ "catalogsRemaining" ] = catalogQuota
						end
						
						result[ "catalogLimit" ] = catalogLimit
				
						REAppLuaInterface.performResultCallback( "updateUserInfo", result )
					
				end ):runUntilDone()
				
			end	-- isCurrentUser
			
		end) -- onDataB
		
	end

	local fullNameB = ozUser:fullNameB()

	local emailB = ozUser:emailB()

	local accountStatusB = ozUser:accountStatusB()
	
	local catalogQuotaB = ozUser:catalogQuotaB()
	
	-- To avoid being 'garbage collected'
	self._monitorUserInfoB = KSReactive.dataCallB( userInfoChanged, fullNameB, emailB, accountStatusB, catalogQuotaB, ozUser )

	ozUser:addCloseTarget( self._monitorUserInfoB )

end

--------------------------------------------------------------------------------

function methods:changeCatalogPollInterval( ozUser, numSeconds )
	WFArguments.check( REScriptArg, 'OzUser', 'number' )
	
	if ozUser then
		ozUser:setCatalogUpdateTime( numSeconds )
		ozUser:startWatchingForCatalogUpdates( nil, true )
		--debug.print( "catalog poll interval=", numSeconds )
	end
end

--------------------------------------------------------------------------------

local function processCatalogList( --[[table]] catalogList, --[[boolean]] isSharedList, self )
	
	if catalogList then
		-- debug.print( "******** catalog list changed" )
		-- create individual lists for each attribute: id, name, etc.
		local idList = {}
		local nameList = {}
		local acceptedInviteList = {}
		local subtypeList = {}
		local startDateList = {}
		local endDateList = {}
		local defaultAlbumList = {}
		local pinCodeList = {}
		local pinStateList = {}
		local pinExpiresList = {}
		local inviteShortUrlList = {}
		
		local broadcastList = {}
		local broadcastTypeList = {}
		local broadcastAutoJoinList = {}
		
		local listIdx = 1
		
		for catIdx = 1, #catalogList do
		
			-- make it easier to reference
			local catalog = catalogList[ catIdx ]
			
			-- subtype
			local subtype = catalog.subtype

			subtype = subtype or ''
			subtypeList[ catIdx ] = subtype
			
						
			if subtype == 'grouppix' or subtype == 'tornado' then 

				idList[ listIdx ] = catalog.id
				nameList[ listIdx ] = catalog.name
				acceptedInviteList[ listIdx ] = tostring( catalog.acceptedInvite )
					
				local payload = catalog.payload
				
				-- Start Date
				local startDate = payload['start_date'] or ''
				startDateList[ listIdx ] = startDate
				
				-- End Date
				local endDate = payload['end_date'] or ''
				endDateList[ listIdx ] = endDate

				-- default album id
				local albumId = payload[ 'X-Event-DefaultAlbumId' ] or ''
				defaultAlbumList[ listIdx ] = albumId
				
				-- grouppix pin code (aka "Invite ID")
				local pinCode = payload[ 'X-Event-Pin' ] or ''
				pinCodeList[ listIdx ] = pinCode
				
				-- grouppix pin code state: active/inactive/expired
				local pinState = payload[ 'X-Event-PinState' ] or ''
				pinStateList[ listIdx ] = pinState
				
				-- grouppix pin expires ISO 8601 format date
				local pinExpires = payload[ 'X-Event-PinExpires' ] or ''
				pinExpiresList[ listIdx ] = pinExpires
				
				-- grouppix pin invite short url
				local inviteShortUrl = payload[ 'X-Event-PinInviteShortUrl' ] or ''
				inviteShortUrlList[ listIdx ] = inviteShortUrl
				
				-- broadcast
				local broadcast = payload[ 'X-Event-Broadcast' ] or ''
				broadcastList[ listIdx ] = broadcast
				
				-- broadcast type
				local broadcastType = payload[ 'X-Event-BroadcastType' ] or ''
				broadcastTypeList[ listIdx ] = broadcastType

				-- broadcast auto join
				local broadcastAutoJoin = payload[ 'X-Event-BroadcastAutoJoin' ] or ''
				broadcastAutoJoinList[ listIdx ] = broadcastAutoJoin
	
				
				if self._currentCatalogId == catalog.id then
					-- request the list of users as well
					-- debug.print( "requesting update of catalog users" )
					self:updateCatalogUsers( self._currentCatalog )
				end
			
				listIdx = listIdx + 1
				
				-- debug.print( catalogList[ listIdx ].name )
			else
				--if not WFPlatformInfo.isAndroid then
					idList[ listIdx ] = catalog.id
					nameList[ listIdx ] = catalog.name
					acceptedInviteList[ listIdx ] = tostring( catalog.acceptedInvite )
	
					startDateList[ listIdx ] = ''
					endDateList[ listIdx ] = ''
					defaultAlbumList[ listIdx ] = ''
					pinCodeList[ listIdx ] = ''
					pinStateList[ listIdx ] = ''
					pinExpiresList[ listIdx ] = ''
					inviteShortUrlList[ listIdx ] = ''
					broadcastList[ listIdx ] = ''
					broadcastTypeList[ listIdx ] = ''
					broadcastAutoJoinList[ listIdx ] = ''
                    listIdx = listIdx + 1
				--end
			end
	
		end
		
		local result = {}
		result[ "catalogIds" ] = idList
		result[ "catalogNames" ] = nameList
		result[ "acceptedInvites" ] = acceptedInviteList
		result[ "startDates" ] = startDateList
		result[ "endDates" ] = endDateList
		result[ "subtypes" ] = subtypeList
		result[ "defaultAlbums" ] = defaultAlbumList
		result[ "pinCodes" ] = pinCodeList
		result[ "pinStates" ] = pinStateList
		result[ "pinExpires" ] = pinExpiresList
		result[ "inviteShortUrls" ] = inviteShortUrlList
		result[ "broadcasts" ] = broadcastList
		result[ "broadcastTypes" ] = broadcastTypeList
		result[ "broadcastAutoJoins" ] = broadcastAutoJoinList
		
		if isSharedList then
			REAppLuaInterface.performResultCallback( "updateSharedEventList", result )
		else
			REAppLuaInterface.performResultCallback( "updateEventList", result )

			if self._newEventB then
				--debug.print("set _newEventB" )
				self._newEventB:set(true)
			end

		end
	end

end

--------------------------------------------------------------------------------

function methods:monitorUserCatalogs( ozUser )
		
	if self._monitorUserCatalogsB then
		self._monitorUserCatalogsB:close()
	end
	self._monitorUserCatalogsB = false

	if not ozUser then return end
	
	self._monitorUserCatalogsB = ozUser:catalogsB()
	self._monitorUserCatalogsB:onDataB( function( catalogList )

		if catalogList then
			processCatalogList( catalogList, false, self)
		end
			
	end )
	
end

--------------------------------------------------------------------------------

function methods:monitorCatalogsSharedWithUser( ozUser )

	if self._monitorCatalogsSharedWithUserB then
		self._monitorCatalogsSharedWithUserB:close()
	end
	self._monitorCatalogsSharedWithUserB = false

	if not ozUser then return end
	
	self._monitorCatalogsSharedWithUserB = ozUser:catalogsSharedWithUserB()
	self._monitorCatalogsSharedWithUserB:onDataB( function( catalogList )

		if catalogList then
			processCatalogList( catalogList, true, self )
		end
			
	end )

end

--------------------------------------------------------------------------------

function methods:deleteAsset( ozUser, catalogId, assetID )
       
        self:setCurrentCatalog( ozUser, catalogId )
        self._currentCatalog:deletePhotoP( assetID ):whenResolvedDo( function( status, newPhotoId )
       
        if status == 'data' then
            --debug.print("newPhotoId = ", newPhotoId)
            local result = {}
            result[ "catalogId" ] = catalogId
            result[ "assetID" ] = assetID
            REAppLuaInterface.performResultCallback( "deletedPhoto", result )
        else
             --debug.print("status = error")
        end
            
        end ):runUntilDone()

end

--------------------------------------------------------------------------------

function methods:deleteAssets( ozUser, catalogId, assetIDs )
       
		reScriptLog:debug( "AssetIds %s", assetIDs )
		
        self:setCurrentCatalog( ozUser, catalogId )
        self._currentCatalog:deletePhotosP( assetIDs ):whenResolvedDo( function( status, data )
       
        if status == 'data' then
            local result = {}
            result[ "catalogId" ] = catalogId
            result[ "data" ] = data
            REAppLuaInterface.performResultCallback( "deletedPhotos", result )
        else
             --debug.print("status = error")
        end
            
        end ):runUntilDone()

end

--------------------------------------------------------------------------------

-- @mark Comments

--------------------------------------------------------------------------------

function methods:canPostComments( photoId )


    local result = {}
    result[ "canPost" ] = '0'
    result[ "photoId" ] = photoId
    REAppLuaInterface.performResultCallback( "enablePostCommentEvent", result )

    --debug.print("starting syn now")
    
    self._currentCatalog:syncNowP():whenResolvedDo( function( status, data)

        if status == 'error' then
            local result = {}
            REAppLuaInterface.performResultCallback( "clearCommentDict", result )
        else
            local result = {}
            result[ "canPost" ] = '1'
            result[ "photoId" ] = photoId
            REAppLuaInterface.performResultCallback( "enablePostCommentEvent", result )
           -- debug.print("ending syn now")

        end
        end):runUntilDone()

end

--------------------------------------------------------------------------------

function methods:addComment( ozUser, catalogId, photoId, comment  )
    self:setCurrentCatalog( ozUser, catalogId )

    self._currentCatalog:addCommentP( nil, photoId, comment ):whenResolvedDo( function( status, commentID )
		
         if status == 'error' then
          --  debug.print(" catalog:addCommentP error = ", status )
          --  debug.print(" catalog:addCommentP commentID = ", commentID )
        end
    end ):runUntilDone()

end

--------------------------------------------------------------------------------
local function concatentCommentFields( list, assetId )

	local commentsArray = {}
	 
	local i = 1
	for j = 1, #list do
		for k = 1, #list[ j ].comments do
			local ownerId  = list[ j ].comments[ k ].author.id or ''
		    local commentId = list[ j ].comments[ k ].id or ''
		    local dateCreated = list[ j ].comments[ k ].created or ''
		    local comment = list[ j ].comments[ k ].body or ''
			local fullName = list[ j ].comments[ k ].author.fullName or ''
		         
			if WFPlatformInfo.isiOS then
				commentsArray[i] = assetId 
				i = i + 1
			end
			commentsArray[i] = ownerId
			i = i + 1
			commentsArray[i] = commentId
			i = i + 1
			commentsArray[i] = dateCreated
			i = i + 1
			commentsArray[i] = comment
			i = i + 1
			if WFPlatformInfo.isAndroid then
				commentsArray[i] = fullName
				i = i + 1
			end			
		end
	end
	
	return commentsArray
	
end

--------------------------------------------------------------------------------

function methods:getCommentList( ozUser, catalogId, photoId )

	WFArguments.check( REScriptArg, 'OzUser', 'string', 'string' )

    self:setCurrentCatalog( ozUser, catalogId )

	if not self._getCommentListBehaviors[ photoId ] or WFCloseSupport.hasBeenOrWillBeClosed( self._getCommentListBehaviors[ photoId ] ) then
		self._getCommentListBehaviors[ photoId ] = self._currentCatalog:getCommentsOnAssetB( photoId, { 'all' } )
		self._currentCatalog:addCloseTarget( self._getCommentListBehaviors[ photoId ] )
	end

    self._getCommentListBehaviors[ photoId ]:onDataB( function( list )
		reScriptLog:debug( "comment list = %s", list )
			
		local result = {}
		
		local commentsArray = {}
	    if list ~= nil then
	    	commentsArray = concatentCommentFields( list, photoId )
	    end
	
	    result[ "commentsArray" ] = commentsArray
	    result[ "eventId" ] = catalogId
	    result[ "photoId" ] = photoId

        REAppLuaInterface.performResultCallback( "updateComments", result )
	end )
	
end

--------------------------------------------------------------------------------

function methods:closeCommentList( ozUser, catalogId, photoId )

    if self._getCommentListBehaviors[ photoId ] ~= nil or not WFCloseSupport.hasBeenOrWillBeClosed( self._getCommentListBehaviors [ photoId ] ) then
        self._getCommentListBehaviors[ photoId ]:close()
		self._getCommentListBehaviors[ photoId ] = nil
    end

end

--------------------------------------------------------------------------------

function methods:deleteComment( ozUser, catalogId, commentId )
      
    self:setCurrentCatalog( ozUser, catalogId )
    
    self._currentCatalog:deleteCommentP( commentId ):whenResolvedDo( function( status, deleted )
        if status == 'error' then
            reScriptLog:debug( " catalog:addCommentP error = %s", status )
            reScriptLog:debug( " catalog:addCommentP commentID = %s", commentId )
        end
     end ):runUntilDone()

end

--------------------------------------------------------------------------------

function methods:closeCommentBehavior( photoId )

	if self._getCommentListBehaviors[ photoId ] then
        reThumbLog:debug("methods:closeCommentBehavior for photoID = %s", photoId )

      	self._getCommentListBehaviors[ photoId ]:close()
      	self._getCommentListBehaviors[ photoId ] = nil
    end

end

--------------------------------------------------------------------------------
function methods:assetCollectionInfo( rows )
	
	if not rows or #rows == 0 then
		return { assetId = 0, favoritedUsers = 0,  }
	end
			
	local row = rows[1]	
		
	local collectionInfo = {
		assetId = row[1],
        favoritedAssets = row[2],
		importedBy = row[3],
		favoritedUsers = row[4]
	}
	return collectionInfo
	
end

--------------------------------------------------------------------------------

function methods:monitorAsset( ozuser, eventId, assetId)
      
	if self._currentCatalog then
	    
		if self._assetChangedB ~= false then
			self._assetChangedB:close()
			self._assetChangedB = false
		end
		
		local resultFcn = function( rows )
		
			if #rows > 0 then
				local assetInfo = self:assetCollectionInfo( rows )
				local result = {}
        		result[ "assetId" ] = assetId
        		result[ "eventId" ] = eventId
        		result[ "favoriteCount" ] = assetInfo.favoritedAssets
				result[ "favoritedUsers" ] = assetInfo.favoritedUsers	
				REAppLuaInterface.performResultCallback( "updateAsset", result )
			end
		end
						
						
		if self._filterByUserId then	
				self._assetChangedB =  self._currentCatalog._indexStore
					:queryB( [[SELECT coreInfo.docId, favoritedCount, importedBy, favoritedUsers
                            FROM coreInfo
                                	WHERE 
										 importedBy == ? and coreInfo.docId == ? ]], self._filterByUserId, assetId )
						:onDataB( resultFcn )
	
		else			
			  self._assetChangedB = self._currentCatalog._indexStore:queryB(
                        [[SELECT coreInfo.docId, favoritedCount, importedBy, favoritedUsers
							FROM coreInfo
								WHERE
									coreInfo.docId == ? ]], assetId)				
				:onDataB( resultFcn)

		end
		
		--self._currentCatalog:addCloseTarget( self._assetChangedB )
		
	end
end

--------------------------------------------------------------------------------

function methods:collaboratorPhotoCountList( )
      
	if self._currentCatalog then
	    
		if self._collaboratorPhotoCountB ~= false then
			self._collaboratorPhotoCountB:close()
			self._collaboratorPhotoCountB = false
		end
		
		local resultFcn = function( rows )
			local ids = {}
			local counts = {}
			for i = 1, #rows do
				ids[ i ] = rows[ i ][ 1 ]
				counts[ i ] = rows[ i ][ 2 ]
			end
			local result = {}
			result[ "importedByIds" ] = ids
			result[ "importedByCounts" ] = counts
        	result[ "eventId" ] = self._currentCatalogId 
			REAppLuaInterface.performResultCallback( "updateCollaboratorPhotoCount", result )
		end					
						
		self._collaboratorPhotoCountB = self._currentCatalog._indexStore:queryB(
			[[SELECT importedBy, COUNT(*), contentTypeNumber
							FROM coreInfo
							WHERE contentTypeNumber == 1001
							GROUP BY importedBy]] )				
				:onDataB( resultFcn)

		self._currentCatalog:addCloseTarget( self._collaboratorPhotoCountB )
		
	end
end

--------------------------------------------------------------------------------

function methods:assetsTotalCount( )
      
	if self._currentCatalog then
	    
		if self._assetTotalCountB ~= false then
			self._assetTotalCountB:close()
			self._assetTotalCountB = false
		end
		
		local resultFcn = function( rows )
			local result = {}
			result[ "totalAssets" ] = #rows
        	result[ "eventId" ] = self._currentCatalogId 
			REAppLuaInterface.performResultCallback( "updateTotalAssetsCount", result )
		end
						
						
		self._assetTotalCountB = self._currentCatalog._indexStore:queryB(
			[[SELECT coreInfo.docId, contentTypeNumber FROM coreInfo WHERE contentTypeNumber == 1001 ]] )				
				:onDataB( resultFcn)

		self._currentCatalog:addCloseTarget( self._assetTotalCountB )
		
	end
end

--------------------------------------------------------------------------------

function methods:assetsLikeCount( )
      
	if self._currentCatalog then
	    
		if self._assetLikedCountB ~= false then
			self._assetLikedCountB:close()
			self._assetLikedCountB = false
		end
		
		local resultFcn = function( rows )
			local result = {}
			result[ "numFavorites" ] = #rows
        	result[ "eventId" ] = self._currentCatalogId 
			REAppLuaInterface.performResultCallback( "updateLikeCount", result )
		end
						
						
		if self._filterByUserId and self._useUserIdSubFiltering then	
				self._assetLikedCountB =  self._currentCatalog._indexStore
					:queryB( [[SELECT coreInfo.docId, favoritedCount, importedBy, contentTypeNumber
                            FROM coreInfo
                                	WHERE importedBy == ? and favoritedCount > 0 and contentTypeNumber == 1001 ]], self._filterByUserId )
						:onDataB( resultFcn )
	
		else			
			  self._assetLikedCountB = self._currentCatalog._indexStore:queryB(
                        [[SELECT coreInfo.docId, favoritedCount, contentTypeNumber
							FROM coreInfo
								WHERE favoritedCount > 0 and contentTypeNumber == 1001 ]] )				
				:onDataB( resultFcn)

		end
		
		self._currentCatalog:addCloseTarget( self._assetLikedCountB )
		
	end
end


--------------------------------------------------------------------------------

function methods:assetsWithCommentsCount( sortOrder )
      
	if self._currentCatalog then
	    
		if self._assetCommentCountB ~= false then
			self._assetCommentCountB:close()
			self._assetCommentCountB = false
		end
		
		local resultFcn = function( rows )
						
			local result = {}
			result[ "assetsWithComments" ] = #rows
			result[ "eventId" ] = self._currentCatalogId
			REAppLuaInterface.performResultCallback( "updatePhotosWithCommentsCount", result )
	
		end
						
						
		if self._filterByUserId and self._useUserIdSubFiltering then
				
				self._assetCommentCountB = self._currentCatalog._indexStore
					:queryB( [[ SELECT DISTINCT coreInfo.docId, coreInfo.importedBy, coreInfo.contentTypeNumber
			                        FROM coreInfo
			                        	INNER JOIN comment
											ON coreInfo.docId == comment.assetId
			                             		WHERE coreInfo.importedBy == ? and coreInfo.contentTypeNumber == 1001 ]], self._filterByUserId )
						:onDataB( resultFcn )
	
		else

			  self._assetCommentCountB = self._currentCatalog._indexStore
					:queryB( [[ SELECT DISTINCT coreInfo.docId, coreInfo.contentTypeNumber
			                        FROM coreInfo
			                        	INNER JOIN comment
											ON coreInfo.docId == comment.assetId
			                             		WHERE coreInfo.contentTypeNumber == 1001 ]] )
						:onDataB( resultFcn )
					:onDataB( resultFcn)
			end
		
		self._currentCatalog:addCloseTarget( self._assetCommentCountB )
		
	end
end

--------------------------------------------------------------------------------

-- @mark Report Abuse

--------------------------------------------------------------------------------

function methods:reportAbuse( ozUser, catalogId, assetID )
	
	self:setCurrentCatalog( ozUser, catalogId )
	
	self._currentCatalog:abuseReportingUrlP( assetID, 'preview' ):whenResolvedDo( function( status, url )

		if status == 'data' then

			-- Invoke the report abuse web page in the Safari browser
				
			if not WFPlatformInfo.isAndroid then
				REAppLuaInterface.openURL( url )
			else 
				local result = {}
				result[ "url" ] = url
				result[ "eventId" ] = catalogId
				result[ "assetId" ] = assetID
				REAppLuaInterface.performResultCallback( "updateReportAbuseUrl", result )
			end

		else

			if not WFPlatformInfo.isAndroid then
				REAppLuaInterface.showServiceUnavailableAlert()
			else
				local result = {}
				result[ "error" ] = url
				result[ "eventId" ] = catalogId
				result[ "assetId" ] = assetID
				REAppLuaInterface.performResultCallback( "updateReportAbuseUrl", result )
			end

		end

	end ):runUntilDone()

end

--------------------------------------------------------------------------------

-- @mark Omniture

--------------------------------------------------------------------------------

function methods:checkOmnitureSetting( ozUser )
   
    local prefsKey = "OmnitureOptIn_Tornado"
    local omnitureSetting = KSPrefs[ prefsKey ]

    if omnitureSetting == nil and self._pipAlertShown == false then
        local result = {}

        REAppLuaInterface.performResultCallback( "queryOmnitureUse", result )

        self._pipAlertShown = true
    else
        self:updateOmnitureSetting( ozUser, omnitureSetting )
    end
end

--------------------------------------------------------------------------------
function methods:updateOmnitureSettingOnStartUp( omnitureSetting )

    if omnitureSetting == nil then
        return
    end

    local prefsKey = "OmnitureOptIn_Tornado"

    KSPrefs[ prefsKey ] = omnitureSetting

end

--------------------------------------------------------------------------------
function methods:updateOmnitureSetting( ozUser, omnitureSetting )

    if omnitureSetting == nil then
        return
    end

    local prefsKey = "OmnitureOptIn_Tornado"

    KSPrefs[ prefsKey ] = omnitureSetting

    --[[
        01/31/2013 mscarafo
        We will always set Oz to true. We will always be sending up metrics, but
        internally based on the prefs, we will only send certain metrics
    --]]
    self:updateOmnitureSettingInOz( ozUser, true )

    WFAnalyticsService.enableTracking( true )

    self._pipAlertShown = false
end

--------------------------------------------------------------------------------

function methods:updateOmnitureSettingInOz( ozUser, omnitureSetting )

    local resultP = ozUser:setOmnitureOptInP( omnitureSetting )

    resultP:whenResolvedDo( function(status, data)
        ozUser:refreshP()
        REAppLuaInterface.performResultCallback( "omnitureSettingSync", {} )
    end ):runUntilDone()

    WFAnalyticsService.enableTracking( omnitureSetting )

    self._pipAlertShown = false
end

--------------------------------------------------------------------------------

-- @mark CurrentEvent

--------------------------------------------------------------------------------

function methods:changeSortOrder( sortOrder, userId )

    reScriptLog:debug("methods:changeSortOrder sortOrder = %s", sortOrder )

	if userId == nil then
		self._filterByUserId = false
	else
		self._filterByUserId = userId
	end
	
	self:assetsWithCommentsCount( sortOrder )

    self:monitorCatalog( sortOrder, userId )

end

-------------------------------------------------------------------------------

function  methods:extractComponent( componentName, list )
		
    local retval = {}
    
    for i, value in ipairs( list ) do
        retval[ i ] = value[ componentName ]	
    end
    
    return retval
    
end

--------------------------------------------------------------------------------
-- For Unit tests ONLY!
function methods:clearFavoriteBehaviors()

    for k,v in pairs(self._favoriteBehaviors) do
        self._favoriteBehaviors[ k ]:close()
        self._favoriteBehaviors[ k ] = nil
    end
    
    for k,v in pairs(self._allFavoritesBehaviours) do
        self._allFavoritesBehaviours[ k ]:close()
        self._allFavoritesBehaviours[ k ] = nil
    end

    REAppLuaInterface.performResultCallback( "clearFavoritesBehaviors", {} )

end

--------------------------------------------------------------------------------
function methods:clearCurrentBehaviors()

	for k,v in pairs(self._getAlbumCoverBehaviors) do
		self._getAlbumCoverBehaviors[k]:close()
		self._getAlbumCoverBehaviors[k] = nil
	end
    for k,v in pairs(self._getCommentListBehaviors) do
        self._getCommentListBehaviors[ k ]:close()
        self._getCommentListBehaviors[ k ] = nil
    end

    for k,v in pairs(self._thumbnailBehaviors) do
        self._thumbnailBehaviors[ k ]:close()
        self._thumbnailBehaviors[ k ] = nil
    end

    for k,v in pairs(self._imageBehaviors) do
        self._imageBehaviors[ k ]:close()
        self._imageBehaviors[ k ] = nil
    end

    for k,v in pairs(self._originalImageBehaviors) do
    	self._originalImageBehaviors[k]:close()
    	self._originalImageBehaviors[k] = nil
    end

    for k,v in pairs(self._favoriteBehaviors) do
        self._favoriteBehaviors[ k ]:close()
        self._favoriteBehaviors[ k ] = nil
    end

    for k,v in pairs(self._allFavoritesBehaviours) do
        self._allFavoritesBehaviours[ k ]:close()
        self._allFavoritesBehaviours[ k ] = nil
    end

	if self._assetCommentCountB then
		self._assetCommentCountB:close()
		self._assetCommentCountB = false
	end
	
	if self._currentCollectionB then
		self._currentCollectionB:close()
		self._currentCollectionB = false
	end
	
	if self._catalogUserLimitB then
		self._catalogUserLimitB:close()
		self._catalogUserLimitB = false
	end
	
	if self._currentCatalogSyncStateB then
		self._currentCatalogSyncStateB:close()
		self._currentCatalogSyncStateB = false
	end
	
	if self._assetLikedCountB then
		self._assetLikedCountB:close()
		self._assetLikedCountB = false
	end
	
	if self._collaboratorPhotoCountB then
		self._collaboratorPhotoCountB:close()
		self._collaboratorPhotoCountB = false
	end
	
	if self._assetTotalCountB then
		self._assetTotalCountB:close()
		self._assetTotalCountB = false
	end

	if self._idleTimerDisabledB then
		self._idleTimerDisabledB:close()
		self._idleTimerDisabledB = false
	end
	
	if self._catalogAccessRequestB then
		self._catalogAccessRequestB:close()
		self._catalogAccessRequestB = false
	end
	
	if self._serviceAvailableB then
		self._serviceAvailableB:close()
		self._serviceAvailableB = false
	end
		
end

--------------------------------------------------------------------------------

function methods:setCurrentCatalog( ozUser, catalogId )

	if not ozUser then return end

    if self._currentCatalog == false then
        self:checkOmnitureSetting( ozUser )
    end

	if self._currentCatalogId ~= catalogId or self._currentCatalog == false then

		if self._currentCatalog then
			self._currentCatalog:setPurgeEnabled( false )
			self._currentCatalog:close()
			self._currentCatalog = false
		end

		if self._currentRenditionServer then
			self._currentRenditionServer:close()
			self._currentRenditionServer = false
		end

        self:clearCurrentBehaviors()
        self._failedUserListCatalog = false
		self._currentCatalogId = false

		if catalogId then
        	self._currentCatalog = OzCatalogService( catalogId, ozUser )
			self._currentCatalog:setSyncEnabled( true )
			self._currentCatalog:setPurgeEnabled( true )

			self._currentCatalogId = catalogId
		
			self._currentRenditionServer = WFOzRenditionServer( self._currentCatalog )
        	self._currentRenditionServer:setUseMinimalThumbnailCache( true )
		--	self._currentRenditionServer:spinUpDataCache()

			self:updateCatalogUsers( self._currentCatalog )
			self:updateCatalogCollaboratorQuota( self._currentCatalog )
			self:monitorCatalogSyncState( self._currentCatalog )

			self:assetsWithCommentsCount( )
			self:assetsLikeCount( )
			self:collaboratorPhotoCountList()
			self:assetsTotalCount()
			self:monitorCatalog( )
			self:setIdleTimeB()
			self:monitorUserAlbums( ozUser )
			self:canAccessB()
			self:serviceAvailable()
		end	
		
	end

end

--------------------------------------------------------------------------------

function methods:logout( --[[OzUser]] ozUser )

	WFArguments.check( REScriptArg, 'OzUser' )
	
	local logoutCallback = function( status, data )
		debug.print('in callback')
		local result = { ozUser = ozUser:id() }
	
		if status == 'data' then
			self:closeMonitors( )
			self:setCurrentCatalog( ozUser, nil )
		end
		
		-- send back a response for the logout
		-- return true if the logout was successful
		result[ "result" ] = status == 'data'
		result[ "tokenDeleted" ] = true
		result[ "status" ] = status
		result[ "data" ] = data
		REAppLuaInterface.performResultCallback( "logOutEvent", result )
	end
				
	ozUser:logoutP():whenResolvedDo(logoutCallback):runUntilDone()
	
end

---------------------

function methods:closeCurrentCatalog( )
	
    if self._currentCatalog then

        self._currentCatalog:close()
        self._currentCatalog = false
        self._currentCatalogId = false
		self._currentRenditionServer:close()
		self._currentRenditionServer = false
		self._assetCommentCountB = false
		self._currentCollectionB = false
		self._catalogAccessRequestB:close()
		self._catalogAccessRequestB = false
        self._failedUserListCatalog = false
        self._currentCatalogSyncStateB:close()
		self._currentCatalogSyncStateB = false
		
    end
end


--------------------------------------------------------------------------------

function methods:monitorCatalog( sortOrder, userID )

	local currentCollectionB = self._currentCatalog:tornadoCollectionInfoB( sortOrder, userID )

	if currentCollectionB ~= nil then
		
		if self._currentCollectionB then
			self._currentCollectionB:close()
		end
		
		self._currentCollectionB = currentCollectionB
	else
		--debug.print( "ERROR : sortedCollection was nil!" )
	end

	self._currentCollectionB:onDataB( function( collectionInfo )
	

		reScriptLog:debug( "collection updated" )
		--[[
			local collectionInfo = {
				name = name,		
				photos = {},
				displayDate = displayDate,
				eventName = eventName,
				startDate = startDate,
				endDate = endDate,
				eventNameFirst = eventNameFirst,
		        favoritedAssets = 0,
				assetCount = nPhotos
			}
		--]]

		if collectionInfo then

			local photos = collectionInfo.photos
			
			local photoIds = {}
			local favoritedCounts = {}
			local captureDates = {}
            local importedByIds = {}
			local aspectRatios = {}
            local favoritedUsers = {}

			local assetsAdded = 0
			local indexAdded = 1
			-- we would like to just send the table over, but it's not possible yet

			debug.print("update asstes : count : " .. #photos)

			for j = 1, #photos do				
				
				local contentType = photos[ j ].contentType
				if contentType == "image/jpeg" or contentType == "image/png" then
					photoIds[ indexAdded ] = photos[ j ].id
					favoritedCounts[ indexAdded ] = photos[ j ].favoritedCount .. ""
					favoritedUsers[ indexAdded ] = photos[ j ].favoritedUsers   
					captureDates[ indexAdded ] = photos[ j ].captureDate .. ""
					importedByIds[ indexAdded ] = photos[ j ].importedBy      
					aspectRatios[ indexAdded] = photos[ j ].aspectRatio
					assetsAdded = assetsAdded + 1
					indexAdded = indexAdded + 1
					
				end
			
			end 
			
			--debug.print( "catalog: " .. catalogList[ i ].name )
			--debug.print( "photo ids: " .. #photoIds .. " md5s: " .. #photoMD5s )
			--debug.print( "favorited: " .. #favoritedCounts .. " captureDates: " .. #captureDates )

			if assetsAdded > 0 then
				-- debug.print("assetsAdded > 0")
				local result = {}
				result[ "catalogId" ] = self._currentCatalogId
				result[ "favoritedAssets" ] = collectionInfo.favoritedAssets
				result[ "assetCount" ] = assetsAdded
				result[ "photoIds" ] = photoIds
				result[ "favoritedCounts" ] = favoritedCounts
                result[ "favoritedUsers" ] = favoritedUsers
				result[ "captureDates" ] = captureDates
				result[ "importedByIds" ] = importedByIds
				result[ "aspectRatios" ] = aspectRatios

				REAppLuaInterface.performResultCallback( "updateEventPhotos", result )
			else
				-- return just the counts (all will be zero)
				local result = {}
				result[ "catalogId" ] = self._currentCatalogId
				result[ "favoritedAssets" ] = collectionInfo.favoritedAssets
				result[ "assetCount" ] = collectionInfo.assetCount
				REAppLuaInterface.performResultCallback( "updateEventPhotos", result )
			end
		end
		
	end )	
	
	self._currentCatalog:addCloseTarget( self._currentCollectionB )
	
end

--------------------------------------------------------------------------------

function methods:getAlbumCover( albumId )
	if self._getAlbumCoverBehaviors[albumId] ~= nil then
		self._getAlbumCoverBehaviors[albumId]:close()
		self._getAlbumCoverBehaviors[albumId] = nil;
	end

	self._getAlbumCoverBehaviors[albumId] = self._currentCatalog:albumCoverImageB(albumId)
	self._getAlbumCoverBehaviors[albumId]:onDataB( function(coverId)
						local coverResult = {}
						coverResult.albumId = albumId
						coverResult.coverId = coverId

						REAppLuaInterface.performResultCallback( "updateAlbumCover", coverResult )
					end
					)
end


function methods:monitorUserAlbums( ozUser )

	debug.print( "monitorUserAlbums called" )
		
	if self._monitorUserAlbumsB then
		self._monitorUserAlbumsB:close()
	end
	self._monitorUserAlbumsB = false

	if not ozUser then return end
	
	self._monitorUserAlbumsB = self._currentCatalog:albumsDateSortedB()
	self._monitorUserAlbumsB:onDataB( function( albumsList )
	
		debug.print( "albums updated" )

		if albumsList then
		
			local idList = {}
			local nameList = {}
			local assetCountList = {}
			
			debug.print( " AlbumList size = " .. tostring(#albumsList) )
		
			for i = 1, #albumsList do
		
				-- make it easier to reference
				local album = albumsList[ i ]
			
				idList[ i ] = album.id
				nameList[ i ] = album.name
				assetCountList[i] = album.assetCount
			
--				debug.print(  'Album id=' .. album.id .. 'album name=' .. album.name )



			end
			
			local result = {}
			result[ "albumIds" ] = idList
			result[ "albumNames" ] = nameList
			result["albumAssetCounts"] = assetCountList

			REAppLuaInterface.performResultCallback( "updateAlbumList", result )
			
		end
			
	end )
	
end

--------------------------------------------------------------------------------

function methods:getUserAlbumColections( albumId )

	debug.print( "monitorUserAlbumColections called" )
		
	if self._currentAlbumCollectionB then
		self._currentAlbumCollectionB:close()
	end
	self._currentAlbumCollectionB = false

	if not albumId then return end
	
	self._currentAlbumCollectionB = self._currentCatalog:globalGridCollectionInfoB( albumId, false )
	self._currentAlbumCollectionB:onDataB( function( collectionInfo )
	
		debug.print( "album collection updated" )

		if collectionInfo then

			local photos = collectionInfo.photos
			
			local photoIds = {}
			
			debug.print( "Photos in this album = " .. tostring(#photos) )
	
			-- we would like to just send the table over, but it's not possible yet
			for i = 1, #photos do
						
				photoIds[ i ] = photos[ i ].id

			end 

			local result = {}
			result[ "catalogId" ] = self._currentCatalogId
			result[ "albumId" ] = albumId
			result[ "assetCount"] = collectionInfo.assetCount
			result[ "photoIds" ] = photoIds

			REAppLuaInterface.performResultCallback( "updateAlbumPhotos", result )
		end
			
	end )
	
end

---------------------------------------------------------------------------------

function methods:isImportingB()

	return importJpegService:requestE( 'getStatusCallback' )
		:mapE( function( status )
			return status ~= 'canceled' and status ~= 'complete'
		end )
		:holdB( false )
		
end

--------------------------------------------------------------------------------

function methods:monitorCatalogSyncState( catalog )

	if self._currentCatalogSyncStateB then
		self._currentCatalogSyncStateB:close()
		self._currentCatalogSyncStateB = false
	end
	
	local canCancelImportB = self:isImportingB()
	local isUploadingToOzB = catalog:isUploadingToOzB()
	local isDownloadingFromOzB = catalog:isDownloadingFromOzB()
	
	self._currentCatalogSyncStateB = KSReactive.callB( function( isUploading, isDownloading, isImporting )
		local hasTask = isUploading or isDownloading or isImporting
		
		-- set sync state, I'm sure there's a better Lua way to do this...
		-- no syncing
		local syncState = 0
		
		if isImporting then
			-- importing photos
			syncState = 1
		elseif isUploading then
			-- uploading photos/metadata
			syncState = 2
		elseif isDownloading then
			-- downloading photos/metadata
			syncState = 3
		end
		
		local result = {}
		result[ "eventId" ] = self._currentCatalogId
		result[ "hasTask" ] = not not hasTask
		result[ "syncState" ] = syncState
				
		REAppLuaInterface.performResultCallback( "updateCatalogSyncState", result )

	end,
	isUploadingToOzB, 
	isDownloadingFromOzB, 
	canCancelImportB )
	:addCloseTarget( isUploadingToOzB )
	:addCloseTarget( isDownloadingFromOzB )
	:addCloseTarget( canCancelImportB )

	if self._statB then
		self._statB:close()
		self._statB = false
	end
	self._statB = catalog:uploadStatusB()
	self._statB:onDataB( function(string, count)
			local numNotifications = 0
			if string == 'originals' then
				numNotifications = (self._prevOriginalLeft or 0) - (count or 0)
				self._prevOriginalLeft = count
				if numNotifications > 0 then
					debug.print('#&^@*%$&^%#@ upload done')
				end
			elseif string == 'idle' and self._prevOriginalLeft == 1 then
				numNotifications = 1
			end
			local result = {}
			local eventId = catalog._catalogService._catalogId
			result['eventId'] = eventId
			for i = 1,numNotifications do
				REAppLuaInterface.performResultCallback( "importDone", result )
			end
		end
		)

	if self._initialSyncB then
		self._initialSyncB:close()
		self._initialSyncB = false
	end

	self._initialSyncB = catalog:hasCompletedInitialSyncB()
	self._initialSyncB:onDataB( function(hasCompletedIntialSync)
		local result = {}
		result['eventId'] = catalog._catalogService._catalogId
		result['hasCompletedIntialSync'] = hasCompletedIntialSync
		REAppLuaInterface.performResultCallback( 'hasCompletedIntialSync', result )

	end
		)

			
end

--------------------------------------------------------------------------------

function methods:updateCatalogUsers( --[[OzCatalogService]] catalog )

    WFArguments.check( REScriptArg, 'OzCatalogService' )
	
	catalog:catalogUsersP():whenResolvedDo( function( status, users )
	
		if status ~= 'error' then
			
			-- we were able to get a list of users
			self._failedUserListCatalog = false
			
			if users then
			
				local userNames = {}
				local userIds = {}
				local userEmails = {}
				local userRoles = {}
				local userStatus = {}

				for j = 1, #users do
					-- debug.print( "full name:", users[ j ].full_name .. ", " .. users[ j ].email )
					userNames[ j ] = users[ j ].full_name or ""
					userIds[ j ] = users[ j ]._id or ""
					userEmails[ j ] = users[ j ].email or ""
					userRoles[ j ] = users[ j ].role or ""
					userStatus[ j ] = users[ j ].status or ""
				end
				
				local result = {}
				result[ "catalogId" ] = catalog:getCatalogId()
				result[ "userNames" ] = userNames
				result[ "userIds" ] = userIds
				result[ "userEmails" ] = userEmails
				result[ "userRoles" ] = userRoles
				result[ "userStatus" ] = userStatus
				
				REAppLuaInterface.performResultCallback( "updateEventUsers", result )
				
			end
		else
			-- getting a list of users failed
			self._failedUserListCatalog = catalog
			
			local result = {}
			result[ "catalogId" ] = catalog:getCatalogId()
			-- return the error, users in this case is the error message
			result[ "error" ] = users

			REAppLuaInterface.performResultCallback( "updateEventUsers", result )
		end
		
	end ):runUntilDone()

end


--------------------------------------------------------------------------------

function methods:updateCatalogCollaboratorQuota( --[[OzCatalogService]] catalog )

    WFArguments.check( REScriptArg, 'OzCatalogService' )
	
	self._catalogUserLimitB = catalog:collaboratorQuotaB()
	self._catalogUserLimitB:onDataB( function( numUsersLeft )

		if numUsersLeft then
			local result = {}
			result[ "catalogId" ] = catalog:getCatalogId()
			result[ "eventUsersLeft" ] = numUsersLeft
			REAppLuaInterface.performResultCallback( "updateEventUsersLeft", result )
			
			
		end
		
	end )

	catalog:addCloseTarget( self._catalogUserLimitB )
end

--------------------------------------------------------------------------------

-- @mark Events

--------------------------------------------------------------------------------

--------------------------------------------------------------------------------

--------------------------------------------------------------------------------

function methods:setDefaultAlbum( --[[OzUser]] ozUser, --[[OzCatalogService]] catalog,  --[[string]] eventId, --[[string]] albumId )

	WFArguments.check( REScriptArg, 'OzUser', 'OzCatalogService', 'string', 'string' )

	local resultP = catalog._docStore:applyOverlayToDocumentP {
		docType = 'catalog',
		docId = eventId,
		overlay = {
			payload = { ['X-Event-DefaultAlbumId' ] = albumId }
		},
	}

	resultP:runUntilDone()
	

	-- [2012-11-02 tdonahue] we need to manually sync with the server since the sync on a catalog is not supported
	
	local catalogUrl = string.format( '%s/%s', OzCatalogUrls.catalogsUrl, eventId )

	local content = KSJsonEncoder.encodeOzUploadJson { payload = { [ 'X-Event-DefaultAlbumId' ] = albumId } } 

	local postArgs = {
		requestHeaders = {
		{ 
			header = 'Content-Type', value = 'application/json' },
		},
		requestMethod = 'POST',
	}
	
	resultP = ozUser:putP( catalogUrl, content, postArgs )
		:mapDataP( function( status, data, headers )
			--debug.print( data )
			return 'data', true
		end )

	resultP:runUntilDone()

end

--------------------------------------------------------------------------------
-- This will share the specified album of the event and return the URL of the webshare

function methods:shareAlbum( ozUser, eventId, albumId )

	WFArguments.check( REScriptArg, 'OzUser', 'string', 'string' )
	
	self:setCurrentCatalog( ozUser, eventId )
	
	local resultP = self._currentCatalog:shareAlbumP( albumId, 'grouppix' ):mapDataP( function( status, data )
		
		local allowDownloadsP = self._currentCatalog:setAllowDownloadsForAlbumP( albumId, false )
		
		-- data will be true if the call succeeded for allow downloads
		return allowDownloadsP:mapDataP( function( status, data )

			-- get the URL for the shared album		
			return self._currentCatalog:sharedAlbumUrlP( albumId, true ):mapDataP( function ( status, url )
			
					local result = {}
					result[ "eventId" ] = eventId
					result[ "albumId" ] = albumId
					result[ "url" ] = url
					result[ "shared" ] = true
					result[ "allowsDownloads"  ] = not data
	
					REAppLuaInterface.performResultCallback( "updateSharedAlbum", result )
					
					
					return status, url
	
				end)
		end)
		
	end)
	
	resultP:runUntilDone()
	
end

--------------------------------------------------------------------------------
-- This will share the specified album of the event and return the URL of the webshare

function methods:unShareAlbum( ozUser, eventId, albumId )

	WFArguments.check( REScriptArg, 'OzUser', 'string', 'string' )
	
	self:setCurrentCatalog( ozUser, eventId )
	
	local resultP = self._currentCatalog:unshareAlbumP( albumId ):mapDataP( function( status, data )
	
		local result = {}
		result[ "eventId" ] = eventId
		result[ "albumId" ] = albumId
		result[ "shared" ] = false
		REAppLuaInterface.performResultCallback( "updateSharedAlbum", result )
		
		
	end)
	
	resultP:runUntilDone()
	
end


--------------------------------------------------------------------------------
-- This will determine if the specified album of the event is shared and return the URL of the webshare if so

function methods:isSharedAlbum( ozUser, eventId, albumId, checkServer )

	WFArguments.check( REScriptArg, 'OzUser', 'string', 'string' )
	
	self:setCurrentCatalog( ozUser, eventId )
	
	local resultP = self._currentCatalog:isAlbumSharedP( albumId ):mapDataP( function( status, data )
			
		if data == true then
			
			-- album is shared get the URL
			local shareUrlP = self._currentCatalog:sharedAlbumUrlP( albumId, checkServer )
			
			return shareUrlP:mapP( function ( status, url )
			
				if status == 'data' then
				
					return self._currentCatalog:getAllowDownloadsForAlbumP( albumId )
						:mapDataP( function( status, allowsDownloads )
							local result = {}
							result[ "eventId" ] = eventId
							result[ "albumId" ] = albumId
							result[ "url" ] = url
							result[ "shared" ] = true
							result[ "allowsDownloads"  ] = allowsDownloads
							REAppLuaInterface.performResultCallback( "updateSharedAlbum", result )
							
							
							return status, allowsDownloads
						end)
				else
					local result = {}
					result[ "eventId" ] = eventId
					result[ "albumId" ] = albumId
					result[ "error" ] = url
					result[ "shared" ] = true
					result[ "allowsDownloads"  ] = false

					REAppLuaInterface.performResultCallback( "updateSharedAlbum", result )
					
					
					return status, nil
				end
				
			end)
		else
			-- the album is not shared
			
			local result = {}
			result[ "eventId" ] = eventId
			result[ "albumId" ] = albumId
			result[ "shared" ] = false
			result[ "allowsDownloads"  ] = false

			REAppLuaInterface.performResultCallback( "updateSharedAlbum", result )
			
		end
		
	end)
	
	resultP:runUntilDone()
	
end

--------------------------------------------------------------------------------
-- This will set whether or not downloads of photos are allowed for the shared album

function methods:sharedAlbumAllowDownloads( ozUser, eventId, albumId, allowDownloads )

	WFArguments.check( REScriptArg, 'OzUser', 'string', 'string', 'boolean' )
	
	self:setCurrentCatalog( ozUser, eventId )
	
	-- set allow downloads for this shared album
	
	local resultP = self._currentCatalog:setAllowDownloadsForAlbumP( albumId, allowDownloads )
		:mapDataP( function( status, data )

		-- get the URL for the shared album
		local shareUrlP = self._currentCatalog:sharedAlbumUrlP( albumId, true )
		
		return shareUrlP:mapDataP( function ( status, url )
		
			local result = {}
			result[ "eventId" ] = eventId
			result[ "albumId" ] = albumId
			result[ "url" ] = url
			result[ "shared" ] = true
			result[ "allowsDownloads"  ] = allowDownloads
	
			REAppLuaInterface.performResultCallback( "updateSharedAlbum", result )
			
			
			return status, allowDownloads
			
		end)
	
	end)
		
	resultP:runUntilDone()
	
end

--------------------------------------------------------------------------------
-- This command deletes the event 
function methods:removeEvent( --[[OzUser]] ozUser, --[[string]] eventId, --[[string]] pinCode )
	
	-- pin code can be null, don't test for it
	WFArguments.check( REScriptArg, 'OzUser', 'string' )

	self._deleteEventFlag = true
	
	self:setCurrentCatalog( ozUser, eventId )
	
	ozUser:requestCatalogReload()
			
	local function handleReturn( status, data, headers )
	
		self:setCurrentCatalog( ozUser, nil )
		
		if status == 'data' then
			self._deleteEventFlag = false
			local result = {}
			result[ "eventId" ] = eventId
			REAppLuaInterface.performResultCallback( "deleteEvent", result )
			
		else
			
			local h = KSJsonDecoder.decodeWebSafeJson( headers )
			local reason = h.errors.catalog[1]
			debug.print( "removeEvent: reason = " .. reason)
			
			if reason == "can not be deleted since it is the last one for this account" then
				debug.print( "removeEvent: failed due to being last event." )
				self:checkForDefaultEvent(ozUser)
				return
			end
			
			self._deleteEventFlag = false
			
			local result = {}
			result[ "eventId" ] = eventId
			result[ "error" ] = data		-- http status code
			REAppLuaInterface.performResultCallback( "deleteEvent", result )
			
		end
	end
	
	
	self._defaultEventB:onDataB( function(data)
		
		if data == false then
			return
		end
		
		if self._deleteEventFlag == false then
			return
		end
		
		if self._currentCatalogId == false or self._currentCatalogId ~= eventId then
			self:setCurrentCatalog( ozUser, eventId )
		end

		self._currentCatalog:setPurgeEnabled( false )
		self._currentCatalog:catalogOwnerIdP():whenResolvedDo( function( status, ownerId )
			if status == 'data' then
				if ozUser:id() == ownerId then
					debug.print("Try to delete catalog")
					ozUser:deleteCatalogP( eventId, self._currentCatalog ):whenResolvedDo( handleReturn ):runUntilDone()
					
					if pinCode and string.len( pinCode ) > 0 then
						groupPixPin:deletePinForCatalog( ozUser, self._currentCatalog, pinCode )
					end
				else
					local emailAddress = ozUser:emailB():valueNow()
					self._currentCatalog:removeUserP_knownNonOwner( emailAddress ):whenResolvedDo( handleReturn ):runUntilDone()
				end
			end
		end ):runUntilDone()
				
	end)


end


--------------------------------------------------------------------------------
	
function methods:renameEvent( --[[OzUser]] ozUser, --[[string]] eventId, --[[string]] newEventName, --[[string]] defaultAlbumId )

	WFArguments.check( REScriptArg, 'OzUser', 'string', 'string' )

	self:setCurrentCatalog( ozUser, eventId )

    self._currentCatalog:renameCatalogP( newEventName ):whenResolvedDo( function(status, data)
        if status ~= 'error' then

			local result = {}
			result[ "eventId" ] = eventId
			result[ "eventName" ] = newEventName
            REAppLuaInterface.performResultCallback( "renameEvent", result )
			self._currentCatalog:renameAlbumP( defaultAlbumId, newEventName, true )
			
			ozUser:requestCatalogReload()
        end
    end):runUntilDone()
		
end

--------------------------------------------------------------------------------

function methods:updateEventDates( --[[OzUser]] ozUser, --[[string]] eventId, --[[string]] startDate, --[[string]] endDate )

	WFArguments.check( REScriptArg, 'OzUser', 'string', 'string', 'string' )

	self:setCurrentCatalog( ozUser, eventId )
		
	local resultP = self._currentCatalog._docStore:applyOverlayToDocumentP {
		docType = 'catalog',
		docId = eventId,
		overlay = {
			payload = {
				['start_date' ] = startDate,
				['end_date' ] = endDate,
			}
		},
	}

	resultP:runUntilDone()


	-- [2012-12-03 tdonahue] we need to manually sync with the server since the sync on a catalog is not supported
	
	local catalogUrl = string.format( '%s/%s', OzCatalogUrls.catalogsUrl, eventId )

	local content = KSJsonEncoder.encodeOzUploadJson {
		payload = {
			['start_date' ] = startDate,
			['end_date' ] = endDate,
		}
	} 

	local postArgs = {
		requestHeaders = {
		{ 
			header = 'Content-Type', value = 'application/json' },
		},
		requestMethod = 'POST',
	}
	
	resultP = ozUser:putP( catalogUrl, content, postArgs )
		:mapDataP( function( status, data, headers )
			--debug.print( data )
			REAppLuaInterface.performResultCallback( "updateDatesEvent", nil )
			return 'data', true
		end )

	resultP:runUntilDone()

end

--------------------------------------------------------------------------------

function methods:updateBroadcastParams( --[[OzUser]] ozUser, --[[string]] eventId, --[[string]] broadcast, --[[string]] broadcastType, --[[string]] broadcastAutoJoin )

	WFArguments.check( REScriptArg, 'OzUser', 'string', 'string', 'string', 'string' )

	self:setCurrentCatalog( ozUser, eventId )
		
	local resultP = self._currentCatalog._docStore:applyOverlayToDocumentP {
		docType = 'catalog',
		docId = eventId,
		overlay = {
			payload = {
				[ 'X-Event-Broadcast' ] = broadcast,
				[ 'X-Event-BroadcastType' ] = broadcastType,
				[ 'X-Event-BroadcastAutoJoin' ] = broadcastAutoJoin
			}
		},
	}

	resultP:runUntilDone()


	-- [2012-12-03 tdonahue] we need to manually sync with the server since the sync on a catalog is not supported
	
	local catalogUrl = string.format( '%s/%s', OzCatalogUrls.catalogsUrl, eventId )

	local content = KSJsonEncoder.encodeOzUploadJson {
		payload = {
			[ 'X-Event-Broadcast' ] = broadcast,
			[ 'X-Event-BroadcastType' ] = broadcastType,
			[ 'X-Event-BroadcastAutoJoin' ] = broadcastAutoJoin
		}
	} 

	local postArgs = {
		requestHeaders = {
		{ 
			header = 'Content-Type', value = 'application/json' },
		},
		requestMethod = 'POST',
	}
	
	resultP = ozUser:putP( catalogUrl, content, postArgs )
		:mapDataP( function( status, data, headers )
			--debug.print( data )
			return 'data', true
		end )

	resultP:runUntilDone()

end

--------------------------------------------------------------------------------
function methods:canAccessB()

	if self._catalogAccessRequestB then
		self._catalogAccessRequestB:close()
		self._catalogAccessRequestB = false
	end
	
	if not self._currentCatalog then
		return
	end
	
	self._catalogAccessRequestB = self._currentCatalog:canAccessB()
			
	self._catalogAccessRequestB:onDataB( function( canAccess, reason )
		
		local accessStatus = (reason == 'noAccess') or (reason == 'doesNotExist')
		if canAccess == false and accessStatus then
			local ozUser = OzAccountService.currentUserB():valueNow()
			ozUser:canAccessP( self._currentCatalogId  ):whenResolvedDo( function( status, canAccess, reason )
				if canAccess == false and reason == 'noAccess' then
					ozUser:requestSharedCatalogReload()
					ozUser:requestCatalogReload()
	
					local result = {}
					result[ "eventId" ] = self._currentCatalogId
					REAppLuaInterface.performResultCallback( "noCatalogAccess", result )
				end
			end):runUntilDone()
		end

		--debug.print("methods:canAccessB() canAccess = ", canAccess)
		--debug.print("methods:canAccessB() reason = ", reason)

	end )
	
	self._currentCatalog:addCloseTarget( self._catalogAccessRequestB )

end
--------------------------------------------------------------------------------

function methods:addEventInvitees( --[[OzUser]] ozUser, --[[string]] eventId, --[[string]] emailInviteList, --[[boolean]] sendInvite )

	WFArguments.check( REScriptArg, 'OzUser', 'string', 'string', 'boolean' )

	self:setCurrentCatalog( ozUser, eventId )

	local emailAddresses
	
	if string.len( emailInviteList ) > 0 then
		emailAddresses = KSStringUtils.split(emailInviteList, ",")
	end

	local resultP = self._currentCatalog:inviteUsersP( emailAddresses, sendInvite ):whenResolvedDo(
		function( status, data )
		
			ozUser:requestCatalogReload()
		
		end )
	
	resultP:runUntilDone()

end


--------------------------------------------------------------------------------

function methods:removeEventInvitee( --[[OzUser]] ozUser, --[[string]] eventId, --[[string]] removedUser )

	WFArguments.check( REScriptArg, 'OzUser', 'string', 'string' )

	self:setCurrentCatalog( ozUser, eventId )

	local resultP = self._currentCatalog:removeUserP( removedUser, true ):whenResolvedDo(
		function( status, data )
		
			ozUser:requestCatalogReload()
		
		end )
	
	resultP:runUntilDone()

end

--------------------------------------------------------------------------------

-- @mark Thumbnails


--------------------------------------------------------------------------------

function methods:garbageCollect()
    collectgarbage()
    collectgarbage()
end

--------------------------------------------------------------------------------

local function tableLength(T)
  local count = 0
  for _ in pairs(T) do count = count + 1 end
  return count
end

--------------------------------------------------------------------------------
function methods:closeThumbnailBehavior( photoId )


    if self._thumbnailBehaviors[ photoId ] then
        reThumbLog:debug( "methods:closeThumbnailBehavior for photoID = %s", photoId )
        self._thumbnailBehaviors[ photoId ]:close()
        self._thumbnailBehaviors[ photoId ] = nil
		
		reThumbLog:debug( "close thumbnail behaviors=", tableLength( self._thumbnailBehaviors ) )

        local result = {}
        result[ "photoId" ] = photoId
        REAppLuaInterface.performResultCallback( "removeRequestedThumbnail", result )
        
    end

	--TODO: Tornado iOS should remove the following code since there are other commands that close these B's:
	if not WFPlatformInfo.isAndroid then
	    if self._getCommentListBehaviors[ photoId ] then
    	    reThumbLog:debug("methods:closeCommentBehavior for photoID = %s", photoId )

        	self._getCommentListBehaviors[ photoId ]:close()
	        self._getCommentListBehaviors[ photoId ] = nil
    	end

    	if self._favoriteBehaviors[ photoId ] then
        	reThumbLog:debug("methods:closeFavoriteBehavior for photoID = %s", photoId )
        	self._favoriteBehaviors[ photoId ]:close()
        	self._favoriteBehaviors[ photoId ] = nil
    	end
	end     
end

--------------------------------------------------------------------------------
function methods:getThumbnail( ozUser, catalogId, albumId, photoId, thumbnailMD5 )

	if ozUser and catalogId and photoId then

		self:setCurrentCatalog( ozUser, catalogId )


 --[[       
		-- nicholas 6-14-2013 we are closing our thumb behaviors manually (and often)
		-- i have spoken to mscarafo and we thought we would pull this code out and 
		-- let it run like it used to run to see what happens.  We have newer WF code
		-- and the original problem may not exist anymore
		
		-- mscarafo 3/2/12
        -- We used to keep the behavior alive so as to get then
        -- larger thumbnail. I don't think this is needed because if a user
        -- isn't scrolling, the behavior will stay alive long enough to get the largr thumbnail

        if self._thumbnailBehaviors[ photoId ] then
            self._thumbnailBehaviors[ photoId ]:close()
            self._thumbnailBehaviors[ photoId ] = nil
        end

--]]

        if not self._thumbnailBehaviors[ photoId ] or WFCloseSupport.hasBeenOrWillBeClosed( self._thumbnailBehaviors[ photoId ] ) then
		--	self._thumbnailBehaviors[ photoId ] = self._currentRenditionServer:thumbnailB( photoId, wantBigThumbnails, thumbnailMD5 )
			self._thumbnailBehaviors[ photoId ] = self._currentRenditionServer:thumbnailB( photoId, wantBigThumbnails )

			reThumbLog:debug( "create ThumbnailBehavior for photoID = %s", photoId )
			reThumbLog:debug( "thumbnail behaviors=", tableLength( self._thumbnailBehaviors ) )

        	self._currentCatalog:addCloseTarget( self._thumbnailBehaviors[ photoId ] )
        end

		self._thumbnailBehaviors[ photoId ]:onDataB( function( thumbnail )
            if thumbnail ~= nil then
                reThumbLog:debug( " got ThumbnailBehavior for photoID = %s", photoId )

				local result = {}
				result[ "thumbnail" ] = thumbnail
				result[ "photoId" ] = photoId
				result[ "eventId" ] = catalogId
				REAppLuaInterface.performResultCallback( "updateThumbnail", result )
				
				-- Since we are only retrieving the big thumbnails on Android, we don't need to keep this B open...
				if WFPlatformInfo.isAndroid and self._thumbnailBehaviors[ photoId ] then
					self._thumbnailBehaviors[ photoId ]:close()
					self._thumbnailBehaviors[ photoId ] = nil
				end
			end
		end )

    end
end

--------------------------------------------------------------------------------
-- Is Oz online...
function methods:serviceAvailable()

	if self._serviceAvailableB then
		self._serviceAvailableB:close()
		self._serviceAvailableB = false
	end
	
	if self._currentCatalog then
		self._serviceAvailableB = self._currentCatalog.serviceAvailableB()
	
		self._serviceAvailableB:onDataB( function(isAvailable)
			local result = {}
			if isAvailable then
				result[ "serviceAvailable" ] = '1'
			else
				result[ "serviceAvailable" ] = '0'
			end
		
			REAppLuaInterface.performResultCallback( "updateServiceAvailable", result )
		
		end)
	end
end

--------------------------------------------------------------------------------
function methods:hostIsReachable()

    self._ozIsReachableB = WFReachability.hostIsReachableB( OzCatalogUrls.baseServerUrl )
    self._ozIsReachableB:onDataB( function( newStatus )
	
		local result = {}

		if newStatus == 'unreachable'  then
			result[ "reachable" ] = '0'
		else
			result[ "reachable" ] = '1'
		end		    

        REAppLuaInterface.performResultCallback( "updateConnectionStatus", result )
		

		if self._imsAuthUrlFailed and newStatus ~= 'unreachable' then
			-- try and get the ims url info again
			self:getImsAuthInfo()	
		end
		
		if self._failedUserListCatalog and newStatus ~= 'unreachable' then
			-- request a catalog reload
			self:updateCatalogUsers( self._failedUserListCatalog )
			self._failedUserListCatalog = false
		end
			
	end )

end

--------------------------------------------------------------------------------

function methods:setIdleTimeB()

	if self._idleTimerDisabledB then
		self._idleTimerDisabledB:close()
		self._idleTimerDisabledB = false
	end

	if self._currentCatalog then
		local isUploadingToOzB = self._currentCatalog:isUploadingToOzB()

		local isDownloadingFromOzB = self._currentCatalog:isDownloadingFromOzB()
	
		self._idleTimerDisabledB = KSReactive.callB( function( isUploading, isDownloading )
			return isUploading or isDownloading
		end, isUploadingToOzB, isDownloadingFromOzB )
	
		self._idleTimerDisabledB.addCloseTarget( isUploadingToOzB )
		self._idleTimerDisabledB.addCloseTarget( isDownloadingFromOzB )
	
		self._idleTimerDisabledB:onDataB( function( disabled )

			local result = {}
			if disabled == true then
				result[ "disable" ] = '1'
			else
				result[ "disable" ] = '0'
			end

			REAppLuaInterface.performResultCallback( "disableIdleTimer", result )
		    
		end )
	
		self._currentCatalog:addCloseTarget( self._idleTimerDisabledB )
	end
	
end

--------------------------------------------------------------------------------
function methods:getImage( ozUser, catalogId, photoId, renditionName )	-- Rendition name is one of: 'preview', 'thumbnail', or 'thumbnail2x'

	local behavName = renditionName .. photoId
	if self._imageBehaviors[ behavName ] and not WFCloseSupport.hasBeenOrWillBeClosed( self._imageBehaviors[ behavName ] ) then
		return
	end

	self._imageBehaviors[ behavName ] = KSReactive.variableB( 0 )

	if ozUser and catalogId then
		
		self:setCurrentCatalog( ozUser, catalogId )
		
		local imageP = self._currentRenditionServer:encodedRenditionP( photoId, renditionName, true )
		
		imageP:whenResolvedDo( function( status, encodedImage )
					
			local result = {}
			result[ "photoId" ] = photoId
			result[ "eventId" ] = catalogId
			result[ "rendition" ] = renditionName
 			result[ "hasImage" ] = true

			if status == 'data' and type( encodedImage ) == 'userdata' then
				result[ "image" ] = encodedImage
		
			else
				if encodedImage == "encodedRendition: no rendition available either locally or on server" then
					result[ "hasImage" ] = false
				end

                result[ "error" ] = encodedImage
			end

			REAppLuaInterface.performResultCallback( "updateImage", result )
								
			self._imageBehaviors[ behavName ]:close()
			self._imageBehaviors[ behavName ] = nil
				
		end ):attachTo( self._imageBehaviors[ behavName ] )
	end
end

--------------------------------------------------------------------------------
function methods:cancelImage( photoId, renditionName )

	local behavName = renditionName .. photoId
	if self._imageBehaviors[ behavName ] and not WFCloseSupport.hasBeenOrWillBeClosed( self._imageBehaviors[ behavName ] ) then
		self._imageBehaviors[ behavName ]:close()
	end
	self._imageBehaviors[ behavName ] = nil
	
end

--------------------------------------------------------------------------------
function methods:getPreview( ozUser, catalogId, photoId )

	if ozUser and catalogId then
		
		self:setCurrentCatalog( ozUser, catalogId )
		
		--[[ mscarafo 3/19/2013
			Bug: 3521446
			Make sure we have access to the catalog before we request the preview
		--]]
		
		ozUser:canAccessP( catalogId ):whenResolvedDo( function( status, data, access )
			
			if status == 'data' and data == true then
				local previewP = self._currentRenditionServer:encodedPreviewP( photoId , true )
		
				previewP:whenResolvedDo( function( status, encodedPreview )
					if status == 'data' and type( encodedPreview ) == 'userdata' then
		               -- debug.print("methods:getPreview got preview")
						local result = {}
		                result[ "error" ] = false
						result[ "preview" ] = encodedPreview
						result[ "photoId" ] = photoId
						result[ "eventId" ] = catalogId
						REAppLuaInterface.performResultCallback( "updatePreview", result )
						
		
					else
						local ozIsReachable = WFReachability.hostIsReachableB( OzCatalogUrls.baseServerUrl ):valueNow()
						
						--debug.print( "Revel Event Preview: Unable to retrieve image preview. Reachable:" .. ozIsReachable)
		                --debug.print( "Revel Event Preview: status = ", status)
		                --debug.print( "Revel Event Preview: encodedPreview = ", encodedPreview)
		
						local hasPreview = true
						
						if encodedPreview == "encodedRendition: no rendition available either locally or on server" then
							hasPreview = false
						end
		                local result = {}
		                result[ "error" ] = true
                        result[ "errortype" ] = encodedPreview
		                result[ "photoId" ] = photoId
						result[ "eventId" ] = catalogId
		 				result[ "hasPreview" ] = hasPreview
		                REAppLuaInterface.performResultCallback( "updatePreview", result )
		                
		
		               -- debug.print("status = ", status)
		                --debug.print("encodedPreview = ", encodedPreview)
		
						if not ozIsReachable then
							return 'error', 'NoNetwork'
						else
							return 'error', 'PreviewUnavailable'
						end
					end
				
				end ):runUntilDone()
			else
				if data == false then
					--debug.print("methods:getPreview catalog is not accessible!")
					local result = {}
		            result[ "error" ] = true
                    result[ "errortype" ] = "catalog is not accessible"
		            result[ "photoId" ] = photoId
					result[ "eventId" ] = catalogId
		            REAppLuaInterface.performResultCallback( "updatePreview", result )
				end
			end
		
		end ):runUntilDone()
		
	end
end


--------------------------------------------------------------------------------
function methods:getOriginal( ozUser, catalogId, photoId )

	WFArguments.check( REScriptArg, 'OzUser', 'string', 'string' )

	if self._originalImageBehaviors[photoId] and not WFCloseSupport.hasBeenOrWillBeClosed(self._originalImageBehaviors[photoId]) then
		self._originalImageBehaviors[photoId]:close()
		self._originalImageBehaviors[photoId] = nil;
	end

	self._originalImageBehaviors[photoId] = KSReactive.variableB(0)
	
	if ozUser and catalogId then
		
		self:setCurrentCatalog( ozUser, catalogId )
		
        local originalP = self._currentCatalog:originalDataPathP( photoId, false )

        originalP = originalP:mapDataP(
        	function(status, pathToOriginal)
        		local catalogService = OzCatalogService( catalogId, ozUser)
				local docStore = catalogService._docStore
				local docValueP = docStore:docValueOnceP('asset', photoId)
				local newValueP = docValueP:mapP(
				function ( status, docValue )
					local result = {}
					if status == 'data' then
						local develop = docValue.develop or {}
						result["status"] = "data"
						result["xmpBase"] = develop.xmpBase or ""
						result["xmpLook"] = develop.xmpLook or ""
						result["userOrientation"] = develop.userOrientation or 1
						
						local contentTypeNumber = 0
				
						local contentType = derefOpt( docValue, 'importSource', 'contentType' )
							or derefOpt( docValue, '_serverProvidedMetadata', 'links', '/rels/master', 'content_type' )
						if contentType then
							contentTypeNumber = WFOzContentTypes.getContentTypeNumberForContentType( contentType )
						else
							local subtype = derefOpt( docValue, '_serverProvidedMetadata', 'subtype' )
							contentTypeNumber = WFOzContentTypes.getContentTypeNumberRangeForSubType( subtype )
						end
						
						if contentTypeNumber == 0 then
							contentTypeNumber = 1001
						end
						
						contentType = WFOzContentTypes.contentTypeForContentTypeNumber(contentTypeNumber, false)
						
						result["contentType"] = contentType
										
					else
						result["status"] = "error"
					end
					-- REAppLuaInterface.performResultCallback("updateEditList", result)
					return status, pathToOriginal, result

				end
				)
				return newValueP
        	end
        	)

		originalP:whenResolvedDo( function( status, pathToOriginal, result )
			
			if status == 'data' then
				result[ "originalPath" ] = pathToOriginal
				result[ "photoId" ] = photoId
				result[ "eventId" ] = catalogId
				REAppLuaInterface.performResultCallback( "updateOriginal", result )
                

			else
			
			    local result = {}
				-- pathToOriginal will hold the error message
                result[ "error" ] = pathToOriginal
                result[ "photoId" ] = photoId
				result[ "eventId" ] = catalogId

                REAppLuaInterface.performResultCallback( "updateOriginal", result )
                

			end
			
		end ):attachTo(self._originalImageBehaviors[photoId])
	end
end

function methods:cancelOriginal( photoId)
	if self._originalImageBehaviors[photoId] and not WFCloseSupport.hasBeenOrWillBeClosed(self._originalImageBehaviors[photoId]) then
		self._originalImageBehaviors[photoId]:close()
	end
	self._originalImageBehaviors[photoId] = nil;
end

--------------------------------------------------------------------------------

-- @mark Favorites

--------------------------------------------------------------------------------

function methods:getMyFavoriteForAssetId( --[[OzUser]] ozUser, --[[string]] catalogId, --[[string]] photoId )

	WFArguments.check( REScriptArg, 'OzUser', 'string', 'string' )

    if self._favoriteBehaviors[ photoId ] and not WFCloseSupport.hasBeenOrWillBeClosed( self._favoriteBehaviors[ photoId ] ) then
        return
    end

    self:setCurrentCatalog( ozUser, catalogId )

	local favoriteB = self._currentCatalog:isMyFavoriteB( photoId )
	self._favoriteBehaviors[ photoId ] = favoriteB

    favoriteB:onDataB( function( isMyFav, isExisting )

		if isExisting ~= nil then
	        local myFav = "false"
	        local result = {}
	        
	        if isMyFav == true then
	            myFav = "true"
	        end
	
			result[ "eventId" ] = catalogId
			result[ "photoId" ] = photoId
	        result[ "myFavorite" ] = myFav
	        REAppLuaInterface.performResultCallback( "updateMyFavorites", result )
	        
		end
		
    end )

	self._currentCatalog:addCloseTarget( self._favoriteBehaviors[ photoId ] )

end

--------------------------------------------------------------------------------

function methods:getAllFavoritesForAssetId( --[[OzUser]] ozUser, --[[string]] catalogId, --[[string]] photoId )

	WFArguments.check( REScriptArg, 'OzUser', 'string', 'string' )

    self:setCurrentCatalog( ozUser, catalogId )

	if not self._allFavoritesBehaviours[ photoId ] or WFCloseSupport.hasBeenOrWillBeClosed( self._allFavoritesBehaviours[ photoId ] ) then
		self._allFavoritesBehaviours[ photoId ] = self._currentCatalog:favoritedByB( photoId )
		self._currentCatalog:addCloseTarget( self._allFavoritesBehaviours[ photoId ] )
	end

    self._allFavoritesBehaviours[ photoId ]:onDataB( function( users, isExisting )
		if isExisting ~= nil then
	        local result = {}

			local userList = ''
			for j = 1, #users do
		        local userId  = users[ j ] or ''
				if j == 1 then
	    	    	userList = userId
				else
					userList = userList .. '|' .. userId
				end
			end
	        
			result[ "eventId" ] = catalogId
			result[ "photoId" ] = photoId
	        result[ "users" ] = userList
	        REAppLuaInterface.performResultCallback( "updateFavoritedUsers", result )       
		end
	end )

end

--------------------------------------------------------------------------------

function methods:closeFavoritesForAssetId( --[[OzUser]] ozUser, --[[string]] catalogId, --[[string]] photoId )

	WFArguments.check( REScriptArg, 'OzUser', 'string', 'string' )

    if self._allFavoritesBehaviours[ photoId ] and not WFCloseSupport.hasBeenOrWillBeClosed( self._allFavoritesBehaviours[ photoId ] ) then
		self._allFavoritesBehaviours[ photoId ]:close()
        self._allFavoritesBehaviours[ photoId ] = nil
    end

end

--------------------------------------------------------------------------------

function methods:setLiked( --[[OzUser]] ozUser, --[[string]] catalogId, --[[string]] photoId, --[[number]]liked )

	WFArguments.check( REScriptArg, 'OzUser', 'string', 'string', 'boolean' )
	
	self:setCurrentCatalog( ozUser, catalogId )
		
	self._currentCatalog:setMyFavoriteP( photoId, liked )
		:whenResolvedDo( function( status, revisionID )
		
			if status == 'data' then
				-- after setting report back the new value 
				self:getMyFavoriteForAssetId( ozUser, catalogId, photoId )
			end
			
		end):runUntilDone()

end

--------------------------------------------------------------------------------

-- @mark Device Token

--------------------------------------------------------------------------------

function methods:deleteDeviceToken( --[[OzUser]] ozUser )

	WFArguments.check( REScriptArg, 'OzUser' )

	local resultP = ozUser:deleteDeviceTokenP( WFPlatformInfo.getAppIdentifier(), 'production' ):mapP( function( status, data )
    	local result = {}
    	
	if status == 'error' and data ~= 'no_token' then
        	result[ "result" ] = false
        	result[ "data" ] = data
    	else
        	result[ "result" ] = true
    	end
    	REAppLuaInterface.performResultCallback( "deviceTokenDelete", result )
	end )
	
	resultP:runUntilDone()

end

--------------------------------------------------------------------------------

function methods:registerDeviceToken( --[[string]] deviceToken )

    WFArguments.check( REScriptArg, 'userdata|KSData|string' )

    -- If the user is logged in, then register the device token now.
    -- If the user is not logged in, register when they do login...in watchCurrentUser().

    self._deviceToken = deviceToken

    local ozUser = OzAccountService.currentUserB():valueNow()

    if ozUser ~= nil then
        registerDeviceToken( ozUser, deviceToken )
        end

end

--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- For GroupPix 1.2, we're calling this with the sole purpose
-- of informing Oz of the communication_preference language, so
-- that Oz will send us properly-localized push notifications.
--------------------------------------------------------------------------------
function methods:listProducts( --[[OzUser]] ozUser )

	WFArguments.check( REScriptArg, 'OzUser' )

    ozUser:listAllProductsP( 'ios' )
        :whenResolvedDo( function( status, data )

            if status == 'data' then

                debug.print( "list all products succeeded" )

            else

                debug.print( string.format( "list all products failed with error %s",  data ) )

            end
        
        end ):runUntilDone()

end

--------------------------------------------------------------------------------

-- @mark Diagnostic Report

--------------------------------------------------------------------------------
function methods:generateDiagnosticReport()

	require 'WFDiagnosticLog'.generateHtmlReportP {
		appName = 'Tornado',
	}
		:whenResolvedDo( function( status, reportHtml, automationOutputOrNil )
		
			local KSPathUtils = require 'KSPathUtils'
			
			local directory = KSPathUtils.getStandardFilePath( WFPlatformInfo.isAndroid and 'home' or 'temp' )
			
			local result = {}
        
			if directory ~= '???' then
				local todaysDate = KSDate.timeInIso8601Format( nil, true ):sub( 1, 10 )
				local fileName = string.format( 'GroupPix Diagnostic Log %s.html', todaysDate )
			
				local targetPath = KSPathUtils.child( directory, fileName )
			
				local KSData = require 'KSData'
				local reportData = KSData.make( reportHtml )
			
				local resultPath = KSData.saveToUniquelyNamedFile( reportData, targetPath )
			
				result[ "reportPath" ] = resultPath
			end
			
			REAppLuaInterface.performResultCallback( "diagnosticReportGenerated", result )
			
			
		end )
		:runUntilDone()
end

--------------------------------------------------------------------------------

WFCloseSupport.addCloseSupportMethods( methods )

return KSClass 'REViewDefinition'( methods )

--------------------------------------------------------------------------------


