--[[----------------------------------------------------------------------------

WFAnalyticsServiceImp.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author allany 2012-01-19

------------------------------------------------------------------------------]]

	-- wichitafoundation.core.lua.threading
local WFCurrentThread = require 'WFCurrentThread'
local WFRequestDispatcher = require 'WFRequestDispatcher'

	-- services.analytics
local WFAnalyticsManager = require 'WFAnalyticsManager'

    -- wichitafoundation/core/utils
local KSTableUtils = require "KSTableUtils"

local WFLocalizationUtils = require "WFLocalizationUtils"

    -- core.os
local KSDate = require 'KSDate'

local WFPlatformInfo = require 'WFPlatformInfo'

--==============================================================================
--[[!
    \brief Implementation for analytics service. This file should not be accessed
    by any code other than WFAnalyticsService.
]]--

--------------------------------------------------------------------------------

WFCurrentThread.setThreadPriority( '< low >' )

--------------------------------------------------------------------------------
--[[!
    \brief Reads device class usage prefs, and returns a comma-separated list of
    device class names used by this user.

    \param ozAccountId
    The user's Oz account Id

    \return A comma-separated list of Revel device class names used
    by this user.
]]--

--local function makeDeviceClassList( ozAccountId )
--
--    -- If any device usage is older than three months, do not include it
--    local secondsInThreeMonths = 7776000
--    local nowTime = KSDate.timeInIso8601Format( nil, false )
--
--    local prefsTable = {}
--    prefsTable[ "computer" ] = KSPrefs[ "timestamp_computer_" .. ozAccountId ]
--    prefsTable[ "phone" ] = KSPrefs[ "timestamp_phone_" .. ozAccountId ]
--    prefsTable[ "tablet" ] = KSPrefs[ "timestamp_tablet_" .. ozAccountId ]
--    prefsTable[ "web" ] = KSPrefs[ "timestamp_web_" .. ozAccountId ]
--
--    local resultArray = {}
--    for key, value in pairs( prefsTable ) do
--        local diffTime = KSDate.differenceInSeconds( value, nowTime, false )
--        if diffTime < secondsInThreeMonths then
--            table.insert( resultArray, key )
--        end
--    end
--
--    -- Ensure alphabetical order
--    table.sort( resultArray )
--
--    return table.concat( resultArray, "," )
--
--end

--------------------------------------------------------------------------------

--local function currentDeviceClass()
--
--	if WFPlatformInfo.isiPhone then
--		return "phone"
--	elseif WFPlatformInfo.isiPad then
--		return "tablet"
--	else
--		return "computer"
--	end
--
--end

--------------------------------------------------------------------------------

local sMeasurement = "Not INITed"
local appStateProperties = {}

-- Initialize standard appState properties
if WFPlatformInfo.isTablet then
	appStateProperties.deviceClass = "tablet"
elseif WFPlatformInfo.isPhone then
	appStateProperties.deviceClass = "phone"
elseif WFPlatformInfo.isMac then
	appStateProperties.deviceClass = "mac"
elseif WFPlatformInfo.isDesktop then
	appStateProperties.deviceClass = "desktop"
end

if WFPlatformInfo.isiPhone then
	appStateProperties.currentlyUsedDevice = "iPhone"
elseif WFPlatformInfo.isiPad then
	appStateProperties.currentlyUsedDevice = "iPad"
elseif WFPlatformInfo.isiOS then
	appStateProperties.currentlyUsedDevice = "iOS"
elseif WFPlatformInfo.isAndroid then
	appStateProperties.currentlyUsedDevice = "android"
end

appStateProperties.locale = WFLocalizationUtils.getLocaleId()
appStateProperties.language = WFLocalizationUtils.getLanguageCode()

if WFPlatformInfo.isiOS then
	appStateProperties.appPlatform = "iOS"
elseif WFPlatformInfo.isAndroid then
	appStateProperties.appPlatform = "Android"
elseif WFPlatformInfo.isMac then
	appStateProperties.appPlatform = "MacOSX"
elseif WFPlatformInfo.isWin7 then
	appStateProperties.appPlatform = "Windows 7"
elseif WFPlatformInfo.isWinRT then
	appStateProperties.appPlatform = "Windows RT"
elseif WFPlatformInfo.isWindows then
	appStateProperties.appPlatform = "Windows"
end

--------------------------------------------------------------------------------

-- Sometimes we have events/evars that we also want to use as sprops. Rather than
-- record them twice, we can just record the event and sniff that for keys that
-- are also known to be sprops

local function updateKnownAppStateProperties( params )
	if params.deviceOrientation then
		appStateProperties.deviceOrientation = params.deviceOrientation
	end
end

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessageDo {

	message = 'init',
	simpleResponses = true,
	
	handler = function( params )
		
		local service = WFAnalyticsManager.initAnalytics( params )
		if service then
			sMeasurement = service
		end
		
		return true
		
	end,
}

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessageDo {

	message = 'storeAppStateData',
	simpleResponses = true,
	
	handler = function( k, v )
	
		appStateProperties[ k ] = v
		
	end
}

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessageDo {

	message = 'send',
	simpleResponses = true,
	
	handler = function( params )
		
		if sMeasurement == "Not INITed" then
			-- if not inited, or analytics is not supported on this platform then
			-- do nothing.
			return
		end
		
		local appState = params.appState
		params.appState = nil

		local channel = params.channel
		params.channel = nil
			
		local eventName = params.eventName
		params.eventName = nil
		
		updateKnownAppStateProperties( params )
	
		if appState then

			local mergedData = KSTableUtils.deepCopy( params )
			for k,v in pairs( appStateProperties ) do
				mergedData[ k ] = v
			end

			-- Convert non-strings to strings.

			for k, v in pairs( mergedData ) do
				mergedData[ k ] = tostring( v )
			end

			mergedData.dayPart = KSDate.timeInIso8601FormatNearestSecond()
			mergedData.pageView = "yes"

			sMeasurement:appStateChanged( channel, appState, mergedData )

		end
	
		if eventName then

			-- Convert non-strings to strings.

			for k, v in pairs( params ) do
				params[ k ] = tostring( v )
			end

			sMeasurement:sendEvent( eventName, params )

		end

	end
}

--------------------------------------------------------------------------------

--WFRequestDispatcher.onMessageDo {
--
--	message = 'send',
--	simpleResponses = true,
--
--	handler = function( pageName, channel, keyValuePairs )
--
--		assert( type( pageName ) == 'string', "WFAnalyticsService.send: pageName must be a string" )
--		assert( type( channel ) == 'string', "WFAnalyticsService.send: channel must be a string" )
--        assert( ( keyValuePairs == nil or type( keyValuePairs ) == 'table' ), "WFAnalyticsService.send: keyValuePairs must be nil or a table" )
--
--        -- Get account id
--        local ozAccountId = KSPrefs.currentUserId
--		if not ozAccountId then
--			return
--		end
--
--        -- Get account status
--		local statusPrefsKey = 'lastSeenStatusFor' .. ozAccountId
--		local ozAccountStatus = KSPrefs[ statusPrefsKey ]
--
--        -- If logged in, generate a unique Id and get the device class list
--        local visitorIdHash
--        local deviceClassList
--        if ozAccountId ~= nil then
--
--            -- Generate the unique Id for this visitor on this device
--            -- (but see if we can use the cached version)
--
--            if lastOzAccountId ~= ozAccountId or lastVisitorIdHash == nil then
--                local macAddress = KSDeviceUtils.macAddress()
--                if macAddress ~= nil then
--                    visitorIdHash = sha256.digest( ozAccountId .. macAddress )
--                    lastVisitorIdHash = visitorIdHash
--                end
--            else
--                visitorIdHash = lastVisitorIdHash
--            end
--
--            lastOzAccountId = ozAccountId
--
--            -- Get the list of Device Classes that the user has used Revel on
--            deviceClassList = makeDeviceClassList( ozAccountId )
--
--        end
--
--		local authenticationService = KSPrefs[ "authentication_service" ]
--
--		WFAnalyticsManager.send( pageName, channel, visitorIdHash, ozAccountId ~= nil, deviceClassList, currentDeviceClass(), ozAccountStatus, authenticationService, keyValuePairs )
--
--	end,
--
--}

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessageDo {

	message = 'enableTracking',
	simpleResponses = true,
	
	handler = function( enable )
		
		if sMeasurement == "Not INITed" then
			-- if not inited, or analytics is not supported on this platform then
			-- do nothing.
			return
		end

		assert( type( enable ) == 'boolean', "WFAnalyticsService.enableTracking: enable must be a boolean" )

		sMeasurement:setEnabled( enable )
		
	end,

}
