--[[----------------------------------------------------------------------------

WFAnalyticsServiceTest.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

This namespace provides a way for client code to send analytics events to SiteCatalyst. It may be accessed
from any thread.

This API is a minimalist wrapper over Adobe's AppMeasurement SiteCatalyst analytics library.

--------------------------------------------------------------------------------

@author allany 2012-01-19

------------------------------------------------------------------------------]]

	-- wichitafoundation.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'

	-- services.analytics
local WFAnalyticsService = require 'WFAnalyticsService'

	-- wichitafoundation.core.os
local WFPlatformInfo = require 'WFPlatformInfo'

--============================================================================--

local WFAnalyticsServiceTest = {}

--------------------------------------------------------------------------------

if WFPlatformInfo.isCocoa then

	function WFAnalyticsServiceTest.testSend( test )
	
		WFAsyncTestCase( test, 2 )
		
		:waitFor( function()	-- step 1
					
			return WFAnalyticsService.send( "test_pageName", "test_channel" )
					
		end )
	
	end

end

--------------------------------------------------------------------------------

if WFPlatformInfo.isCocoa then

	function WFAnalyticsServiceTest.testSendAnalyticsP( test )
	
		local KSPromises = require 'KSPromises'
		
		local p, r = KSPromises.promiseP()
		
		local p2 = WFAnalyticsService.sendAnalyticsP( p, { data = { eventName = 'Some fake analytics!' } } )
		
		local x
		
		p2:whenResolvedDo( function( status, value ) x = value end ):runUntilDone()
		
		r( 'data', "foo" )
		
		assert( x == "foo", "Failed to pass value through analytics code." )
		
	end

end

--------------------------------------------------------------------------------

-- FIXME [allany 2012-01-19]: Should also test various error conditions.

--------------------------------------------------------------------------------

return WFAnalyticsServiceTest
