--[[----------------------------------------------------------------------------

WFBasePropertiesTest.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

local WFBaseProperties = require 'WFBaseProperties'
local WFBasePropertiesTest = require 'WFBasePropertiesTestHelper'

local KSData = require "KSData"

--------------------------------------------------------------------------------

local WFBasePropertiesLuaTest = { }

--------------------------------------------------------------------------------

function WFBasePropertiesLuaTest.testAddPropertyInvalidArgs()

	local props = WFBaseProperties.create()
	assert( props )
	assert( pcall( props.addProperty, props, props ) == false, "should have failed: need three or more args" )
	assert( pcall( props.addProperty, props, props, "key" ) == false, "should have failed: need three or more args" )
	assert( pcall( props.addProperty, props, props, "key", "type" ) == false, "should have failed: need three or more args" )
	assert( pcall( props.addProperty, props, props, 1, 2, 3 ) == false, "should have failed: invalid arg types" )
	assert( pcall( props.addProperty, props, props, "key", "type", "value" ) == false, "should have failed: invalid type" )
	props:close()

end

--------------------------------------------------------------------------------

function WFBasePropertiesLuaTest.testAddPropertyValidValues()

	local props = WFBaseProperties.create()
	assert( props )
	
	local resultOfCompare
	
	props.addProperty( props, "testString", "string", "test"  )
	
	resultOfCompare = WFBasePropertiesTest.extractValueAndCompare( props, "testString", "string", "test"   )
	assert( resultOfCompare == true, "should be true" )
	
	props.addProperty( props, "testInt", "int", 1 )
	
	resultOfCompare = WFBasePropertiesTest.extractValueAndCompare( props,  "testInt", "int", 1)
	assert( resultOfCompare == true, "should be true" )
	
	props.addProperty( props, "testUInt", "uint", 10 )
	
	resultOfCompare = WFBasePropertiesTest.extractValueAndCompare( props,  "testUInt", "uint", 10 )
	assert( resultOfCompare == true, "should be true" )
	
	props.addProperty( props, "testBoolTrue", "bool", true )
	
	resultOfCompare = WFBasePropertiesTest.extractValueAndCompare( props, "testBoolTrue", "bool", true  )
	assert( resultOfCompare == true, "should be true" )
	
	props.addProperty( props, "testBoolFalsebool", "bool", false )
	
	resultOfCompare = WFBasePropertiesTest.extractValueAndCompare( props,  "testBoolFalsebool", "bool", false )
	assert( resultOfCompare == true, "should be true" )
	
	props.addProperty( props, "testFloat", "float", 1.12 )
	
	resultOfCompare = WFBasePropertiesTest.extractValueAndCompare( props,  "testFloat", "float", 1.12)
	assert( resultOfCompare == true, "should be true" )
	
	props.addProperty( props, "testDouble", "double", 1.1234 )
	
	resultOfCompare = WFBasePropertiesTest.extractValueAndCompare( props, "testDouble", "double", 1.1234 )
	assert( resultOfCompare == true, "should be true" )
	
	props.addProperty( props, "testStringList", "stringlist", { "One", "Two", "Three" } )
	
	local KSData_instance = KSData.make( "lalalala" )
	props.addProperty( props, "testKSData", "KSData", KSData_instance )
	
	resultOfCompare = WFBasePropertiesTest.extractValueAndCompare( props, "testKSData", "KSData", KSData_instance )
	assert( resultOfCompare == true, "should be true" )

	props:close()

end

--------------------------------------------------------------------------------

function WFBasePropertiesLuaTest.testOverwritingProperty()

	local props = WFBaseProperties.create()
	assert( props )
	
	props.addProperty( props, "testString", "string", "test"  )
	props.addProperty( props, "testString", "string", "replaced test" )
	
	props:close()

end

--------------------------------------------------------------------------------

function WFBasePropertiesLuaTest.testAddPropertyInvalidValues()


	-- Types to test:
	-- bool, float, double, KSData, stringlist, int, uint, object, string
	
	local props = WFBaseProperties.create()
	assert( props )
	
	-- bool:				Lua boolean
	assert( pcall( props.addProperty, props, props, "key", "bool", function() end ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "bool", 1 ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "bool", "< foo >" ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "bool", { } ) == false, "should have failed" )
	
	-- int:			Lua number
	assert( pcall( props.addProperty, props, props, "key", "int", function() end ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "int", false ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "int", "< foo >" ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "int", { } ) == false, "should have failed" )
	
	-- uint:			Lua number
	assert( pcall( props.addProperty, props, props, "key", "uint", function() end ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "uint", false ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "uint", "< foo >" ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "uint", -1 ) == false, "should have failed" )
	
	-- float:			Lua number
	assert( pcall( props.addProperty, props, props, "key", "float", function() end ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "float", false ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "float", "< foo >" ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "float", { } ) == false, "should have failed" )

	-- string:			Lua string
	assert( pcall( props.addProperty, props, props, "key", "int", function() end ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "int", false ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "int", 1 ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "string", { } ) == false, "should have failed" )
	
	-- KSData:			Lua string
	assert( pcall( props.addProperty, props, props, "key", "KSData", function() end ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "KSData", false ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "KSData", 1 ) == false, "should have failed" )
	assert( pcall( props.addProperty, props, props, "key", "KSData", "testString" ) == false, "should have failed" )
	
	props:close()

end
--------------------------------------------------------------------------------

return WFBasePropertiesLuaTest
