--[[----------------------------------------------------------------------------

WFDiagnosticLogImp.lua

@author scouten 2012-05-31

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

This message processor is used in the implementation of the WFDiagnosticLog
namespace. Do not access directly.

------------------------------------------------------------------------------]]

	-- wichitafoundation.core.diagnostic_log
local WFDiagnosticLogNative = require 'WFDiagnosticLogNative'

	-- wichitafoundation.core.lua.threading
local WFRequestDispatcher = require 'WFRequestDispatcher'
local WFTimer = require 'WFTimer'

	-- wichitafoundation.core.os
local KSDate = require 'KSDate'

	-- wichitafoundation.core.string
local KSStringUtils = require 'KSStringUtils'

	-- wichitafoundation.os.notifications
local KSSystemNotifications = require 'KSSystemNotifications'
	-- FIXME [scouten 2012-06-12]: Dependency outside of core.
	-- Think KSSystemNotifications will have to move into core.

	-- wichitafoundation.os.prefs
local KSPrefs -- require 'KSPrefs' -- lazy loaded
	-- FIXME [scouten 2012-10-03]: Dependency outside of core.
	-- Think KSPrefs will have to move into core.


--------------------------------------------------------------------------------

local reportingInterval = 15 -- seconds (how long do we wait for DL sections to come in?)

--------------------------------------------------------------------------------

local htmlEscape = KSStringUtils.escapeHtml

--------------------------------------------------------------------------------

local kSparklineJS = [[<script>var color='#0000aa';var height=23;function doSparkline(container){var data=container.innerHTML;container.innerHTML="";data=data.split(",");var i;var x=0;var y=0;var width=50;if(width<data.length){width=data.length}var itemGap=width/data.length;var cap=0;for(i=0;i<data.length;i++){var val=parseInt(data[i]);if(val>cap){cap=val}}var canvas=document.createElement('canvas');canvas.width=width;canvas.height=height;container.appendChild(canvas);var ctx=canvas.getContext('2d');ctx.fillStyle='#ffffff';ctx.fillRect(0,0,canvas.width,canvas.height);ctx.fillStyle=color;ctx.strokeStyle=color;var first=1;ctx.beginPath();for(i=0;i<data.length;i++){var val=parseInt(data[i]);var barheight=height-Math.floor(val/cap*(height-2));x+=itemGap;if(first){ctx.moveTo(Math.floor(x),barheight);first=0}else{ctx.lineTo(Math.floor(x),barheight)}}ctx.stroke()}var containers=document.getElementsByClassName('sparkline');for(var i=0;i<containers.length;i++){var container=containers[i];doSparkline(container)}</script>]]

--------------------------------------------------------------------------------

local kBoxAndWhiskerJS = [[<script>function getIndex (node) { var parent=node.parentElement||node.parentNode, i=-1, child; while (parent && (child=parent.childNodes[++i])) if (child==node) return i; return -1; } function _getPath (node) { var parent, path=[], index=getIndex(node); (parent=node.parentElement||node.parentNode) && (path=_getPath(parent)); index > -1 && path.push(index); return path; } function getPath (container) { parent = container.parentNode.parentNode; return _getPath(parent).join(","); } var height=30; var boxMax = {}; function parseData( container ) { var data=container.innerHTML; var path=getPath(container); var max=boxMax[path]; if (typeof max === 'undefined') { max = 0; } data=data.split(","); if (data.length < 2) { return; } for(i=0; i<data.length; i++){ var val=parseFloat(data[i]); data[ i ] = val; if (val > max) { max = val; } } boxMax[path]= max; return data; } var kBoxWhiskerWidth=200; function getCoord( data, ratio ) {; var path=getPath(container); var max=boxMax[path]; var x = Math.floor( ( data.length - 1 ) * ratio ); x = data[ x ]; x = Math.floor((kBoxWhiskerWidth-2) * x / max)+1; return x; } function doBoxAndWhisker(container){ var data = parseData(container); if (!data || data.length<2) { return; }data.sort(function(a,b){return a-b}); var wMax = getCoord( data, 1 ); var wQHi = getCoord( data, 0.75 ); var wMedian = getCoord( data, 0.5 ); var wQLo = getCoord( data, 0.25 ); var wMin = getCoord( data, 0 ); container.innerHTML=""; var en = document.createElement('span'); en.innerHTML=data[ 0 ].toPrecision( 3 ); container.appendChild(en); var canvas=document.createElement('canvas'); canvas.width=kBoxWhiskerWidth; canvas.height=height; container.appendChild(canvas); var en = document.createElement('span'); en.innerHTML=data[ data.length - 1 ].toPrecision( 3 ); container.appendChild(en); var ctx=canvas.getContext('2d'); ctx.fillStyle='#eeeeee'; ctx.fillRect(0,0,canvas.width,canvas.height); ctx.fillStyle=color; ctx.strokeStyle='#000000'; var first=1; ctx.beginPath(); ctx.moveTo(wMax,5); ctx.lineTo(wMax,25); ctx.moveTo(wMin,5); ctx.lineTo(wMin,25); ctx.moveTo(wMin,15); ctx.lineTo(wQLo,15); ctx.moveTo(wMax,15); ctx.lineTo(wQHi,15); ctx.moveTo(wMedian,8); ctx.lineTo(wMedian,22); ctx.moveTo(wQHi,8); ctx.lineTo(wQHi,22); ctx.lineTo(wQLo,22); ctx.lineTo(wQLo,8); ctx.closePath(); ctx.stroke(); } var boxAndWhiskerEls=document.getElementsByClassName('box-and-whisker'); for(var bwEl=0; bwEl<boxAndWhiskerEls.length; bwEl++){ var container=boxAndWhiskerEls[bwEl]; parseData(container); } for(var bwEl=0; bwEl<boxAndWhiskerEls.length; bwEl++){ var container=boxAndWhiskerEls[bwEl]; doBoxAndWhisker(container); } </script>]]


--------------------------------------------------------------------------------

local automationOutputSections

--------------------------------------------------------------------------------

local function automationOutputEnabled()
	
	if not KSPrefs then
		KSPrefs = require 'KSPrefs'
	end
	
	local enabled = not _G.DISTRIBUTION and KSPrefs[ 'WFTroubleshooting_automationOutputEnabled' ]
	
	if enabled and not automationOutputSections then
		automationOutputSections = {}
	end
	
	return enabled

end


--============================================================================--

-- @mark HTML report generation

local generateVisualizedHtml = function( node, selector )
	
	local data = node[ "< " .. selector .. " >" ]
	local title = node.title
	
	return string.format( "%s <div class='%s'>%s</div>", title or "",
		selector, table.concat( data, "," ) )
end

--------------------------------------------------------------------------------

local function tableToHtml( t )
	
	-- Support preformatted representaion
	if t[ "< preformatted html >" ] then
		return t[ "< preformatted html >" ]
	end
	
	-- Support preformatted representaion
	if t[ "< sparkline >" ] then
		return generateVisualizedHtml( t, 'sparkline' )
	end
	
	-- Support preformatted representaion
	if t[ "< box-and-whisker >" ] then
		return generateVisualizedHtml( t, 'box-and-whisker' )
	end
	
	-- Sort scalar keys. Ignore (with warning) non-scalar keys.

	local keysInTable = {}
	local nKeys = 0
	local ignoredNonScalarKeys = false

	for k in pairs( t ) do
		local tk = type( k )
		if tk == 'number' or tk == 'string' then
			nKeys = nKeys + 1
			keysInTable[ nKeys ] = k
		else
			ignoredNonScalarKeys = true
		end
	end
	
	table.sort( keysInTable, function( a, b )
		local ta = type( a )
		local tb = type( b )
		if ta ~= tb then
			return ta < tb
		else
			return a < b
		end
	end )

	if nKeys == 0 and not ignoredNonScalarKeys then
		return "<i>(empty table)</i>"
	end
	
	local htmlForKeyAtIndex = {}
	
	for i = 1, nKeys do
	
		local key = keysInTable[ i ]
		local htmlKey = htmlEscape( key )
		
		local value = t[ key ]
		local tv = type( value )
		
		local htmlValue
		
		if tv == 'table' then
			htmlValue = tableToHtml( value )
		else
			htmlValue = htmlEscape( value )
		end
		
		htmlForKeyAtIndex[ i ] = string.format( "<li>%s: %s</li>", htmlKey, htmlValue )
	
	end
	
	if ignoredNonScalarKeys then
		htmlForKeyAtIndex[ nKeys + 1 ] = "<li><i>(ignored non-scalar keys)</i></li>"
	end
	
	return string.format( "<ul>%s</ul>\n", table.concat( htmlForKeyAtIndex, "\n" ) )

end

--------------------------------------------------------------------------------

local function makeHtmlPreamble( args )

	local escapedAppName = htmlEscape( args.appName or "(unnamed Wichita Foundation app)" )

	local appUptime = KSDate.timeIntervalSinceAppLaunch()
	local appUptimeStr
	
	if appUptime >= 3600 then
		appUptimeStr = string.format( "%.0d seconds (%.0d hours, %.0d minutes, %.0d seconds)",
									  appUptime,
									  appUptime / 3600,
									  ( appUptime / 60 % 60 ),
									  appUptime % 60 )
	elseif appUptime >= 60 then
		appUptimeStr = string.format( "%.2d seconds (%.0d minutes, %.2d seconds)",
									  appUptime, appUptime / 60, appUptime % 60 )
	else
		appUptimeStr = string.format( "%.2d seconds", appUptime )
	end

	return string.format(
[[
<html>
<head>
<title>%s Diagnostic Log</title>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
<style>
.sparkline {
display:inline-block;
}
.box-and-whisker * {
vertical-align:middle;
}
.box-and-whisker span {
width: 200px;
display: inline-block;
}
.box-and-whisker>span:first-of-type {
text-align:right;
}
.box-and-whisker canvas {
margin: 20px 10px;
padding: 2px;
}
</style>
</head>
<body>
<h1>%s Diagnostic Log</h1>
<p>Generated: %s%s</p>
<p>App has been running for %s.</p>
<p>This information will be used by Adobe engineers to help diagnose the problem you are reporting.</p>
<p>&nbsp;</p>
]],
		escapedAppName,
		escapedAppName,
		KSDate.timeInIso8601FormatNearestSecond( KSDate.timeNow(), true ),
		KSDate.timeZoneInIso8601Format(),
		appUptimeStr
	)

end

--------------------------------------------------------------------------------

local sections = {}

--------------------------------------------------------------------------------

local function sortSectionList()

	local sortedSections = {}
	local n = 1
	
	for _, section in pairs( sections ) do
		sortedSections[ n ] = section
		n = n + 1
	end

	table.sort( sortedSections,
		function( a, b )
		
			if a.weight ~= b.weight then
				return a.weight < b.weight
			else
				return a.id < b.id
			end
		
		end )

	return sortedSections

end

--------------------------------------------------------------------------------

local function anchorNameForSection( sectionProvider )

	-- FIXME [scouten 2012-05-30]: Assumes section provider IDs are HTML clean.

	return string.format( '%d-%s', sectionProvider.weight, sectionProvider.id )

end

--------------------------------------------------------------------------------

local function makeHtmlToc( args, sortedSectionProviders )

	local tocForEach = {}
	
	for i = 1, #sortedSectionProviders do

		local sectionProvider = sortedSectionProviders[ i ]

		tocForEach[ i ] = string.format( '<li><a href="#%s">%s</a></li>',
							anchorNameForSection( sectionProvider ),
							htmlEscape( sectionProvider.name ) )

	end
	
	return string.format(
[[<h2>Table of Contents</h2>
<ol>
%s
</ol>
<p>&nbsp;</p>
]],
		table.concat( tocForEach, '\n' ) )

end

--------------------------------------------------------------------------------

local function makeHtmlSections( args, sortedSections )

	local sortedHtmlSections = {}
	
	for i = 1, #sortedSections do
		
		local section = sortedSections[ i ]
		
		sortedHtmlSections[ i ] = string.format(
[[<hr /><a name="%s" />
<h2>%s</h2>
%s
<p>&nbsp;</p>
]],
							anchorNameForSection( section ),
							htmlEscape( section.name ),
							section.body )
	
	end

	return table.concat( sortedHtmlSections, '\n' )

end

--------------------------------------------------------------------------------

local threadStats = {}

--------------------------------------------------------------------------------

local function stripPreformattedHtml( node )
	if type( node ) == 'table' then
		if node[ "< raw data >" ] then
			return node[ "< raw data >" ]
		elseif node[ "< sparkline >" ] then
			return {
				title = node.title,
				data = node[ "< sparkline >" ],
			}
		else
			local strippedCopy = {}
			for k,v in pairs( node ) do
				strippedCopy[ k ] = stripPreformattedHtml( v )
			end
			return strippedCopy
		end
	else
		return node
	end
end

--------------------------------------------------------------------------------

local function gatherAutomationOutput()
	
	local resultTable = {}
	
	for k, v in pairs( automationOutputSections ) do
		if v.tableData then
			resultTable[ k ] = stripPreformattedHtml( v.tableData )
		end
	end
	
	return resultTable
	
end

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessage( 'registerDiagnosticUtility', function( namespace )

	local x = require( namespace )
	if x ~= nil then
		return WFRequestDispatcher.getInstalledMessages()
	else
		return false
	end

end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessage( 'reportingInterval', function()

	return reportingInterval

end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessage( 'generateHtmlReportP', function( args )

	local includeAutomationOutput = automationOutputEnabled()

	WFDiagnosticLogNative.sendDiagnosticLogRequest()

	args = args or {}
	threadStats = {}
	sections = {}
	
	if includeAutomationOutput then
		automationOutputSections = {}
	end
	
	return WFTimer.afterDelayP( reportingInterval )
		:mapP( function( status, message )
		
			local automationOutputOrNil
			
			if includeAutomationOutput then
				automationOutputOrNil = gatherAutomationOutput()
			end
		
			-- Special case: Gather thread performance stats and build a section
			-- for them.
			
			if #threadStats > 0 then
		
				table.sort( threadStats, function( a, b )
				
					if a.currentSize ~= b.currentSize then
						return a.currentSize > b.currentSize
					else
						return a.threadName > b.threadName
					end
				
				end )
				
				local threadStatsHtml = {}
				
				for i, stat in pairs( threadStats ) do
				
					local peakStr = stat.peakSize
						and string.format( ", peak size = %sK", htmlEscape( stat.peakSize ) )
						or ""
					
					local wallClockTimeStr = type( stat.wallClockTime ) == 'number'
						and string.format( ", wall clock time = %.2f seconds", stat.wallClockTime )
						or ""
					
					threadStatsHtml[ i ] = string.format( '<li><b>%s</b>: GC size = %sK (after GC = %sK)%s%s</li>\n',
												htmlEscape( stat.threadName ),
												htmlEscape( stat.currentSize ),
												htmlEscape( stat.sizeAfterGC ),
												peakStr,
												wallClockTimeStr )
				
				end
				
				sections.WFLuaMessagingProcessor = {
			
					id = 'WFLuaMessagingProcessor',
					name = "Messaging Processors",
					weight = 1500,
					body = string.format( '<ul>\n%s</ul>\n', table.concat( threadStatsHtml ) ),

				}
			
			end
			
			-- Now roll up all the sections that we received.
		
			local sortedSections = sortSectionList()
		
			local result = makeHtmlPreamble( args )
						.. makeHtmlToc( args, sortedSections )
						.. makeHtmlSections( args, sortedSections )
						.. kSparklineJS
						.. kBoxAndWhiskerJS
						.. "</html>\n"
						
			return 'data', result, automationOutputOrNil

		end )

end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessageNoCancel( 'addSectionToLog', function( args )

	if type( args.body ) == 'table' then
	
		if automationOutputEnabled() and args.outputForAutomation then
			args.tableData = args.body
			automationOutputSections[ args.id ] = args
		end
		
		pcall( function()
			args.body = tableToHtml( args.body )
		end )
	
	end
	
	sections[ args.id ] = args

end )

--------------------------------------------------------------------------------

KSSystemNotifications.notificationStreamE( 'receiveThreadStats' )
	:onDataE( function( stats )

		threadStats[ #threadStats + 1 ] = stats

	end )


--============================================================================--

-- @mark built-in section providers

--------------------------------------------------------------------------------

local WFDiagnosticLog = require 'WFDiagnosticLog'

WFDiagnosticLog.addLogSectionFunc( function()

	local WFPlatformInfo = require 'WFPlatformInfo'
	local WFPlatformInfo_copy = {}
	
	if not WFPlatformInfo.isWindows then
	
		-- WINRT_FIXME [scouten 2012-05-30]: Cloak Windows platform flags until
		-- we're ready to announce.
	
		for k, v in pairs( WFPlatformInfo ) do
			if type( v ) == 'function' then
				v = v()
			end
			if not k:match( 'Win' ) and WFDiagnosticLog.isSimpleValue( v ) then
				WFPlatformInfo_copy[ k ] = v
			end
		end
		
		WFPlatformInfo_copy.folderSeparator = nil
	
	end

	return {

		id = 'WFPlatformInfo',
		name = "Platform Description",
		weight = -1000,
		body = WFPlatformInfo_copy,
	
	}

end )
