--[[----------------------------------------------------------------------------

WFDigest.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local WFDigest = {}

-- @mark WFDigest namespace

--[[!
	\class WFDigest
	\brief This namespace provides static WFDigestProvider instances
		for these digest algorithms:
		<ul>
		<li>WFDigest.MD4 (only available in Lightroom)</li>
		<li>WFDigest.MD5</li>
		<li>WFDigest.SHA1</li>
		<li>WFDigest.SHA256</li>
		<li>WFDigest.SHA384</li>
		<li>WFDigest.SHA512</li>
		<li>WFDigest.HMAC</li></ul>
	These instances are static properties of the namespace. They are NOT
	functions, but due to limitations of our documentation engine, they
	are presented here as functions.

	You can use these instances to encode data in a single operation.
	You can also use these singleton instances to create your own instances
	of WFDigest to perform streaming digest operations.  Streaming data in
	smaller chunks reduces the memory overhead of the digest operation.
--]]
	
--[[!
	\brief (WFDigestProvider) The shared singleton instance for MD4.
	THIS IS NOT A FUNCTION, it is a static property of the namespace.
]]
function WFDigest.MD4()
	-- DOXYGEN STUB
end
	
--[[!
	\brief (WFDigestProvider) The shared singleton instance for MD5.
	THIS IS NOT A FUNCTION, it is a static property of the namespace.
]]
function WFDigest.MD5()
	-- DOXYGEN STUB
end
	
--[[!
	\brief (WFDigestProvider) The shared singleton instance for SHA1.
	THIS IS NOT A FUNCTION, it is a static property of the namespace.
]]
function WFDigest.SHA1()
	-- DOXYGEN STUB
end
	
--[[!
	\brief (WFDigestProvider) The shared singleton instance for SHA256.
	THIS IS NOT A FUNCTION, it is a static property of the namespace.
]]
function WFDigest.SHA256()
	-- DOXYGEN STUB
end
	
--[[!
	\brief (WFDigestProvider) The shared singleton instance for SHA384.
	THIS IS NOT A FUNCTION, it is a static property of the namespace.
]]
function WFDigest.SHA384()
	-- DOXYGEN STUB
end
	
--[[!
	\brief (WFDigestProvider) The shared singleton instance for SHA512.
	THIS IS NOT A FUNCTION, it is a static property of the namespace.
]]
function WFDigest.SHA512()
	-- DOXYGEN STUB
end
	
--[[!
	\brief (WFDigestProvider) The shared singleton instance for HMAC.
	THIS IS NOT A FUNCTION, it is a static property of the namespace.
]]
function WFDigest.HMAC()
	-- DOXYGEN STUB
end

--------------------------------------------------------------------------------

local WFDigestProvider = {}
-- @mark WFDigestProvider namespace

--[[!
	\class WFDigestProvider
	\brief Provides the ability to encode a string or KSBlob in a supported
	digest protocol, directly using the \c digest() function, or by
	streaming data through a WFDigest instance created by the
	WFDigestProvider's \c init() function.
	
	Shared singleton instances for each supported digest algorithm
	are available as static properties of the WFDigest namespace.
--]]

--[[!
	\brief Encodes data according to the digest algorithm,
	returning both the binary digest and a hex-string version of the digested data.
		
	\param stringOrBlob The data to be encoded, as a string or KSData object.
	\param ... For the HMAC protocol ONLY, there are two additional parameters:
	<ul><li>(string) The name of a secondary digest algorithm (such as "SHA1")</li>
		<li> (string or KSData) A secret.</li></ul>
	\return The binary digest data as a string, and as a hex string.
--]]

function WFDigestProvider.digest( --[[string or blob]] stringOrBlob, ... )
	-- WFArguments.check( 'string|KSData' )
	-- DOXYGEN STUB
end

--[[!
	\brief Creates a WFDigest instance that you can use for a streaming digest operation
		using the algorithm supported by the singleton WFDigestProvider instance.
	
	\param ... For the HMAC protocol ONLY, there are two additional parameters:
	<ul><li>(string) The name of a secondary digest algorithm (such as "SHA1")</li>
		<li> (string or KSData) A secret.</li></ul>

	\return (WFDigest) The new digest object
--]]

function WFDigestProvider.init( ... )
	-- DOXYGEN STUB
	
	-- here to eliminate lint warning
	return WFDigestProvider
end

-- @mark WFDigest namespace

--[[!
	\brief Allows additional input to the ongoing streaming digest
		operation.
	
	\param input (string or KSData) The additional input.
--]]

function WFDigest:update( --[[string or KSData]] input )
	-- DOXYGEN STUB
end

--[[!
	\brief Computes the digest for the accumulated bytes in a
		streaming digest operation.
 
 	As a side-effect this WFDigest instance is reset and is ready
	to compute another hash.
	
	\return The binary digest data as a string, and as a hex string.
--]]
function WFDigest:digest( )
	-- DOXYGEN STUB
end

--[[!
	\brief Clears this digest without computing the final digest;
	the instance is then ready to compute another hash.
 --]]
function WFDigest:reset( )
	-- DOXYGEN STUB
end

--[[!
	\cond PRIVATE
	Doxygen will not try to parse anything after this point.
--]]

-- On Wichita, this goes directly to the native export
-- In Lightroom, we have a stub .lua file that provides this.
local WFDigestImpl = require 'WFDigestImpl'

--==============================================================================

for k, suite in pairs( WFDigestImpl ) do
	local init = suite.init
	suite.digest = function( input, ... )
		local ctx = init( ... )
		assert( ctx, "Invalid params" )
		ctx:update( input )
		return ctx:digest( ctx )
	end
	WFDigest[ k ] = suite
end

--==============================================================================

return WFDigest
--[[!
	\endcond
--]]
