--[[----------------------------------------------------------------------------

WFDigestTest.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local WFDigest = require 'WFDigest'

--==============================================================================

local tests = {}

--==============================================================================

local function checkEqual( a, b, why )
	assert( a == b, why )
end
--==============================================================================

local hexstr = '0123456789abcdef'
local hex = {}

local ct = 0
for c in string.gmatch( hexstr, '.' ) do
	hex[ c ] = ct
	ct = ct + 1
end

local function verify( expected, digest, bin, name )

	checkEqual( digest, expected, name )
	local t = {}

	for i = 1, math.floor( #expected/2 ) do
		local achr = string.sub( expected, i * 2 - 1, i * 2 - 1 )
		local bchr = string.sub( expected, i * 2, i * 2 )
		local a = assert( hex[ achr ], achr )
		local b = assert( hex[ bchr ], bchr )
		local c = string.char( a * 16 + b )
		table.insert( t, c )
	end
	assert( bin == table.concat( t ), name )

end

--------------------------------------------------------------------------------

local function quickDigest( test, digestType, input, expected, ... )
	
	local digest, bin = WFDigest[ digestType ].digest( input, ... )
	test:log( "quick: ", digestType, input, digest )
	verify( expected, digest, bin, digestType .. "_digest" )
	
end

--------------------------------------------------------------------------------

local function updateDigest( test, digestType, input, expected, ... )
	
	local digester = WFDigest[ digestType ].init( ... )
	
	-- turn this input string into a series of parts by splitting it on 'a' chars
	local parts = {}
	for s in string.gmatch( input, "[^a]+" ) do
		table.insert( parts, s )
		table.insert( parts, "a" )
	end
	parts[ #parts ] = nil
	for _, s in ipairs( parts ) do
		digester:update( s )
	end
	
	local digest, bin = digester:digest()
	test:log( "update: ", digestType, input, digest )
	verify( expected, digest, bin, digestType .. "_digest" )
	
end

--------------------------------------------------------------------------------

local function getTestData()
	local input = "Calculate a hash (aka message digest) of data. Implementations are from Sun (java.security.MessageDigest) and GNU."
	local t = {}
	local function add( ... )
		t[ #t + 1 ] = { ... }
	end

--	add( "MD4", input, "28aa04c0d6ab8fa0e5f0fc00dcd821fc" )
	add( "MD5", input, "889de37475010a39510c419e5657b6a6" )
	add( "SHA1", input, "3f22f2af267f5ceff036f9b1429c48ccf4995a63" )
	add( "SHA256", input, "2ed398c94aad35718659e6263855d5321a87d0bdf05085dc4aa7f34757a83249" )
	add( "SHA512", input, "7ed4cb153957127c0c277b771dcfcc6d7de3ce557424cbedf48c42aa608b1c9a6547282fe9a2054d80c937baefeaae8fba975007fa70189832ba870a3387a5f4" )

-- CommonCrypto on Mac/iOS does not support HMAC+MD4
--	add( "HMAC", input, "d6f19f0ff19b7d21be742ee331417ccc", "MD4", "secret" )
	add( "HMAC", input, "99f3a240ea9fee53a4d69763f8462e6d", "MD5", "secret" )
	add( "HMAC", input, "a029b8b6888d9a11851895401d14ac122cd30760", "SHA1", "secret" )
	add( "HMAC", input, "53654f988ffe55549b566a73c0b2779768999d6a39074fa1a84f2ad53160393c", "SHA256", "secret" )
	add( "HMAC", input, "39eedf9024e13fff87460d92be201ac6a9f31b82697195f2221199b0b25a98f61d307e259c102a9bb8a21099e661650fcc62ed00e13dccde7bd2380bd8c701c8", "SHA512", "secret" )
	return t
	
end

--------------------------------------------------------------------------------

function tests.testDigests( test )
	
	for _, params in ipairs( getTestData() ) do
		quickDigest( test, unpack( params ) )
	end
	
end

--------------------------------------------------------------------------------

function tests.testUpdate( test )
	
	for _, params in ipairs( getTestData() ) do
		updateDigest( test, unpack( params ) )
	end
	
end

--==============================================================================

return tests
