--[[----------------------------------------------------------------------------

WFDocStoreImpTest.lua

@author scouten 2013-04-01

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'

	-- WF.core.utils
local KSIdGenerator = require 'KSIdGenerator'
local KSTestUtils = require 'KSTestUtils'
local WFArguments = require 'WFArguments'

	-- WF.oz_client.doc_store
local WFDocStoreImp = require 'WFDocStoreImp'

local WFPlatformInfo = require 'WFPlatformInfo'

local TEST_WAIT_TIME = WFPlatformInfo.isAndroid and 6 or 4

--============================================================================--

local WFDocStoreImpTest = {}

local check = assert( WFArguments.check )

local docStore

--------------------------------------------------------------------------------

function WFDocStoreImpTest.setUp()

	docStore = WFDocStoreImp( ':memory:' )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.tearDown( coveragePath )

	if docStore then

		if coveragePath then
			docStore:DEBUG_dumpCoverage( coveragePath )
		end

		docStore:close()
		docStore = nil

	end

end

--------------------------------------------------------------------------------

-- @mark whole document values

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_emptyDoc( test )

	local hash

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue._hash = _hash
			hash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDocValueB_emptyDoc( test )

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local docValueB = docStore:docValueB( 'testDocId' )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( docValueB )
		:continueWhenValueMatches
			{
				_id = 'testDocId',
				_slot = 'main',
				_empty = true,
			}
		
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}

			expectedValue._hash = hash

			return hash
		
		end )

		:waitFor( docValueB )
		:continueWhenValueMatches( expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDocValueB_reuseExisting( test )

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local docValueB = docStore:docValueB( 'testDocId' )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
			
			return docStore:docValueB( 'testDocId' )
			
		end )
		:continueWhenValueMatches
			{
				_id = 'testDocId',
				_slot = 'main',
				_empty = true,
			}
		
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValue._hash = hash

			return hash
		
		end )

		:waitFor( docValueB )
		:continueWhenValueMatches( expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDocValueB_slotCrossContamination( test )

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local docValueB = docStore:docValueB( 'testDocId', 'other' )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( docValueB )
		:continueWhenValueMatches
			{
				_id = 'testDocId',
				_slot = 'other',
				_empty = true,
			}
		
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValue._hash = hash

			return hash
		
		end )

		:waitFor( docValueB )
		:continueWhenValueMatches
			{
				_id = 'testDocId',
				_slot = 'other',
				_empty = true,
			}

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_replacingHash( test )

	local hash

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 45,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			return hash
		
		end )
		
		:waitFor( function()
			
			hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				_hash = hash,
				test = 45,
			}
			
			expectedValue._hash = hash
			return hash
		
		end )
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_replacingChangeCounter( test )

	local hash

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 45,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			return hash
		
		end )
		
		:waitFor( function()
			
			hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				_changeCounter = 1,
				test = 45,
			}
			
			expectedValue._hash = hash
			return hash
		
		end )
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_emptyDocNestedArray( test )

	local hash

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
		array = { 'a', 'b', 'c' },
		hash = { a = 1, b = 2, c = 3 },
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
				array = { 'a', 'b', 'c' },
				hash = { a = 1, b = 2, c = 3 },
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue._hash = _hash
			hash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_allowSyncStatusSpecialKey( test )

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
		_syncStatus = { 'a', 'b', 'c' },
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
				_syncStatus = { 'a', 'b', 'c' },
			}

			expectedValue._hash = hash
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_allowDeletedSpecialKey( test )

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValue_afterDelete = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_deleted = true,
		_changeCounter = 2,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}

			expectedValue._hash = hash
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

		:waitFor( function()
			
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				_deleted = true,
			}

			expectedValue_afterDelete._hash = hash
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue_afterDelete )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_emptyDocTempDB( test )

	-- Same test as above, but proving we can use ':temp:' pseudo-path as well.

	docStore:close()	
	docStore = WFDocStoreImp( ':temp:' )

	return WFDocStoreImpTest.testGetSetDocValue_emptyDoc( test )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetDocValue_empty( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42
			}
		
		end )

		:waitFor( function()
		
			return docStore:docValue( 'bogusDocId' )
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'bogusDocId',
				_slot = 'main',
				_empty = true,
			} )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorReplacingWrongHash( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )

		:waitFor( function()
		
			KSTestUtils.assertFails( function()
				return docStore:setDocValue {
					_id = 'testDocId',
					_slot = 'main',
					_hash = 'notTheRightHash',
					test = 45,
				}
			end )
			
			return true
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorChangingTransientStatus_trueToFalse( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				_transient = true,
				test = 42,
			}
		
		end )

		:waitFor( function()
		
			KSTestUtils.assertFails( function()
				return docStore:setDocValue {
					_id = 'testDocId',
					_slot = 'main',
					_transient = false,
					test = 45,
				}
			end )
			
			return true
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorChangingTransientStatus_falseToTrue( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				_transient = false,
				test = 42,
			}
		
		end )

		:waitFor( function()
		
			KSTestUtils.assertFails( function()
				return docStore:setDocValue {
					_id = 'testDocId',
					_slot = 'main',
					_transient = true,
					test = 45,
				}
			end )
			
			return true
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorChangingTransientStatus_nilToTrue( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )

		:waitFor( function()
		
			KSTestUtils.assertFails( function()
				return docStore:setDocValue {
					_id = 'testDocId',
					_slot = 'main',
					_transient = true,
					test = 45,
				}
			end )
			
			return true
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorReplacingWrongChangeCounter( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )

		:waitFor( function()
		
			KSTestUtils.assertFails( function()
				return docStore:setDocValue {
					_id = 'testDocId',
					_slot = 'main',
					_changeCounter = 71,
					test = 45,
				}
			end )
			
			return true
		
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueMissingTable()

	KSTestUtils.assertFails( function()
		return docStore:setDocValue()
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueMissingId()

	KSTestUtils.assertFails( function()
		return docStore:setDocValue {}
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueIdNotString()

	KSTestUtils.assertFails( function()
		return docStore:setDocValue { _id = 4 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueIdEmptyString()

	KSTestUtils.assertFails( function()
		return docStore:setDocValue { _id = '' }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueMissingSlot()

	KSTestUtils.assertFails( function()
		return docStore:setDocValue { id = 'ok' }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueSlotNotString()

	KSTestUtils.assertFails( function()
		return docStore:setDocValue { _id = 'ok', _slot = 4 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueSlotEmptyString()

	KSTestUtils.assertFails( function()
		return docStore:setDocValue { _id = 'ok', _slot = '' }
	end )

end

--------------------------------------------------------------------------------

local function setDocValueShouldFail( value )

	value._id = 'testDocId'
	value._slot = 'main'

	KSTestUtils.assertFails( function()
		return docStore:setDocValue( value )
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testSetDocValue_errorDocValueIsInvalid( test )

	local valueWithMT = { test = 42 }
	setmetatable( valueWithMT, {} )

	setDocValueShouldFail { _transient = "true", blah = 42 }

	setDocValueShouldFail( valueWithMT )

	setDocValueShouldFail { [ 42 ] = 'string keys only, please' }
	
	setDocValueShouldFail { [ '' ] = 'non-empty string keys' }
	
	setDocValueShouldFail { _blah = 'must not start with underscore' }
	
	setDocValueShouldFail { func = function() end }
	
	setDocValueShouldFail
		{
			nested = {
				[ true ] = 'enforce non-string/number rule in nested tables',
			},
		}
	
	setDocValueShouldFail
		{
			nested = {
				_stillNotOK = 'enforce leading underscore rule in nested tables',
			},
		}
	
	setDocValueShouldFail
		{
			nested = {
				[ '' ] = 'enforce non-empty string rule in nested tables',
			},
		}

	setDocValueShouldFail
		{
			nested = {
				nested2 = {
					[ true ] = 'enforce non-string/number rule in deeply nested tables',
				},
			},
		}
	
	setDocValueShouldFail
		{
			_deleted = true,
			nothingElseAllowed = true,
				-- If _deleted = true, then no other values are allowed in the table.
		}
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_eraseOldValue( test )

	local firstHash, secondHash

	local expectedValue1 = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValue2 = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		blah = 95,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue1._hash = _hash
			firstHash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue1 )

		-- Update doc value and test read-back again.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				blah = 95,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			assert( firstHash ~= _hash )
			
			expectedValue2._hash = _hash
			secondHash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue2 )

		-- Make sure old value no longer exists in DB.
		-- (Yes, grungy reaching around into DB internals.
		-- Don't try this at home, kids.)
		
		:waitFor( function()
		
			return docStore._db:exec( [[SELECT 1 FROM content WHERE contentHash == ?]],
									  firstHash ) or ':empty:'
		
		end )
		:continueWhenValueMatches( 'data', ':empty:' )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_differentSlots( test )

	local firstHash, secondHash

	local expectedValue1 = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValue2 = {
		_id = 'testDocId',
		_slot = 'other',
		-- _hash = (inserted later),
		_changeCounter = 3,
		blah = 95,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue1._hash = _hash
			firstHash = _hash
		
		end )
	
		-- Set second slot to same doc value.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'other',
				test = 42,
			}
		
		end )
	
		-- Now change second slot value, leaving first slot at existing value.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'other',
				blah = 95,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			assert( firstHash ~= _hash )
			
			expectedValue2._hash = _hash
			secondHash = _hash
		
		end )
	
		-- Now read both values back.
	
		:waitFor( function()
			return docStore:docValue( 'testDocId', 'main' )
		end )
		:continueWhenValueMatches( 'data', expectedValue1 )

		:waitFor( function()
			return docStore:docValue( 'testDocId', 'other' )
		end )
		:continueWhenValueMatches( 'data', expectedValue2 )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testGetSetDocValue_reuseExistingValue( test )

	local hash

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue._hash = _hash
			hash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

		-- Set doc value to same value and capture rev ID.
		-- Should not bump change counter.
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue._hash = _hash
			hash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

-- @mark partial document values

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_empty( test )

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		-- Set doc value and capture rev ID.
	
		:waitFor( function()
		
			return docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
		
		end )
		:whenResolvedDo( function( status, _hash )
			
			check( 'string', 'string' )
			
			assert( status == 'data' )
			assert( #_hash == 40 )
			assert( _hash:match( '^[0-9a-f]+$' ) )
			
			expectedValue._hash = _hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_afterSetDocValue( test )

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValueAfterOverlay = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
		blah = 45,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				blah = 45,
			}
			
			expectedValueAfterOverlay._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueAfterOverlay )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_nestedTable( test )

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
		blah = {
			a = 1,
			b = 2,
			[ "foo" ] = 42,
		},
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				blah = {
					a = 1,
					b = 2,
					[ "foo" ] = 42,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_nestedTableReplacingScalar( test )

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
		blah = {
			a = 1,
			b = 2,
			[ "foo" ] = 42,
		},
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
				blah = 95,
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				blah = {
					a = 1,
					b = 2,
					[ "foo" ] = 42,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_mergeNestedTable( test )

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
		blah = {
			a = 1,
			b = 2,
			[ "foo" ] = 42,
		},
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
				blah = {
					a = 1,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				blah = {
					b = 2,
					[ "foo" ] = 42,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_deleteNestedTable( test )

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
				blah = {
					a = 1,
					b = 2,
					[ "foo" ] = 42,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				blah = {
					_delete = true,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_deleteScalar( test )

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
				blah = "zip",
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				blah = {
					_delete = true,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_deleteNothing( test )

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				blah = {
					_delete = true,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_replaceExisting( test )

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
		nested = {
			a = 1,
			b = 2,
			[ 42 ] = 'c',
		},
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
				nested = {
					a = 1,
					b = 2,
					[ "foo" ] = 42,
				}
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				nested = {
					_replace = true,
					a = 1,
					b = 2,
					[ 42 ] = 'c',
				},
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_replaceNothing( test )

	local expectedValue = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
		nested = {
			a = 1,
			b = 2,
			[ 42 ] = 'c',
		},
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )

		:waitFor( function()
		
			local hash = docStore:applyOverlay {
				_id = 'testDocId',
				_slot = 'main',
				nested = {
					_replace = true,
					a = 1,
					b = 2,
					[ 42 ] = 'c',
				},
			}
			
			expectedValue._hash = hash
			
			return hash
		
		end )
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValue )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_errorMissingTable()

	KSTestUtils.assertFails( function()
		return docStore:applyOverlay()
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_errorMissingId()

	KSTestUtils.assertFails( function()
		return docStore:applyOverlay {}
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_errorIdNotString()

	KSTestUtils.assertFails( function()
		return docStore:applyOverlay { _id = 4 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_errorIdEmptyString()

	KSTestUtils.assertFails( function()
		return docStore:applyOverlay { _id = '' }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_errorMissingSlot()

	KSTestUtils.assertFails( function()
		return docStore:applyOverlay { id = 'ok' }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_errorSlotNotString()

	KSTestUtils.assertFails( function()
		return docStore:applyOverlay { _id = 'ok', _slot = 4 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testApplyOverlay_errorSlotEmptyString()

	KSTestUtils.assertFails( function()
		return docStore:applyOverlay { _id = 'ok', _slot = '' }
	end )

end

--------------------------------------------------------------------------------

-- @mark slot management

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testCopySlotToSlot( test )

	local expectedValueMain = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValueOther = {
		_id = 'testDocId',
		_slot = 'other',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValueMain._hash = hash
			
			return hash

		end )

		:waitFor( function()

			local hash = docStore:copySlotToSlot( 'testDocId', 'main', 'other' )
			
			assert( hash == expectedValueMain._hash,
				"Huh. How did the hash value change?" )
			
			expectedValueOther._hash = hash
		
			return hash
			
		end )
		
		-- Make sure we can read both back.
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'main' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueMain )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'other' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueOther )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testCopySlotToSlot_docValueB( test )

	local expectedValueMain = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValueOther = {
		_id = 'testDocId',
		_slot = 'other',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
	}
	
	local docValueB_main = docStore:docValueB( 'testDocId', 'main' )
	local docValueB_other = docStore:docValueB( 'testDocId', 'other' )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValueMain._hash = hash
			
			return hash

		end )
		
		:waitFor( docValueB_main )
		:continueWhenValueMatches( expectedValueMain )

		:waitFor( docValueB_other )
		:continueWhenValueMatches {
			_id = 'testDocId',
			_slot = 'other',
			_empty = true,
		}

		:waitFor( function()

			local hash = docStore:copySlotToSlot( 'testDocId', 'main', 'other' )
			
			assert( hash == expectedValueMain._hash,
				"Huh. How did the hash value change?" )
			
			expectedValueOther._hash = hash
		
			return hash
			
		end )
		
		-- Make sure we can read both back.
	
		:waitFor( docValueB_main )
		:continueWhenValueMatches( expectedValueMain )

		:waitFor( docValueB_other )
		:continueWhenValueMatches( expectedValueOther )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testCopySlotToSlot_shortCircuitExistingValueMatches( test )

	local expectedValueMain = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
			-- should not bump to 3 when we do :copySlotToSlot
		test = 42,
	}

	local expectedValueOther = {
		_id = 'testDocId',
		_slot = 'other',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValueMain._hash = hash
			
			return hash

		end )

		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'other',
				test = 42,
			}
			
			assert( hash == expectedValueMain._hash,
				"Huh. How did the hash value change?" )

			expectedValueOther._hash = hash
			
			return hash

		end )

		:waitFor( function()

			local hash = docStore:copySlotToSlot( 'testDocId', 'other', 'main' )
			
			assert( hash == expectedValueMain._hash,
				"Huh. How did the hash value change?" )
			
			expectedValueOther._hash = hash
		
			return hash
			
		end )
		
		-- Make sure we can read both back.
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'main' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueMain )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'other' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueOther )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testCopySlotToSlot_errorNoSuchSource( test )

	local expectedValueMain = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValueMain._hash = hash
			
			return hash

		end )

		:waitFor( function()

			KSTestUtils.assertFails( function()
				return docStore:copySlotToSlot( 'testDocId', 'bogus', 'other' )
			end )

			return true
			
		end )
		
		-- Make sure we can read back expected results.
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'main' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueMain )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'other' )

		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'testDocId',
				_slot = 'other',
				_empty = true,
			} )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testCopySlotToSlot_changeTransientStatus_true( test )

	local expectedValueMain = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValueOther = {
		_id = 'testDocId',
		_slot = 'other',
		-- _hash = (inserted later),
		_changeCounter = 2,
		_transient = true,
		test = 42,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValueMain._hash = hash
			
			return hash

		end )

		:waitFor( function()

			local hash = docStore:copySlotToSlot( 'testDocId', 'main', 'other', true )
			
			assert( hash == expectedValueMain._hash,
				"Huh. How did the hash value change?" )
			
			expectedValueOther._hash = hash
		
			return hash
			
		end )
		
		-- Make sure we can read both back.
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'main' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueMain )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'other' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueOther )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testCopySlotToSlot_changeTransientStatus_false( test )

	local expectedValueMain = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValueOther = {
		_id = 'testDocId',
		_slot = 'other',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 42,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValueMain._hash = hash
			
			return hash

		end )

		:waitFor( function()

			local hash = docStore:copySlotToSlot( 'testDocId', 'main', 'other', false )
			
			assert( hash == expectedValueMain._hash,
				"Huh. How did the hash value change?" )
			
			expectedValueOther._hash = hash
		
			return hash
			
		end )
		
		-- Make sure we can read both back.
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'main' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueMain )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'other' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueOther )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testCopySlotToSlot_overwriteExistingValue( test )

	local expectedValueMain = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValueOther = {
		_id = 'testDocId',
		_slot = 'other',
		-- _hash = (inserted later),
		_changeCounter = 3,
		_transient = true,
		test = 42,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValueMain._hash = hash
			
			return hash

		end )

		:waitFor( function()

			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'other',
				_transient = true,
				blah = 57,
			}
			
		end )

		:waitFor( function()

			local hash = docStore:copySlotToSlot( 'testDocId', 'main', 'other' )
			
			assert( hash == expectedValueMain._hash,
				"Huh. How did the hash value change?" )
			
			expectedValueOther._hash = hash
		
			return hash
			
		end )
		
		-- Make sure we can read both back.
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'main' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueMain )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'other' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueOther )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testCopySlotToSlot_errorChangingTransientStatus_trueToFalse( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				_transient = true,
				test = 42,
			}
		
		end )

		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'other',
				_transient = true,
				test = 45,
			}
		
		end )

		:waitFor( function()
		
			KSTestUtils.assertFails( function()
				return docStore:copySlotToSlot( 'testDocId', 'main', 'other', false )
			end )
			
			return true
			
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testCopySlotToSlot_errorChangingTransientStatus_falseToTrue( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				_transient = true,
				test = 42,
			}
		
		end )

		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'other',
				_transient = false,
				test = 45,
			}
		
		end )

		:waitFor( function()
		
			KSTestUtils.assertFails( function()
				return docStore:copySlotToSlot( 'testDocId', 'main', 'other', true )
			end )
			
			return true
			
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testCopySlotToSlot_errorChangingTransientStatus_nilToTrue( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				_transient = true,
				test = 42,
			}
		
		end )

		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'other',
				test = 45,
			}
		
		end )

		:waitFor( function()
		
			KSTestUtils.assertFails( function()
				return docStore:copySlotToSlot( 'testDocId', 'main', 'other', true )
			end )
			
			return true
			
		end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testQuietlyDeleteSlot( test )

	local firstHash

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			firstHash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'blah',
				test = 42,
			}
			
			return firstHash

		end )
		
		:waitFor( function()
			
			local result = docStore:docValue( 'testDocId', 'blah' )
			result._hash = nil
			return result
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'testDocId',
				_slot = 'blah',
				_changeCounter = 1,
				test = 42,
			} )
		
		:waitFor( function()
		
			docStore:quietlyDeleteSlot( 'testDocId', 'blah' )
		
			return true
		
		end )
		
		:waitFor( function()
			
			return docStore:docValue( 'testDocId', 'blah' )
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'testDocId',
				_slot = 'blah',
				_empty = true,
			} )

		-- Make sure old value no longer exists in DB.
		-- (Yes, grungy reaching around into DB internals.
		-- Don't try this at home, kids.)
		
		:waitFor( function()
		
			return docStore._db:exec( [[SELECT 1 FROM content WHERE contentHash == ?]],
									  firstHash ) or ':empty:'
		
		end )
		:continueWhenValueMatches( 'data', ':empty:' )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testQuietlyDeleteSlot_doesntExist( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
			
			return docStore:docValue( 'testDocId', 'blah' )
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'testDocId',
				_slot = 'blah',
				_empty = true,
			} )
		
		:waitFor( function()
		
			docStore:quietlyDeleteSlot( 'testDocId', 'blah' )
		
			return true
		
		end )
		
		:waitFor( function()
			
			return docStore:docValue( 'testDocId', 'blah' )
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'testDocId',
				_slot = 'blah',
				_empty = true,
			} )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testQuietlyDeleteSlot_updatesDocValueB( test )

	local expectedValue =
		{
			_id = 'testDocId',
			_slot = 'blah',
			_changeCounter = 1,
			test = 42,
		}
	
	local docValueB = docStore:docValueB( 'testDocId', 'blah' )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'blah',
				test = 42,
			}
			
			expectedValue._hash = hash
			
			return hash

		end )
		
		:waitFor( docValueB )
		:continueWhenValueMatches( expectedValue )
		
		:waitFor( function()
		
			docStore:quietlyDeleteSlot( 'testDocId', 'blah' )
		
			return true
		
		end )
		
		:waitFor( docValueB )
		:continueWhenValueMatches(
			{
				_id = 'testDocId',
				_slot = 'blah',
				_empty = true,
			} )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testQuietlyDeleteSlot_errorCantDeleteMain( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}

		end )
		
		:waitFor( function()
			
			local result = docStore:docValue( 'testDocId', 'main' )
			result._hash = nil
			return result
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'testDocId',
				_slot = 'main',
				_changeCounter = 1,
				test = 42,
			} )
		
		:waitFor( function()
		
			KSTestUtils.assertFails( function()
				docStore:quietlyDeleteSlot( 'testDocId', 'main' )
			end )
			
			return true
		
		end )
		
end

--------------------------------------------------------------------------------

-- @mark transient slots

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDeleteTransientValues( test )

	local expectedValueMain = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValueOther = {
		_id = 'testDocId',
		_slot = 'other',
		-- _hash = (inserted later),
		_changeCounter = 2,
		_transient = true,
		test = 45,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		-- Set two slots on same doc ID.
	
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValueMain._hash = hash
			
			return hash

		end )

		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'other',
				_transient = true,
				test = 45,
			}
			
			expectedValueOther._hash = hash
		
			return hash
			
		end )
		
		-- Make sure we can read both back.
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'main' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueMain )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'other' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueOther )

		-- Now delete transients.
		
		:waitFor( function()
		
			docStore:deleteTransientValues()
			return true
		
		end )
		
		-- Should only be able to read main back.
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'main' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueMain )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'other' )
		
		end )
		:continueWhenValueMatches( 'data',
			{
				_id = 'testDocId',
				_slot = 'other',
				_empty = true,
			} )
		
		-- Also verify that content table got cleaned up.
		-- (Yes, grungy reaching around into DB internals.
		-- Don't try this at home, kids.)
		
		:waitFor( function()
		
			return docStore._db:exec( [[SELECT count() FROM content]] )
		
		end )
		:continueWhenValueMatches( 'data', { [ 'count()' ] = 1 } )

		-- But also verify that change counter won't fall backwards.
		
		:waitFor( function()
		
			return docStore._db:exec( [[SELECT max( changeCounter ) FROM documents]] )
		
		end )
		:continueWhenValueMatches( 'data', { [ 'max( changeCounter )' ] = 2 } )
		
end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDeleteTransientValues_nothingToDelete( test )

	local expectedValueMain = {
		_id = 'testDocId',
		_slot = 'main',
		-- _hash = (inserted later),
		_changeCounter = 1,
		test = 42,
	}

	local expectedValueOther = {
		_id = 'testDocId',
		_slot = 'other',
		-- _hash = (inserted later),
		_changeCounter = 2,
		test = 45,
	}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		-- Set two slots on same doc ID.
	
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			
			expectedValueMain._hash = hash
			
			return hash

		end )

		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'other',
				test = 45,
			}
			
			expectedValueOther._hash = hash
		
			return hash
			
		end )
		
		-- Make sure we can read both back.
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'main' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueMain )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'other' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueOther )

		-- Now delete transients. (There are none in this test.)
		
		:waitFor( function()
		
			docStore:deleteTransientValues()
			return true
		
		end )
		
		-- Make sure we can still read both back.
	
		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'main' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueMain )

		:waitFor( function()
		
			return docStore:docValue( 'testDocId', 'other' )
		
		end )
		:continueWhenValueMatches( 'data', expectedValueOther )

end

--------------------------------------------------------------------------------

-- @mark change notification

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDocChangeCounterB( test )

	local docChangeCounterB = docStore:docChangeCounterB()

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( docChangeCounterB )
		:continueWhenValueMatches( 0 )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}

		end )

		:waitFor( docChangeCounterB )
		:continueWhenValueMatches( 1 )

		:waitFor( function()

			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				rating = 3,
			}
			
		end )

		:waitFor( docChangeCounterB )
		:continueWhenValueMatches( 2 )
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testDocChangeCounter( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
	
		:waitFor( function()

			return tostring( docStore:docChangeCounter() )
				-- because WFAsyncTestCase interprets a number as a #seconds
				-- to wait :-(
		
		end )
		:continueWhenValueMatches( 'data', '0' )
	
		:waitFor( function()
		
			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}

		end )

		:waitFor( function()

			return tostring( docStore:docChangeCounter() )
		
		end )
		:continueWhenValueMatches( 'data', '1' )

		:waitFor( function()

			return docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				rating = 3,
			}
			
		end )

		:waitFor( function()

			return tostring( docStore:docChangeCounter() )
		
		end )
		:continueWhenValueMatches( 'data', '2' )
	
end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs( test )

	local expectedChangedDocsResult =
		{

			{
				_id = 'testDocId',
				_slot = 'main',
				_changeCounter = 1,
				_hash = nil, -- filled in later
				test = 42,
			},
			{
				_id = 'anotherPhoto',
				_slot = 'main',
				_changeCounter = 2,
				_hash = nil, -- filled in later
				favorite = true,
				title = "My Nifty Photo",
			},
			
			continueFrom = 3,
		
		}

	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		-- Write two documents.
		
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'testDocId',
				_slot = 'main',
				test = 42,
			}
			expectedChangedDocsResult[ 1 ]._hash = hash

			return hash

		end )
		
		:waitFor( function()

			local hash = docStore:setDocValue {
				_id = 'anotherPhoto',
				_slot = 'main',
				favorite = true,
				title = "My Nifty Photo",
			}

			expectedChangedDocsResult[ 2 ]._hash = hash

			return hash

		end )
	
		-- Read them back via :changedDocs.
		
		:waitFor( function()
			return docStore:changedDocs { fromChangeCounter = 1 }
		end )
		
		:continueWhenValueMatches( 'data', expectedChangedDocsResult )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_empty( test )

	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( function()
			return docStore:changedDocs { fromChangeCounter = 1 }
		end )
		
		:continueWhenValueMatches( 'data', { continueFrom = 1 } )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorFromChangeCounterMissing( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs {}
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorLimitNotString( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 1, limit = 'bogus' }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorLimitNotPositive( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 1, limit = 0 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorSlotNotString( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 1, slot = 0 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorSlotNameEmpty( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 1, slot = '' }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_observesLimit( test )

	local docIds = {}
	for i = 1, 100 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = { continueFrom = 51 }
	for i = 1, 50 do
		expectedResult[ i ] = {
			_id = docIds[ i ],
			_slot = 'main',
			_changeCounter = i,
			favorite = true,
		}
	end
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( function()

			for i = 1, #docIds do

				local hash = docStore:setDocValue {
					_id = docIds[ i ],
					_slot = 'main',
					favorite = true,
				}

				if i <= 50 then
					expectedResult[ i ]._hash = hash
				end

			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs { fromChangeCounter = 1, limit = 1000 }
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_observesSlotName( test )

	local docIds = {}
	for i = 1, 100 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = { continueFrom = 101 }
	for i = 1, 50 do
		expectedResult[ i ] = {
			_id = docIds[ i * 2 ],
			_slot = 'even',
			_changeCounter = i * 2,
			favorite = true,
		}
	end
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( function()

			for i = 1, #docIds do

				local hash = docStore:setDocValue {
					_id = docIds[ i ],
					_slot = i % 2 == 0 and 'even' or 'odd',
					favorite = true,
				}

				if i % 2 == 0 then
					expectedResult[ i / 2 ]._hash = hash
				end

			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs { fromChangeCounter = 1, slot = 'even' }
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_observesSlotName_empty( test )

	local docIds = {}
	for i = 1, 100 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = { continueFrom = 101 }
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( function()

			for i = 1, #docIds do
				docStore:setDocValue {
					_id = docIds[ i ],
					_slot = 'main',
					favorite = true,
				}
			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs { fromChangeCounter = 1, slot = 'nonesuch' }
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_observesDocInclusionList( test )

	local docIds = {}
	for i = 1, 100 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = { continueFrom = 101 }

	expectedResult[ 1 ] = {
		_id = docIds[ 17 ],
		_slot = 'main',
		_changeCounter = 17,
		favorite = true,
	}
	
	expectedResult[ 2 ] = {
		_id = docIds[ 50 ],
		_slot = 'main',
		_changeCounter = 50,
		favorite = true,
	}
	
	expectedResult[ 3 ] = {
		_id = docIds[ 51 ],
		_slot = 'main',
		_changeCounter = 51,
		favorite = true,
	}
	
	expectedResult[ 4 ] = {
		_id = docIds[ 87 ],
		_slot = 'main',
		_changeCounter = 87,
		favorite = true,
	}
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( function()

			for i = 1, #docIds do

				local hash = docStore:setDocValue {
					_id = docIds[ i ],
					_slot = 'main',
					favorite = true,
				}

				if i == 17 then
					expectedResult[ 1 ]._hash = hash
				elseif i == 50 then
					expectedResult[ 2 ]._hash = hash
				elseif i == 51 then
					expectedResult[ 3 ]._hash = hash
				elseif i == 87 then
					expectedResult[ 4 ]._hash = hash
				end

			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs {
				fromChangeCounter = 1,
				docs = { docIds[ 87 ], docIds[ 50 ], docIds[ 51 ], docIds[ 17 ] },
					-- Intentionally not in order of creation.
					-- Want to ensure that changes are reported in chronological
					-- order, not the order that they appear in docs array.
			}
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_observesDocInclusionListAndSlotName( test )

	local docIds = {}
	for i = 1, 100 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = { continueFrom = 101 }

	expectedResult[ 1 ] = {
		_id = docIds[ 17 ],
		_slot = 'odd',
		_changeCounter = 17,
		favorite = true,
	}
	
	expectedResult[ 2 ] = {
		_id = docIds[ 51 ],
		_slot = 'odd',
		_changeCounter = 51,
		favorite = true,
	}
	
	expectedResult[ 3 ] = {
		_id = docIds[ 87 ],
		_slot = 'odd',
		_changeCounter = 87,
		favorite = true,
	}
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( function()

			for i = 1, #docIds do

				local hash = docStore:setDocValue {
					_id = docIds[ i ],
					_slot = i % 2 == 0 and 'even' or 'odd',
					favorite = true,
				}

				if i == 17 then
					expectedResult[ 1 ]._hash = hash
				elseif i == 51 then
					expectedResult[ 2 ]._hash = hash
				elseif i == 87 then
					expectedResult[ 3 ]._hash = hash
				end

			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs {
				fromChangeCounter = 1,
				slot = 'odd',
				docs = { docIds[ 87 ], docIds[ 50 ], docIds[ 51 ], docIds[ 17 ] },
					-- Intentionally not in order of creation.
					-- Want to ensure that changes are reported in chronological
					-- order, not the order that they appear in docs array.
			}
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorDocInclusionListNotTable( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 1, docs = 42 }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_errorDocInclusionListEmptyTable( test )

	KSTestUtils.assertFails( function()
		return docStore:changedDocs { fromChangeCounter = 1, docs = {} }
	end )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_warnDocInclusionListIsLarge( test )

	local docIds = {}
	for i = 1, 120 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = { continueFrom = 51 }

	for i = 1, 50 do
		expectedResult[ i ] = {
			_id = docIds[ i ],
			_slot = 'main',
			_changeCounter = i,
			favorite = true,
		}
	end

	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( function()

			for i = 1, #docIds do

				local hash = docStore:setDocValue {
					_id = docIds[ i ],
					_slot = 'main',
					favorite = true,
				}

				if i <= 50 then
					expectedResult[ i ]._hash = hash
				end

			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs {
				fromChangeCounter = 1,
				docs = docIds, -- yes, that's too many, but we'll warn and continue
			}
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_idsOnlyObservesLimit( test )

	local docIds = {}
	for i = 1, 2000 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = { continueFrom = 1001 }
	for i = 1, 1000 do
		expectedResult[ i ] = docIds[ i ]
	end
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( function()

			for i = 1, #docIds do
				docStore:setDocValue {
					_id = docIds[ i ],
					_slot = 'main',
					favorite = true,
				}
			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs {
				fromChangeCounter = 1,
				idsOnly = true,
				limit = 1787,
			}
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testChangedDocs_idsOnlyObservesDocInclusionList( test )

	local docIds = {}
	for i = 1, 100 do
		docIds[ i ] = KSIdGenerator.uuidInLowercaseHex()
	end
	
	local expectedResult = {
		continueFrom = 101,
		docIds[ 17 ],
		docIds[ 50 ],
		docIds[ 51 ],
		docIds[ 87 ],
	}
	
	WFAsyncTestCase( test, TEST_WAIT_TIME )
		
		:waitFor( function()

			for i = 1, #docIds do
				docStore:setDocValue {
					_id = docIds[ i ],
					_slot ='main',
					favorite = true,
				}
			end

			return true
			
		end )
		
		:waitFor( function()
		
			return docStore:changedDocs {
				fromChangeCounter = 1,
				idsOnly = true,
				docs = { docIds[ 87 ], docIds[ 50 ], docIds[ 51 ], docIds[ 17 ] },
					-- Intentionally not in order of creation.
					-- Want to ensure that changes are reported in chronological
					-- order, not the order that they appear in docs array.
			}
		
		end )
		:continueWhenValueMatches( 'data', expectedResult )

end

--------------------------------------------------------------------------------

-- @mark miscellaneous

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testPath()

	local path = docStore:path()
	
	assert( type( path ) == 'string' and #path > 0,
		":path() should have returned a non-empty string" )

end

--------------------------------------------------------------------------------

function WFDocStoreImpTest.testToString()

	local docStoreAsString = tostring( docStore )
	
	assert( type( docStoreAsString ) == 'string' and #docStoreAsString > 0,
		"tostring() should have returned a non-empty string" )

end

--------------------------------------------------------------------------------

return WFDocStoreImpTest
