--[[----------------------------------------------------------------------------

WFHttpClientHeaderParser.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author arahn 2012-05-04
		May the Fourth be with you, always
		
----------------------------------------------------------------------------]]--

local function parseHeaderString( headerString )

	local parsed = {}
	local curKey = nil
	local curValue = {}
	
	local function enterCurrentKey()
		if curKey and curValue[ 1 ] then
			local parsedValue = table.concat( curValue )

			if ( curKey == 'Set-Cookie' ) then
				-- add cookie to the array if there's one there, else create a new array of cookies
				local existingCookies = parsed[ curKey ]
				if existingCookies then
					table.insert( existingCookies, parsedValue )
					parsedValue = existingCookies
				else
					parsedValue = { parsedValue }
				end
			end
			
			parsed[ curKey ] = parsedValue
			curValue = {}
			curKey = nil
		end
	end
	
	for row in string.gfind( headerString, "[^\r\n]+" ) do
		-- check for multi-line headers
		if string.find( row, "^ " ) then
			if curKey then
				curValue[ #curValue + 1 ] = row
			end
		else
			enterCurrentKey()
			local _, _, k, v = string.find( row, "([^:]+):[ \t]*(.+)$" )
			curKey = k
			curValue[ 1 ] = v
		end
	end
	enterCurrentKey()
	
	return parsed

end

--==============================================================================

return {
	parseHeaderString = parseHeaderString,
}
