--[[----------------------------------------------------------------------------

WFKeyboardState.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author ktieskoe 2012-03-16

----------------------------------------------------------------------------]]--

--============================================================================--
--[[!
	\class 	WFKeyboardState
	\brief This namespace allows you to test for modifier key and mouse button status.
	       None of this applies in iOS, and all methods return \c false in that
	       environment.
]]--

-- @mark WFKeyboardState namespace

local WFPlatformInfo = require 'WFPlatformInfo'
local MAC_PLATFORM = WFPlatformInfo.isMac

	-- core.os
local WFKeyboardStateImp = MAC_PLATFORM and require 'WFKeyboardStateImp' or {}

local WFKeyboardState = {}

--------------------------------------------------------------------------------
--[[!
	\brief Reports whether the Option key is down. Not applicable in iOS.

	\return (boolean) \c true if the key is down. Always \c false in iOS.
]]--

function WFKeyboardState.isOptionKeyDown()
	return MAC_PLATFORM and WFKeyboardStateImp.isOptionKeyDown() or false
end

--------------------------------------------------------------------------------
--[[!
	\brief Reports whether the CapsLock key is down. Not applicable in iOS.

	\return (boolean) \c true if the key is down. Always \c false in iOS.
 ]]--

function WFKeyboardState.isCapsLockKeyDown()
	return MAC_PLATFORM and WFKeyboardStateImp.isCapsLockKeyDown() or false
end

--------------------------------------------------------------------------------
--[[!
	\brief Reports whether the Shift key is down. Not applicable in iOS.

	\return (boolean) \c true if the key is down. Always \c false in iOS.
 ]]--

function WFKeyboardState.isShiftKeyDown()
	return MAC_PLATFORM and WFKeyboardStateImp.isShiftKeyDown() or false
end

--------------------------------------------------------------------------------
--[[!
	\brief Reports whether the Command key is down. Not applicable in iOS.

	\return (boolean) \c true if the key is down. Always \c false in iOS.
 ]]--

function WFKeyboardState.isCommandKeyDown()
	return MAC_PLATFORM and WFKeyboardStateImp.isCommandKeyDown() or false
end

--------------------------------------------------------------------------------
--[[!
	\brief Reports whether the mouse button is down. Not applicable in iOS.

	\return (boolean) \c true if the key is down. Always \c false in iOS.
]]--

function WFKeyboardState.isMouseButtonDown()
	return MAC_PLATFORM and WFKeyboardStateImp.isMouseButtonDown() or false
end

--------------------------------------------------------------------------------

return WFKeyboardState
