--[[----------------------------------------------------------------------------

WFLimboTest.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author mhamburg 2012-07-29

------------------------------------------------------------------------------]]

local WFLimboTest = {}

--------------------------------------------------------------------------------

local assert = assert

-- local KSTestUtils = require 'KSTestUtils'

--------------------------------------------------------------------------------

function WFLimboTest.testExistence()

	local WFLimbo = require 'WFLimbo'
	
	assert( WFLimbo.addToLimbo, "No addToLimbo function" )
	assert( WFLimbo.removeFromLimbo, "No removeFromLimbo function" )
	assert( WFLimbo.withPoolDo, "No withPoolDo function" )
	assert( WFLimbo.processCurrentLimboPool, "No processCurrentLimboPool function" )

end

--------------------------------------------------------------------------------

function WFLimboTest.testAddToLimbo()

	local WFLimbo = require 'WFLimbo'
	
	local didHandle = { }

	local function handler( obj, success, traceback )
		didHandle[ obj ] = success
	end
	
	local objects = { { }, { }, { } }
	
	for _, obj in pairs( objects ) do
		WFLimbo.addToLimbo( obj, handler )
	end
	
	assert( WFLimbo.processLimboPoolIfRoot(), "Processing limbo pool failed" )
	
	for _, obj in pairs( objects ) do
		assert( didHandle[ obj ] == true, "Object failed to be processed by the handler." )
	end
	
	local rmap = { }
	for k, v in pairs( objects ) do
		rmap[ v ] = k
	end
	
	for obj in pairs( didHandle ) do
		assert( rmap[ obj ], "Appear to have handled an object that shouldn't have been in limbo" )
	end

end

--------------------------------------------------------------------------------

function WFLimboTest.testRemoveFromLimbo()

	local WFLimbo = require 'WFLimbo'
	
	local didHandle = { }

	local function handler( obj, success, traceback )
		didHandle[ obj ] = success
	end
	
	local objects = { { }, { }, { } }
	
	for _, obj in pairs( objects ) do
		WFLimbo.addToLimbo( obj, handler )
	end
	
	WFLimbo.removeFromLimbo( objects[ 2 ] )
	objects[ 2 ] = nil
	
	assert( WFLimbo.processLimboPoolIfRoot(), "Processing limbo pool failed" )
	
	for _, obj in pairs( objects ) do
		assert( didHandle[ obj ] == true, "Object failed to be processed by the handler." )
	end
	
	local rmap = { }
	for k, v in pairs( objects ) do
		rmap[ v ] = k
	end
	
	for obj in pairs( didHandle ) do
		assert( rmap[ obj ], "Appear to have handled an object that shouldn't have been in limbo" )
	end

end

--------------------------------------------------------------------------------

return WFLimboTest

