--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remains the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and are protected by all applicable
intellectual property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--
local WFPlatformInfo = require 'WFPlatformInfo'

local WFOzContentTypes = {}

--------------------------------------------------------------------------------

local kImageContentTypeNumberLowEnd = 1000
local kImageContentTypeNumber = 1000
local kImageJpegContentTypeNumber = 1001
local kImagePngContentTypeNumber = 1002
local kImageContentTypeNumberHighEnd = 1999

local kVideoContentTypeNumberLowEnd = 2000
local kVideoContentTypeNumber = 2000
local kVideoMp4ContentTypeNumber = 2001
local kVideoQuicktimeContentTypeNumber = 2002
local kVideoMpegContentTypeNumber = 2003
local kVideoContentTypeNumberHighEnd = 2999

local kContentTypeStringToNumber = {
	[ 'image/*' ] = kImageContentTypeNumber,
	[ 'image/jpeg' ] = kImageJpegContentTypeNumber,
	[ 'image/png' ] = kImagePngContentTypeNumber,

	[ 'video/*' ] = kVideoContentTypeNumber,
	[ 'video/mp4' ] = kVideoMp4ContentTypeNumber,
	[ 'video/mpeg' ] = kVideoMpegContentTypeNumber,
	[ 'video/quicktime' ] = kVideoQuicktimeContentTypeNumber,
}

local kContentTypeNumberToString = {}

for contentType,number in pairs( kContentTypeStringToNumber ) do
	kContentTypeNumberToString[ number ] = contentType
end

--------------------------------------------------------------------------------

function WFOzContentTypes.getContentTypeNumberRangeForSubType( subtype )
	
	if WFPlatformInfo.isTornado then

		--[[ mscarafo 9/13/13
			Torando_Hack
			Why are we not returning the correct value for subtype
			if we know it?
		--]]
		if subtype == 'video' then
			return kVideoContentTypeNumberLowEnd, kVideoContentTypeNumberHighEnd
		elseif subtype == 'jpeg' then
			return kImageJpegContentTypeNumber, kImageJpegContentTypeNumber
		elseif subtype == 'png' then
			return kImagePngContentTypeNumber, kImagePngContentTypeNumber
		elseif subtype == 'image' or subtype == nil then
			return kImageContentTypeNumberLowEnd, kImageContentTypeNumberHighEnd
		end
		error( "Unknown subtype: " .. subtype )
	else
		if subtype == 'video' then
			return kVideoContentTypeNumberLowEnd, kVideoContentTypeNumberHighEnd
		elseif subtype == 'jpeg' or subtype == 'image' or subtype == nil then
			return kImageContentTypeNumberLowEnd, kImageContentTypeNumberHighEnd
		end
		error( "Unknown subtype: " .. subtype )
	end	
end

--------------------------------------------------------------------------------

function WFOzContentTypes.isVideo( contentType )
	if type( contentType ) ~= 'string' then
		return false
	end
	return not not ( string.find( contentType, "^video/" ) )
end

--------------------------------------------------------------------------------

function WFOzContentTypes.getContentTypeNumberForContentType( contentType )
	
	if not contentType then
		return
	end
	
	contentType = contentType:gsub( ";.*", "" )
	
	local result = kContentTypeStringToNumber[ contentType ]
	
	if not result then
		contentType = string.gsub( contentType, '/.+$', '/*' )
		result = kContentTypeStringToNumber[ contentType ]
	end
	
	return result
	
end

--------------------------------------------------------------------------------

function WFOzContentTypes.contentTypeForContentTypeNumber( contentTypeNumber, optOmitJpegImage )
	if optOmitJpegImage and contentTypeNumber == kImageJpegContentTypeNumber then
		return
	end
	return contentTypeNumber and kContentTypeNumberToString[ contentTypeNumber ]
end

--==============================================================================

return WFOzContentTypes
