--[[----------------------------------------------------------------------------

WFOzDocStorePath.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@reviewed scouten 2012-05-23

------------------------------------------------------------------------------]]

	-- wichitafoundation.core.os
local KSFileUtils = require 'KSFileUtils'
local KSPathUtils = require 'KSPathUtils'

	-- wichitafoundation.core.utils
local WFArguments = require 'WFArguments'


--------------------------------------------------------------------------------

local check = assert( WFArguments.check )


--============================================================================--
--[[!
	\class WFOzDocStorePath
	\brief This namespace provides shared code for calculating
		where on disk a local Oz catalog should be stored.

	\author Eric Scouten <scouten@adobe.com> 2012-05-23
]]--

-- @mark WFOzDocStorePath namespace

local WFOzDocStorePath = {}

--------------------------------------------------------------------------------
--[[!
	\brief Finds the directory that should be used for a catalog, creating it
	if it does not already exist.
	
	\param catalogId (string) The catalog's unique ID.
	
	\return (string) The full path to the directory on the local disk.
--]]

function WFOzDocStorePath.dirPathForCatalogId( --[[string]] catalogId )

	check( 'string' )

	local documentsDir = KSPathUtils.getStandardFilePath 'carouselDocuments'
	
	local catalogIdDir = KSPathUtils.child( documentsDir, catalogId )
	
	local ozCatDir = KSPathUtils.child( catalogIdDir, 'Catalog.ozcat' )

	KSFileUtils.mkdir( ozCatDir )
	
	return ozCatDir

end

--------------------------------------------------------------------------------

return WFOzDocStorePath
