--[[----------------------------------------------------------------------------

WFPlatformInfo.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author nicholas 2012-05-15

------------------------------------------------------------------------------]]

local WFPlatformInfoImp = require 'WFPlatformInfoImp'

--============================================================================--

local WFPlatformInfo = {

	-- booleans
	isiOS = not not WFPlatformInfoImp.isiOS,
	isiPhone = not not WFPlatformInfoImp.isiPhone,
	isiPad = not not WFPlatformInfoImp.isiPad,
	isHighDPIDevice = not not WFPlatformInfoImp.isHighDPIDevice,
	isMac = not not WFPlatformInfoImp.isMac,
	isCocoa = not not WFPlatformInfoImp.isCocoa,
	isWindows = not not WFPlatformInfoImp.isWindows,
	isWin7 = not not WFPlatformInfoImp.isWin7,
	isWinRT = not not WFPlatformInfoImp.isWinRT,
	isAndroid = not not WFPlatformInfoImp.isAndroid,
	isTornado = not not WFPlatformInfoImp.isTornado,
	isPhone = not not WFPlatformInfoImp.isPhone,
	isTablet = not not WFPlatformInfoImp.isTablet,
	isDesktop = not not WFPlatformInfoImp.isDesktop,
	isConstrainedMemoryDevice = not not WFPlatformInfoImp.isConstrainedMemoryDevice,
	
	isDebug = not not WFPlatformInfoImp.isDebug,
	isProfile = not not WFPlatformInfoImp.isProfile,
	isProduction = not not WFPlatformInfoImp.isProduction,
	isRelease = not not WFPlatformInfoImp.isRelease,

	-- non booleans
	folderSeparator = WFPlatformInfoImp.folderSeparator or "/",
	
	getBuild = WFPlatformInfoImp.getBuild,
	getAppVersion = WFPlatformInfoImp.getAppVersion,
	getDeviceManufacturer = WFPlatformInfoImp.getDeviceManufacturer,
	getDeviceModel = WFPlatformInfoImp.getDeviceModel,
	getOSVersion = WFPlatformInfoImp.getOSVersion,
	getAppName = WFPlatformInfoImp.getAppName,
	getAppIdentifier = WFPlatformInfoImp.getAppIdentifier,
	getUserAgentAppName = WFPlatformInfoImp.getUserAgentAppName,
	
}

--------------------------------------------------------------------------------

function WFPlatformInfo.getOS()

	local appPlatform
	
	if WFPlatformInfoImp.isiOS then
		appPlatform = "iOS"
	elseif WFPlatformInfoImp.isAndroid then
		appPlatform = "Android"
	elseif WFPlatformInfoImp.isMac then
		appPlatform = "MacOSX"
	elseif WFPlatformInfoImp.isWin7 then
		appPlatform = "Windows 7"
	elseif WFPlatformInfoImp.isWinRT then
		appPlatform = "Windows RT"
	elseif WFPlatformInfoImp.isWindows then
		appPlatform = "Windows"
	else
		appPlatform = "Unknown"
	end
	
	return appPlatform
	
end

--------------------------------------------------------------------------------

function WFPlatformInfo.getAppDeviceClass()

	local deviceClass
	
	if WFPlatformInfoImp.isTablet then
		deviceClass = "tablet"
	elseif WFPlatformInfoImp.isPhone or WFPlatformInfoImp.isAndroid then
		deviceClass = "phone"
	elseif WFPlatformInfoImp.isMac then
		deviceClass = "mac"
	elseif WFPlatformInfoImp.isDesktop then
		deviceClass = "desktop"
	else
		deviceClass = "Unknown"
	end
	
	return deviceClass
	
end

--------------------------------------------------------------------------------

function WFPlatformInfo.getDeviceModel()

	local deviceModel
	
	if WFPlatformInfoImp.isiPhone then
		deviceModel = "iPhone"
	elseif WFPlatformInfoImp.isiPad then
		deviceModel = "iPad"
	elseif WFPlatformInfoImp.getDeviceModel then
		return WFPlatformInfoImp.getDeviceModel()
	else
		deviceModel = "Unknown"
	end
	
	return deviceModel
	
end

--------------------------------------------------------------------------------

function WFPlatformInfo.getBranchName()

	local jenkinsHome = os.getenv( "JENKINS_HOME" )
	local branch
	
	if jenkinsHome then
		branch = os.getenv( "GIT_BRANCH" )
	else
		branch = "master"
	end
	
	return branch
	
end

--------------------------------------------------------------------------------

function WFPlatformInfo.getBuildTarget()

	local buildTarget
	
	if WFPlatformInfoImp.isDebug then
		buildTarget = "Debug"
	elseif WFPlatformInfoImp.isProfile then
		buildTarget = "Profile"
	elseif WFPlatformInfoImp.isProduction then
		buildTarget = "Production"
	elseif WFPlatformInfoImp.isRelease then
		buildTarget = "Release"
	else
		buildTarget = "Unknown"
	end
	
	return buildTarget
		
end

--------------------------------------------------------------------------------

return WFPlatformInfo
