--[[----------------------------------------------------------------------------

WFReachability.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author scouten 2011-07-15

------------------------------------------------------------------------------]]

	-- wichitafoundation.core.reactive
local KSReactive = require 'KSReactive'

	-- wichitafoundation.core.utils
local WFCloseSupport = require 'WFCloseSupport'

	-- wichitafoundation.web.http_client
local WFReachabilityImp = require 'WFReachabilityImp'
	-- (private to this service)


--============================================================================--
--[[!
	\class  WFReachability
	\brief This namespace helps you determine whether
	network access is available, and if so, what class of service may be available.

 	This information should be considered advisory and is obtained at the least
 	possible cost in spinning up network services.
		
	Use this as a hint as to when you should retry network connections
	if they have failed in the past, and also a hint about the type
	of network connection that is available (3G vs Wi-Fi).
	
	THIS NAMESPACE DOES NOT WORK WELL and should be considered DEPRECATED
	until a better implementation can be found.
]]--

-- @mark WFReachability namespace

local WFReachability = {}

--------------------------------------------------------------------------------

local reachabilityBehaviorCache = setmetatable( {}, { __mode = 'kv' } )

--------------------------------------------------------------------------------

WFReachabilityImp.registerReachabilityHandler( function( url, status )

	local resultB = WFCloseSupport.fetchCached( reachabilityBehaviorCache, url )
	
	if resultB then
		resultB:set( status )
	end

end )

--------------------------------------------------------------------------------
--[[!
	\brief Creates a behavior that indicates whether a specific named host on
	the Internet is thought to be reachable at this time.
 
	\param url (string) The URL to query.
	
	\return B< string > network status. The behavior returns one of these network status codes:
	
		- \c "unreachable": No connection is thought to be available. This is the initial state.
		- \c "interventionRequired": A connection might be available, but the user
			will have to enter a password or perform some other manual intervention
			to use it.
		- \c "wifi": A Wi-Fi (or similarly fast) connection is thought to be available.
		- \c "wwan": A mobile data connection (3G, etc.) connection is thought to be
			available.
		- \c "other": Some other connection is available.
	
	Some of these status codes may never be reported on some platforms.
	(For instance, Mac OS has no notion of \c "wwan" or \c "interventionRequired".)
	
	The behavior starts in the \c "unreachable" state and later
	updates to another state as more information becomes available.

--]]

function WFReachability.hostIsReachableB( --[[string]] url )

	local resultB = WFCloseSupport.fetchCached( reachabilityBehaviorCache, url )

	if not resultB then
	
		resultB = KSReactive.variableB( 'unreachable' )
		
		local request = WFReachabilityImp.hostIsReachable( url )
		
		rawset( resultB, '_request', request )
		resultB:addCloseTarget( request )
		
		reachabilityBehaviorCache[ url ] = resultB
	
	end
	
	return resultB:relayB()

end

--------------------------------------------------------------------------------

return WFReachability
