--[[----------------------------------------------------------------------------

WFRequestTestHandler.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author scouten 2011-02-04

@reviewed erwright 2011-03-08
@reviewed mhamburg 2011-03-13

------------------------------------------------------------------------------]]

	-- wichitafoundation.core.lua.threading
local WFRequestDispatcher = require 'WFRequestDispatcher'
local WFTimer = require 'WFTimer'

	-- wichitafoundation.core.reactive
local KSReactive = require 'KSReactive'


--============================================================================--

WFRequestDispatcher.onMessage( 'echo', function( ... )

	return select( '#', ... ), ...

end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessage( 'echoDelay', function( interval, str )

	return WFTimer.afterDelayP( interval )
		:mapDataP( function()
		
			return 'data', str
		
		end )

end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessage( 'sendError', function( message )

	local KSPromises = require "KSPromises"
	return KSPromises.resolvedPromiseP( 'error', message )

end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessage( 'setPref', function( k, v )

	local KSPrefs = require "KSPrefs"
	KSPrefs[ k ] = v

end )

--------------------------------------------------------------------------------

WFRequestDispatcher.onMessage( 'counterE', function( start, stop )

	local resultE = KSReactive.sourceE()
	
	local counter = start
	
	local WFTimer = require 'WFTimer'
	
	local function bumpCounterAndSend()
	
		WFTimer.afterDelayDo( .1, function()

			if counter <= stop then
				resultE:post( counter )
				bumpCounterAndSend()
			else
				resultE:close()
			end

			counter = counter + 1
			
		end )
	
	end
	
	bumpCounterAndSend()
	
	return resultE

end )
