--[[----------------------------------------------------------------------------

WFSqliteUtilsTest.lua

@author scouten 2011-04-26
@reviewed scouten 2012-08-05

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2011 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- wichitafoundation.core.utils
local KSTableUtils = require 'KSTableUtils'
local KSTestUtils = require 'KSTestUtils'

	-- wichitafoundation.sqlite.database
local WFSqliteUtils = require 'WFSqliteUtils'


--============================================================================--

-- @mark WFSqliteUtils test case

local WFSqliteUtilsTest = {}

--------------------------------------------------------------------------------

local function testValuePair( luaValue, sqliteValue, sqliteDataType )

	local actualSqliteValue, actualSqliteDataType
		= WFSqliteUtils.luaValueToSqliteValueAndType( luaValue )
	
	if actualSqliteValue ~= sqliteValue then
		error( string.format(
			"luaValueToSqliteValueAndType returned incorrect value: actual %s, expected %s",
				actualSqliteValue, sqliteValue ), 2 )
	end

	if actualSqliteDataType ~= sqliteDataType then
		error( string.format(
			"luaValueToSqliteValueAndType returned incorrect dataType: actual %s, expected %s",
				actualSqliteDataType, sqliteDataType ), 2 )
	end
	
	local actualLuaValue
		= WFSqliteUtils.sqliteValueAndTypeToLuaValue( sqliteValue, sqliteDataType )

	if type( actualLuaValue ) ~= type( luaValue ) then
		error( string.format(
			"luaValueToSqliteValueAndType returned incorrect dataType: actual %s, expected %s",
				type( actualLuaValue ), type( luaValue ) ), 2 )
	end

	if type( luaValue ) == 'table' then
	
		if not KSTableUtils.deepEquals( actualLuaValue, luaValue, 10, true ) then

			error( "luaValueToSqliteValueAndType returned incorrect value (see debug log for mismatch location)", 2 )
		
		end
	
	else
	
		if actualLuaValue ~= luaValue then
			error( string.format(
				"luaValueToSqliteValueAndType returned incorrect value: actual %s, expected %s",
					actualLuaValue, luaValue ), 2 )
		end

	end

end


--------------------------------------------------------------------------------

function WFSqliteUtilsTest.testNil()

	testValuePair( nil, nil, nil )

end

--------------------------------------------------------------------------------

function WFSqliteUtilsTest.testBoolean()

	testValuePair( false, 'false', 'boolean' )
	testValuePair( true, 'true', 'boolean' )

end

--------------------------------------------------------------------------------

function WFSqliteUtilsTest.testString()

	testValuePair( '', '', nil )
	testValuePair( 'false', 'false', nil )
	testValuePair( 'true', 'true', nil )
	testValuePair( '42', '42', nil )
	testValuePair( '{}', '{}', nil )

end

--------------------------------------------------------------------------------

function WFSqliteUtilsTest.testNumber()

	testValuePair( 0, 0, nil )
	testValuePair( 42, 42, nil )

end

--------------------------------------------------------------------------------

function WFSqliteUtilsTest.testTable()

	testValuePair( {}, '{\n}', 'table' )
	testValuePair( { a = 42 }, '{\n\ta = 42,\n}', 'table' )

end

--------------------------------------------------------------------------------

function WFSqliteUtilsTest.testTableNameFromCreateTableStatement()

	local fn = WFSqliteUtils.tableNameFromCreateTableStatement

	KSTestUtils.testResults( fn, 'CREATE TABLE blah' )( nil )
	KSTestUtils.testResults( fn, 'Create Table blah( foo )' )( 'blah' )
	KSTestUtils.testResults( fn, 'CREATE TABLE blah( foo )' )( 'blah' )
	KSTestUtils.testResults( fn, 'CREATE TABLE blah( foo ) bar( blop )' )( 'blah' )

end

--------------------------------------------------------------------------------

return WFSqliteUtilsTest
