--[[----------------------------------------------------------------------------

WFSyncAgentTest.lua

@author scouten 2013-04-11

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2013 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.core.self_test
local WFAsyncTestCase = require 'WFAsyncTestCase'

	-- WF.oz_client.doc_store
local WFDocStore = require 'WFDocStore'

	-- WF.oz_client.sync_agent
local WFSyncAgent = require 'WFSyncAgent'

	-- WF.oz_client.sync_agent.test
local WFTestSyncPeer = require 'WFTestSyncPeer'


--============================================================================--

local WFSyncAgentTest = {}

local docStore, testSyncPeer, syncAgent

--------------------------------------------------------------------------------

function WFSyncAgentTest.setUp()

	docStore = WFDocStore( ':memory:' )

	testSyncPeer = WFTestSyncPeer()

	syncAgent = WFSyncAgent( docStore, testSyncPeer, 'test' )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.tearDown()

	if syncAgent then
		syncAgent:close()
		syncAgent = nil
	end

	if testSyncPeer then
		testSyncPeer:close()
		testSyncPeer = nil
	end

	if docStore then
		docStore:close()
		docStore = nil
	end

end

--------------------------------------------------------------------------------

-- @mark :getChangesP

--------------------------------------------------------------------------------

function WFSyncAgentTest.testGetChangesP_noop( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			return syncAgent:getChangesP()
		
		end )
		:continueWhenValueMatches( 'data', false )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testGetChangesP_noopTwice( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			return syncAgent:getChangesP()
		
		end )
		:continueWhenValueMatches( 'data', false )

		:waitFor( function()

			return syncAgent:getChangesP()
		
		end )
		:continueWhenValueMatches( 'data', false )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testGetChangesP_overlappingRequestsCoalesced( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			testSyncPeer:delayNextChangedDocsP( .25 )

			local p1 = syncAgent:getChangesP()
			local p2 = syncAgent:getChangesP()
			
			assert( p1 == p2, "requests weren't coalesced" )
			
			return p1
		
		end )
		:continueWhenValueMatches( 'data', false )

end

--------------------------------------------------------------------------------

local function mapChangedDocs_stripHashAndSyncStatus( _, changedDocs )

	for i = 1, #changedDocs do
	
		local docValue = changedDocs[ i ]
		
		docValue._hash = nil
		docValue._syncStatus = nil
	
	end

	return 'data', changedDocs

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testGetChangesP_oneDoc( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			return syncAgent:getChangesP()
		
		end )
		:continueWhenValueMatches( 'data', false )

		:waitFor( function()
			
			return testSyncPeer:docStore():setDocValueP
				{
					_id = 'testDocId',
					_slot = 'main',
					foo = 42,
				}
			
		end )
		
		-- :getChangesP() should signal activity this time.
		
		:waitFor( function()

			return syncAgent:getChangesP()
		
		end )
		:continueWhenValueMatches( 'data', true )

		-- And we should see doc values in main database.
		
		:waitFor( function()
		
			return docStore:changedDocsP
				{
					fromChangeCounter = 0,
					slot = 'main',
				}
				:mapDataP( mapChangedDocs_stripHashAndSyncStatus )
		
		end )
		:continueWhenValueMatches( 'data',
			{
				continueFrom = 3,
				{
					_id = 'testDocId',
					_slot = 'main',
					_changeCounter = 2,
					foo = 42,
				}
			} )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testGetChangesP_gripeResponseNotTable( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			testSyncPeer:overrideNextChangedDocsPResponse( 'data', 42 )

			return syncAgent:getChangesP()
			
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testGetChangesP_gripeResponseItemNotTable( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			testSyncPeer:overrideNextChangedDocsPResponse( 'data', 
				{
					continueFrom = 99,
					"not a table",
				} )

			return syncAgent:getChangesP()
			
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testGetChangesP_gripeResponseItemMissingId( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			testSyncPeer:overrideNextChangedDocsPResponse( 'data', 
				{
					continueFrom = 99,
					{},
				} )

			return syncAgent:getChangesP()
			
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testGetChangesP_gripeResponseItemIdEmptyString( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			testSyncPeer:overrideNextChangedDocsPResponse( 'data', 
				{
					continueFrom = 99,
					{ _id = '' },
				} )

			return syncAgent:getChangesP()
			
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testGetChangesP_gripeResponseItemIdNotString( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			testSyncPeer:overrideNextChangedDocsPResponse( 'data', 
				{
					continueFrom = 99,
					{ _id = 56 },
				} )

			return syncAgent:getChangesP()
			
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testGetChangesP_gripeResponseItemTimestampMissing( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			testSyncPeer:overrideNextChangedDocsPResponse( 'data', 
				{
					continueFrom = 99,
					{ _id = 'blah' },
				} )

			return syncAgent:getChangesP()
			
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testGetChangesP_gripeResponseItemTimestampNotStringOrNumber( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			testSyncPeer:overrideNextChangedDocsPResponse( 'data', 
				{
					continueFrom = 99,
					{ _id = 'blah', _timestamp = false },
				} )

			return syncAgent:getChangesP()
			
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

end

--------------------------------------------------------------------------------

-- @mark :sendUpdatesP

--------------------------------------------------------------------------------

function WFSyncAgentTest.testSendUpdatesP_noop( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			return syncAgent:sendUpdatesP()
		
		end )
		:continueWhenValueMatches( 'data', false )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testSendUpdatesP_noopTwice( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			return syncAgent:sendUpdatesP()
		
		end )
		:continueWhenValueMatches( 'data', false )

		:waitFor( function()

			return syncAgent:sendUpdatesP()
		
		end )
		:continueWhenValueMatches( 'data', false )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testSendUpdatesP_overlappingRequestsCoalesced( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			-- testSyncPeer:delayNextChangedDocsP( .25 )

			local p1 = syncAgent:sendUpdatesP()
			local p2 = syncAgent:sendUpdatesP()
			
			assert( p1 == p2, "requests weren't coalesced" )
			
			return p1
		
		end )
		:continueWhenValueMatches( 'data', false )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testSendUpdatesP_gripeMaxUpdatesResponseNotNumber( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			testSyncPeer:overrideNextMaxUpdatesPerSendResponse( 'bogus' )

			return syncAgent:sendUpdatesP()
			
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testSendUpdatesP_gripeMaxUpdatesResponseNegative( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			testSyncPeer:overrideNextMaxUpdatesPerSendResponse( -42 )

			return syncAgent:sendUpdatesP()
			
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testSendUpdatesP_gripeMaxUpdatesResponseZero( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			testSyncPeer:overrideNextMaxUpdatesPerSendResponse( 0 )

			return syncAgent:sendUpdatesP()
			
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testSendUpdatesP_gripeMaxUpdatesResponseNotInteger( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			testSyncPeer:overrideNextMaxUpdatesPerSendResponse( 3.14159 )

			return syncAgent:sendUpdatesP()
			
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testSendUpdatesP_oneDoc( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			return syncAgent:sendUpdatesP()
		
		end )
		:continueWhenValueMatches( 'data', false )

		:waitFor( function()
			
			return docStore:setDocValueP
				{
					_id = 'testDocId',
					_slot = 'main',
					foo = 42,
				}
			
		end )
		
		-- :sendUpdatesP() should signal activity this time.
		
		:waitFor( function()

			return syncAgent:sendUpdatesP()
		
		end )
		:continueWhenValueMatches( 'data', true )

		-- And we should see doc values in remote database.
		
		:waitFor( function()
		
			return testSyncPeer:docStore():changedDocsP
				{
					fromChangeCounter = 0,
					slot = 'main',
				}
				:mapDataP( mapChangedDocs_stripHashAndSyncStatus )
		
		end )
		:continueWhenValueMatches( 'data',
			{
				continueFrom = 2,
				{
					_id = 'testDocId',
					_slot = 'main',
					_changeCounter = 1,
					foo = 42,
				}
			} )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testSendUpdatesP_oneDoc_errorPeerFailure( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			return syncAgent:sendUpdatesP()
		
		end )
		:continueWhenValueMatches( 'data', false )

		:waitFor( function()
			
			return docStore:setDocValueP
				{
					_id = 'testDocId',
					_slot = 'main',
					foo = 42,
				}
			
		end )
		
		-- :sendUpdatesP() responds with failure.
		
		:waitFor( function()

			testSyncPeer:overrideNextSendUpdatesPResponse( 'error', 'failfailfail' )

			return syncAgent:sendUpdatesP()
		
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testSendUpdatesP_oneDoc_errorPerDocFailure( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			return syncAgent:sendUpdatesP()
		
		end )
		:continueWhenValueMatches( 'data', false )

		:waitFor( function()
			
			return docStore:setDocValueP
				{
					_id = 'testDocId',
					_slot = 'main',
					foo = 42,
				}
			
		end )
		
		-- :sendUpdatesP() responds with per-document failure.
		
		:waitFor( function()

			testSyncPeer:partiallyFailNextSendUpdatesPResponse
				{
					testDocId = "totally failed, dude!",
				}

			return syncAgent:sendUpdatesP()
		
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testSendUpdatesP_oneDoc_gripePerDocFailureTableEmpty( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			return syncAgent:sendUpdatesP()
		
		end )
		:continueWhenValueMatches( 'data', false )

		:waitFor( function()
			
			return docStore:setDocValueP
				{
					_id = 'testDocId',
					_slot = 'main',
					foo = 42,
				}
			
		end )
		
		-- :sendUpdatesP() responds with per-document failure.
		
		:waitFor( function()

			testSyncPeer:overrideNextSendUpdatesPResponse( 'error', {} )

			return syncAgent:sendUpdatesP()
		
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

		-- Can't test for this, but error message should say something
		-- about empty error table response.

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testSendUpdatesP_oneDoc_gripePerDocFailureTableMalformed( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()

			return syncAgent:sendUpdatesP()
		
		end )
		:continueWhenValueMatches( 'data', false )

		:waitFor( function()
			
			return docStore:setDocValueP
				{
					_id = 'testDocId',
					_slot = 'main',
					foo = 42,
				}
			
		end )
		
		-- :sendUpdatesP() responds with per-document failure.
		
		:waitFor( function()

			testSyncPeer:overrideNextSendUpdatesPResponse( 'error', { 'abc', 'def' } )
				-- Error response table *should* be [id] = message mapping.
	
			return syncAgent:sendUpdatesP()
		
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

		-- Can't test for this, but error message should say something
		-- about empty error table response.

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testSendUpdatesP_twoDocs( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
			
			return docStore:setDocValueP
				{
					_id = 'testDocId',
					_slot = 'main',
					foo = 42,
				}
			
		end )
		
		:waitFor( function()
			
			return docStore:setDocValueP
				{
					_id = 'testDocTwo',
					_slot = 'main',
					blah = 79,
				}
			
		end )
		
		-- :sendUpdatesP() should signal activity this time.
		
		:waitFor( function()

			return syncAgent:sendUpdatesP()
		
		end )
		:continueWhenValueMatches( 'data', true )

		-- And we should see doc values in remote database.
		
		:waitFor( function()
		
			return testSyncPeer:docStore():changedDocsP
				{
					fromChangeCounter = 0,
					slot = 'main',
				}
				:mapDataP( mapChangedDocs_stripHashAndSyncStatus )
		
		end )
		:continueWhenValueMatches( 'data',
			{
				continueFrom = 3,
				{
					_id = 'testDocId',
					_slot = 'main',
					_changeCounter = 1,
					foo = 42,
				},
				{
					_id = 'testDocTwo',
					_slot = 'main',
					_changeCounter = 2,
					blah = 79,
				}
			} )

end

--------------------------------------------------------------------------------

function WFSyncAgentTest.testSendUpdatesP_twoDocs_partialFailure( test )

	WFAsyncTestCase( test )
	
		:waitFor( function()
			
			return docStore:setDocValueP
				{
					_id = 'testDocId',
					_slot = 'main',
					foo = 42,
				}
			
		end )
		
		:waitFor( function()
			
			return docStore:setDocValueP
				{
					_id = 'testDocTwo',
					_slot = 'main',
					blah = 79,
				}
			
		end )
		
		-- :sendUpdatesP() configured to succeed on one doc, fail the other.
		
		:waitFor( function()

			testSyncPeer:partiallyFailNextSendUpdatesPResponse
				{
					testDocId = "totally failed, dude!",
				}

			return syncAgent:sendUpdatesP()
		
		end )
		:continueWhenValueMatches( 'error', ':ignore:' )

		-- And we should see doc values in remote database.
		
		:waitFor( function()
		
			return testSyncPeer:docStore():changedDocsP
				{
					fromChangeCounter = 0,
					slot = 'main',
				}
				:mapDataP( mapChangedDocs_stripHashAndSyncStatus )
		
		end )
		:continueWhenValueMatches( 'data',
			{
				continueFrom = 2,
				{
					_id = 'testDocTwo',
					_slot = 'main',
					_changeCounter = 1,
					blah = 79,
				}
			} )

end

--------------------------------------------------------------------------------

return WFSyncAgentTest
