--[[----------------------------------------------------------------------------

WFTestMetrics.lua

@author ppaurav 2013-02-22

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

local WFPlatformInfo
local WFPlatformInfoImp

--============================================================================--
--[[!
	\class WFTestMetrics
	\brief This class provides a way to capture a
		metrics about hardware and software where tests are run.
]]--

local WFTestMetrics = {}

--------------------------------------------------------------------------------

function WFTestMetrics.populateTestMetrics( test )

	local KSDeviceUtils = require 'KSDeviceUtils'
	local KSPathUtils = require 'KSPathUtils'
	
	if not WFPlatformInfo then
		WFPlatformInfo = require 'WFPlatformInfo'
	end
	
	if not WFPlatformInfoImp then
		WFPlatformInfoImp = require 'WFPlatformInfoImp'
	end

	local metrics = {}

	local fileName = KSPathUtils.leafName( test:fileName() )
	local fileNameLen = fileName:len()
	local className = string.sub( fileName, -fileNameLen, -5 ) -- stripping .lua
	
	metrics[ "wfp_file_name" ] = fileName
	metrics[ "wfp_class_name" ] = className
	metrics[ "wfp_test_name" ] = test:testName()
	metrics[ "product_name" ] = "WichitaFoundation"
	metrics[ "product_family" ] = "Revel"
	metrics[ "branch" ] = WFPlatformInfo.getBranchName()
	metrics[ "platform" ] = WFPlatformInfo.getAppDeviceClass()
	metrics[ "build_target" ] = WFPlatformInfo.getBuildTarget()
	metrics[ "changelist" ] = WFPlatformInfo.getBuild()
	metrics[ "test_machine" ] = KSDeviceUtils.deviceName()
	metrics[ "model" ] = KSDeviceUtils.deviceModel()
	metrics[ "os" ] = WFPlatformInfo.getOS()
	metrics[ "cpu" ] = KSDeviceUtils.cpuType()
	metrics[ "os_version" ] = WFPlatformInfoImp.getOSVersion()
	metrics[ "test" ] = "Performance"
	metrics[ "name" ] = KSDeviceUtils.userName()
	
	return metrics

end

--------------------------------------------------------------------------------

function WFTestMetrics.writeTestMetrics( metrics )

	local KSJsonEncoder = require 'KSJsonEncoder'
	local KSData = require 'KSData'
	local KSPathUtils = require 'KSPathUtils'
	local testMetrics = {}
	
	testMetrics[ "wf_performance_test" ] = { metrics }

	local jsonData = KSJsonEncoder.prettyPrint( testMetrics )
	jsonData = KSData.make( jsonData )
	local basePath = KSPathUtils.getStandardFilePath( 'app' )
	local fileName = metrics [ "wfp_test_name" ]
	fileName = string.format( "%s.%s", fileName, 'json' )
	if jsonData then
		KSData.saveToUniquelyNamedFile( jsonData, KSPathUtils.child( basePath, fileName ) )
	end
	
end

--------------------------------------------------------------------------------

return WFTestMetrics
