--[[----------------------------------------------------------------------------

WFTimerTest.lua


--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2012 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

--------------------------------------------------------------------------------

@author scouten 2010-09-27

------------------------------------------------------------------------------]]

	-- wichitafoundation.core.lua.threading
local WFTimer = require 'WFTimer'

	-- wichitafoundation.core.os
local KSDate = require 'KSDate'

	-- wichitafoundation.core.utils
local KSTestUtils = require 'KSTestUtils'


--------------------------------------------------------------------------------

local WFTimerTest = {}

--------------------------------------------------------------------------------

function WFTimerTest.testRejectsNegativeDelay()

	KSTestUtils.assertFails( function()
		
		WFTimer.afterDelayP( -1 )
	
	end )

end

--------------------------------------------------------------------------------

function WFTimerTest.testAfterDelayP( test )

	local startTime = KSDate.timeNowForAnimation()
	local secondTimerElapsedTime = 0

	-- NOTE: Intentionally not using WFAsyncTestCase here because that class
	-- depends on WFIdle.onIdleP. Need an independent test case.
	
	WFTimer.afterDelayP( .5 )
		:whenResolvedDo( function( status, blah )
			
			if status == 'data' and blah == true then

				local elapsedTime = KSDate.timeNowForAnimation() - startTime
				
				if secondTimerElapsedTime == 0 then
					test:log( "timers are blocking" )
					test:notifyTestDidFail()
				elseif elapsedTime < .5 or secondTimerElapsedTime < .1 then
					test:log( "promises resolved too soon" )
					test:notifyTestDidFail()
				elseif elapsedTime > .75 or secondTimerElapsedTime > .35 then
					test:log( "promises resolved too late" )
					test:notifyTestDidFail()
				else
					test:notifyTestDidSucceed()
				end

			else
				test:log( "Unexpected resolution on WFIdle.onIdleP: ", status, blah )
				test:notifyTestDidFail()
			end
		
		end )
		:runUntilDone()
		
	WFTimer.afterDelayP( .1 )
		:whenResolvedDo( function( status, blah )
			
			if status == 'data' and blah == true then
				secondTimerElapsedTime = KSDate.timeNowForAnimation() - startTime
			else
				test:log( "Unexpected resolution on WFIdle.onIdleP: ", status, blah )
				test:notifyTestDidFail()
			end
		
		end )
		:runUntilDone()
		
	
	test:waitForTestToFinish( 4 )

end

--------------------------------------------------------------------------------

function WFTimerTest.testAfterDelayDo( test )

	local startTime = KSDate.timeNowForAnimation()

	WFTimer.afterDelayDo( .25,
		function( status, blah )
			
			local elapsedTime = KSDate.timeNowForAnimation() - startTime
			
			if elapsedTime < .25 then
				test:log( "function called too soon" )
				test:notifyTestDidFail()
			elseif elapsedTime > .5 then
				test:log( "function called too late" )
				test:notifyTestDidFail()
			else
				test:notifyTestDidSucceed()
			end

		end )
	
	test:waitForTestToFinish( 3 )

end

--------------------------------------------------------------------------------

return WFTimerTest
